/*****************************************************************************
 *  Rohde&Schwarz SFU Broadcast Test System instrument driver     
 *  VXIpnp Style Instrument Driver
 *  Original Release: 10/2005
 *  By: Jiri Kominek
 *
 *  Should you have any technical questions please contact the hotline of
 *  Rohde & Schwarz Vertriebs-GmbH Rohde & Schwarz Support Center
 *
 *  e-mail: CustomerSupport@rohde-schwarz.com
 *
 *
 *  Modification History:
 *    see rssfu_vxi.chm
 *****************************************************************************/

#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "rssfu.h"

/*****************************************************************************
 *- Useful Macros & Constants -----------------------------------------------*
 *****************************************************************************/

#define RSSFU_SIMULATION_ID_QUERY      "Rohde&Schwarz,SFU,2110.2500k02/101251,02.80.00.08"  /* Default definition of ID Query Response for simulation */
#define RSSFU_VALID_ID_RESPONSE_STRING "Rohde&Schwarz"      /* Valid response for identification query */
#define RSSFU_SIMULATION_OPT_QUERY     "SFU-B1,SFU-B3,SFU-B4,SFU-B30"      /* Simulated response for *OPT? command */

#define RSSFU_IO_BUFFER_SIZE           1024L   /* I/O buffer size */
#define RSSFU_OPC_TIMEOUT              30000L   /* Maximum time to wait for OPC in milliseconds */
#define IEEE_488_2_ERROR_MASK          0x3C    /* 488.2 Event Status Register (ESR) Bits */
#define RSSFU_TOKEN_BUFFER_SIZE        50      /* Size for rep capability values */
#define MAX_OPC_TIME                   60000L 


/*****************************************************************************
 *- Value Definition and Range Tables ---------------------------------------*
 *****************************************************************************/

/*****************************************************************************/
/*= INSTRUMENT-DEPENDENT COMMAND ARRAYS =====================================*/
/*****************************************************************************/

static ViString  noiseOffsetArr[] = {"1e2","1e3","1e4","1e5","1e6"};
static ViString  stateArr[] = {"OFF","ON"};  
static ViString  statusRegisterArr[] = {"COND","EVEN","ENAB","PTR","NTR"};
static ViString  waveformFileTagArr[] = {"TYPE","CRESTFACTOR","LEVEL OFFS","CLOCK","SAMPLES","IQ SWAP","DATE","COMMENT"};
static ViString  cmmbPlchRsArr[] = {"0","1","2","3"};      
static ViString  cmmbPlchLdpdArr[] = {"R1_2","R3_4"};   
static ViString  cmmbPlchConstArr[] = {"C2","C4","C16"};

/*****************************************************************************
 *- Utility Function Declarations (Non-Exported) ----------------------------*
 *****************************************************************************/

static ViStatus rssfu_InitAttributes (ViSession vi);
static ViStatus rssfu_DefaultInstrSetup (ViSession openInstrSession);
static ViStatus rssfu_CheckStatus (ViSession vi);
static ViStatus rssfu_WaitForOPC (ViSession vi, ViInt32 maxTime);
static ViStatus rssfu_ReadDataUnknownLength(ViSession instrSession,
                ViChar **outBuffer, ViUInt32 *count);
static ViStatus rssfu_ReplaceString(char *str, char *orig, char *rep);


static ViStatus rssfu_GetActualRepCaps (ViSession instrSession, ViAttr attribute, 
                                        ViString repCaps, ViInt32 index,ViChar retVal[]);
/*****************************************************************************
 *- Callback Declarations ---------------------------------------------------*
 *****************************************************************************/

ViStatus rssfu_CheckStatusCallback (ViSession io);
ViStatus rssfu_WaitForOPCCallback (ViSession io);

/*****************************************************************************
 *- User-Callable Functions (Exportable Functions) --------------------------*
 *****************************************************************************/

/*****************************************************************************
 * Function: rssfu_init
 * Purpose:  VXIplug&play required function. Calls the
 *           rssfu_InitWithOptions function.
 *****************************************************************************/
ViStatus _VI_FUNC rssfu_init (ViRsrc resourceName, ViBoolean IDQuery,
                                 ViBoolean resetDevice, ViSession *newInstrSession)
{
    ViStatus    error = VI_SUCCESS;

    if (newInstrSession == VI_NULL)
        {
        (void) Rs_SetErrorInfo (VI_NULL, VI_FALSE, RS_ERROR_INVALID_PARAMETER,
                          VI_ERROR_PARAMETER4, "Null address for Instrument Handle");
        checkErr( RS_ERROR_INVALID_PARAMETER);
        }

    checkErr( rssfu_InitWithOptions (resourceName, IDQuery, resetDevice,
                                      "QueryInstrStatus=1", newInstrSession));

Error:
    return error;
}

/*****************************************************************************
 * Function: rssfu_InitWithOptions
 * Purpose:  This function creates a new RS session and calls the
 *           RsInit function.
 *****************************************************************************/
ViStatus _VI_FUNC rssfu_InitWithOptions(
    ViRsrc          resourceName,
    ViBoolean       IDQuery,
    ViBoolean       resetDevice,
    ViString        optionString,
    ViSession      *newInstrSession
)
{
    ViStatus                error = VI_SUCCESS;
    ViChar                  newResourceName[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar                  newOptionString[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViBoolean               isLogicalName = 0;
    ViSession               instrSession = 0, rmSession = 0;
    RsAttrPropertiesPtr     *attrList=NULL;
    RsRepCapPtr             repCapTable = NULL;

    if (newInstrSession == VI_NULL)
        {
        (void) Rs_SetErrorInfo (VI_NULL, VI_FALSE, RS_ERROR_INVALID_PARAMETER,
                         VI_ERROR_PARAMETER5, "Null address for Instrument Handle");
        checkErr( RS_ERROR_INVALID_PARAMETER);
        }

    /* Prepare resource manager session handle */
    checkErr (viOpenDefaultRM (&rmSession));

    /* Establish communication session with the instrument */

    viCheckErr( viOpen (rmSession, resourceName, VI_NULL, VI_NULL, &instrSession));
    
    /* Configure VISA Formatted I/O */
    viCheckErr( viSetAttribute (instrSession, VI_ATTR_TMO_VALUE, 30000 ));
    viCheckErr( viSetBuf (instrSession, (ViUInt16) (VI_READ_BUF | VI_WRITE_BUF), 4000));
    viCheckErr( viSetAttribute (instrSession, VI_ATTR_WR_BUF_OPER_MODE, VI_FLUSH_ON_ACCESS));
    viCheckErr( viSetAttribute (instrSession, VI_ATTR_RD_BUF_OPER_MODE, VI_FLUSH_ON_ACCESS));

    checkErr( Rs_GetInfoFromResourceName (resourceName,
                                          (ViString)optionString,
                                          newResourceName,
                                          newOptionString,
                                          &isLogicalName));

    ATTRLIST(attrList,rssfu); 
    checkErr( Rs_SpecificDriverNew (instrSession, "rssfu", newOptionString, attrList));

    /* Build repeated capability table */
    REPCAPTABLE(repCapTable,rssfu);
    checkErr( Rs_BuildRepCapTable (instrSession, repCapTable));

    checkErr (Rs_SetAttribute (instrSession, "", RS_ATTR_VISA_RM_SESSION, 0, &rmSession));

    /* With remote control via Ethernet, remote control is not automatically
       set by means of a command. The instrument must be explicitly set to
       the REMOTE state, e.g. by sending the intf command &GTR
    */
    checkErr (viWrite (instrSession, (ViBuf) "&GTR\n", 5, NULL));
    
    /*- Reset instrument ----------------------------------------------------*/
    if (resetDevice == VI_TRUE)
    {
        checkErr( rssfu_reset (instrSession));
    }
    else  /*- Send Default Instrument Setup ---------------------------------*/
        checkErr( rssfu_DefaultInstrSetup (instrSession));

    /*- Identification Query ------------------------------------------------*/
    if (IDQuery == VI_TRUE)
        {
        ViChar rdBuffer[RSSFU_IO_BUFFER_SIZE] = "";

        checkErr( Rs_GetAttribute (instrSession, VI_NULL, (ViAttr) RSSFU_ATTR_ID_QUERY_RESPONSE,
                                   0, RSSFU_IO_BUFFER_SIZE, rdBuffer));

        if (strncmp (rdBuffer, RSSFU_VALID_ID_RESPONSE_STRING, strlen(RSSFU_VALID_ID_RESPONSE_STRING)) != 0)
            {
            viCheckErr( VI_ERROR_FAIL_ID_QUERY);
            }
        }

    if (isLogicalName == VI_FALSE)
        {
        ViInt32 oldFlag = 0;

        checkErr (Rs_GetAttributeFlags (instrSession, RS_ATTR_IO_RESOURCE_DESCRIPTOR, &oldFlag));
        checkErr (Rs_SetAttributeFlags (instrSession, RS_ATTR_IO_RESOURCE_DESCRIPTOR, oldFlag & 0xfffb | 0x0010));
        checkErr (Rs_SetAttribute (instrSession, "", RS_ATTR_IO_RESOURCE_DESCRIPTOR, 0, newResourceName));
        }
    

    if (isLogicalName == VI_TRUE)
        checkErr( Rs_ApplyDefaultSetup (instrSession));

    /* Instrument session accessible by the end user is native VISA session,
       or when simulated virtual instrument (vi) session. */
    checkErr( rssfu_CheckStatus (instrSession));

Error:

    if (error < VI_SUCCESS)
    {
        ViSession   rmSess = VI_NULL;

        (void) Rs_GetAttribute (instrSession, VI_NULL, RS_ATTR_VISA_RM_SESSION, 0, (ViInt32) sizeof (rmSess), &rmSess);
        
        if (instrSession != 0)
            (void) viClose (instrSession);
        (void) viClose (rmSess);
    }
    else
    {
        *newInstrSession = instrSession;
    }

    if (error < VI_SUCCESS)
        (void) Rs_Dispose (*newInstrSession);

    return error;
}

/*****************************************************************************
 * Function: rssfu_close
 * Purpose:  This function closes the instrument.
 *
 *           Note:  This function must unlock the session before calling
 *           Rs_Dispose.
 *****************************************************************************/
ViStatus _VI_FUNC rssfu_close (ViSession instrSession)
{
    ViStatus    error   = VI_SUCCESS;

    // checkErr( Rs_LockSession (vi, VI_NULL));
    
    checkErr( rssfu_RsClose (instrSession));

Error:

    // Rs_UnlockSession (vi, VI_NULL);
    (void) Rs_Dispose (instrSession);

    return error;
}

/*****************************************************************************
 * Function: rssfu_RsClose
 * Purpose:  This function performs all of the drivers clean-up operations
 *           except for closing the RS session.  This function is called by
 *           rssfu_close or by an RS class driver.
 *
 *           Note:  This function must close the I/O session and set
 *           RS_ATTR_IO_SESSION to 0.
 *****************************************************************************/
ViStatus _VI_FUNC rssfu_RsClose (ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;
    ViSession   rmSession = VI_NULL;

    /* Do not lock here. The caller manages the lock. */

    (void) Rs_SetAttribute (instrSession, VI_NULL, RS_ATTR_IO_SESSION, 0, VI_NULL);
    (void) Rs_GetAttribute (instrSession, VI_NULL, RS_ATTR_VISA_RM_SESSION, 0, (ViInt32) sizeof (rmSession), &rmSession);
        
    (void) viClose (instrSession);
    if (rmSession != 0)
        (void) viClose (rmSession);
   
    return error;
}


ViStatus _VI_FUNC rssfu_SetErrorInfo (ViSession session,
                          ViBoolean overWrite,
                          ViStatus primaryError,
                          ViStatus secondaryError,
                          ViConstString errorElaboration)
{
    return Rs_SetErrorInfo (session,
                          overWrite,
                          primaryError,
                          secondaryError,
                          errorElaboration);
}

ViStatus _VI_FUNC rssfu_GetErrorInfo (ViSession session,
                          ViStatus *primaryError,
                          ViStatus *secondaryError,
                          ViChar errorElaboration[])
{
    return Rs_GetErrorInfo (session,
                          primaryError,
                          secondaryError,
                          errorElaboration);
}



/*****************************************************************************
 * Function: rssfu_reset
 * Purpose:  This function resets the instrument.
 *****************************************************************************/
ViStatus _VI_FUNC rssfu_reset (ViSession instrSession)
{
    ViStatus    error   = VI_SUCCESS;
    ViUInt32    tout    = 0;

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (Rs_Simulating(instrSession) == 0)                /* call only when locked */
        {
            viCheckErr (viGetAttribute (instrSession, VI_ATTR_TMO_VALUE, &tout));
            viCheckErr (viSetAttribute (instrSession, VI_ATTR_TMO_VALUE, 30000));
          /*
            viCheckErr (Rs_WriteInstrData (instrSession, "*RST;*OPC?"));
            viCheckErr (viScanf (instrSession, "%hu", &opc));
            */
                 
            viCheckErr (Rs_WriteInstrData (instrSession, "*RST;*CLS;*WAI")); 
            
            viCheckErr (viSetAttribute (instrSession, VI_ATTR_TMO_VALUE, tout));
        }

    checkErr( rssfu_DefaultInstrSetup (instrSession));
    checkErr( rssfu_CheckStatus (instrSession));

Error:

    (void) Rs_UnlockSession (instrSession, VI_NULL);

    return error;
}

/*****************************************************************************
 * Function: rssfu_ResetWithDefaults
 * Purpose:  This function resets the instrument and applies default settings
 *           from the RS Configuration Store based on the logical name
 *           from which the session was created.
 *****************************************************************************/
ViStatus _VI_FUNC rssfu_ResetWithDefaults (ViSession instrSession)
{
    ViStatus    error   = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    checkErr( rssfu_reset(instrSession));
    checkErr( Rs_ApplyDefaultSetup(instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/****************************************************************************
 *  Function: rssfu_Disable
 *  Purpose:  This function places the instrument in a quiescent state as
 *            quickly as possible.
 ****************************************************************************/
ViStatus _VI_FUNC rssfu_Disable (ViSession instrSession)
{
    ViStatus    error   = VI_SUCCESS;
    instrSession;
    
/*    checkErr( Rs_LockSession (vi, VI_NULL));

    if (Rs_Simulating(vi) == 0)
        {
        }
Error:
//    (void) Rs_UnlockSession (vi, VI_NULL);
        */
    return error;
}

/*****************************************************************************
 * Function: rssfu_self_test
 * Purpose:  This function executes the instrument self-test and returns the
 *           result.
 *****************************************************************************/
ViStatus _VI_FUNC rssfu_self_test (ViSession instrSession, ViInt16 *testResult,
                                      ViChar testMessage[])
{
    ViStatus    error   = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (testResult == VI_NULL)
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Null address for Test Result")
    if (testMessage == VI_NULL)
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 3, "Null address for Test Message");

    if (Rs_Simulating(instrSession) == 0)                /* call only when locked */
        {
        viCheckErr( viPrintf (instrSession, "*TST?"));
        viCheckErr( viScanf (instrSession, "%hd,\"%256[^\"]", testResult, testMessage));

        }
    else
        {
        /* Simulate Self Test */
        if (testResult != NULL)
            *testResult = 0;
        strcpy (testMessage, "No error.");
        }

    checkErr( rssfu_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: rssfu_revision_query
 * Purpose:  This function returns the driver and instrument revisions.
 *****************************************************************************/
ViStatus _VI_FUNC rssfu_revision_query (ViSession instrSession, ViChar driverRev[],
                                           ViChar instrRev[])
{
    ViStatus    error   = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (driverRev == VI_NULL)
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Null address for Driver Revision");
    if (instrRev == VI_NULL)
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 3, "Null address for Instrument Revision");

    checkErr( Rs_GetAttribute (instrSession, VI_NULL, RSSFU_ATTR_SPECIFIC_DRIVER_REVISION,
                                        0, 256, driverRev));
    checkErr( Rs_GetAttribute (instrSession, "", RSSFU_ATTR_INSTRUMENT_FIRMWARE_REVISION,
                                        0, 256, instrRev));
    checkErr( rssfu_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: rssfu_error_query
 * Purpose:  This function queries the instrument error queue and returns
 *           the result.
 *****************************************************************************/
ViStatus _VI_FUNC rssfu_error_query (ViSession instrSession, ViInt32 *errCode,
                                        ViChar errMessage[])
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer [256] = "SYST:ERR?\n";

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (errCode == VI_NULL)
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Null address for Error Code");
    if (errMessage == VI_NULL)
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 3, "Null address for Error Message");

    if (Rs_Simulating(instrSession) == 0)                /* call only when locked */
    {
        viCheckErr (viWrite (instrSession, (ViBuf) buffer, (ViUInt32)strlen (buffer), NULL));
        viCheckErr (viRead (instrSession, (ViBuf) buffer, 255, NULL));
        sscanf (buffer, "%ld,\"%255[^\"]", errCode, errMessage);
    }
    else
    {
            /* Simulate Error Query */
        if (errCode  != NULL)
            *errCode = 0;
        strcpy (errMessage, "No error.");
    }

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: rssfu_error_message
 * Purpose:  This function translates the error codes returned by this
 *           instrument driver into user-readable strings.
 *
 *           Note:  The caller can pass VI_NULL for the vi parameter.  This
 *           is useful if one of the init functions fail.
 *****************************************************************************/
ViStatus _VI_FUNC rssfu_error_message (ViSession instrSession, ViStatus errorCode,
                                          ViChar errorMessage[])
{
    ViStatus    error   = VI_SUCCESS;

    static      RsStringValueTable errorTable =
        {
 
            RSSFU_ERROR_CODES_AND_MSGS,
            {VI_NULL,                               VI_NULL}
        };

    (void) Rs_LockSession(instrSession, VI_NULL);

        /* all VISA and RS error codes are handled as well as codes in the table */
    if (errorMessage == VI_NULL)
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 3, "Null address for Error Message");

    checkErr( Rs_GetSpecificDriverStatusDesc(instrSession, errorCode, errorMessage, errorTable));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: rssfu_GetError and rssfu_ClearError Functions
 * Purpose:  These functions enable the instrument driver user to
 *           get or clear the error information the driver associates with the
 *           RS session.
 *****************************************************************************/
ViStatus _VI_FUNC rssfu_GetError (ViSession instrSession,
                                     ViStatus *errorCode,
                                     ViInt32 bufferSize,
                                     ViChar description[])
{
    ViStatus    error       = VI_SUCCESS;
    ViStatus    primary     = VI_SUCCESS,
                secondary   = VI_SUCCESS;
    ViChar      elaboration[RS_MAX_MESSAGE_BUF_SIZE]    = "";
    ViChar      errorMessage[1024]  = "";
    ViChar     *appendPoint = errorMessage;
    ViInt32     actualSize  = 0;
    ViBoolean   locked      = VI_FALSE;

    /* Lock Session */
    checkErr( Rs_LockSession(instrSession, &locked));

    /* Test for nulls and acquire error data */
    if (bufferSize != 0)
        {
        if (errorCode == VI_NULL)
            {
            viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Null address for Error");
            }
        if (description == VI_NULL)
            {
            viCheckParm( RS_ERROR_INVALID_PARAMETER, 4, "Null address for Description");
            }
        checkErr( Rs_GetErrorInfo (instrSession, &primary, &secondary, elaboration));
        }

    else
        {
        checkErr( Rs_GetAttribute (instrSession, VI_NULL, RS_ATTR_ERROR_ELABORATION, 0, RS_MAX_MESSAGE_BUF_SIZE, elaboration));
        checkErr( Rs_GetAttribute (instrSession, VI_NULL, RS_ATTR_SECONDARY_ERROR, 0, (ViInt32) sizeof (secondary), &secondary));
        checkErr( Rs_GetAttribute (instrSession, VI_NULL, RS_ATTR_PRIMARY_ERROR, 0, (ViInt32) sizeof (primary), &primary));
        }

    /* Format data */
    if (primary != VI_SUCCESS)
        {
        ViChar msg[256] = "";
        checkErr( rssfu_error_message (instrSession, primary, msg));
        appendPoint += sprintf(appendPoint, "Primary Error: (Hex 0x%08X) %s\n", (unsigned int) primary, msg);
        }

    if (secondary != VI_SUCCESS)
        {
        ViChar msg[256] = "";
        checkErr( rssfu_error_message (instrSession, secondary, msg));
        appendPoint += sprintf(appendPoint, "Secondary Error: (Hex 0x%08X) %s\n", (unsigned int) secondary, msg);
        }

    if (elaboration[0] != '\0')
        {
        sprintf(appendPoint, "Elaboration: %s", elaboration);
        }

    actualSize = (ViInt32) strlen(errorMessage) + 1;

    /* Prepare return values */
    if (bufferSize == 0)
        {
        error = actualSize;
        }
    else
        {
        if (bufferSize > 0)
            {
            if (actualSize > bufferSize)
                {
                error = actualSize;
                actualSize = bufferSize;
                }
            }
        memcpy(description, errorMessage, (size_t)(actualSize-1));
        if (description != NULL)
            description[actualSize-1] = '\0';
        }

    if (errorCode != 0)
        {
        *errorCode = primary;
        }

Error:
    /* Unlock Session */
    (void) Rs_UnlockSession (instrSession, &locked);
    return error;
}

ViStatus _VI_FUNC rssfu_ClearError (ViSession instrSession)
{
    return Rs_ClearErrorInfo (instrSession);
}

/*****************************************************************************
 * Function: rssfu_LockSession and rssfu_UnlockSession Functions
 * Purpose:  These functions enable the instrument driver user to lock the
 *           session around a sequence of driver calls during which other
 *           execution threads must not disturb the instrument state.
 *
 *           NOTE:  The callerHasLock parameter must be a local variable
 *           initialized to VI_FALSE and passed by reference, or you can pass
 *           VI_NULL.
 *****************************************************************************/
ViStatus _VI_FUNC rssfu_LockSession (ViSession instrSession, ViBoolean *callerHasLock)
{
    return Rs_LockSession(instrSession, callerHasLock);
}

ViStatus _VI_FUNC rssfu_UnlockSession (ViSession instrSession, ViBoolean *callerHasLock)
{
    return Rs_UnlockSession(instrSession,callerHasLock);
}

/*****************************************************************************
 *------------ User-Callable Functions (Exportable Functions) ---------------*
 *****************************************************************************/

/*****************************************************************************
 *------------------------- TX Application ----------------------------------*
 *****************************************************************************/
/*****************************************************************************
 *---------------------------- Frequency ------------------------------------*
 *****************************************************************************/

/*===========================================================================*
 * Function: Configure Frequency
 * Purpose:  This function configures Tx Frequency.
 *===========================================================================*/
ViStatus _VI_FUNC rssfu_confTxFreq(ViSession instrSession,
                                   ViReal64 frequency,
                                   ViReal64 frequencyOffset)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      model [RSSFU_IO_BUFFER_SIZE] = "";  

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckErr (rssfu_GetAttributeViString (instrSession, "",
                                               RS_ATTR_INSTRUMENT_MODEL,
                                               RSSFU_IO_BUFFER_SIZE, model));   
    
    if (strstr (model, "SFU") != NULL) {    
        viCheckErr( rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_FREQ_MODE, RSSFU_VAL_FREQ_MODE_CW_FIX));
    }
    
    viCheckParm( rssfu_SetAttributeViReal64 (instrSession, "", RSSFU_ATTR_FREQ_CW, frequency), 2, "Frequency");
    
    if (strstr (model, "SFC") == NULL) {
        viCheckParm( rssfu_SetAttributeViReal64 (instrSession, "", RSSFU_ATTR_FREQ_OFFSET, frequencyOffset), 3, "Frequency Offset");
    }
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Set Tx Actual Center Frequency
 * Purpose:  This function sets the RF output frequency.
/============================================================================*/
ViStatus _VI_FUNC rssfu_SetTxActualCenterFrequency (ViSession instrSession,
                                                    ViReal64 RFOutputFrequency)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    viCheckParm( rssfu_SetAttributeViReal64 (instrSession, "", RSSFU_ATTR_FREQ_ACT_CENTER, RFOutputFrequency),
                                            2, "RF Output Frequency");
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}                                                    


/*============================================================================/
 * Function: Set Tx Frequency Channel
 * Purpose:  This function sets the channel and RF output frequency.
/============================================================================*/
ViStatus _VI_FUNC rssfu_SetTxFrequencyChannel (ViSession instrSession,
                                               ViInt32 frequencyChannel)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    viCheckParm( rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_FREQ_CHANNEL, frequencyChannel),
                                            2, "Frequency Channel");
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}                                                    
                                               


/*============================================================================/
 * Function: Frequency Channel Table Operations
 * Purpose:  This function selects frequency channel table operations.
/============================================================================*/
ViStatus _VI_FUNC rssfu_FrequencyChannelTableOperations (ViSession instrSession,
                                                         ViInt32 operation,
                                                         ViString channelName)
{
    ViStatus    error   = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
  
    if (Rs_Simulating(instrSession) == 0){                /* call only when locked */
    switch (operation){
        case RSSFU_VAL_FREQ_CHAN_TAB_SELECT:
             viCheckParm (rssfu_SetAttributeViString (instrSession, "", RSSFU_ATTR_FREQ_CHANNEL_TABLE_SELECT, channelName),
                         3, "Channel Name"); 
        break;
        
        case RSSFU_VAL_FREQ_CHAN_TAB_STORE:
             viCheckParm (rssfu_SetAttributeViString (instrSession, "", RSSFU_ATTR_FREQ_CHANNEL_TABLE_STORE, channelName),
                         3, "Channel Name"); 
        break;
        
        case RSSFU_VAL_FREQ_CHAN_TAB_DELETE:
             viCheckParm (rssfu_SetAttributeViString (instrSession, "", RSSFU_ATTR_FREQ_CHANNEL_TABLE_DELETE, channelName),
                         3, "Channel Name"); 
        break;
        }
    }
 
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}                                                         


/*============================================================================/
 * Function: Configure Channel Table Frequency
 * Purpose:  This function configures channel table frequency.
/============================================================================*/
ViStatus _VI_FUNC rssfu_ConfigureChannelTableFrequency (ViSession instrSession,
                                                        ViInt32 channel,
                                                        ViString channelName,
                                                        ViReal64 channelFrequency)
{
    ViStatus    error   = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
  
    if ((channel < 1) || (channel > 100))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Channel");
    if ((channelFrequency < 300.0e3) || (channelFrequency > 3.0e9))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 4, "Channel Frequency");         
    
    if (Rs_Simulating(instrSession) == 0){     
    
        viCheckErr( viPrintf (instrSession, ":FREQ:CHAN:TABL:NAME %ld,\"%s\"", channel, channelName)); 
        
        viCheckErr( viPrintf (instrSession, ":FREQ:CHAN:TABL:FREQ %ld,%lf", channel, channelFrequency));   
    }

    viCheckErr(rssfu_CheckStatus (instrSession));
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;   
}
       

/*============================================================================/
 * Function: Set Frequency VSB Reference Point
 * Purpose:  This function determines the reference point for the frequency
 *           setting in the 8VSB modulation type.
/============================================================================*/
ViStatus _VI_FUNC rssfu_SetFrequencyVSBReferencePoint (ViSession instrSession,
                                                       ViInt32 frequencyRefPoint)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    viCheckParm( rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_FREQ_VSB_REF_POINT, frequencyRefPoint),
                                            2, "Frequency Reference Point");
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}                                                    

/*============================================================================/
 * Function: Read TX Frequency
 * Purpose:  This function queries the RF output frequency.
/============================================================================*/
ViStatus _VI_FUNC rssfu_readTxFreq (ViSession instrSession, ViReal64 *frequency)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (!frequency) viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "TX Frequency");
    viCheckErr (rssfu_GetAttributeViReal64 (instrSession, "", RSSFU_ATTR_FREQ_CW,
                                            frequency));
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Read Peak Envelope Power
 * Purpose:  This function returns the peak envelope power(PEP) in dBm for
 *           digital modulation and all digital standards.
/============================================================================*/
ViStatus _VI_FUNC rssfu_readPeakEnvelopePower (ViSession instrSession,
                                               ViReal64 *peakEnvelopePower)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (!peakEnvelopePower) viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Peak Envelope Power");
    viCheckErr (rssfu_GetAttributeViReal64 (instrSession, "", RSSFU_ATTR_POW_PEAK_ENVELOPE_POWER,
                                            peakEnvelopePower));
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 *----------------------------------- Sweep ---------------------------------*
 *****************************************************************************/
/*============================================================================/
 * Function: Enable Frequency Sweep
 * Purpose:  Determines the mode of the device and also by what commands the
 *           output frequency is determined.
/============================================================================*/
ViStatus _VI_FUNC rssfu_actEnableFrequencySweep (ViSession instrSession,
                                                 ViBoolean state)
{
    ViStatus    error = VI_SUCCESS;
    ViInt32     mode =  0;
    ViChar      model [RSSFU_IO_BUFFER_SIZE] = "";  

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckErr (rssfu_GetAttributeViString (instrSession, "",
                                               RS_ATTR_INSTRUMENT_MODEL,
                                               RSSFU_IO_BUFFER_SIZE, model));
    if (strstr (model, "BTC") != NULL) {
    viCheckParm (rssfu_SetAttributeViBoolean (instrSession, "", RSSFU_ATTR_FREQ_MODE_BTC, state),
                                2, "State");
    }
    else {
        if (state != 0)
        {                                         
            mode = RSSFU_VAL_FREQ_MODE_SWEEP;
        }
        else
        {
            mode = RSSFU_VAL_FREQ_MODE_CW_FIX;  
        }
    
        viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_FREQ_MODE,
                                                mode), 2, "State");
    }
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Configure Frequency Sweep Range
 * Purpose:  This function configures Tx Frequency Sweep range.
/============================================================================*/
ViStatus _VI_FUNC rssfu_confTxFreqSweepRange(ViSession instrSession,
                                             ViInt32 settingType,
                                             ViReal64 startFrequency,
                                             ViReal64 stopFrequency,
                                             ViReal64 centerFrequency,
                                             ViReal64 frequencySpan)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if ((settingType < RSSFU_SWEEP_START) || (settingType > RSSFU_SWEEP_CENTER))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Setting Type");
   
    switch (settingType){
        case  RSSFU_SWEEP_START:
            viCheckParm( rssfu_SetAttributeViReal64 (instrSession, "",
                                                     RSSFU_ATTR_FREQ_SWE_START, startFrequency), 
                                                     3, "Start Frequency");
            viCheckParm( rssfu_SetAttributeViReal64 (instrSession, "",
                                                     RSSFU_ATTR_FREQ_SWE_STOP, stopFrequency), 
                                                     4, "Stop Frequency");  
        break;
        case RSSFU_SWEEP_CENTER:
            viCheckParm( rssfu_SetAttributeViReal64 (instrSession, "",
                                                     RSSFU_ATTR_FREQ_SWE_CENTER, centerFrequency), 
                                                     5, "Center Frequency");
            viCheckParm( rssfu_SetAttributeViReal64 (instrSession, "",
                                                     RSSFU_ATTR_FREQ_SWE_SPAN, frequencySpan), 
                                                     6, "Frequency Span"); 
        break;
    }    

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Configure Frequency Sweep Step
 * Purpose:  This function configures Tx Frequency Sweep Step.
/============================================================================*/
ViStatus _VI_FUNC rssfu_confTxFreqSweepStep (ViSession instrSession,
                                             ViInt32 sweepSpacing,
                                             ViReal64 sweepStep,
                                             ViReal64 sweepDwell)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_FREQ_SWE_SPACING, sweepSpacing), 
                2, "Sweep Spacing");
    
    switch (sweepSpacing)
    {
        case RSSFU_VAL_FREQ_SWE_SPAC_LIN:
            viCheckParm (rssfu_SetAttributeViReal64 (instrSession, "", RSSFU_ATTR_FREQ_SWE_STEP_LIN, sweepStep), 
                        3, "Sweep Step"); 
        break;
        case RSSFU_VAL_FREQ_SWE_SPAC_LOG:
            viCheckParm (rssfu_SetAttributeViReal64 (instrSession, "", RSSFU_ATTR_FREQ_SWE_STEP_LOG, sweepStep), 
                        3, "Sweep Step"); 
        break;
    }

    viCheckParm (rssfu_SetAttributeViReal64 (instrSession, "", RSSFU_ATTR_FREQ_SWE_DWELL, sweepDwell),
                4, "Sweep Dwell");
                
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Configure Frequency Sweep Mode
 * Purpose:  This function configures Tx Frequency Sweep Mode.
/============================================================================*/
ViStatus _VI_FUNC rssfu_confTxFreqSweepMode(ViSession instrSession,
                                            ViInt32 sweepMode)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      model [RSSFU_IO_BUFFER_SIZE] = "";  

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckErr (rssfu_GetAttributeViString (instrSession, "",
                                               RS_ATTR_INSTRUMENT_MODEL,
                                               RSSFU_IO_BUFFER_SIZE, model));   
    
    if (strstr (model, "SFU") != NULL) {   

        viCheckErr (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_FREQ_MODE, RSSFU_VAL_FREQ_MODE_SWEEP));
    }
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_FREQ_SWE_MODE, sweepMode),
                                2, "Sweep Mode"); 
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Frequency Sweep Action
 * Purpose:  This function does selected sweep action.
/============================================================================*/
ViStatus _VI_FUNC rssfu_actTxFreqSweep(ViSession instrSession,
                                       ViInt32 actionType,
                                       ViReal64 manualSweepFrequency)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    if ((actionType < RSSFU_SWEEP_ACT_TRIGGER) || (actionType > RSSFU_SWEEP_ACT_SET_FREQ))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Action Type");

    manualSweepFrequency;
    
    switch (actionType){
        case  RSSFU_SWEEP_ACT_TRIGGER:
              viCheckErr (rssfu_SetAttributeViString (instrSession, "", RSSFU_ATTR_FREQ_SWE_TRIG, ""));
        break;
        case  RSSFU_SWEEP_ACT_RESET:
              viCheckErr (rssfu_SetAttributeViString (instrSession, "", RSSFU_ATTR_FREQ_SWE_RESET, ""));
        break; 
        case  RSSFU_SWEEP_ACT_SET_FREQ:
            /*
            viCheckParm (rssfu_SetAttributeViReal64 (instrSession, "", RSSFU_ATTR_FREQ_SWE_MAN, manualSweepFrequency),
                                3, "Manual Sweep Frequency");        
            */
        break;
    }    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 *---------------------------- Level ----------------------------------------*
 *****************************************************************************/
/*============================================================================/
 * Function: Configure Level
 * Purpose:  This function configures Tx Level.
/============================================================================*/
ViStatus _VI_FUNC rssfu_confTxLevel(ViSession   instrSession,
                                    ViReal64    level,
                                    ViBoolean   rf,
                                    ViReal64    offset,
                                    ViReal64    userLimit,
                                    ViInt32 attenuatorMode,
                                    ViInt32 levelUnit)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      model [RSSFU_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
        
    viCheckErr (rssfu_GetAttributeViString (instrSession, "",
                                               RS_ATTR_INSTRUMENT_MODEL,
                                               RSSFU_IO_BUFFER_SIZE, model));
    if (strstr (model, "BTC") != NULL) {
      if ((attenuatorMode < RSSFU_RFOUT_AMODE_AUTO) || (attenuatorMode > RSSFU_RFOUT_AMODE_NORM))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 6, "attenuatorMode");
    }
    
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_POW_LEV_UNIT, levelUnit),
                                7, "Level Unit");
    if (levelUnit == RSSFU_VAL_POW_LEV_DBM) {
        viCheckParm (rssfu_SetAttributeViReal64 (instrSession, "", RSSFU_ATTR_POW_LEVEL, level),
                                2, "Level");
    }
    else {
    
        viCheckParm (rssfu_SetAttributeViReal64 (instrSession, "", RSSFU_ATTR_TX_VOLATGE_LEVEL, level),
                                2, "Level");
    }
    viCheckParm (rssfu_SetAttributeViBoolean (instrSession, "", RSSFU_ATTR_RFOUT_STATE, rf),
                                3, "RF");
    if (strstr (model, "SFC") == NULL) {
        viCheckParm (rssfu_SetAttributeViReal64 (instrSession, "", RSSFU_ATTR_POW_OFFSET, offset),
                                4, "Offset"); 
    }
    viCheckParm (rssfu_SetAttributeViReal64 (instrSession, "", RSSFU_ATTR_POW_USER_LIMIT, userLimit),
                                5, "User Limit");  
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_RFOUT_AMODE, attenuatorMode),
                                6, "Attenuator Mode"); 
                                  
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Read TX Level
 * Purpose:  This function queries the RF output level in CW mode.
/============================================================================*/
ViStatus _VI_FUNC rssfu_readTxLevel (ViSession instrSession, ViReal64 *level)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (!level) viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "TX Level");
    viCheckErr (rssfu_GetAttributeViReal64 (instrSession, "", RSSFU_ATTR_POW_LEVEL,
                                            level));
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Get Att Fixed Range
 * Purpose:  This function queries the minimum and maximum level that can be 
 *           set without altering the attanuator in the mode Attenuator 
 *           Fixed.
/============================================================================*/
ViStatus _VI_FUNC rssfu_getTxAttFixedRange(ViSession   instrSession,
                                           ViReal64*   lower,
                                           ViReal64*   upper)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rssfu_GetAttributeViReal64 (instrSession, "", RSSFU_ATTR_RFOUT_ATT_MIN, lower),
                                2, "Lower");    
    viCheckParm (rssfu_GetAttributeViReal64 (instrSession, "", RSSFU_ATTR_RFOUT_ATT_MAX, upper),
                                3, "Upper");                                    
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: TX RF Output Protection Tripped
 * Purpose:  This function queries the status of the protection circuit.
/============================================================================*/
ViStatus _VI_FUNC rssfu_txRfOutputProtectionTripped (ViSession instrSession,
                                                     ViBoolean* tripped)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL)); 
    
    viCheckParm (rssfu_GetAttributeViBoolean (instrSession, "", RSSFU_ATTR_RFOUT_TRIPPED, tripped),
                                2, "Tripped");
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}                                

/*============================================================================/
 * Function: TX RF Output Clear
 * Purpose:  Resets the protection circuit after triggering.
/============================================================================*/
ViStatus _VI_FUNC rssfu_actTxRfOutputClear (ViSession instrSession)

{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));    

    viCheckErr (rssfu_SetAttributeViString (instrSession, "", RSSFU_ATTR_RFOUT_CLEAR, "")); 

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 *----------------------------------- ALC -----------------------------------*
 *****************************************************************************/
/*============================================================================/
 * Function: Configure ALC
 * Purpose:  This function configures Automatic Level Control system.
/============================================================================*/
ViStatus _VI_FUNC rssfu_confTxALC(ViSession   instrSession,
                                  ViInt32 state,
                                  ViInt32 alcoffMode)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      model [RSSFU_IO_BUFFER_SIZE] = "";  

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckErr (rssfu_GetAttributeViString (instrSession, "",
                                               RS_ATTR_INSTRUMENT_MODEL,
                                               RSSFU_IO_BUFFER_SIZE, model));
    if (strstr (model, "SFC") == NULL) { 
        viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_POW_ALC_STATE, state),
                                2, "State"); 
    }
    if (strstr (model, "SFU") != NULL) {
        viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_POW_ALCOFF_MODE, alcoffMode),
                                3, "ALCOFF Mode"); 
    }

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Search Once
 * Purpose:  Briefly switches on internal level control for correction.
/============================================================================*/
ViStatus _VI_FUNC rssfu_actTxALCSearchOnce(ViSession instrSession)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckErr (rssfu_SetAttributeViString (instrSession, "", RSSFU_ATTR_POW_ALC_SEARCH_ONCE, ""));
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 *---------------------------- Modulation -----------------------------------*
 *****************************************************************************/
/*============================================================================/
 * Function: Configure Modulation
 * Purpose:  This function configures Tx modulator.
/============================================================================*/
ViStatus _VI_FUNC rssfu_confTxModulation (ViSession instrSession,
                                          ViBoolean modulation,
                                          ViInt32 signalSource,
                                          ViInt32 transmissionStandard,
                                          ViInt32 spectrum)
                                         
{
    ViStatus    error = VI_SUCCESS;
    ViChar      model [RSSFU_IO_BUFFER_SIZE] = "";
    ViUInt32    attr = RSSFU_ATTR_MOD_DM_SIGNAL_SOURCE;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    if (signalSource == RSSFU_VAL_DM_SOURCE_INT)
        signalSource = RSSFU_VAL_DM_SOURCE_DTV; 

    viCheckErr (rssfu_GetAttributeViString (instrSession, "",
                                               RS_ATTR_INSTRUMENT_MODEL,
                                               RSSFU_IO_BUFFER_SIZE, model));   
    if (strstr (model, "BTC") != NULL)
        attr = RSSFU_ATTR_MOD_DM_SIGNAL_SOURCE_BTC;

    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", attr, signalSource),
                                3, "Signal Source");

    if((signalSource == RSSFU_VAL_DM_SOURCE_INT)||(signalSource == RSSFU_VAL_DM_SOURCE_DTV))
        viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_MOD_DM_TRANSMISSION_STANDARD, transmissionStandard),
                                    4, "Transmission Standard");
                                    
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_MOD_DM_SPECTRUM, spectrum),
                                5, "Spectrum");
    viCheckParm (rssfu_SetAttributeViBoolean (instrSession, "", RSSFU_ATTR_MOD_DM_MODULATION, modulation),
                                2, "Modulation"); 

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Configure Modulation ATV Transmission Standard
 * Purpose:  This function configures the transmission standard.
/============================================================================*/
ViStatus _VI_FUNC rssfu_confAtvTransStandard (ViSession instrSession,
                                         ViInt32 ATVTransmissionStandard)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      model [RSSFU_IO_BUFFER_SIZE] = "";  

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckErr (rssfu_GetAttributeViString (instrSession, "",
                                               RS_ATTR_INSTRUMENT_MODEL,
                                               RSSFU_IO_BUFFER_SIZE, model));   
    if (strstr (model, "BTC") != NULL) {
         
        if ((ATVTransmissionStandard < RSSFU_VAL_DM_ATV_STANDARD_BG) || (ATVTransmissionStandard > RSSFU_VAL_DM_ATV_STANDARD_L)||(ATVTransmissionStandard == RSSFU_VAL_DM_ATV_STANDARD_BGN))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "ATV Transmission Standard");

    }

    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_MOD_DM_ATV_TRANSMISSION_STANDARD,
                            ATVTransmissionStandard), 2, "ATV Transmission Standard");
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

  

/*============================================================================/
 * Function: Configure Interferer
 * Purpose:  This function configures Tx modulator interferer.
/============================================================================*/
ViStatus _VI_FUNC rssfu_confTxModulationInterferer (ViSession instrSession,
                                                    ViInt32 interfererSource,
                                                    ViReal64 interfererAttenuation,
                                                    ViInt32 interfererAddition,
                                                    ViReal64 interfererFrequencyOffset,
                                                    ViReal64 signalFrequencyOffset)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      model [RSSFU_IO_BUFFER_SIZE] = "";  
        
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckErr (rssfu_GetAttributeViString (instrSession, "",
                                               RS_ATTR_INSTRUMENT_MODEL,
                                               RSSFU_IO_BUFFER_SIZE, model));

    //for SFC, skip all the settings
    
    if (strstr (model, "SFU") != NULL)
    { 
        viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_MOD_DM_INTERFERER_SOURCE,
                                                interfererSource), 2, "Interferer Source"); 
        viCheckParm (rssfu_SetAttributeViReal64 (instrSession, "", RSSFU_ATTR_MOD_DM_INTERFERER_ATTENUATION,
                                                 interfererAttenuation), 3, "Interferer Attenuation");
        viCheckParm (rssfu_SetAttributeViReal64 (instrSession, "", RSSFU_ATTR_MOD_DM_INTERFERER_FREQ_OFFSET,
                                             interfererFrequencyOffset), 5, "Interferer Frequency Offset");
        viCheckParm (rssfu_SetAttributeViReal64 (instrSession, "", RSSFU_ATTR_MOD_DM_SIGNAL_FREQ_OFFSET,
                                         signalFrequencyOffset), 6, "Signal Frequency Offset");   
        viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_MOD_DM_INTERFERER_ADDITION,
                                            interfererAddition), 4, "Interferer Addition");
    }
    
    else if (strstr (model, "BTC") != NULL)
    {
        viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_MOD_DM_INTERFERER_SOURCE,
                                                interfererSource), 2, "Interferer Source");
        
        
        if (interfererSource == RSSFU_VAL_DM_INTERFER_SOUR_ARB)
        {
            // For ARB interferer, use special BTC attributes
            viCheckParm (rssfu_SetAttributeViReal64 (instrSession, "", RSSFU_ATTR_MOD_DM_INTERFERER_ARB_ATTENUATION,
                                                 interfererAttenuation), 3, "Interferer Attenuation");
            viCheckParm (rssfu_SetAttributeViReal64 (instrSession, "", RSSFU_ATTR_MOD_DM_INTERFERER_ARB_FREQ_OFFSET,
                                                interfererFrequencyOffset), 6, "Signal Frequency Offset");   
        }
        else
        {
            // For other interferer sources, use the same attributes as for the SFU
            viCheckParm (rssfu_SetAttributeViReal64 (instrSession, "", RSSFU_ATTR_MOD_DM_INTERFERER_ATTENUATION,
                                                 interfererAttenuation), 3, "Interferer Attenuation");
            viCheckParm (rssfu_SetAttributeViReal64 (instrSession, "", RSSFU_ATTR_MOD_DM_INTERFERER_FREQ_OFFSET,
                                                interfererFrequencyOffset), 5, "Interferer Frequency Offset");
    }
    
        viCheckParm (rssfu_SetAttributeViReal64 (instrSession, "", RSSFU_ATTR_MOD_DM_SIGNAL_FREQ_OFFSET,
                                         signalFrequencyOffset), 6, "Signal Frequency Offset");
    }                                           

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Configure Interferer Signal
 * Purpose:  This function configures Tx modulator interferer signal.
/============================================================================*/
ViStatus _VI_FUNC rssfu_confTxModulationInterfererSignal (ViSession instrSession,
                                                          ViInt32 interfererReference,
                                                          ViReal64 interfererLevel)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      model [RSSFU_IO_BUFFER_SIZE] = "";  

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckErr (rssfu_GetAttributeViString (instrSession, "",
                                               RS_ATTR_INSTRUMENT_MODEL,
                                               RSSFU_IO_BUFFER_SIZE, model));


    if (strstr (model, "BTC") != NULL)
    {
        ViInt32 interfererSource;
        
        viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_TX_MOD_INTERFERER_REFERENCE,
                                                interfererReference), 2, "Interferer Reference");
        //For BTC, differentiate between ARB interferer and other interferers
        viCheckErr (rssfu_GetAttributeViInt32 (instrSession, "", RSSFU_ATTR_MOD_DM_INTERFERER_SOURCE, &interfererSource));
        if (interfererSource == RSSFU_VAL_DM_INTERFER_SOUR_ARB) 
        {
            if (interfererReference == RSSFU_VAL_DM_INTERFERER_REF_LEV)
            {
                viCheckParm (rssfu_SetAttributeViReal64 (instrSession, "", RSSFU_ATTR_MOD_DM_INTERFERER_ARB_LEVEL,
                                                    interfererLevel), 3, "Interferer Level");
            }
            else
            {
                viCheckParm (rssfu_SetAttributeViReal64 (instrSession, "", RSSFU_ATTR_MOD_DM_INTERFERER_ARB_ATTENUATION,
                                                    interfererLevel), 3, "Interferer Level");
            }

        }
        else
        {
            if (interfererReference == RSSFU_VAL_DM_INTERFERER_REF_LEV)
            {
                viCheckParm (rssfu_SetAttributeViReal64 (instrSession, "", RSSFU_ATTR_TX_MOD_INTERFERER_LEVEL,
                                                    interfererLevel), 3, "Interferer Level"); 
            }
            else
            {
                viCheckParm (rssfu_SetAttributeViReal64 (instrSession, "", RSSFU_ATTR_MOD_DM_INTERFERER_ATTENUATION,
                                                    interfererLevel), 3, "Interferer Level");
            }
        }
    }
    else
    {
        //For all other instruments
        viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_TX_MOD_INTERFERER_REFERENCE,
                                            interfererReference), 2, "Interferer Reference"); 
        if (interfererReference == RSSFU_VAL_DM_INTERFERER_REF_LEV)
        {
            viCheckParm (rssfu_SetAttributeViReal64 (instrSession, "", RSSFU_ATTR_TX_MOD_INTERFERER_LEVEL,
                                             interfererLevel), 3, "Interferer Level"); 
        }
        else
        {
            viCheckParm (rssfu_SetAttributeViReal64 (instrSession, "", RSSFU_ATTR_MOD_DM_INTERFERER_ATTENUATION,
                                                interfererLevel), 3, "Interferer Level");
        }
    }
    
    
    
    

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Configure Interferer ATV Predefined Standard
 * Purpose:  This function configures the Interferer ATV Predefined Standard.
/============================================================================*/
ViStatus _VI_FUNC rssfu_confTxModInterfAtvPredefStd (ViSession instrSession,
                                                     ViInt32 ATVPredefinedStandard)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_MOD_DM_ATV_PREDEF_STANDARD,
                                            ATVPredefinedStandard), 2, "ATV Predefined Standard"); 
                                           
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Configure Modulation Settings
 * Purpose:  This function configures Tx modulator settings.
/============================================================================*/
ViStatus _VI_FUNC rssfu_confTxModulationSettings(ViSession instrSession,
                                                 ViReal64 iqWidebandInCrestFactor,
                                                 ViBoolean iqWidebandRFFiltering,
                                                 ViInt32 iqDigitalInMode,
                                                 ViInt32 iqDigitalOutMode)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      model [RSSFU_IO_BUFFER_SIZE] = "";  

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckErr (rssfu_GetAttributeViString (instrSession, "",
                                               RS_ATTR_INSTRUMENT_MODEL,
                                               RSSFU_IO_BUFFER_SIZE, model));   
    if (strstr (model, "SFC") == NULL) {
        viCheckParm (rssfu_SetAttributeViReal64 (instrSession, "", RSSFU_ATTR_MOD_WIDEBAND_IN_CREST_FACTOR,
                                             iqWidebandInCrestFactor) , 2, "I/Q Wideband In Crest Factor"); 
        viCheckParm (rssfu_SetAttributeViBoolean (instrSession, "", RSSFU_ATTR_MOD_WIDEBAND_RF_FILTERING,
                                              iqWidebandRFFiltering), 3, "I/Q Wideband RF Filtering");
    }
    
    if (strstr (model, "BTC") == NULL) { 
        viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_MOD_DIGITAL_IN_MODE,
            iqDigitalInMode), 4, "I/Q Digital In Mode");
    }
        
    if (strstr (model, "SFU") != NULL) {    
        viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "",  RSSFU_ATTR_MOD_DIGITAL_OUTPUT,
                                            iqDigitalOutMode), 5, "I/Q Digital Out Mode");
    }
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Configure Modulator Level
 * Purpose:  This function sets the gain of the I/Q modulator in relation to
 *           the analog I/Q signal. This enables the gain of the I/Q
 *           modulator to be matched to different measurement requirements.
/============================================================================*/
ViStatus _VI_FUNC rssfu_confTxModulatorLevel (ViSession instrSession,
                                              ViInt32 modulatorLevel)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      model [RSSFU_IO_BUFFER_SIZE] = "";  
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckErr (rssfu_GetAttributeViString (instrSession, "",
                                               RS_ATTR_INSTRUMENT_MODEL,
                                               RSSFU_IO_BUFFER_SIZE, model));

    if (strstr (model, "SFU") != NULL) {
        viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_MOD_MODULATOR_LEVEL,
                                            modulatorLevel), 2, "Modulator Level");
    }
    if (strstr (model, "BTC") != NULL) {
        ViInt32 btc_level[5] = {RSSFU_VAL_IQ_GAIN_DB4, RSSFU_VAL_IQ_GAIN_DB8, RSSFU_VAL_IQ_GAIN_DB4, RSSFU_VAL_IQ_GAIN_DB0, RSSFU_VAL_IQ_GAIN_DBM4};
        viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_MOD_MODULATOR_LEVEL_BTC,
                                            btc_level[modulatorLevel]), 2, "Modulator Level");
    }

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Modulation Signal Info and Statistics
 * Purpose:  This function reads selected coding value.
/============================================================================*/
ViStatus _VI_FUNC rssfu_readModulationSigInfoStat (ViSession instrSession,
                                                   ViInt32 value, ViReal64* result)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      model [RSSFU_IO_BUFFER_SIZE] = "";  
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
        
    viCheckErr (rssfu_GetAttributeViString (instrSession, "",
                                               RS_ATTR_INSTRUMENT_MODEL,
                                               RSSFU_IO_BUFFER_SIZE, model));
    if ((value < RSSFU_VAL_MOD_AN_IO_INP_RMS) || (value > RSSFU_VAL_MOD_DIG_IO_INP_PEAK))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Value");
        
    switch (value) {
        case RSSFU_VAL_MOD_AN_IO_INP_RMS:
            if (strstr (model, "SFU") != NULL) { 
                viCheckParm (rssfu_GetAttributeViReal64 (instrSession, "", RSSFU_ATTR_MOD_ANALOG_IQ_INPUT_RMS_VAL,
                                                    result), 3, "Result"); 
            }
            break;
        case RSSFU_VAL_MOD_AN_IO_INP_PEAK:
            if (strstr (model, "SFU") != NULL) { 
                viCheckParm (rssfu_GetAttributeViReal64 (instrSession, "", RSSFU_ATTR_MOD_ANALOG_IQ_INPUT_PEAK_VAL,
                                                     result), 3, "Result");
            }
        break;
        case RSSFU_VAL_MOD_DIG_IO_INP_RMS:
            if (strstr (model, "BTC") == NULL) {
                viCheckParm (rssfu_GetAttributeViReal64 (instrSession, "", RSSFU_ATTR_MOD_DIGITAL_IQ_INPUT_RMS_VAL,
                                                     result), 3, "Result");
            }
        break;
        case RSSFU_VAL_MOD_DIG_IO_INP_PEAK:
            if (strstr (model, "BTC") == NULL) { 
                viCheckParm (rssfu_GetAttributeViReal64 (instrSession, "", RSSFU_ATTR_MOD_DIGITAL_IQ_INPUT_PEAK_VAL,
                                                     result), 3, "Result");
            }
        break;
     }           
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Modulation IQ Input Refresh
 * Purpose:  This function activates the recalculation of the output power
 *           according to the level measured at the I/Q DIGITAL or I/Q ANALOG
 *           input.
/============================================================================*/
ViStatus _VI_FUNC rssfu_confTxModIqInputRefresh (ViSession instrSession,
                                                 ViInt32 refresh)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      model [RSSFU_IO_BUFFER_SIZE] = "";  
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckErr (rssfu_GetAttributeViString (instrSession, "",
                                               RS_ATTR_INSTRUMENT_MODEL,
                                               RSSFU_IO_BUFFER_SIZE, model));
    
    if ((refresh < RSSFU_VAL_MOD_IQ_INPUT_AN) || (refresh > RSSFU_VAL_MOD_IQ_INPUT_DIG))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Refresh");
        
    switch (refresh) {
        case RSSFU_VAL_MOD_IQ_INPUT_AN:
            if (strstr (model, "SFU") != NULL) {  
                viCheckErr (rssfu_SetAttributeViString (instrSession, "", RSSFU_ATTR_MOD_ANALOG_IQ_INPUT_REFRESH, "")); 
            }
            break;
        
        case RSSFU_VAL_MOD_IQ_INPUT_DIG:
            if (strstr (model, "BTC") == NULL) { 
                viCheckErr (rssfu_SetAttributeViString (instrSession, "", RSSFU_ATTR_MOD_DIGITAL_IQ_INPUT_REFRESH, "")); 
            }
        break;
            
     }           

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/*===========================================================================*/
/* Function: Audio Transmission Standard
/* Purpose:  This function sets the audio broadcasting transmission 
/*           standard.
/*===========================================================================*/
ViStatus _VI_FUNC rssfu_audioTransmissionStandard(
    ViSession   instrSession,
    ViInt32 transmissionStandard
)
{    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_MOD_AUDIO_BC_STANDARD,
                                            transmissionStandard), 2, "Transmission Standard"); 
                                           
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;

}

/*===========================================================================*/
/* Function: Read Digital IQ Symbol Rate
/* Purpose:  This function queries the symbolrate present at the digital I/Q 
/*           Input.
/*===========================================================================*/
ViStatus _VI_FUNC rssfu_readDiditalIqSymbolRate(
    ViSession   instrSession,
    ViReal64*   iqSymbolRate
)
{
       ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
     
    viCheckParm (rssfu_GetAttributeViReal64 (instrSession, "",
        RSSFU_ATTR_MODULATION_DIGITAL_IQ_INPUT_SYMBOL_RATE, iqSymbolRate), 2, "IQ Symbol Rate");
   Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;

}


/*****************************************************************************
 *---------------------------- Coding ---------------------------------------*
 *****************************************************************************/
/*****************************************************************************
 *----------------------------------- DVB-T/H -------------------------------*
 *****************************************************************************/

/*============================================================================/
 * Function: Read DVBT Coder Status
 * Purpose:  This function reads the status of DVBT coder.
/============================================================================*/
ViStatus _VI_FUNC rssfu_readDVBTCoderStatus(ViSession instrSession,
                                            ViInt32 priorityPath,
                                            ViChar  status[]
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSSFU_IO_BUFFER_SIZE] = "\0";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if ((priorityPath < RSSFU_VAL_PRIORITY_HIGH) || (priorityPath > RSSFU_VAL_PRIORITY_NONE))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Priority Path");
    if (status == NULL)
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 3, "Status");        
        
    switch (priorityPath){
        case RSSFU_VAL_PRIORITY_HIGH:
            viCheckErr (rssfu_GetAttributeViString (instrSession, "", RSSFU_ATTR_IQC_DVBT_LOCK_HIGH, RSSFU_IO_BUFFER_SIZE, buffer));
        break;      
        case RSSFU_VAL_PRIORITY_LOW: 
            viCheckErr (rssfu_GetAttributeViString (instrSession, "", RSSFU_ATTR_IQC_DVBT_LOCK_LOW, RSSFU_IO_BUFFER_SIZE, buffer));
        break;
        case RSSFU_VAL_PRIORITY_NONE:
            viCheckErr (rssfu_GetAttributeViString (instrSession, "", RSSFU_ATTR_IQC_DVBT_LOCK, RSSFU_IO_BUFFER_SIZE, buffer));
        break;  
        
    }
    if (status != NULL)
    {
        strncpy (status, buffer, strlen(buffer));
        status[strlen(buffer)] = '\0';
    }    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 *------------------------- Input Signal ------------------------------------*
 *****************************************************************************/
/*============================================================================/
 * Function: Configure DVBT Input Signal
 * Purpose:  This function configures DVBT Input Signal parameters.
/============================================================================*/
ViStatus _VI_FUNC rssfu_confDVBTInputSignal(ViSession   instrSession,
                                            ViInt32 priorityPath,
                                            ViInt32 source,
                                            ViInt32 input,
                                            ViInt32 stuffing)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      model [RSSFU_IO_BUFFER_SIZE] = "";  

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckErr (rssfu_GetAttributeViString (instrSession, "",
                                               RS_ATTR_INSTRUMENT_MODEL,
                                               RSSFU_IO_BUFFER_SIZE, model));
    
    if ((priorityPath < RSSFU_VAL_PRIORITY_HIGH) || (priorityPath > RSSFU_VAL_PRIORITY_LOW))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Priority Path");
        
    switch (priorityPath){
        case RSSFU_VAL_PRIORITY_HIGH: {
            viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_IQC_DVBT_SOUR, source),
                         3, "Source"); 
            viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_IQC_DVBT_INP, input),
                         4, "Input");
            viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_IQC_DVBT_STUF, stuffing),
                         5, "Stuffing");  
                                      }
        break;
        case RSSFU_VAL_PRIORITY_LOW: {
            viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_IQC_DVBT_SOUR_LOW, source),
                         3, "Source"); 
            viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_IQC_DVBT_INP_LOW, input),
                         4, "Input");
            viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_IQC_DVBT_STUF_LOW, stuffing),
                         5, "Stuffing"); 
                                     }
        break;
    }
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Configure DVBT Test Signal
 * Purpose:  This function configures DVBT Test Signal.
/============================================================================*/
ViStatus _VI_FUNC rssfu_confDVBTTestSignal(ViSession   instrSession,
                                           ViInt32 priorityPath,
                                           ViInt32 testSignal)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    if ((priorityPath < RSSFU_VAL_PRIORITY_HIGH) || (priorityPath > RSSFU_VAL_PRIORITY_LOW))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Priority Path");
    
    switch (priorityPath){
        case RSSFU_VAL_PRIORITY_HIGH:
            viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_IQC_DVBT_TEST, testSignal),
                         3, "Test Signal"); 
        break;
        case RSSFU_VAL_PRIORITY_LOW:
            viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_IQC_DVBT_TEST_LOW, testSignal),
                         3, "Test Signal"); 
        break;
    }
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Read DVBT Values
 * Purpose:  This function reads selected coding value.
/============================================================================*/
ViStatus _VI_FUNC rssfu_readDVBTValues(ViSession   instrSession,
                                       ViInt32 priorityPath,
                                       ViInt32 value,
                                       ViReal64*   result)
{
    ViStatus    error = VI_SUCCESS;
    ViInt32     int_result = 0;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    if ((priorityPath < RSSFU_VAL_PRIORITY_HIGH) || (priorityPath > RSSFU_VAL_PRIORITY_LOW))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Priority Path");
    if ((value < RSSFU_VAL_PACKET_LENGTH) || (value > RSSFU_VAL_USEFUL_DRATE))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 3, "Value");        

    switch (priorityPath){
        case RSSFU_VAL_PRIORITY_HIGH:
            switch (value){
                case RSSFU_VAL_PACKET_LENGTH:
                    viCheckParm (rssfu_GetAttributeViInt32 (instrSession, "", RSSFU_ATTR_IQC_DVBT_PACKET_LENGTH,
                                                            &int_result), 4, "Result"); 
                    *result = (ViReal64)int_result;
                break;
                
                case RSSFU_VAL_MAX_USEFUL_DRATE:
                    viCheckParm (rssfu_GetAttributeViReal64 (instrSession, "", RSSFU_ATTR_IQC_DVBT_MAX_USEFUL_DATA_RATE,
                                                             result), 4, "Result");
                break;
                case RSSFU_VAL_MEAS_DRATE:
                    viCheckParm (rssfu_GetAttributeViReal64 (instrSession, "", RSSFU_ATTR_IQC_DVBT_MEASURED_DATA_RATE,
                                                             result), 4, "Result");
                break;
                case RSSFU_VAL_USEFUL_DRATE:
                    viCheckParm (rssfu_GetAttributeViReal64 (instrSession, "", RSSFU_ATTR_IQC_DVBT_USEFUL_DATA_RATE,
                                                             result), 4, "Result");
                break;
             }
            break;
        case RSSFU_VAL_PRIORITY_LOW:
            switch (value){
                case RSSFU_VAL_PACKET_LENGTH:
                    viCheckParm (rssfu_GetAttributeViInt32 (instrSession, "", RSSFU_ATTR_IQC_DVBT_PACKET_LENGTH_LOW,
                                                            &int_result), 4, "Result"); 
                    *result = (ViReal64)int_result;
                break;
                
                case RSSFU_VAL_MAX_USEFUL_DRATE:
                    viCheckParm (rssfu_GetAttributeViReal64 (instrSession, "", RSSFU_ATTR_IQC_DVBT_MAX_USEFUL_DATA_RATE_LOW,
                                                             result), 4, "Result");
                break;
                case RSSFU_VAL_MEAS_DRATE:
                    viCheckParm (rssfu_GetAttributeViReal64 (instrSession, "", RSSFU_ATTR_IQC_DVBT_MEASURED_DATA_RATE_LOW,
                                                             result), 4, "Result");
                break;
                case RSSFU_VAL_USEFUL_DRATE:
                    viCheckParm (rssfu_GetAttributeViReal64 (instrSession, "", RSSFU_ATTR_IQC_DVBT_USEFUL_DATA_RATE_LOW,
                                                             result), 4, "Result");
                break;
             }
            break;
                
    }    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Configure DVBT Coding
 * Purpose:  This function configures DVBT coding parameters.
/============================================================================*/
ViStatus _VI_FUNC rssfu_confDVBTCoding(ViSession instrSession,
                                       ViInt32 channelBandwidth,
                                       ViReal64 usedBandwidth,
                                       ViInt32 fftMode,
                                       ViInt32 guardInterval,
                                       ViInt32 hierarchy,
                                       ViInt32 constellation)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      model [RSSFU_IO_BUFFER_SIZE] = "";  

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckErr (rssfu_GetAttributeViString (instrSession, "",
                                               RS_ATTR_INSTRUMENT_MODEL,
                                               RSSFU_IO_BUFFER_SIZE, model));   
    switch (channelBandwidth){ 
        case RSSFU_IQC_DVBT_CHBW_VAR:
             if (strstr (model, "SFC") == NULL) {   /* read only for SFC */
                 viCheckParm (rssfu_SetAttributeViReal64 (instrSession, "", RSSFU_ATTR_IQC_DVBT_USED_RESULT, usedBandwidth),
                         3, "Used Bandwidth"); 
             }
        break;
        default:
            viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_IQC_DVBT_CHAN, channelBandwidth),
                                 2, "Channel Bandwidth"); 
        break;
    }
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_IQC_DVBT_FFT_MODE, fftMode),
                         4, "FFT Mode"); 
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_IQC_DVBT_GUAR_INT, guardInterval),
                         5, "Guard Interval"); 
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_IQC_DVBT_CONS, constellation),
                         7, "Constellation");                          
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_IQC_DVBT_HIER, hierarchy),
                         6, "Hierarchy");                          

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Configure DVBT Code Rate
 * Purpose:  This function configures DVBT code rate.
/============================================================================*/
ViStatus _VI_FUNC rssfu_confDVBTCodeRate(ViSession instrSession,
                                         ViInt32 priorityPath,
                                         ViInt32 codeRate)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if ((priorityPath < RSSFU_VAL_PRIORITY_HIGH) || (priorityPath > RSSFU_VAL_PRIORITY_LOW))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Priority Path");
    
    switch (priorityPath){
        case RSSFU_VAL_PRIORITY_HIGH:
            viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_IQC_DVBT_RATE, codeRate),
                         3, "Code Rate"); 
        break;
        case RSSFU_VAL_PRIORITY_LOW:
            viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_IQC_DVBT_RATE_LOW, codeRate),
                         3, "Code Rate"); 
        break;
    }
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Configure DVBH Coding
 * Purpose:  This function configures DVB-H coding parameters.
/============================================================================*/
ViStatus _VI_FUNC rssfu_confDVBHCoding(ViSession instrSession,
                                       ViBoolean state,
                                       ViInt32 symbolInterleaverMode)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    viCheckParm (rssfu_SetAttributeViBoolean (instrSession, "", RSSFU_ATTR_IQC_DVBT_DVHS, state),
                         2, "State");
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_IQC_DVBT_DVBH_SYMB, symbolInterleaverMode),
                         3, "Symbol Interleaver Mode");                         
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Enable DVBT Special Settings
 * Purpose:  This function enables use of special DVBT coding parameters - 
 *           part 1.
/============================================================================*/
ViStatus _VI_FUNC rssfu_actEnableDVBTSpecial(ViSession instrSession,
                                             ViBoolean state)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    viCheckParm (rssfu_SetAttributeViBoolean (instrSession, "", RSSFU_ATTR_IQC_DVBT_SETT, state),
                         2, "State");
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Configure DVBT Special Settings
 * Purpose:  This function configures special DVBT coding parameters - part 
 *           1.
/============================================================================*/
ViStatus _VI_FUNC rssfu_confDVBTSpecial(ViSession   instrSession,
                                        ViInt32 priorityPath,
                                        ViBoolean   scrambler,
                                        ViBoolean   syncByteInversion,
                                        ViBoolean   reedSolomon,
                                        ViBoolean   convolutionalInterleaver)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      model [RSSFU_IO_BUFFER_SIZE] = "";  

    viCheckErr (rssfu_GetAttributeViString (instrSession, "",
                                               RS_ATTR_INSTRUMENT_MODEL,
                                               RSSFU_IO_BUFFER_SIZE, model));   
        
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    if ((priorityPath < RSSFU_VAL_PRIORITY_HIGH) || (priorityPath > RSSFU_VAL_PRIORITY_LOW))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Priority Path");
        
    switch (priorityPath){
        case RSSFU_VAL_PRIORITY_HIGH:
            if (strstr (model, "SFC") == NULL) {
                viCheckParm (rssfu_SetAttributeViBoolean (instrSession, "", RSSFU_ATTR_IQC_DVBT_SCR, scrambler),
                             3, "Scrambler"); 
                viCheckParm (rssfu_SetAttributeViBoolean (instrSession, "", RSSFU_ATTR_IQC_DVBT_SYNC_INV, syncByteInversion),
                             4, "Sync Byte Inversion");
          
                viCheckParm (rssfu_SetAttributeViBoolean (instrSession, "", RSSFU_ATTR_IQC_DVBT_CONV, convolutionalInterleaver),
                             6, "Convolutional Interleaver");
            }
            viCheckParm (rssfu_SetAttributeViBoolean (instrSession, "", RSSFU_ATTR_IQC_DVBT_REED, reedSolomon),
                         5, "Reed Solomon"); 
        break;
        case RSSFU_VAL_PRIORITY_LOW:
            if (strstr (model, "SFC") == NULL) {            
                viCheckParm (rssfu_SetAttributeViBoolean (instrSession, "", RSSFU_ATTR_IQC_DVBT_SCR_LOW, scrambler),
                             3, "Scrambler"); 
                viCheckParm (rssfu_SetAttributeViBoolean (instrSession, "", RSSFU_ATTR_IQC_DVBT_SYNC_INV_LOW, syncByteInversion),
                             4, "Sync Byte Inversion");
                viCheckParm (rssfu_SetAttributeViBoolean (instrSession, "", RSSFU_ATTR_IQC_DVBT_CONV_LOW, convolutionalInterleaver),
                             6, "Convolutional Interleaver");                          
            }
            viCheckParm (rssfu_SetAttributeViBoolean (instrSession, "", RSSFU_ATTR_IQC_DVBT_REED_LOW, reedSolomon),
                         5, "Reed Solomon");   
            
        break;
    }
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Configure DVBT Special Interleaver
 * Purpose:  This function configures special DVBT coding parameters - part 
 *           2.
/============================================================================*/
ViStatus _VI_FUNC rssfu_confDVBTSpecialInterleaver(ViSession   instrSession,
                                                   ViBoolean   bitInterleaver,                                                  
                                                   ViBoolean   symbolInterleaver,
                                                   ViBoolean   guardInsertion)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rssfu_SetAttributeViBoolean (instrSession, "", RSSFU_ATTR_IQC_DVBT_BIT, bitInterleaver),
                         2, "Bit Interleaver"); 
    viCheckParm (rssfu_SetAttributeViBoolean (instrSession, "", RSSFU_ATTR_IQC_DVBT_SYMB, symbolInterleaver),
                         3, "Symbol Interleaver"); 
    viCheckParm (rssfu_SetAttributeViBoolean (instrSession, "", RSSFU_ATTR_IQC_DVBT_GUAR_STAT, guardInsertion),
                         4, "Guard Insertion");

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Configure DVBT Special Carriers
 * Purpose:  This function configures special DVBT coding parameters - part 
 *           1.
/============================================================================*/
ViStatus _VI_FUNC rssfu_confDVBTSpecialCarriers(ViSession   instrSession,
                                                ViBoolean   modulationDataCarrier,
                                                ViBoolean   modulationPilotCarrier,
                                                ViBoolean   modulationTPSCarrier,
                                                ViBoolean   dataCarrier,
                                                ViBoolean   pilotCarrier,
                                                ViBoolean   tpsCarrier,
                                                ViBoolean   allCarriersEqualCW,
                                                ViString    disableCarriers)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      model [RSSFU_IO_BUFFER_SIZE] = "";  
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
        
    viCheckErr (rssfu_GetAttributeViString (instrSession, "",
                                               RS_ATTR_INSTRUMENT_MODEL,
                                               RSSFU_IO_BUFFER_SIZE, model));
    
    if (strstr (model, "SFU") != NULL) { 
        viCheckParm (rssfu_SetAttributeViBoolean (instrSession, "", RSSFU_ATTR_IQC_DVBT_DATA_MOD, modulationDataCarrier),
                             2, "Modulation Data Carrier"); 
        viCheckParm (rssfu_SetAttributeViBoolean (instrSession, "", RSSFU_ATTR_IQC_DVBT_PIL_MOD, modulationPilotCarrier),
                             3, "Modulation Pilot Carrier"); 
        viCheckParm (rssfu_SetAttributeViBoolean (instrSession, "", RSSFU_ATTR_IQC_DVBT_TPS_MOD, modulationTPSCarrier),
                             4, "Modulation TPS Carrier");
        viCheckParm (rssfu_SetAttributeViBoolean (instrSession, "", RSSFU_ATTR_IQC_DVBT_DATA, dataCarrier),
                             5, "Data Carrier"); 
        viCheckParm (rssfu_SetAttributeViBoolean (instrSession, "", RSSFU_ATTR_IQC_DVBT_PIL, pilotCarrier),
                             6, "Pilot Carrier"); 
        viCheckParm (rssfu_SetAttributeViBoolean (instrSession, "", RSSFU_ATTR_IQC_DVBT_TPS, tpsCarrier),
                             7, "TPS Carrier");   
        viCheckParm (rssfu_SetAttributeViBoolean (instrSession, "", RSSFU_ATTR_IQC_DVBT_EQU, allCarriersEqualCW),
                         8, "All Carriers Equal CW"); 
    }
    viCheckParm (rssfu_SetAttributeViString (instrSession, "", RSSFU_ATTR_IQC_DVBT_DIS_CARR, disableCarriers),
                         9, "Disable Carriers");                           
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Store and Load DVBT Carriers
 * Purpose:  This function stores or recalls application settings in the
 *           specified file.
/============================================================================*/
ViStatus _VI_FUNC rssfu_storeLoadDVBTCarriers (ViSession instrSession,
                                                         ViInt32 operation,
                                                         ViString source)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if ((operation < RSSFU_VAL_SAVE) || (operation > RSSFU_VAL_RECALL))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Operation");
        
    if (Rs_Simulating(instrSession) == 0) {                /* call only when locked */
                    
    switch (operation){
            case RSSFU_VAL_SAVE:
                viCheckParm (rssfu_SetAttributeViString (instrSession, "", RSSFU_ATTR_DVBT_SPECIAL_STORE_CARRIERS, source),
                         2, "Source");                           
            break;
        
            case RSSFU_VAL_RECALL:
                viCheckParm (rssfu_SetAttributeViString (instrSession, "", RSSFU_ATTR_DVBT_SPECIAL_LOAD_CARRIERS, source),
                         2, "Source");                           
            break;
        }   
    }
     
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}
 
/*============================================================================/
 * Function: Configure DVBT Packets and Payload
 * Purpose:  This function configures DVBT coding settings - packets.
/============================================================================*/
ViStatus _VI_FUNC rssfu_confDVBTPacketsPayload(ViSession   instrSession,
                                               ViInt32 testTSPacket,
                                               ViInt32 pidTestPacket,
                                               ViInt32 pid,
                                               ViInt32 payloadTestStuff,
                                               ViInt32 prbs)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      model [RSSFU_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckErr (rssfu_GetAttributeViString (instrSession, "",
                                               RS_ATTR_INSTRUMENT_MODEL,
                                               RSSFU_IO_BUFFER_SIZE, model));   
    
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_IQC_DVBT_TSP, testTSPacket),
                         2, "Test TS Packet");
    if (strstr (model, "SFC") == NULL) {
        viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_IQC_DVBT_PIDT, pidTestPacket),
                             3, "PID Test Packet"); 
        viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_IQC_DVBT_PID, pid),
                         4, "PID"); 
    }
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_IQC_DVBT_PAYL, payloadTestStuff),
                         5, "Payload Test/Stuff");      
    switch (payloadTestStuff){
        case  RSSFU_IQC_PRBS:
            viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_IQC_DVBT_PRBS, prbs),
                         6, "PRBS");     
        break;
    }    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Configure DVBT TPS
 * Purpose:  This function configures DVBT coding TPS settings.
/============================================================================*/
ViStatus _VI_FUNC rssfu_confDVBTTPS(ViSession instrSession,
                                    ViInt32 tpsCellID,
                                    ViBoolean tpsResevedState,
                                    ViInt32 tpsReservedValue)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_IQC_DVBT_CELL_ID, tpsCellID),
                         2, "TPS Cell ID");
    viCheckParm (rssfu_SetAttributeViBoolean (instrSession, "", RSSFU_ATTR_IQC_DVBT_TPSR_STAT,
                                              tpsResevedState), 3, "TPS Reseved State"); 
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_IQC_DVBT_TPSR_VAL,
                                            tpsReservedValue), 4, "TPS Reserved Value"); 
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Configure DVBT Timeslicing and MPE
 * Purpose:  This function configures DVBT coding Time Slicing and MPE.
/============================================================================*/
ViStatus _VI_FUNC rssfu_confDVBTTimeMPE(ViSession instrSession,
                                        ViInt32 priorityPath,
                                        ViBoolean timeSlicing,
                                        ViBoolean mpeFEC)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    if ((priorityPath < RSSFU_VAL_PRIORITY_HIGH) || (priorityPath > RSSFU_VAL_PRIORITY_LOW))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Priority Path");
        
    switch (priorityPath){
        case RSSFU_VAL_PRIORITY_HIGH:
            viCheckParm (rssfu_SetAttributeViBoolean (instrSession, "", RSSFU_ATTR_IQC_DVBT_TIM, timeSlicing),
                         3, "Time Slicing"); 
            viCheckParm (rssfu_SetAttributeViBoolean (instrSession, "", RSSFU_ATTR_IQC_DVBT_MPEF, mpeFEC),
                         4, "MPE FEC");
        break;
        case RSSFU_VAL_PRIORITY_LOW:
            viCheckParm (rssfu_SetAttributeViBoolean (instrSession, "", RSSFU_ATTR_IQC_DVBT_TIM_LOW, timeSlicing),
                         3, "Time Slicing"); 
            viCheckParm (rssfu_SetAttributeViBoolean (instrSession, "", RSSFU_ATTR_IQC_DVBT_MPEF_LOW, mpeFEC),
                         4, "MPE FEC");
        break;
    }
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 *----------------------------------- DVB-C ---------------------------------*
 *****************************************************************************/
/*============================================================================/
 * Function: Read DVBC Coder Status
 * Purpose:  This function reads the status of DVBC coder.
/============================================================================*/
ViStatus _VI_FUNC rssfu_readDVBCCoderStatus(ViSession instrSession,
                                            ViChar status[]
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSSFU_IO_BUFFER_SIZE]="\0";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (status == NULL) viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Status");
    viCheckErr (rssfu_GetAttributeViString (instrSession, "", RSSFU_ATTR_IQC_DVBC_LOCK,
                                            RSSFU_IO_BUFFER_SIZE, buffer));
    if (status != NULL)
    {
        strncpy (status, buffer, strlen(buffer));
        status [strlen(buffer)] = '\0';
    }
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Configure DVBC Input Signal
 * Purpose:  This function configures DVBC Input Signal parameters.
/============================================================================*/
ViStatus _VI_FUNC rssfu_confDVBCInputSignal(ViSession instrSession,
                                            ViInt32 source,
                                            ViInt32 input,
                                            ViInt32 stuffing)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      model [RSSFU_IO_BUFFER_SIZE] = "";  

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckErr (rssfu_GetAttributeViString (instrSession, "",
                                               RS_ATTR_INSTRUMENT_MODEL,
                                               RSSFU_IO_BUFFER_SIZE, model));
    if (strstr (model, "BTC") == NULL) {
        viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_IQC_DVBC_SOUR, source),
                             2, "Source"); 
        viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_IQC_DVBC_INP, input),
                             3, "Input");
    }
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_IQC_DVBC_STUF, stuffing),
                         4, "Stuffing");
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Configure DVBC Test Signal
 * Purpose:  This function configures DVBC Test Signal.
/============================================================================*/
ViStatus _VI_FUNC rssfu_confDVBCTestSignal(ViSession instrSession,
                                           ViInt32 testSignal)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_IQC_DVBC_TEST, testSignal),
                         2, "Test Signal"); 
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Read DVBC Values
 * Purpose:  This function reads selected coding value.
/============================================================================*/
ViStatus _VI_FUNC rssfu_readDVBCValues(ViSession instrSession,
                                       ViInt32 value,
                                       ViReal64* result
)
{
    ViStatus    error = VI_SUCCESS;
    ViInt32     int_result = 0;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    if ((value < RSSFU_VAL_PACKET_LENGTH) || (value > RSSFU_VAL_USEFUL_DRATE))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Value");
        
    switch (value){
        case RSSFU_VAL_PACKET_LENGTH:
            viCheckParm (rssfu_GetAttributeViInt32 (instrSession, "", RSSFU_ATTR_IQC_DVBC_PACKET_LENGTH,
                                                    &int_result), 3, "Result"); 
            *result = (ViReal64)int_result;
        break;
    
        case RSSFU_VAL_MAX_USEFUL_DRATE:
            viCheckParm (rssfu_GetAttributeViReal64 (instrSession, "", RSSFU_ATTR_IQC_DVBC_MAX_USEFUL_DATA_RATE,
                                                     result), 3, "Result");
        break;
        case RSSFU_VAL_MEAS_DRATE:
            viCheckParm (rssfu_GetAttributeViReal64 (instrSession, "", RSSFU_ATTR_IQC_DVBC_MEASURED_DATA_RATE,
                                                     result), 3, "Result");
        break;
        case RSSFU_VAL_USEFUL_DRATE:
            viCheckParm (rssfu_GetAttributeViReal64 (instrSession, "", RSSFU_ATTR_IQC_DVBC_USEFUL_DATA_RATE,
                                                     result), 3, "Result");
        break;
     }           
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Configure DVBC Coding
 * Purpose:  This function configures DVBC coding parameters.
/============================================================================*/
ViStatus _VI_FUNC rssfu_confDVBCCoding(ViSession instrSession,
                                       ViReal64 symbolRate,
                                       ViInt32 constellation,
                                       ViInt32 rollOff)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rssfu_SetAttributeViReal64 (instrSession, "", RSSFU_ATTR_IQC_DVBC_SYMB, symbolRate),
                         2, "Symbol Rate"); 
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_IQC_DVBC_CONS, constellation),
                         3, "Constellation");
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_IQC_DVBC_ROLL, rollOff),
                         4, "Roll Off");
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Enable DVBC Special Settings
 * Purpose:  This function enables use of special DVBC coding parameters - 
 *           part 1.
/============================================================================*/
ViStatus _VI_FUNC rssfu_actEnableDVBCSpecial(ViSession   instrSession,
                                             ViBoolean   state)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    viCheckParm (rssfu_SetAttributeViBoolean (instrSession, "", RSSFU_ATTR_IQC_DVBC_SETT, state),
                         2, "State");
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Configure DVBC Special Settings
 * Purpose:  This function configures special DVBC coding parameters.
/============================================================================*/
ViStatus _VI_FUNC rssfu_confDVBCSpecial(ViSession   instrSession,
                                        ViBoolean   scrambler,
                                        ViBoolean   syncByteInversion,
                                        ViBoolean   reedSolomon,
                                        ViBoolean   convolutionalInterleaver)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      model [RSSFU_IO_BUFFER_SIZE] = "";  

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckErr (rssfu_GetAttributeViString (instrSession, "",
                                               RS_ATTR_INSTRUMENT_MODEL,
                                               RSSFU_IO_BUFFER_SIZE, model));   
    if (strstr (model, "SFC") == NULL) {
        viCheckParm (rssfu_SetAttributeViBoolean (instrSession, "", RSSFU_ATTR_IQC_DVBC_SCR, scrambler),
                     2, "Scrambler"); 
        viCheckParm (rssfu_SetAttributeViBoolean (instrSession, "", RSSFU_ATTR_IQC_DVBC_SYNC_INV, syncByteInversion),
                     3, "Sync Byte Inversion");
    }
    viCheckParm (rssfu_SetAttributeViBoolean (instrSession, "", RSSFU_ATTR_IQC_DVBC_REED, reedSolomon),
                 4, "Reed Solomon");   
    
    if (strstr (model, "SFC") == NULL) {
        viCheckParm (rssfu_SetAttributeViBoolean (instrSession, "", RSSFU_ATTR_IQC_DVBC_INT, convolutionalInterleaver),
                 5, "Convolutional Interleaver");
    }
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Configure DVBC Settings
 * Purpose:  This function configures DVBC coding settings.
/============================================================================*/
ViStatus _VI_FUNC rssfu_confDVBCSettings(ViSession   instrSession,
                                         ViInt32 testTSPacket,
                                         ViInt32 pidTestPacket,
                                         ViInt32 pid,
                                         ViInt32 payloadTestStuff,
                                         ViInt32 prbs)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      model [RSSFU_IO_BUFFER_SIZE] = ""; 
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckErr (rssfu_GetAttributeViString (instrSession, "",
                                               RS_ATTR_INSTRUMENT_MODEL,
                                               RSSFU_IO_BUFFER_SIZE, model));   
    
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_IQC_DVBC_TSP, testTSPacket),
                         2, "Test TS Packet");
    if (strstr (model, "SFC") == NULL) {
        viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_IQC_DVBC_PIDT, pidTestPacket),
                             3, "PID Test Packet"); 
        viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_IQC_DVBC_PID, pid),
                             4, "PID"); 
    }       
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_IQC_DVBC_PAYL, payloadTestStuff),
                         5, "Payload Test/Stuff");      
    switch (payloadTestStuff){
        case  RSSFU_IQC_PRBS:
            viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_IQC_DVBC_PRBS, prbs),
                         6, "PRBS"); 
    }                         
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 *----------------------------------- DVB-S ---------------------------------*
 *****************************************************************************/
/*============================================================================/
 * Function: Read DVBS Coder Status
 * Purpose:  This function reads the status of DVBS coder.
/============================================================================*/
ViStatus _VI_FUNC rssfu_readDVBSCoderStatus(ViSession   instrSession,
                                            ViChar  status[])
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSSFU_IO_BUFFER_SIZE]="\0";  
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    if (status == NULL) viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Status");
    viCheckErr (rssfu_GetAttributeViString (instrSession, "", RSSFU_ATTR_IQC_DVBS_LOCK,
                                            RSSFU_IO_BUFFER_SIZE, buffer));
    if (status != NULL)
        strncpy (status, buffer, strlen(buffer)); 
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Configure DVBS Input Signal
 * Purpose:  This function configures DVBS Input Signal parameters.
/============================================================================*/
ViStatus _VI_FUNC rssfu_confDVBSInputSignal(ViSession   instrSession,
                                            ViInt32 source,
                                            ViInt32 input,
                                            ViInt32 stuffing)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      model [RSSFU_IO_BUFFER_SIZE] = "";
        
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckErr (rssfu_GetAttributeViString (instrSession, "",
                                               RS_ATTR_INSTRUMENT_MODEL,
                                               RSSFU_IO_BUFFER_SIZE, model));    
    if (strstr (model, "BTC") == NULL) {
        viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_IQC_DVBS_SOUR, source),
                             2, "Source"); 
        viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_IQC_DVBS_INP, input), 
            3, "Input");
    }
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_IQC_DVBS_STUF, stuffing),
                         4, "Stuffing");
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Configure DVBS Test Signal
 * Purpose:  This function configures DVBS Test Signal.
/============================================================================*/
ViStatus _VI_FUNC rssfu_confDVBSTestSignal(ViSession   instrSession,
                                           ViInt32 testSignal)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_IQC_DVBS_TEST, testSignal),
                         2, "Test Signal"); 
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Read DVBS Values
 * Purpose:  This function reads selected coding value.
/============================================================================*/
ViStatus _VI_FUNC rssfu_readDVBSValues(ViSession   instrSession,
                                       ViInt32 value,
                                       ViReal64*   result)
{
    ViStatus    error = VI_SUCCESS;
    ViInt32     int_result = 0; 

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    if ((value < RSSFU_VAL_PACKET_LENGTH) || (value > RSSFU_VAL_USEFUL_DRATE))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Value");
        
    switch (value){
        case RSSFU_VAL_PACKET_LENGTH:
            viCheckParm (rssfu_GetAttributeViInt32 (instrSession, "", RSSFU_ATTR_IQC_DVBS_PACKET_LENGTH,
                                                    &int_result), 3, "Result"); 
            *result = (ViReal64)int_result;
        break;
    
        case RSSFU_VAL_MAX_USEFUL_DRATE:
            viCheckParm (rssfu_GetAttributeViReal64 (instrSession, "", RSSFU_ATTR_IQC_DVBS_MAX_USEFUL_DATA_RATE,
                                                     result), 3, "Result");
        break;
        case RSSFU_VAL_MEAS_DRATE:
            viCheckParm (rssfu_GetAttributeViReal64 (instrSession, "", RSSFU_ATTR_IQC_DVBS_MEASURED_DATA_RATE,
                                                     result), 3, "Result");
        break;
        case RSSFU_VAL_USEFUL_DRATE:
            viCheckParm (rssfu_GetAttributeViReal64 (instrSession, "", RSSFU_ATTR_IQC_DVBS_USEFUL_DATA_RATE,
                                                     result), 3, "Result");
        break;
     }           
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Configure DVBS Coding
 * Purpose:  This function configures DVBS coding parameters.
/============================================================================*/
ViStatus _VI_FUNC rssfu_confDVBSCoding(ViSession instrSession,
                                       ViReal64 symbolRate,
                                       ViInt32 constellation,
                                       ViInt32 rollOff,
                                       ViInt32 codeRate)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rssfu_SetAttributeViReal64 (instrSession, "", RSSFU_ATTR_IQC_DVBS_SYMB, symbolRate),
                         2, "Symbol Rate"); 
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_IQC_DVBS_CONS, constellation),
                         3, "Constellation");
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_IQC_DVBS_ROLL, rollOff),
                         4, "Roll Off");
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_IQC_DVBS_RATE, codeRate),
                         5, "Code Rate");                        
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Enable DVBS Special Settings
 * Purpose:  This function enables use of special DVBS coding parameters.
/============================================================================*/
ViStatus _VI_FUNC rssfu_actEnableDVBSSpecial(ViSession instrSession,
                                             ViBoolean state)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rssfu_SetAttributeViBoolean (instrSession, "", RSSFU_ATTR_IQC_DVBS_SETT, state),
                         2, "State");
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Configure DVBS Special Settings
 * Purpose:  This function configures special DVBS coding parameters.
/============================================================================*/
ViStatus _VI_FUNC rssfu_confDVBSSpecial(ViSession instrSession,
                                        ViBoolean scrambler,
                                        ViBoolean syncByteInversion,
                                        ViBoolean reedSolomon,
                                        ViBoolean convolutionalInterleaver)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      model [RSSFU_IO_BUFFER_SIZE] = "";  

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckErr (rssfu_GetAttributeViString (instrSession, "",
                                               RS_ATTR_INSTRUMENT_MODEL,
                                               RSSFU_IO_BUFFER_SIZE, model));   
    
    if (strstr (model, "SFC") == NULL) {
        viCheckParm (rssfu_SetAttributeViBoolean (instrSession, "", RSSFU_ATTR_IQC_DVBS_SCR, scrambler),
                     2, "Scrambler"); 
        viCheckParm (rssfu_SetAttributeViBoolean (instrSession, "", RSSFU_ATTR_IQC_DVBS_SYNC_INV, syncByteInversion),
                     3, "Sync Byte Inversion");
    }
    viCheckParm (rssfu_SetAttributeViBoolean (instrSession, "", RSSFU_ATTR_IQC_DVBS_REED, reedSolomon),
                 4, "Reed Solomon");   
    if (strstr (model, "SFC") == NULL) {
        viCheckParm (rssfu_SetAttributeViBoolean (instrSession, "", RSSFU_ATTR_IQC_DVBS_INT, convolutionalInterleaver),
                 5, "Convolutional Interleaver");
    }
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Configure DVBS Settings
 * Purpose:  This function configures DVBS coding settings.
/============================================================================*/
ViStatus _VI_FUNC rssfu_confDVBSSettings(ViSession   instrSession,
                                         ViInt32 testTSPacket,
                                         ViInt32 pidTestPacket,
                                         ViInt32 pid,
                                         ViInt32 payloadTestStuff,
                                         ViInt32 prbs)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      model [RSSFU_IO_BUFFER_SIZE] = "";

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckErr (rssfu_GetAttributeViString (instrSession, "",
                                               RS_ATTR_INSTRUMENT_MODEL,
                                               RSSFU_IO_BUFFER_SIZE, model));   
    
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_IQC_DVBS_TSP, testTSPacket),
                         2, "Test TS Packet");
    if (strstr (model, "SFC") == NULL) {   
        viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_IQC_DVBS_PIDT, pidTestPacket),
                             3, "PID Test Packet"); 
        viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_IQC_DVBS_PID, pid),
                             4, "PID"); 
    }
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_IQC_DVBS_PAYL, payloadTestStuff),
                         5, "Payload Test/Stuff");      
    switch (payloadTestStuff){
        case  RSSFU_IQC_PRBS:
            viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_IQC_DVBS_PRBS, prbs),
                         6, "PRBS"); 
    }   
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}
 

/*****************************************************************************
 *----------------------------------- 8VSB ----------------------------------*
 *****************************************************************************/
/*============================================================================/
 * Function: Read 8VSB Coder Status
 * Purpose:  This function reads the status of 8VSB coder.
/============================================================================*/
ViStatus _VI_FUNC rssfu_read8VSBCoderStatus(ViSession   instrSession,
                                            ViChar  status[])
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSSFU_IO_BUFFER_SIZE]="\0";  
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    if (status == NULL) viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Status");
    viCheckErr (rssfu_GetAttributeViString (instrSession, "", RSSFU_ATTR_IQC_8VSB_LOCK,
                                            RSSFU_IO_BUFFER_SIZE, buffer));
    if (status != NULL)
        strncpy (status, buffer, strlen(buffer)); 
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
} 
 
/*============================================================================/
 * Function: Configure 8VSB Input Signal
 * Purpose:  This function configures 8VSB Input Signal parameters.
/============================================================================*/
ViStatus _VI_FUNC rssfu_conf8VSBInputSignal(ViSession   instrSession,
                                            ViInt32 source,
                                            ViInt32 input,
                                            ViInt32 stuffing)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      model [RSSFU_IO_BUFFER_SIZE] = "";  

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckErr (rssfu_GetAttributeViString (instrSession, "",
                                               RS_ATTR_INSTRUMENT_MODEL,
                                               RSSFU_IO_BUFFER_SIZE, model));
    if (strstr (model, "BTC") == NULL) { 
        viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_IQC_8VSB_SOURCE, source),
                             2, "Source"); 
        viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_IQC_8VSB_INPUT_SIGNAL, input),
                             3, "Input");
    }
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_IQC_8VSB_STUFFING, stuffing),
                         4, "Stuffing");
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}
 
     
/*============================================================================/
 * Function: Configure 8VSB Test Signal
 * Purpose:  This function configures 8VSB Test Signal.
/============================================================================*/
ViStatus _VI_FUNC rssfu_conf8VSBTestSignal(ViSession   instrSession,
                                           ViInt32 testSignal)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_IQC_8VSB_TEST, testSignal),
                         2, "Test Signal"); 
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Read 8VSB Packed Length
 * Purpose:  This function reads 8VSB packed length.
/============================================================================*/
ViStatus _VI_FUNC rssfu_read8VSBPackedLength (ViSession instrSession,
                                                 ViInt32* packedLength)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    if (!packedLength) viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Packed Length");
    viCheckParm (rssfu_GetAttributeViInt32 (instrSession, "", RSSFU_ATTR_IQC_8VSB_PACKET_LENGTH,
                                            packedLength), 2, "Packed Length");
   Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

 
/*============================================================================/
 * Function: Read 8VSB Values
 * Purpose:  This function reads selected coding value.
/============================================================================*/
ViStatus _VI_FUNC rssfu_read8VSBValues(ViSession instrSession,
                                        ViInt32 value,
                                        ViReal64* result)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    
    if ((value < RSSFU_VAL_MAX_USEFUL_DRATE) || (value > RSSFU_VAL_USEFUL_DRATE))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Value");
        
    switch (value){                                                       
        case RSSFU_VAL_MAX_USEFUL_DRATE:
            viCheckParm (rssfu_GetAttributeViReal64 (instrSession, "", RSSFU_ATTR_IQC_8VSB_MAX_USEFUL_DATA_RATE,
                                                     result), 3, "Result");
        break;
        case RSSFU_VAL_MEAS_DRATE:
            viCheckParm (rssfu_GetAttributeViReal64 (instrSession, "", RSSFU_ATTR_IQC_8VSB_MEASURED_DATA_RATE,
                                                     result), 3, "Result");
        break;
        case RSSFU_VAL_USEFUL_DRATE:
            viCheckParm (rssfu_GetAttributeViReal64 (instrSession, "", RSSFU_ATTR_IQC_8VSB_USEFUL_DATA_RATE,
                                                     result), 3, "Result");
        break;
       
     }           
     
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}
 
/*============================================================================/
 * Function: Set 8VSB Symbol Rate
 * Purpose:  This function allows setting of the symbol rate in the range
 *           10.762 MS/s +/-5%.
/============================================================================*/
ViStatus _VI_FUNC rssfu_set8VSBSymbolRate (ViSession instrSession,
                                           ViReal64 symbolRate)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rssfu_SetAttributeViReal64 (instrSession, "", RSSFU_ATTR_IQC_8VSB_SYMBOL_RATE, symbolRate),
                         2, "Symbol Rate"); 
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}
 
/*============================================================================/
 * Function: Read 8VSB Constellation
 * Purpose:  This function queries the VSB constellation firmly set to 8VSB.
/============================================================================*/
ViStatus _VI_FUNC rssfu_read8VSBConstellation (ViSession instrSession,
                                                 ViChar constellation[])
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSSFU_IO_BUFFER_SIZE]="\0";  
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    if (constellation == NULL) viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Constellation");
    viCheckErr (rssfu_GetAttributeViString (instrSession, "", RSSFU_ATTR_IQC_8VSB_CONSTELLATION,
                                            RSSFU_IO_BUFFER_SIZE, buffer));
    strncpy (constellation, buffer, strlen(buffer)); 
    if (constellation != NULL)
        constellation[strlen(buffer)] = '\0';
   
   Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}
 
/*============================================================================/
 * Function: Read 8VSB Roll Off
 * Purpose:  This function queries the rolloff, which is always 0.115 for  VSB.
/============================================================================*/
ViStatus _VI_FUNC rssfu_read8VSBRollOff (ViSession instrSession,
                                         ViReal64* rollOff)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rssfu_GetAttributeViReal64 (instrSession, "", RSSFU_ATTR_IQC_8VSB_ROLL_OFF,
                                                     rollOff), 2, "Roll Off");
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Enable 8VSB Special Settings
 * Purpose:  This function enables use of special 8VSB coding parameters.
/============================================================================*/
ViStatus _VI_FUNC rssfu_actEnable8VSBSpecial(ViSession instrSession,
                                             ViBoolean state)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rssfu_SetAttributeViBoolean (instrSession, "", RSSFU_ATTR_IQC_8VSB_SPECIAL_SETTINGS, state),
                         2, "State");
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

 
/*============================================================================/
 * Function: Configure 8VSB Special Settings
 * Purpose:  This function configures special 8VSB coding parameters.
/============================================================================*/
ViStatus _VI_FUNC rssfu_conf8VSBSpecial (ViSession instrSession,
                                         ViBoolean randomizer,
                                         ViBoolean reedSolomon,
                                         ViBoolean interleaver,
                                         ViBoolean pilot,
                                         ViReal64 pilotValue)

{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rssfu_SetAttributeViBoolean (instrSession, "", RSSFU_ATTR_IQC_8VSB_RANDOMIZER, randomizer),
                 2, "Randomizer"); 
    viCheckParm (rssfu_SetAttributeViBoolean (instrSession, "", RSSFU_ATTR_IQC_8VSB_REED_SOLOMON, reedSolomon),
                 3, "Reed Solomon");
    viCheckParm (rssfu_SetAttributeViBoolean (instrSession, "", RSSFU_ATTR_IQC_8VSB_INTERLEAVER, interleaver),
                 4, "Interleaver");   
    viCheckParm (rssfu_SetAttributeViBoolean (instrSession, "", RSSFU_ATTR_IQC_8VSB_PILOT, pilot),
                 5, "Pilot");
    if (pilot)
        viCheckParm (rssfu_SetAttributeViReal64 (instrSession, "", RSSFU_ATTR_IQC_8VSB_PILOT_VALUE, pilotValue),
                 6, "Pilot Value");                 
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/*============================================================================/
 * Function: Configure 8VSB Settings
 * Purpose:  This function configures 8VSB coding settings.
/============================================================================*/
ViStatus _VI_FUNC rssfu_conf8VSBSettings(ViSession   instrSession,
                                         ViInt32 testTSPacket,
                                         ViInt32 pidTestPacket,
                                         ViInt32 pid,
                                         ViInt32 payloadTestStuff,
                                         ViInt32 prbs)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      model [RSSFU_IO_BUFFER_SIZE] = "";  

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckErr (rssfu_GetAttributeViString (instrSession, "",
                                               RS_ATTR_INSTRUMENT_MODEL,
                                               RSSFU_IO_BUFFER_SIZE, model));   
    
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_IQC_8VSB_TEST_TS_PACKED, testTSPacket),
                         2, "Test TS Packet");
    if (strstr (model, "SFC") == NULL) {
        viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_IQC_8VSB_PID_TEST_PACKET, pidTestPacket),
                             3, "PID Test Packet"); 
        viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_IQC_8VSB_PID, pid),
                             4, "PID"); 
    }   
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_IQC_8VSB_PAYLOAD_TEST, payloadTestStuff),
                         5, "Payload Test/Stuff");      
    switch (payloadTestStuff){
        case  RSSFU_IQC_PRBS:
            viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_IQC_8VSB_PRBS, prbs),
                         6, "PRBS"); 
    }   
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

  
/*****************************************************************************
 *----------------------------------- J83B ----------------------------------*
 *****************************************************************************/
/*============================================================================/
 * Function: Read J83B Coder Status
 * Purpose:  This function reads the status of J83B coder.
/============================================================================*/
ViStatus _VI_FUNC rssfu_readJ83BCoderStatus(ViSession   instrSession,
                                            ViChar  status[])
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSSFU_IO_BUFFER_SIZE]="\0";  
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    if (status == NULL) viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Status");
    viCheckErr (rssfu_GetAttributeViString (instrSession, "", RSSFU_ATTR_IQC_J83B_LOCK,
                                            RSSFU_IO_BUFFER_SIZE, buffer));
    if (status != NULL)
        strncpy (status, buffer, strlen(buffer)); 
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
} 
 
/*============================================================================/
 * Function: Configure J83B Input Signal
 * Purpose:  This function configures J83B Input Signal parameters.
/============================================================================*/
ViStatus _VI_FUNC rssfu_confJ83BInputSignal(ViSession   instrSession,
                                            ViInt32 source,
                                            ViInt32 input,
                                            ViInt32 stuffing)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      model [RSSFU_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_IQC_J83B_SOURCE, source),
                         2, "Source"); 
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_IQC_J83B_INPUT_SIGNAL, input),
                         3, "Input");
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_IQC_J83B_STUFFING, stuffing),
                         4, "Stuffing");
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}
 
     
/*============================================================================/
 * Function: Configure J83B Test Signal
 * Purpose:  This function configures J83B Test Signal.
/============================================================================*/
ViStatus _VI_FUNC rssfu_confJ83BTestSignal(ViSession   instrSession,
                                           ViInt32 testSignal)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_IQC_J83B_TEST, testSignal),
                         2, "Test Signal"); 
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Read J83B Packed Length
 * Purpose:  This function reads J83B packed length.
/============================================================================*/
ViStatus _VI_FUNC rssfu_readJ83BPackedLength (ViSession instrSession,
                                              ViInt32* packedLength)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    if (!packedLength) viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Packed Length");
    viCheckParm (rssfu_GetAttributeViInt32 (instrSession, "", RSSFU_ATTR_IQC_J83B_PACKET_LENGTH,
                                            packedLength), 2, "Packed Length");
   Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

 
/*============================================================================/
 * Function: Read J83B Values
 * Purpose:  This function reads selected coding value.
/============================================================================*/
ViStatus _VI_FUNC rssfu_readJ83BValues(ViSession instrSession,
                                        ViInt32 value,
                                        ViReal64* result)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    
    if ((value < RSSFU_VAL_MAX_USEFUL_DRATE) || (value > RSSFU_VAL_USEFUL_DRATE))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Value");
        
    switch (value){                                                       
        case RSSFU_VAL_MAX_USEFUL_DRATE:
            viCheckParm (rssfu_GetAttributeViReal64 (instrSession, "", RSSFU_ATTR_IQC_J83B_MAX_USEFUL_DATA_RATE,
                                                     result), 3, "Result");
        break;
        case RSSFU_VAL_MEAS_DRATE:
            viCheckParm (rssfu_GetAttributeViReal64 (instrSession, "", RSSFU_ATTR_IQC_J83B_MEASURED_DATA_RATE,
                                                     result), 3, "Result");
        break;
        case RSSFU_VAL_USEFUL_DRATE:
            viCheckParm (rssfu_GetAttributeViReal64 (instrSession, "", RSSFU_ATTR_IQC_J83B_USEFUL_DATA_RATE,
                                                     result), 3, "Result");
        break;
       
     }           
     
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}
 

/*============================================================================/
 * Function: Configure J83B Coding
 * Purpose:  This function configures J83B coding parameters.
/============================================================================*/
ViStatus _VI_FUNC rssfu_confJ83BCoding (ViSession instrSession,
                                        ViReal64 symbolRate,
                                        ViInt32 constellation,
                                        ViInt32 interleaver)

{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_IQC_J83B_CONSTELLATION, constellation),
                         3, "Constellation");
    
    viCheckParm (rssfu_SetAttributeViReal64 (instrSession, "", RSSFU_ATTR_IQC_J83B_SYMBOL_RATE, symbolRate),
                         2, "Symbol Rate"); 

    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_IQC_J83B_INTERLEAVER_MODE, interleaver),
                         4, "Interleaver")
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/*============================================================================/
 * Function: Read J83B Roll Off
 * Purpose:  This function queries the rolloff, which is always 0.18 (64QAM)
 *           or 0.12 (256QAM) for J.83B.
/============================================================================*/
ViStatus _VI_FUNC rssfu_readJ83BRollOff (ViSession instrSession,
                                         ViReal64* rollOff)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rssfu_GetAttributeViReal64 (instrSession, "", RSSFU_ATTR_IQC_J83B_ROLL_OFF,
                                                     rollOff), 2, "Roll Off");
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Enable J83B Special Settings
 * Purpose:  This function enables use of special J83B coding parameters.
/============================================================================*/
ViStatus _VI_FUNC rssfu_actEnableJ83BSpecial(ViSession instrSession,
                                             ViBoolean state)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rssfu_SetAttributeViBoolean (instrSession, "", RSSFU_ATTR_IQC_J83B_SPECIAL_SETTINGS, state),
                         2, "State");
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

 
/*============================================================================/
 * Function: Configure J83B Special Settings
 * Purpose:  This function configures special J83B coding parameters.
/============================================================================*/
ViStatus _VI_FUNC rssfu_confJ83BSpecial (ViSession instrSession,
                                         ViBoolean checksumgen,
                                         ViBoolean reedSolomon,
                                         ViBoolean interleaver,
                                         ViBoolean randomizer)

{
    ViStatus    error = VI_SUCCESS;
    ViChar      model [RSSFU_IO_BUFFER_SIZE] = "";  

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckErr (rssfu_GetAttributeViString (instrSession, "",
                                               RS_ATTR_INSTRUMENT_MODEL,
                                               RSSFU_IO_BUFFER_SIZE, model));   
    if (strstr (model, "SFC") == NULL) {
        viCheckParm (rssfu_SetAttributeViBoolean (instrSession, "", RSSFU_ATTR_IQC_J83B_CHECKSUMGEN, checksumgen),
                     2, "Checksumgen"); 
        viCheckParm (rssfu_SetAttributeViBoolean(instrSession, "", RSSFU_ATTR_IQC_J83B_INTERLEAVER, interleaver),
                     4, "Interleaver");   
        viCheckParm (rssfu_SetAttributeViBoolean (instrSession, "", RSSFU_ATTR_IQC_J83B_RANDOMIZER, randomizer),
                     5, "Randomizer");
    }
    viCheckParm (rssfu_SetAttributeViBoolean (instrSession, "", RSSFU_ATTR_IQC_J83B_REED_SOLOMON, reedSolomon),
                 3, "Reed Solomon");
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/*============================================================================/
 * Function: Configure J83B Settings
 * Purpose:  This function configures 8VSB coding settings.
/============================================================================*/
ViStatus _VI_FUNC rssfu_confJ83BSettings(ViSession   instrSession,
                                         ViInt32 testTSPacket,
                                         ViInt32 pidTestPacket,
                                         ViInt32 pid,
                                         ViInt32 payloadTestStuff,
                                         ViInt32 prbs)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      model [RSSFU_IO_BUFFER_SIZE] = ""; 
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckErr (rssfu_GetAttributeViString (instrSession, "",
                                               RS_ATTR_INSTRUMENT_MODEL,
                                               RSSFU_IO_BUFFER_SIZE, model));   
    
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_IQC_J83B_TEST_TS_PACKED, testTSPacket),
                         2, "Test TS Packet");
    if (strstr (model, "SFC") == NULL) 
    {
        viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_IQC_J83B_PID_TEST_PACKET, pidTestPacket),
                             3, "PID Test Packet");
        if (pidTestPacket == RSSFU_IQC_PIDT_VAR)
        {
            viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_IQC_J83B_PID, pid),
                             4, "PID");
        }
    }
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_IQC_J83B_PAYLOAD_TEST, payloadTestStuff),
                         5, "Payload Test/Stuff");      
    switch (payloadTestStuff){
        case  RSSFU_IQC_PRBS:
            viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_IQC_J83B_PRBS, prbs),
                         6, "PRBS"); 
    }   
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
} 

 
/*****************************************************************************
 *----------------------------------- ISDB-T --------------------------------*
 *****************************************************************************/

/*============================================================================/
 * Function: Read ISDBT Coder Status
 * Purpose:  This function reads the status of ISDBT coder.
/============================================================================*/
ViStatus _VI_FUNC rssfu_readISDBTCoderStatus (ViSession instrSession,
                                              ViChar status[])

{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSSFU_IO_BUFFER_SIZE]="\0";  
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    if (status == NULL) viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Status");
    viCheckErr (rssfu_GetAttributeViString (instrSession, "", RSSFU_ATTR_IQC_ISDBT_LOCK,
                                            RSSFU_IO_BUFFER_SIZE, buffer));
    if (status != NULL)
        strncpy (status, buffer, strlen(buffer)); 
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}
 
/*============================================================================/
 * Function: Configure ISDBT Input Signal
 * Purpose:  This function configures ISDBT Input Signal parameters.
/============================================================================*/
ViStatus _VI_FUNC rssfu_confISDBTInputSignal (ViSession instrSession,
                                              ViInt32 layer, ViInt32 source,
                                              ViInt32 input)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buf1[RSSFU_TOKEN_BUFFER_SIZE] = "";

    if ((layer < RSSFU_VAL_IQC_ISDBT_LAYER_A) || (layer > RSSFU_VAL_IQC_ISDBT_LAYER_C))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Layer"); 

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    checkErr (rssfu_GetActualRepCaps (instrSession, RSSFU_ATTR_IQC_ISDBT_SOURCE, "Layer", layer, buf1));

    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, buf1, RSSFU_ATTR_IQC_ISDBT_SOURCE, source), 
                    3, "Source"); 

    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_IQC_ISDBT_INPUT_SIGNAL, input), 
                    4, "Input");                     
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
} 

/*============================================================================/
 * Function: Configure ISDBT Input Parameters
 * Purpose:  This function configures the remultiplexer and stuffing
 *           parameters.
/============================================================================*/
ViStatus _VI_FUNC rssfu_ConfigureISDBTInputParameters (ViSession instrSession,
                                                        ViBoolean remux,
                                                       ViBoolean stuffing)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      model [RSSFU_IO_BUFFER_SIZE] = "";  

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckErr (rssfu_GetAttributeViString (instrSession, "",
                                               RS_ATTR_INSTRUMENT_MODEL,
                                               RSSFU_IO_BUFFER_SIZE, model));

        viCheckParm (rssfu_SetAttributeViBoolean (instrSession, "", RSSFU_ATTR_ISDBT_INPUT_SIGNAL_REMUX, remux), 
                    2, "Remux"); 
    if (strstr (model, "BTC") == NULL) {   /* read only for BTC */  
         viCheckParm (rssfu_SetAttributeViBoolean (instrSession, "", RSSFU_ATTR_ISDBT_INPUT_SIGNAL_STUFFING, stuffing), 
                    3, "Stuffing");  
    }
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
} 

/*============================================================================/
 * Function: Configure ISDBT Test Signal
 * Purpose:  This function configures ISDBT Test Signal.
/============================================================================*/ 
ViStatus _VI_FUNC rssfu_confISDBTTestSignal (ViSession instrSession,
                                             ViInt32 layer, ViInt32 testSignal)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buf1[RSSFU_TOKEN_BUFFER_SIZE] = "";

    if ((layer < RSSFU_VAL_IQC_ISDBT_LAYER_A) || (layer > RSSFU_VAL_IQC_ISDBT_LAYER_C))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Layer"); 

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    checkErr (rssfu_GetActualRepCaps (instrSession, RSSFU_ATTR_IQC_ISDBT_TEST, "Layer", layer, buf1));

    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, buf1, RSSFU_ATTR_IQC_ISDBT_TEST, testSignal), 
                    3, "Test Signal"); 
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
} 
                                            

/*============================================================================/
 * Function: Read ISDBT Input Data Rate
 * Purpose:  This function returns the input data rate.
/============================================================================*/ 
ViStatus _VI_FUNC rssfu_readISDBTInputDataRate (ViSession instrSession,
                                                ViReal64* inputDataRate)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    if (!inputDataRate) viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Input Data Rate");
    viCheckParm (rssfu_GetAttributeViReal64 (instrSession, "", RSSFU_ATTR_IQC_ISDBT_INPUT_DATA_RATE,
                                            inputDataRate), 2, "Input Data Rate");
   Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}                                                

/*============================================================================/
 * Function: Read ISDBT Values
 * Purpose:  This function reads selected coding value.
/============================================================================*/ 
ViStatus _VI_FUNC rssfu_readISDBTValues (ViSession instrSession, ViInt32 layer,
                                         ViInt32 value, ViReal64* result)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buf1[RSSFU_TOKEN_BUFFER_SIZE] = "";

    if ((layer < RSSFU_VAL_IQC_ISDBT_LAYER_A) || (layer > RSSFU_VAL_IQC_ISDBT_LAYER_C))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Layer"); 
     
    if ((value < RSSFU_VAL_MAX_USEFUL_DRATE) || (value > RSSFU_VAL_MEAS_USEFUL_DRATE))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 3, "Value");

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    checkErr (rssfu_GetActualRepCaps (instrSession, RSSFU_ATTR_IQC_ISDBT_MEAS_USEFUL_DATA_RATE, "Layer", layer, buf1));

    if (!result) viCheckParm (RS_ERROR_INVALID_PARAMETER, 4, "Result");
    
    switch (value){ 
        case RSSFU_VAL_MEAS_USEFUL_DRATE:
            viCheckParm (rssfu_GetAttributeViReal64 (instrSession, buf1, RSSFU_ATTR_IQC_ISDBT_MEAS_USEFUL_DATA_RATE,
            result), 4, "Result");                 
        break;

        case RSSFU_VAL_MAX_USEFUL_DRATE:
            viCheckParm (rssfu_GetAttributeViReal64 (instrSession, buf1, RSSFU_ATTR_IQC_ISDBT_MAX_USEFUL_DATA_RATE,
            result), 4, "Result");                 
        break;                                                    
    }
   
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
} 


/*============================================================================/
 * Function: Configure ISDBT System
 * Purpose:  This function configures ISDBT coding parameters.
/============================================================================*/
ViStatus _VI_FUNC rssfu_confISDBTSystem (ViSession instrSession,
                                         ViInt32 ISDBTSystem)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_IQC_ISDBT_CODING_SYSTEM, ISDBTSystem),
                         2, "ISDBT System");
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}                                         
 

/*============================================================================/
 * Function: Configure ISDBT Coding Control
 * Purpose:  This function configures the mode of the coder.
/============================================================================*/
ViStatus _VI_FUNC rssfu_ConfigureISDBTCodingControl (ViSession instrSession,
                                                     ViInt32 control)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_ISDBT_CODING_CONTROL, control),
                         2, "Control");
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
} 

/*============================================================================/
 * Function: Configure ISDBT Layer Coding
 * Purpose:  This function configures ISDBT coding parameters for layer A, B
 *           or C.
/============================================================================*/
ViStatus _VI_FUNC rssfu_confISDBTLayerCoding (ViSession instrSession,
                                              ViInt32 portion, ViInt32 layer,
                                              ViInt32 constellation,
                                              ViInt32 codeRate,
                                              ViInt32 timeInterleaving)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buf1[RSSFU_TOKEN_BUFFER_SIZE] = "";

    if ((layer < RSSFU_VAL_IQC_ISDBT_LAYER_A) || (layer > RSSFU_VAL_IQC_ISDBT_LAYER_C))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 3, "Layer"); 

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    checkErr (rssfu_GetActualRepCaps (instrSession, RSSFU_ATTR_IQC_ISDBT_CONSTELLATION, "Layer", layer, buf1));


    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_IQC_ISDBT_PORTION, portion), 
                    2, "Portion");
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, buf1, RSSFU_ATTR_IQC_ISDBT_CONSTELLATION, constellation), 
                    4, "Constellation");
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, buf1, RSSFU_ATTR_IQC_ISDBT_CODE_RATE, codeRate), 
                    5, "Code Rate"); 
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, buf1, RSSFU_ATTR_IQC_ISDBT_TIME_INTERLEAVING, timeInterleaving), 
                    6, "Time Interleaving");                     
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/*============================================================================/
 * Function: Configure ISDBT Coding
 * Purpose:  This function configures ISDBT coding parameters.
/============================================================================*/
ViStatus _VI_FUNC rssfu_confISDBTCoding (ViSession instrSession,
                                         ViInt32 ISDBTMode, ViInt32 guardInterval,
                                         ViInt32 bandwidthVariation)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      model [RSSFU_IO_BUFFER_SIZE] = "";  

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckErr (rssfu_GetAttributeViString (instrSession, "",
                                               RS_ATTR_INSTRUMENT_MODEL,
                                               RSSFU_IO_BUFFER_SIZE, model));
        
    
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_IQC_ISDBT_MODE, ISDBTMode),
                         2, "ISDBT Mode"); 
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_IQC_ISDBT_GUARD_INTERVAL, guardInterval),
                         3, "Guard Interval");
    if (strstr (model, "SFC") == NULL) { 
        viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_IQC_ISDBT_BANDWIDTH_VARIATION, bandwidthVariation),
                         4, "Bandwidth Variation");
    }
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}                                         
                                         
 
/*============================================================================/
 * Function: Configure ISDBT Sub Channel
 * Purpose:  This function sets the sub-channel of the ISDB-TSB system.
/============================================================================*/
ViStatus _VI_FUNC rssfu_confISDBTSubChannel (ViSession instrSession,
                                         ViInt32 subChannel)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_IQC_ISDBT_CODING_SUB_CHANNEL, subChannel),
                         2, "Sub-Channel"); 
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}                                         
 
/*============================================================================/
 * Function: Configure ISDBT Segments
 * Purpose:  This function sets the segments for layer A, B or C.
/============================================================================*/
ViStatus _VI_FUNC rssfu_confISDBTSegments (ViSession instrSession,
                                           ViInt32 layer, ViInt32 segments)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buf1[RSSFU_TOKEN_BUFFER_SIZE] = "";

    if ((layer < RSSFU_VAL_IQC_ISDBT_LAYER_A) || (layer > RSSFU_VAL_IQC_ISDBT_LAYER_C))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Layer"); 

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    checkErr (rssfu_GetActualRepCaps (instrSession, RSSFU_ATTR_IQC_ISDBT_SEGMENTS, "Layer", layer, buf1));

    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, buf1, RSSFU_ATTR_IQC_ISDBT_SEGMENTS, segments), 
                    3, "Segments");
                
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

 /*============================================================================/
 * Function: Configure ISDBT Channel Bandwidth
 * Purpose:  This function configures the channel bandwidth for the ISDBT signal.
/============================================================================*/
ViStatus _VI_FUNC rssfu_ConfigureISDBTChannelBandwidth (ViSession instrSession,
                                          ViInt32 channelBandwidth)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_ISDBT_CODING_CHANNEL_BANDWIDTH, channelBandwidth),
                         2, "Channel Bandwidth");                          
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Read ISDBT Bandwidth
 * Purpose:  This function queries the bandwidth of the ISDB-T signal.
/============================================================================*/
ViStatus _VI_FUNC rssfu_readISDBTBandwidth (ViSession instrSession,
                                            ViReal64* bandwidth)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    if (!bandwidth) viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Bandwidth");
    viCheckParm (rssfu_GetAttributeViReal64 (instrSession, "", RSSFU_ATTR_IQC_ISDBT_BANDWIDTH,
                                            bandwidth), 2, "Bandwidth");
   Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}                                             


/*============================================================================/
 * Function: Enable ISDBT Special Settings
 * Purpose:  This function enables use of special ISDBT coding parameters.
/============================================================================*/
ViStatus _VI_FUNC rssfu_actEnableISDBTSpecial (ViSession instrSession,
                                               ViBoolean state)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rssfu_SetAttributeViBoolean (instrSession, "", RSSFU_ATTR_IQC_ISDBT_SPECIAL_SETTINGS, state),
                         2, "State");
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}                                               


/*============================================================================/
 * Function: Configure ISDBT Special Settings
 * Purpose:  This function configures special ISDBT coding parameters.
/============================================================================*/
ViStatus _VI_FUNC rssfu_confISDBTSpecial (ViSession instrSession,
                                          ViBoolean scrambler,
                                          ViBoolean reedSolomon,
                                          ViBoolean byteInterleaver,
                                          ViBoolean bitInterleaver,
                                          ViBoolean frequencyInterleaver,
                                          ViBoolean alertBroadcastFlag)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      model [RSSFU_IO_BUFFER_SIZE] = "";  

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckErr (rssfu_GetAttributeViString (instrSession, "",
                                               RS_ATTR_INSTRUMENT_MODEL,
                                               RSSFU_IO_BUFFER_SIZE, model));   
 
    if (strstr (model, "SFC") == NULL) {
        
        viCheckParm (rssfu_SetAttributeViBoolean (instrSession, "", RSSFU_ATTR_IQC_ISDBT_SCRAMBLER, scrambler),
                     2, "Scrambler"); 
        viCheckParm (rssfu_SetAttributeViBoolean (instrSession, "", RSSFU_ATTR_IQC_ISDBT_REED_SOLOMON, reedSolomon),
                     3, "Reed Solomon");
        viCheckParm (rssfu_SetAttributeViBoolean (instrSession, "", RSSFU_ATTR_IQC_ISDBT_BYTE_INTERLEAVER, byteInterleaver),
                     4, "Byte Interleaver");   
        viCheckParm (rssfu_SetAttributeViBoolean (instrSession, "", RSSFU_ATTR_IQC_ISDBT_BIT_INTERLEAVER, bitInterleaver),
                     5, "Bit Interleaver");
        viCheckParm (rssfu_SetAttributeViBoolean (instrSession, "", RSSFU_ATTR_IQC_ISDBT_FREQUENCY_INTERLEAVER, frequencyInterleaver),
                     6, "Frequency Interleaver");
    }
    viCheckParm (rssfu_SetAttributeViBoolean (instrSession, "", RSSFU_ATTR_IQC_ISDBT_ALERT_BROADCAST_FLAG, alertBroadcastFlag),
                 7, "Alert Broadcast Flag");

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}                                          
  

/*============================================================================/
 * Function: Configure ISDBT Special Pilots Settings
 * Purpose:  This function configures special ISDBT coding parameters.
/============================================================================*/
ViStatus _VI_FUNC rssfu_confISDBTSpecialPilotSettings (ViSession instrSession,
                                                       ViBoolean dataCarriers,
                                                       ViBoolean scatteredPilots,
                                                       ViBoolean continualPilots,
                                                       ViBoolean TMCCPilots,
                                                       ViBoolean ACPilots)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rssfu_SetAttributeViBoolean (instrSession, "", RSSFU_ATTR_IQC_ISDBT_DATA_CARRIERS, dataCarriers),
                 2, "Data Carriers"); 
    viCheckParm (rssfu_SetAttributeViBoolean (instrSession, "", RSSFU_ATTR_IQC_ISDBT_SCATTERED_PILOTS, scatteredPilots),
                 3, "Scattered Pilots");
    viCheckParm (rssfu_SetAttributeViBoolean (instrSession, "", RSSFU_ATTR_IQC_ISDBT_CONTINUAL_PILOTS, continualPilots),
                 4, "Continual Pilots");   
    viCheckParm (rssfu_SetAttributeViBoolean (instrSession, "", RSSFU_ATTR_IQC_ISDBT_TMCC_PILOTS, TMCCPilots),
                 5, "TMCC Pilots");
    viCheckParm (rssfu_SetAttributeViBoolean (instrSession, "", RSSFU_ATTR_IQC_ISDBT_AC_PILOTS, ACPilots),
                 6, "AC Pilots");
 
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}                                          
                                                        

/*============================================================================/
 * Function: Configure ISDBT Special Pilot Modulation Settings
 * Purpose:  This function configures special ISDBT modulation coding
 *           parameters.
/============================================================================*/
ViStatus _VI_FUNC rssfu_confISDBTSpecialPilotModulationSettings
             (ViSession instrSession, ViBoolean modDataCarriers,
              ViBoolean modScatteredPilots, ViBoolean modContinualPilots,
              ViBoolean modTMCCPilots, ViBoolean modACPilots)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rssfu_SetAttributeViBoolean (instrSession, "", RSSFU_ATTR_IQC_ISDBT_MOD_DATA_CARRIERS, modDataCarriers),
                 2, "Mod Data Carriers"); 
    viCheckParm (rssfu_SetAttributeViBoolean (instrSession, "", RSSFU_ATTR_IQC_ISDBT_MOD_SCATTERED_PILOTS, modScatteredPilots),
                 3, "Mod Scattered Pilots");
    viCheckParm (rssfu_SetAttributeViBoolean (instrSession, "", RSSFU_ATTR_IQC_ISDBT_MOD_CONTINUAL_PILOTS, modContinualPilots),
                 4, "Mod Continual Pilots");   
    viCheckParm (rssfu_SetAttributeViBoolean (instrSession, "", RSSFU_ATTR_IQC_ISDBT_MOD_TMCC_PILOTS, modTMCCPilots),
                 5, "Mod TMCC Pilots");
    viCheckParm (rssfu_SetAttributeViBoolean (instrSession, "", RSSFU_ATTR_IQC_ISDBT_MOD_AC_PILOTS, modACPilots),
                 6, "Mod AC Pilots");
 
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}                

/*============================================================================/
 * Function: Configure ISDBT Special Segment Settings
 * Purpose:  This function configures segments for special ISDBT coding.
/============================================================================*/
ViStatus _VI_FUNC rssfu_confISDBTSpecialSegmentSettings (ViSession instrSession,
                                                         ViInt32 segment,
                                                         ViBoolean state)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      model [RSSFU_IO_BUFFER_SIZE] = "";  

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckErr (rssfu_GetAttributeViString (instrSession, "",
                                               RS_ATTR_INSTRUMENT_MODEL,
                                               RSSFU_IO_BUFFER_SIZE, model));   

    if ((segment < 0) || (segment > 13))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Segment");
        
    if (strstr (model, "SFC") == NULL) {   
        if (Rs_Simulating(instrSession) == 0) {                /* call only when locked */

        viCheckErr(viPrintf (instrSession, ":IQC:ISDB:SEGM %ld,%s", segment, stateArr[state])); 
        }   
    
    }
        viCheckErr(rssfu_CheckStatus (instrSession));       
     
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}
                                                         
/*============================================================================/
 * Function: Configure ISDBT Special Segment Settings
 * Purpose:  This function configures AC data for special ISDBT modulation
 *           coding.
/============================================================================*/
ViStatus _VI_FUNC rssfu_confISDBTSpecialAcDataSettings (ViSession instrSession,
                                                        ViInt32 ACData_AC1,
                                                        ViInt32 ACData_AC2)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_IQC_ISDBT_AC_DATA1, ACData_AC1), 
                 2, "AC Data AC1");
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_IQC_ISDBT_AC_DATA2, ACData_AC2), 
                 3, "AC Data AC2");                     
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}                                                           
  
  
/*============================================================================/
 * Function: Configure ISDBT Special Guard Insertion Settings
 * Purpose:  This function configures Guard Insertion for special ISDBT
 *           cmodulation coding
/============================================================================*/
ViStatus _VI_FUNC rssfu_confISDBTSpecialGuardInsertionSettings
             (ViSession instrSession, ViBoolean guardInsertion)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
 
    viCheckParm (rssfu_SetAttributeViBoolean (instrSession, "", RSSFU_ATTR_IQC_ISDBT_GUARD_INSERTION, guardInsertion),
                 2, "Guard Insertion");
 
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}                                                           
    
/*============================================================================/
 * Function: Configure ISDBT TMCC Special Settings
 * Purpose:  This function configures special ISDBT coding parameters.
/============================================================================*/
ViStatus _VI_FUNC rssfu_ConfigureISDBTTmccSpecialSettings (ViSession instrSession,
                                                       ViInt32 TXParamSwitchingIndicator,
                                                       ViInt32 TMCCNext)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_ISDBT_SPECIAL_SETT_TX_PARAM_SWITCH_IND, TXParamSwitchingIndicator), 
                        2, "TX Param witching Indicator");
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_ISDBT_SPECIAL_SETTINGS_MOD_TMCC_NEXT, TMCCNext), 
                        3, "TMCC Next");
                         
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}  
    
  
/*============================================================================/
 * Function: Configure ISDBT Settings
 * Purpose:  This function configures ISDBT coding settings.
/============================================================================*/
ViStatus _VI_FUNC rssfu_confISDBTSettings (ViSession instrSession,
                                           ViInt32 layer, ViInt32 testTSPacket,
                                           ViInt32 PIDTestPacket, ViInt32 pid,
                                           ViInt32 prbs)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buf1[RSSFU_TOKEN_BUFFER_SIZE] = "";
    ViChar      model [RSSFU_IO_BUFFER_SIZE] = ""; 

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if ((layer < RSSFU_VAL_IQC_ISDBT_LAYER_A) || (layer > RSSFU_VAL_IQC_ISDBT_LAYER_C))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Layer"); 
    
    
    viCheckErr (rssfu_GetAttributeViString (instrSession, "",
                                               RS_ATTR_INSTRUMENT_MODEL,
                                               RSSFU_IO_BUFFER_SIZE, model));

    if (strstr (model, "SFU") != NULL) {      
            
        checkErr (rssfu_GetActualRepCaps (instrSession, RSSFU_ATTR_IQC_ISDBT_TEST_TS_PACKED, "Layer", layer, buf1));

        viCheckParm (rssfu_SetAttributeViInt32 (instrSession, buf1, RSSFU_ATTR_IQC_ISDBT_TEST_TS_PACKED, testTSPacket), 
                    3, "Test TS Packet");

        viCheckParm (rssfu_SetAttributeViInt32 (instrSession, buf1, RSSFU_ATTR_IQC_ISDBT_PID_TEST_PACKET, PIDTestPacket), 
                        4, "PID Test Packet");
    
        if (PIDTestPacket == RSSFU_IQC_PIDT_VAR)
            viCheckParm (rssfu_SetAttributeViInt32 (instrSession, buf1, RSSFU_ATTR_IQC_ISDBT_PID, pid), 
                            5, "PID");
    }
    else {  /*for BTC instrument Layer A, B, C not supported */ 
    
        viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_IQC_ISDBT_TEST_TS_PACKED_BTC, testTSPacket), 
                    3, "Test TS Packet");

        viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_IQC_ISDBT_PID_TEST_PACKET_BTC, PIDTestPacket), 
                        4, "PID Test Packet");
    
        if (PIDTestPacket == RSSFU_IQC_PIDT_VAR)
            viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_IQC_ISDBT_PID_BTC, pid), 
                            5, "PID");
    }
        
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_IQC_ISDBT_PRBS, prbs), 
                    6, "PRBS");                    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}                                           

/*============================================================================/
 * Function: Configure ISDBT Payload Test Stuff
 * Purpose:  This function allows selection of the payload in the TS packets
 *           of the ISDBT signal.
/============================================================================*/    
ViStatus _VI_FUNC rssfu_ConfigureISDBTPayloadTestStuff (ViSession instrSession,
                                          ViInt32 layer, ViInt32 payloadTestStuff)
{    
    ViStatus    error = VI_SUCCESS;
    ViChar      buf1[RSSFU_TOKEN_BUFFER_SIZE] = "";
    ViChar      model [RSSFU_IO_BUFFER_SIZE] = "";      

    viCheckErr (rssfu_GetAttributeViString (instrSession, "",
                                               RS_ATTR_INSTRUMENT_MODEL,
                                               RSSFU_IO_BUFFER_SIZE, model));
    
    if ((layer < RSSFU_VAL_IQC_ISDBT_LAYER_A) || (layer > RSSFU_VAL_IQC_ISDBT_LAYER_C))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Layer"); 
   
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
        
    if (strstr (model, "SFU") != NULL) {      
        checkErr (rssfu_GetActualRepCaps (instrSession, RSSFU_ATTR_ISDBT_SETTINGS_PAYLOAD_TEST_STUFF, "Layer", layer, buf1));

        viCheckParm (rssfu_SetAttributeViInt32 (instrSession, buf1, RSSFU_ATTR_ISDBT_SETTINGS_PAYLOAD_TEST_STUFF,
                        payloadTestStuff), 3, "Payload Test/Stuff");                  
    }
    else {  /*for BTC instrument Layer A, B, C not supported */ 
        viCheckParm (rssfu_SetAttributeViInt32 (instrSession, buf1, RSSFU_ATTR_ISDBT_SETTINGS_PAYLOAD_TEST_STUFF_BTC,
                        payloadTestStuff), 3, "Payload Test/Stuff");                  
    }
        
        
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/*****************************************************************************
 *--------------------------------- ISDB-Tmm --------------------------------*
 *****************************************************************************/
 /*============================================================================/
 * Function: Configure ISDBTmm Input Signal
 * Purpose:  This function configures ISDBTmm Input Signal parameters.
/============================================================================*/
ViStatus _VI_FUNC rssfu_confISDBTmmInputSignal (ViSession instrSession,
                                              ViInt32 layer, ViInt32 source,
                                              ViInt32 input, ViBoolean stuffing)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buf1[RSSFU_TOKEN_BUFFER_SIZE] = "";

    checkErr( Rs_LockSession (instrSession, VI_NULL));


    if ((layer < RSSFU_VAL_IQC_ISDBTMM_LAYER_A1A) || (layer > RSSFU_VAL_IQC_ISDBTMM_LAYER_B7))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Layer"); 

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    checkErr (rssfu_GetActualRepCaps (instrSession, RSSFU_ATTR_ISDBTMM_INPUT_SIGNAL_SOURCE, "SegLayer", layer, buf1));

    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, buf1, RSSFU_ATTR_ISDBTMM_INPUT_SIGNAL_SOURCE, source), 
                    3, "Source"); 
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_ISDBTMM_INPUT_SIGNAL_INPUT, input), 
                        4, "Input");
    viCheckParm (rssfu_SetAttributeViBoolean (instrSession, "", RSSFU_ATTR_ISDBTMM_INPUT_SIGNAL_STUFFING, stuffing), 
                    5, "Stuffing");       
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
} 


/*============================================================================/
 * Function: Configure ISDBTmm Test Signal
 * Purpose:  This function configures ISDBTmm Test Signal.
/============================================================================*/ 
ViStatus _VI_FUNC rssfu_confISDBTmmTestSignal (ViSession instrSession, ViInt32 testSignal)
{
    ViStatus    error = VI_SUCCESS;

    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_ISDBTMM_INPUT_SIGNAL_TEST_SIGNAL, testSignal), 
                    2, "Test Signal"); 
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
} 
                                            

/*============================================================================/
 * Function: Read ISDBTmm Input Data Rate
 * Purpose:  This function returns the input data rate.
/============================================================================*/ 
ViStatus _VI_FUNC rssfu_readISDBTmmInputDataRate (ViSession instrSession,
                                                ViReal64* inputDataRate)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    if (!inputDataRate) viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Input Data Rate");
    viCheckParm (rssfu_GetAttributeViReal64 (instrSession, "", RSSFU_ATTR_READ_ISDBTMM_INPUT_DATA_RATE,
                                            inputDataRate), 2, "Input Data Rate");
   Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}                                                

/*============================================================================/
 * Function: Read ISDBTmm Values
 * Purpose:  This function reads selected coding value.
/============================================================================*/ 
ViStatus _VI_FUNC rssfu_readISDBTmmValues (ViSession instrSession, ViInt32 layer,
                                         ViInt32 value, ViReal64* result)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buf1[RSSFU_TOKEN_BUFFER_SIZE] = "";

    if ((layer < RSSFU_VAL_IQC_ISDBTMM_LAYER_A1A) || (layer > RSSFU_VAL_IQC_ISDBTMM_LAYER_B7))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Layer"); 

     
    if ((value < RSSFU_VAL_MAX_USEFUL_DRATE) || (value > RSSFU_VAL_MAX_USEFUL_DRATE))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 3, "Value");

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    checkErr (rssfu_GetActualRepCaps (instrSession, RSSFU_ATTR_READ_ISDBTMM_MAX_USEFUL_DATA_RATE, "SegLayer", layer, buf1));

    if (!result) viCheckParm (RS_ERROR_INVALID_PARAMETER, 4, "Result");
    
    switch (value){ 
        
        case RSSFU_VAL_MAX_USEFUL_DRATE:
            viCheckParm (rssfu_GetAttributeViReal64 (instrSession, buf1, RSSFU_ATTR_READ_ISDBTMM_MAX_USEFUL_DATA_RATE,
            result), 4, "Result");                 
        break;                                                    
    }
   
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
} 

/*============================================================================/
 * Function: Configure ISDBTmm Coding Parameters
 * Purpose:  This function configures ISDB-Tmm coding parameters.
/============================================================================*/    
ViStatus _VI_FUNC rssfu_ConfigureISDBTmmCodingParameters (ViSession instrSession,
                                                        ViInt32 control, 
                                                          ViInt32 coder,
                                                          ViBoolean partialReception,
                                                          ViInt32 numberSuperSegments)
{    
    ViStatus    error = VI_SUCCESS;
    ViChar      buf1[RSSFU_TOKEN_BUFFER_SIZE] = "";  
   
    if ((coder < RSSFU_VAL_IQC_TMM_RECEPTION_CODER_A1) || (coder > RSSFU_VAL_IQC_TMM_RECEPTION_CODER_A2))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 3, "Coder"); 
   
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    checkErr (rssfu_GetActualRepCaps (instrSession, RSSFU_ATTR_ISDBTMM_CODING_PARTIAL_RECEPTION, "Coder", coder, buf1)); 
   
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_ISDBTMM_CODING_CONTROL,
                    control), 2, "Control");
    if (control == RSSFU_VAL_IQC_CONTROL_MODE_MANUAL) {
    
        viCheckParm (rssfu_SetAttributeViBoolean (instrSession, buf1, RSSFU_ATTR_ISDBTMM_CODING_PARTIAL_RECEPTION, 
                partialReception), 3, "Partial Reception");
  
        viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_ISDBTMM_CODING_NUMBER_OF_SUPER_SEGMENTS,
                    numberSuperSegments), 4, "Number Super Segments");                
    }
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/*============================================================================/
 * Function: Configure ISDBTmm Coding Segment Parameters
 * Purpose:  This function configures the ISDB-Tmm segment-specific
 *           parameters.
/============================================================================*/    
ViStatus _VI_FUNC rssfu_ConfigureISDBTmmCodingSegmentParameters
             (ViSession instrSession, ViInt32 superSegments,
              ViInt32 superSegmentsType, ViInt32 numSegments)
{    
    ViStatus    error = VI_SUCCESS;
    ViChar      buf1[RSSFU_TOKEN_BUFFER_SIZE] = "";

    if ((superSegments < 1) || (superSegments > 5))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Super Segments"); 
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    checkErr (rssfu_GetActualRepCaps (instrSession, RSSFU_ATTR_ISDBTMM_CODING_SUPER_SEGMENTS_TYPE, "SuperSegment", superSegments, buf1));

   
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, buf1, RSSFU_ATTR_ISDBTMM_CODING_SUPER_SEGMENTS_TYPE,
                    superSegmentsType), 3, "Super Segments Type");

    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, buf1, RSSFU_ATTR_ISDBTMM_CODING_NUMBER_SEGMENTS,
                    numSegments), 4, "Num Segments");                 
  
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Configure ISDBTmm Coding Layer Parameters
 * Purpose:  This function configures the ISDB-Tmm layer-specific parameters.
/============================================================================*/    
ViStatus _VI_FUNC rssfu_ConfigureISDBTmmCodingLayerParameters
             (ViSession instrSession, ViInt32 layer,
              ViInt32 OFDMSegments, ViInt32 centerSubchannel, ViInt32 constellation,
              ViInt32 codeRate, ViInt32 timeInterleaving)
{    
    ViStatus    error = VI_SUCCESS;
    ViChar      buf1[RSSFU_TOKEN_BUFFER_SIZE] = "";

    if ((layer < RSSFU_VAL_IQC_ISDBTMM_LAYER_A1A) || (layer > RSSFU_VAL_IQC_ISDBTMM_LAYER_B7))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Layer"); 


    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    checkErr (rssfu_GetActualRepCaps (instrSession, RSSFU_ATTR_ISDBTMM_CODING_LAYER_OFDM_SEGMENTS, "SegLayer", layer, buf1));

    if (layer < RSSFU_VAL_IQC_ISDBTMM_LAYER_B1) {  
        viCheckParm (rssfu_SetAttributeViInt32 (instrSession, buf1, RSSFU_ATTR_ISDBTMM_CODING_LAYER_OFDM_SEGMENTS,
                    OFDMSegments), 3, "OFDM Segments");
    }
    
    if (layer >= RSSFU_VAL_IQC_ISDBTMM_LAYER_B1) {    
        viCheckParm (rssfu_SetAttributeViInt32 (instrSession, buf1, RSSFU_ATTR_ISDBTMM_CODING_LAYER_CENTER_SUBCHANNEL,
                    centerSubchannel), 4, "Center Subchannel");
    }
    
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, buf1, RSSFU_ATTR_ISDBTMM_CODING_LAYER_CONSTELLATION,
                    constellation), 5, "Constellation");

    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, buf1, RSSFU_ATTR_ISDBTMM_CODING_LAYER_CODE_RATE,
                    codeRate), 6, "Code Rate");
    
     if ((layer < RSSFU_VAL_IQC_ISDBTMM_LAYER_A1A) || (layer > RSSFU_VAL_IQC_ISDBTMM_LAYER_A2C)) {
        if (timeInterleaving  == RSSFU_VAL_IQC_ISDBT_TIME_INTER_32 )
            viCheckParm( RS_ERROR_INVALID_PARAMETER, 7, "Time Interleaving"); 
     }
    
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, buf1, RSSFU_ATTR_ISDBTMM_CODING_LAYER_TIME_INTERLEAVING,
                    timeInterleaving), 7, "Time Interleaving");
  
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Configure ISDBTmm Coding
 * Purpose:  This function configures ISDB-Tmm coding parameters.
/============================================================================*/    
ViStatus _VI_FUNC rssfu_ConfigureISDBTmmCoding (ViSession instrSession,
                                                ViBoolean emergencyAlarm,
                                                ViInt32 ISDBTMode,
                                                ViInt32 guardInterval,
                                                ViInt32 referenceChannel)
{    
    ViStatus    error = VI_SUCCESS;
   
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
   
    viCheckParm (rssfu_SetAttributeViBoolean (instrSession, "", RSSFU_ATTR_ISDBTMM_CODING_EMERGENCY_ALARM, 
                emergencyAlarm), 2, "Emergency Alarm");
  
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_ISDBTMM_CODING_ISDBT_MODE,
                    ISDBTMode), 3, "ISDBT Mode");                 
    
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_ISDBTMM_CODING_GUARD_INTERVAL,
                    guardInterval), 4, "Guard Interval");
    
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_ISDBTMM_CODING_REFERENCE_CHANNEL,
                    referenceChannel), 5, "Reference Channel");
  
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Read Absolute Output Power
 * Purpose:  This function displays the nominal frequency bandwidth of the
 *           ISDB-Tmm signal. It is a function of the ISDB-Tmmm mode and the
 *           reference channel
./============================================================================*/
ViStatus _VI_FUNC rssfu_ReadISDBTmmRFBandwidth (ViSession instrSession,
                                                ViReal64 *bandwidth)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rssfu_GetAttributeViReal64 (instrSession, "", RSSFU_ATTR_ISDBTMM_CODING_BANDWIDTH, 
                bandwidth), 2, "Bandwidth");

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Enable ISDBTmm Special Settings
 * Purpose:  This function enables use of special ISDBTmm coding parameters.
/============================================================================*/
ViStatus _VI_FUNC rssfu_actEnableISDBTmmSpecial (ViSession instrSession,
                                               ViBoolean state)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rssfu_SetAttributeViBoolean (instrSession, "", RSSFU_ATTR_ISDBTMM_SPECIAL_SETTINGS_STATE, state),
                         2, "State");
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}                                               


/*============================================================================/
 * Function: Configure ISDBTmm Special Settings
 * Purpose:  This function configures special ISDBTmm coding parameters.
/============================================================================*/
ViStatus _VI_FUNC rssfu_confISDBTmmSpecialSettings (ViSession instrSession,
                                          ViBoolean scrambler,
                                          ViBoolean reedSolomon,
                                          ViBoolean byteInterleaver,
                                          ViBoolean bitInterleaver,
                                          ViBoolean frequencyInterleaver)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rssfu_SetAttributeViBoolean (instrSession, "", RSSFU_ATTR_ISDBTMM_SPECIAL_SCRAMBLER, scrambler),
                 2, "Scrambler"); 
    viCheckParm (rssfu_SetAttributeViBoolean (instrSession, "", RSSFU_ATTR_ISDBTMM_SPECIAL_REED_SOLOMON, reedSolomon),
                 3, "Reed Solomon");
    viCheckParm (rssfu_SetAttributeViBoolean (instrSession, "", RSSFU_ATTR_ISDBTMM_SPECIAL_SETTINGS_BYTE_INTERLEAVER, byteInterleaver),
                 4, "Byte Interleaver");   
    viCheckParm (rssfu_SetAttributeViBoolean (instrSession, "", RSSFU_ATTR_ISDBTMM_SPECIAL_SETTINGS_BIT_INTERLEAVER, bitInterleaver),
                 5, "Bit Interleaver");
    viCheckParm (rssfu_SetAttributeViBoolean (instrSession, "", RSSFU_ATTR_ISDBTMM_SPECIAL_SETTINGS_FREQUENCY_INTERLEAVER, frequencyInterleaver),
                 6, "Frequency Interleaver");
   

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}                                          
  

/*============================================================================/
 * Function: Configure ISDBTmm TMCC Special Settings
 * Purpose:  This function configures special ISDBTmm coding parameters.
/============================================================================*/
ViStatus _VI_FUNC rssfu_ConfigureISDBTmmTmccSpecialSettings (ViSession instrSession,
                                                       ViInt32 TXParamSwitchingIndicator,
                                                       ViInt32 TMCCNext)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_ISDBTMM_SPECIAL_SETTINGS_TX_PARAM_SWITCHING_INDICATOR, TXParamSwitchingIndicator), 
                        2, "TX Param witching Indicator");
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_ISDBTMM_SPECIAL_SETTINGS_TMCC_NEXT, TMCCNext), 
                        3, "TMCC Next");
                         
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}  
    
    
  
/*============================================================================/
 * Function: Configure ISDBTmm Settings
 * Purpose:  This function configures ISDBTmm coding settings.
/============================================================================*/
ViStatus _VI_FUNC rssfu_confISDBTmmSettings (ViSession instrSession,
                                           ViInt32 testTSPacket,
                                           ViInt32 PIDTestPacket, ViInt32 pid,
                                           ViInt32 payloadTestStuff, ViInt32 prbs)
{
    ViStatus    error = VI_SUCCESS;
   
    checkErr( Rs_LockSession (instrSession, VI_NULL));

  
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_ISDBTMM_SETTINGS_TEST_TS_PACKET, testTSPacket), 
                    2, "Test TS Packet");
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_ISDBTMM_SETTINGS_PID_TEST_PACKET, PIDTestPacket), 
                    3, "PID Test Packet");
    
    if (PIDTestPacket == RSSFU_IQC_PIDT_VAR)
        viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_ISDBTMM_SETTINGS_PID, pid), 4, "PID");
   
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_ISDBTMM_SETTINGS_PAYLOAD_TEST_TS_PACKET,
                    payloadTestStuff), 5, "Payload Test/Stuff");    
    
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_ISDBTMM_SETTINGS_PRBS, prbs), 
                    6, "PRBS");                    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}                                           


/*****************************************************************************
 *----------------------------------- DMB-T ---------------------------------*
 *****************************************************************************/
/*============================================================================/
 * Function: Read DMBT Coder Status
 * Purpose:  This function reads the status of DMB-T coder.
/============================================================================*/
ViStatus _VI_FUNC rssfu_readDMBTCoderStatus(ViSession instrSession,
                                            ViChar  status[])
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSSFU_IO_BUFFER_SIZE] = "\0"; 
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    if (status == NULL) viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Status");
    viCheckErr (rssfu_GetAttributeViString (instrSession, "", RSSFU_ATTR_IQC_DMBT_LOCK,
                                            RSSFU_IO_BUFFER_SIZE, buffer));
    if (status  != NULL)
    {
        strncpy (status, buffer, strlen(buffer));
        status [strlen(buffer)] = '\0';
    }
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Configure DMBT Input Signal
 * Purpose:  This function configures DMBT Input Signal parameters.
/============================================================================*/
ViStatus _VI_FUNC rssfu_confDMBTInputSignal(ViSession instrSession,
                                            ViInt32 source,
                                            ViInt32 input,
                                            ViInt32 stuffing)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_IQC_DMBT_SOUR, source),
                         2, "Source"); 
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_IQC_DMBT_INP, input),
                         3, "Input");
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_IQC_DMBT_STUF, stuffing),
                         4, "Stuffing");
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Configure DMBT Test Signal
 * Purpose:  This function configures DMBT Test Signal.
/============================================================================*/
ViStatus _VI_FUNC rssfu_confDMBTTestSignal(ViSession instrSession,
                                           ViInt32 testSignal)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_IQC_DMBT_TEST, testSignal),
                         2, "Test Signal"); 
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Read DMBT Values
 * Purpose:  This function reads selected coding value.
/============================================================================*/
ViStatus _VI_FUNC rssfu_readDMBTValues(ViSession instrSession,
                                       ViInt32 value,
                                       ViReal64* result)
{
    ViStatus    error = VI_SUCCESS;
    ViInt32     int_result = 0; 
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    if ((value < RSSFU_VAL_PACKET_LENGTH) || (value > RSSFU_VAL_USEFUL_DRATE))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Value");
        
    switch (value){
        case RSSFU_VAL_PACKET_LENGTH:
            viCheckParm (rssfu_GetAttributeViInt32 (instrSession, "", RSSFU_ATTR_IQC_DMBT_PACKET_LENGTH,
                                                    &int_result), 3, "Result"); 
            *result = (ViReal64)int_result;
        break;
    
        case RSSFU_VAL_MAX_USEFUL_DRATE:
            viCheckParm (rssfu_GetAttributeViReal64 (instrSession, "", RSSFU_ATTR_IQC_DMBT_MAX_USEFUL_DATA_RATE,
                                                     result), 3, "Result");
        break;
        case RSSFU_VAL_MEAS_DRATE:
            viCheckParm (rssfu_GetAttributeViReal64 (instrSession, "", RSSFU_ATTR_IQC_DMBT_MEASURED_DATA_RATE,
                                                     result), 3, "Result");
        break;
        case RSSFU_VAL_USEFUL_DRATE:
            viCheckParm (rssfu_GetAttributeViReal64 (instrSession, "", RSSFU_ATTR_IQC_DMBT_USEFUL_DATA_RATE,
                                                     result), 3, "Result");
        break;
     }           
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Configure DMBT Network
 * Purpose:  This function configures DMBT Network parameters.
/============================================================================*/
ViStatus _VI_FUNC rssfu_confDMBTNetworkMode(ViSession instrSession,
                                        ViInt32 networkMode)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rssfu_SetAttributeViInt32(instrSession, "", RSSFU_ATTR_IQC_DMBT_NETW,
                                                     networkMode), 2, "Network Mode");
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/*============================================================================/
 * Function: Configure DMBT Network - Obsolete
 * Purpose:  This function configures DMBT Network parameters.
/============================================================================*/
ViStatus _VI_FUNC rssfu_confDVBCNetwork(ViSession instrSession,
                                        ViInt32 networkMode,
                                        ViReal64 additionalDelay)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rssfu_SetAttributeViInt32(instrSession, "", RSSFU_ATTR_IQC_DMBT_NETW,
                                                     networkMode), 2, "Network Mode");
    viCheckParm (rssfu_SetAttributeViReal64 (instrSession, "", RSSFU_ATTR_IQC_DMBT_DEL_ADD,
                                                     additionalDelay), 3, "Additional Delay");
                                                     
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Configure DMBT Control
 * Purpose:  This function configures DMBT coding control.
/============================================================================*/
ViStatus _VI_FUNC rssfu_confDMBTControl(ViSession instrSession,
                                        ViInt32 control,
                                        ViInt32 channelBandwidth)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rssfu_SetAttributeViInt32(instrSession, "", RSSFU_ATTR_IQC_DMBT_CONT,
                                                     control), 2, "Control");
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_IQC_DMBT_CHAN_BAND,
                                                     channelBandwidth), 3, "Channel Bandwidth");
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Configure DMBT Coding
 * Purpose:  This function configures DMBT coding parameters.
/============================================================================*/
ViStatus _VI_FUNC rssfu_confDMBTCoding(ViSession instrSession,
                                       ViInt32 constellation,
                                       ViInt32 codeRate,
                                       ViInt32 guardInterval,
                                       ViInt32 timeInterleaver)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_IQC_DMBT_CONS, constellation),
                         2, "Constellation"); 
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_IQC_DMBT_RATE, codeRate),
                         3, "Code Rate"); 
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_IQC_DMBT_GUAR, guardInterval),
                         4, "Guard Interval"); 
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_IQC_DMBT_TIME, timeInterleaver),
                         5, "Time Interleaver");       
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Read DMBT SFN Values
 * Purpose:  This function reads selected SFN value.
/============================================================================*/
ViStatus _VI_FUNC rssfu_readDMBTSFNValues (ViSession instrSession,
                                           ViInt32 value, ViReal64 *result)
{
    ViStatus    error = VI_SUCCESS;

    switch (value)
    {
        case RSSFU_VAL_SIGN_TIMESTAMP:
            viCheckParm (rssfu_GetAttributeViReal64 (instrSession, "", RSSFU_ATTR_IQC_DMBT_SIGN,
            result), 3, "Result");                 
        break;
        
        case RSSFU_VAL_MAX_DELAY:
            viCheckParm (rssfu_GetAttributeViReal64 (instrSession, "",
                                                     RSSFU_ATTR_IQC_DMBT_DEL_MAX, result),
                                 3, "Result");
            break;
        case RSSFU_VAL_LOC_DELAY:
            viCheckParm (rssfu_GetAttributeViReal64 (instrSession, "",
                                                     RSSFU_ATTR_IQC_DMBT_DEL_LOC, result),
                                 3, "Result");
            break;
            
        case RSSFU_VAL_ADD_DELAY:
            viCheckParm (rssfu_GetAttributeViReal64 (instrSession, "",
                                                     RSSFU_ATTR_IQC_DMBT_DEL_ADD, result),
                                 3, "Result");
            break; 
                    
            default:
            viCheckParm( RS_ERROR_INVALID_PARAMETER, 3, "Value"); 

    }
Error:
    return error;
}

/*============================================================================/
 * Function: Enable DMBT Special Settings
 * Purpose:  This function enables use of special DMBT coding parameters.
/============================================================================*/
ViStatus _VI_FUNC rssfu_actEnableDMBTSpecial(ViSession instrSession,
                                             ViBoolean state)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rssfu_SetAttributeViBoolean (instrSession, "", RSSFU_ATTR_IQC_DMBT_SETT, state),
                         2, "State");
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Configure DMBT Special Settings
 * Purpose:  This function configures special DMBT coding parameters.
/============================================================================*/    
ViStatus _VI_FUNC rssfu_confDMBTSpecial (ViSession instrSession,
                                         ViBoolean syncWordRandomized,
                                         ViBoolean randomizer,
                                         ViInt32 randomizerRestart,
                                         ViBoolean byteInterleaver,
                                         ViBoolean pilotData,
                                         ViBoolean GIPowerBoost)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rssfu_SetAttributeViBoolean (instrSession, "", RSSFU_ATTR_IQC_DMBT_SYNC, syncWordRandomized),
                         2, "Sync Word Randomized");
    viCheckParm (rssfu_SetAttributeViBoolean (instrSession, "", RSSFU_ATTR_IQC_DMBT_RAND, randomizer),
                         3, "Randomizer");
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_IQC_DMBT_RAND_REST, randomizerRestart),
                         4, "Randomizer Restart");
    viCheckParm (rssfu_SetAttributeViBoolean (instrSession, "", RSSFU_ATTR_IQC_DMBT_BYTE, byteInterleaver),
                         5, "Byte Interleaver");
    viCheckParm (rssfu_SetAttributeViBoolean (instrSession, "", RSSFU_ATTR_IQC_DMBT_TPS, pilotData),
                         6, "Pilot Data");       
    viCheckParm (rssfu_SetAttributeViBoolean (instrSession, "", RSSFU_ATTR_IQC_DMBT_GIP, GIPowerBoost),
                         7, "GI Power Boost");                          
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Configure DMBT Settings
 * Purpose:  This function configures DMBT coding settings.
/============================================================================*/    
ViStatus _VI_FUNC rssfu_confDMBTSettings (ViSession instrSession,
                                          ViBoolean controlFrames,
                                          ViInt32 testTSPacket,
                                          ViInt32 PIDTestPacket, 
                                          ViInt32 PID,
                                          ViInt32 payloadTestStuff, 
                                          ViInt32 PRBS)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rssfu_SetAttributeViBoolean (instrSession, "", RSSFU_ATTR_IQC_DMBT_FRAM, controlFrames),
                         2, "Control Frames");
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_IQC_DMBT_TSP, testTSPacket),
                         3, "Test TS Packet");
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_IQC_DMBT_PIDT, PIDTestPacket),
                         4, "PID Test Packet");
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_IQC_DMBT_PID, PID),
                         5, "PID");
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_IQC_DMBT_PAYL, payloadTestStuff),
                         6, "Payload Test/Stuff"); 
    switch(payloadTestStuff){
        case RSSFU_IQC_PRBS:
            viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_IQC_DMBT_PRBS, PRBS),
                         7, "PRBS");  
    }                         
                         
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


                                                                                 
/*****************************************************************************   
 *----------------------------------- DTMB ----------------------------------*  
 *****************************************************************************/ 

/*============================================================================/
 * Function: Read DTMB Coder Status
 * Purpose:  This function reads the status of DTMB coder.
/============================================================================*/
ViStatus _VI_FUNC rssfu_readDTMBCoderStatus (ViSession instrSession,
                                              ViChar status[])

{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSSFU_IO_BUFFER_SIZE]="\0";  
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    if (status == NULL) viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Status");
    viCheckErr (rssfu_GetAttributeViString (instrSession, "", RSSFU_ATTR_IQC_DTMB_LOCK,
                                            RSSFU_IO_BUFFER_SIZE, buffer));
    if (status != NULL)
        strncpy (status, buffer, strlen(buffer)); 
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
} 


/*============================================================================/
 * Function: Configure DTMB Input Signal
 * Purpose:  This function configures DTMB Input Signal parameters.
/============================================================================*/
ViStatus _VI_FUNC rssfu_confDTMBInputSignal (ViSession instrSession,
                                              ViInt32 source, ViInt32 input,
                                              ViBoolean stuffing)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      model [RSSFU_IO_BUFFER_SIZE] = "";  

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckErr (rssfu_GetAttributeViString (instrSession, "",
                                               RS_ATTR_INSTRUMENT_MODEL,
                                               RSSFU_IO_BUFFER_SIZE, model));
    if (strstr (model, "BTC") == NULL) {
        viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_IQC_DTMB_CODING_SOURCE, source),
                             2, "Source"); 
        viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_IQC_DTMB_CODING_INPUT, input),
                             3, "Input");
    }
    viCheckParm (rssfu_SetAttributeViBoolean (instrSession, "", RSSFU_ATTR_IQC_DTMB_CODING_STUFFING, stuffing),
                         4, "Stuffing");
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}
 
/*============================================================================/
 * Function: Configure DTMB Test Signal
 * Purpose:  This function configures DTMB Test Signal.
/============================================================================*/
ViStatus _VI_FUNC rssfu_confDTMBTestSignal (ViSession instrSession,
                                             ViInt32 testSignal)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    if ((testSignal < RSSFU_DTMB_TEST_TTSP) || (testSignal > RSSFU_DTMB_TEST_TTSP))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Test Signal");
    
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_IQC_DTMB_CODER_TEST_SIGNAL, testSignal),
                         2, "Test Signal"); 
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}
 

/*============================================================================/
 * Function: Read DTMB Values
 * Purpose:  This function reads selected coding value.
/============================================================================*/
ViStatus _VI_FUNC rssfu_readDTMBValues(ViSession instrSession,
                                       ViInt32 value,
                                       ViReal64* result)
{
    ViStatus    error = VI_SUCCESS;
    ViInt32     int_result = 0; 
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    if ((value < RSSFU_VAL_PACKET_LENGTH) || (value > RSSFU_VAL_USEFUL_DRATE))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Value");
        
    switch (value){
        case RSSFU_VAL_PACKET_LENGTH:
            viCheckParm (rssfu_GetAttributeViInt32 (instrSession, "", RSSFU_ATTR_IQC_DTMB_CODING_PACKED_LENGTH,
                                                    &int_result), 3, "Result"); 
            *result = (ViReal64)int_result;
        break;
    
        case RSSFU_VAL_MAX_USEFUL_DRATE:
            viCheckParm (rssfu_GetAttributeViReal64 (instrSession, "", RSSFU_ATTR_IQC_DTMB_CODING_MAX_USEFUL_DATA_RATE,
                                                     result), 3, "Result");
        break;
        case RSSFU_VAL_MEAS_DRATE:
            viCheckParm (rssfu_GetAttributeViReal64 (instrSession, "", RSSFU_ATTR_IQC_DTMB_CODING_MEAS_DATA_RATE,
                                                     result), 3, "Result");
        break;
        case RSSFU_VAL_USEFUL_DRATE:
            viCheckParm (rssfu_GetAttributeViReal64 (instrSession, "", RSSFU_ATTR_IQC_DTMB_CODING_USEFUL_DATA_RATE,
                                                     result), 3, "Result");
        break;
     }           
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/*============================================================================/
 * Function: Configure DTMB Network
 * Purpose:  This function configures DTMB Network parameters.
/============================================================================*/
ViStatus _VI_FUNC rssfu_confDTMBNetworkMode (ViSession instrSession,
                                             ViInt32 networkMode,
                                             ViBoolean singleCarrierMode,
                                             ViBoolean controlFrames)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_IQC_DTMB_CODING_NETWORK_MODE, networkMode),
                         2, "Network Mode"); 
    viCheckParm (rssfu_SetAttributeViBoolean (instrSession, "", RSSFU_ATTR_IQC_DTMB_CODING_SINGLE_CARRIER_MODE, singleCarrierMode),
                         3, "Single Carrier Mode"); 
    viCheckParm (rssfu_SetAttributeViBoolean (instrSession, "", RSSFU_ATTR_IQC_DTMB_CODING_CONTROL_FRAMES, controlFrames),
                         4, "Control Frames"); 
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Configure DTMB Dual Pilot Tone
 * Purpose:  This function switches the pilot tone insertion  on or off.
/============================================================================*/
ViStatus _VI_FUNC rssfu_confDTMBDualPilotTone (ViSession instrSession,
                                             ViBoolean dualPilotTone)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rssfu_SetAttributeViBoolean (instrSession, "", RSSFU_ATTR_IQC_DTMB_CODING_DUAL_PILOT_TONE, dualPilotTone),
                         2, "Dual Pilot Tone");
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}
    
/*============================================================================/
 * Function: Configure DTMB Channel Bandwidth
 * Purpose:  This function configures the channel bandwidth for the DTMB signal.
/============================================================================*/
ViStatus _VI_FUNC rssfu_confDTMBChannelBandwidth (ViSession instrSession,
                                          ViInt32 channelBandwidth)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_IQC_DTMB_CODING_CHANNEL_BANDWIDTH, channelBandwidth),
                         2, "Channel Bandwidth");                          
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}



/*============================================================================/
 * Function: Configure DTMB Coding
 * Purpose:  This function configures DTMB coding parameters.
/============================================================================*/
ViStatus _VI_FUNC rssfu_confDTMBCoding (ViSession instrSession,
                                         ViInt32 constellation, ViInt32 codeRate,
                                         ViInt32 guardInterval,
                                         ViInt32 timeInterleaver)

{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_IQC_DTMB_CODING_CONSTELLATION, constellation),
                         2, "Constellation"); 
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_IQC_DTMB_CODING_CODE_RATE, codeRate),
                         3, "Code Rate");
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_IQC_DTMB_CODING_GUARD_INTERVAL, guardInterval),
                         4, "Guard Interval");
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_IQC_DTMB_CODING_TIME_INTERLEAVER, timeInterleaver),
                         5, "Time Interleaver");                         
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Configure DTMB Guard Interval PN
 * Purpose:  This function configures DTMB guard interval PN.
/============================================================================*/
ViStatus _VI_FUNC rssfu_ConfigureDTMBGuardIntervalPN (ViSession instrSession,
                                         ViInt32 guardIntervalPN)

{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_DTMB_CODING_GUARD_INTERVAL_PN, guardIntervalPN),
                         2, "GuardInterval PN"); 
                
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Read DTMB SFN Values
 * Purpose:  This function reads selected SFN value.
/============================================================================*/
ViStatus _VI_FUNC rssfu_readDTMBSFNValues (ViSession instrSession,
                                            ViInt32 value, ViReal64* result)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      model [RSSFU_IO_BUFFER_SIZE] = "";  

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckErr (rssfu_GetAttributeViString (instrSession, "",
                                               RS_ATTR_INSTRUMENT_MODEL,
                                               RSSFU_IO_BUFFER_SIZE, model)); 
    
    if (!result) viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Result");
    
    switch (value){ 

        case RSSFU_VAL_MAX_DELAY:
            viCheckParm (rssfu_GetAttributeViReal64 (instrSession, "", RSSFU_ATTR_IQC_DTMB_CODING_LOCAL_MAX_DELAY,
            result), 3, "Result");                 
        break;
    
        case RSSFU_VAL_LOC_DELAY:
            viCheckParm (rssfu_GetAttributeViReal64 (instrSession, "", RSSFU_ATTR_IQC_DTMB_CODING_LOCAL_DELAY,
            result), 3, "Result");                 
        break;


        case RSSFU_VAL_SFN_DISTRIB_DELAY:
            if (strstr (model, "SFC") == NULL) {
                viCheckParm (rssfu_GetAttributeViReal64 (instrSession, "",
                                                     RSSFU_ATTR_DTMB_SFN_DISTRIBUTION_DELAY, result),
                                 3, "Result");
            }
        break;
                    
        case RSSFU_VAL_SFN_ADJUST_DELAY:
            if (strstr (model, "SFC") == NULL) {
                viCheckParm (rssfu_GetAttributeViReal64 (instrSession, "",
                                                     RSSFU_ATTR_DTMB_SFN_DELAY_ADJUST, result),
                                 3, "Result");
            }
        break;
                
        case RSSFU_VAL_SFN_FREQ_OFFSET:
            if (strstr (model, "SFC") == NULL) { 
                viCheckParm (rssfu_GetAttributeViReal64 (instrSession, "",
                                                     RSSFU_ATTR_DTMB_SFN_FREQUENCY_OFFSET, result),
                                 3, "Result");
            }
        break;
        case RSSFU_VAL_SFN_TX_POWER:
            if (strstr (model, "SFC") == NULL) {
                viCheckParm (rssfu_GetAttributeViReal64 (instrSession, "",
                                                     RSSFU_ATTR_DTMB_SFN_TX_POWER, result),
                                 3, "Result");
            }
        break;          
                    
        default:
            viCheckParm( RS_ERROR_INVALID_PARAMETER, 3, "Value"); 
    }
        
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
} 


/*============================================================================/
 * Function: Configure DTMB SFN Settings
 * Purpose:  This function configures DTMB SFN coding parameters.
/============================================================================*/
ViStatus _VI_FUNC rssfu_ConfigureDTMBSFNSettings (ViSession instrSession,
                                                  ViReal64 RFDelay,
                                                  ViReal64 delayOffset,
                                                  ViInt32 TXAddress)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      model [RSSFU_IO_BUFFER_SIZE] = "";  

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckErr (rssfu_GetAttributeViString (instrSession, "",
                                               RS_ATTR_INSTRUMENT_MODEL,
                                               RSSFU_IO_BUFFER_SIZE, model));

    viCheckParm (rssfu_SetAttributeViReal64 (instrSession, "", RSSFU_ATTR_DTMB_SFN_DELAY_OFFSET, delayOffset),
                         3, "Delay Offset");

    if (strstr (model, "SFU") != NULL) {    
        viCheckParm (rssfu_SetAttributeViReal64 (instrSession, "", RSSFU_ATTR_DTMB_SFN_RF_DELAY, RFDelay),
                             2, "RF Delay"); 

        viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_DTMB_SFN_TX_ADDRESS, TXAddress),
                             4, "TX Address");
}
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/*============================================================================/
 * Function: Enable DTMB Special Settings
 * Purpose:  This function enables use of special DTMB coding parameters.
/============================================================================*/
ViStatus _VI_FUNC rssfu_actEnableDTMBSpecial (ViSession instrSession,
                                               ViBoolean state)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rssfu_SetAttributeViBoolean (instrSession, "", RSSFU_ATTR_IQC_DTMB_SPECIAL_SETTINGS, state),
                         2, "State");
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Enable DTMB Special Settings
 * Purpose:  This function configures special DTMB coding parameters.
/============================================================================*/
ViStatus _VI_FUNC rssfu_ConfigureDTMBSpecialSettings (ViSession instrSession,
                                                      ViBoolean GIPowerBoost,
                                                      ViBoolean SIPowerNormalization,
                                                      ViBoolean coChannelInterferer)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      model [RSSFU_IO_BUFFER_SIZE] = "";  

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckErr (rssfu_GetAttributeViString (instrSession, "",
                                               RS_ATTR_INSTRUMENT_MODEL,
                                               RSSFU_IO_BUFFER_SIZE, model));
    
    if (strstr (model, "SFC") == NULL) {
        viCheckParm (rssfu_SetAttributeViBoolean (instrSession, "", RSSFU_ATTR_IQC_DTMB_SPECIAL_GI_POWER_BOOST, GIPowerBoost),
                           2, "GI Power Boost");
    
    viCheckParm (rssfu_SetAttributeViBoolean (instrSession, "", RSSFU_ATTR_DTMB_SPECIAL_SI_POWER_NORMALIZATION, SIPowerNormalization),
                           3, "SI Power Normalization");
    }
    if (strstr (model, "SFU") != NULL) {   
        viCheckParm (rssfu_SetAttributeViBoolean (instrSession, "", RSSFU_ATTR_DTMB_SPECIAL_CO_CHANNEL_INTERFERER, coChannelInterferer),
                         4, "Co-Channel Interferer");   
    }
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Configure DTMB Settings
 * Purpose:  This function configures DTMB coding settings.
/============================================================================*/
ViStatus _VI_FUNC rssfu_confDTMBSettings (ViSession instrSession,
                                           ViInt32 testTSPacket,
                                           ViInt32 PIDTestPacket, ViInt32 pid,
                                           ViInt32 payloadTestStuff, ViInt32 prbs)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      model [RSSFU_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckErr (rssfu_GetAttributeViString (instrSession, "",
                                               RS_ATTR_INSTRUMENT_MODEL,
                                               RSSFU_IO_BUFFER_SIZE, model));
    
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_IQC_DTMB_SETTINGS_TEST_TS_PACKET, testTSPacket),
                         2, "Test TS Packet");
    if (strstr (model, "SFC") == NULL) {
        viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_IQC_DTMB_SETTINGS_PID_TEST_PACKET, PIDTestPacket),
                             3, "PID Test Packet"); 
        viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_IQC_DTMB_SETTINGS_PID, pid),
                             4, "PID Test Packet"); 
    }
    
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_IQC_DTMB_SETTINGS_PAYLOAD_TEST_STUFF, payloadTestStuff),
                         5, "Payload Test/Stuff");      
    switch (payloadTestStuff){
        case  RSSFU_IQC_PRBS:
            viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_IQC_DTMB_SETTINGS_PRBS, prbs),
                         6, "PRBS"); 
    }   
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
} 


/*****************************************************************************
 *----------------------------------- DVB-S2 --------------------------------*
 *****************************************************************************/
/*============================================================================/
 * Function: Read DVBS2 Coder Status
 * Purpose:  This function reads the status of DVBS coder.
/============================================================================*/
ViStatus _VI_FUNC rssfu_readDVBS2CoderStatus(
    ViSession   instrSession,
    ViChar  status[]
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSSFU_IO_BUFFER_SIZE] = "\0";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    if (status == NULL) viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Status");
    viCheckErr (rssfu_GetAttributeViString (instrSession, "", RSSFU_ATTR_IQC_DVBS2_LOCK,
                                            RSSFU_IO_BUFFER_SIZE, buffer));
    strcpy (status, buffer); 
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Configure DVBS2 Input Signal
 * Purpose:  This function configures DVBS2 Input Signal parameters.
/============================================================================*/
ViStatus _VI_FUNC rssfu_confDVBS2InputSignal(ViSession  instrSession,
                                             ViInt32    source,
                                             ViInt32    input,
                                             ViInt32    stuffing)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      model [RSSFU_IO_BUFFER_SIZE] = "";  

    checkErr( Rs_LockSession (instrSession, VI_NULL)); 

    viCheckErr (rssfu_GetAttributeViString (instrSession, "",
                                               RS_ATTR_INSTRUMENT_MODEL,
                                               RSSFU_IO_BUFFER_SIZE, model));
    if (strstr (model, "BTC") == NULL) { 
        viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_IQC_DVBS2_SOUR, source),
                             2, "Source"); 
        viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_IQC_DVBS2_INP, input),
                                 3, "Input");
    }
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_IQC_DVBS2_STUF, stuffing),
                         4, "Stuffing");
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Configure DVBS2 ACM Mode
 * Purpose:  This function is used to switch ACM mode (Adaptive Coding and
 *           Modulation) on or off.
/============================================================================*/
ViStatus _VI_FUNC rssfu_confDVBS2ACMMode (ViSession instrSession,
                                          ViBoolean ACMMode)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rssfu_SetAttributeViBoolean (instrSession, "", RSSFU_ATTR_IQC_DVBS2_ACM_MODE, ACMMode),
                         2, "ACM Mode");
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Configure DVBS2 Test Signal
 * Purpose:  This function configures DVBS2 Test Signal.
/============================================================================*/
ViStatus _VI_FUNC rssfu_confDVBS2TestSignal(
    ViSession   instrSession,
    ViInt32 testSignal
)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_IQC_DVBS2_TEST, testSignal),
                         2, "Test Signal");
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Read DVBS2 Values
 * Purpose:  This function reads selected coding value.
/============================================================================*/
ViStatus _VI_FUNC rssfu_readDVBS2Values(ViSession instrSession,
                                        ViInt32 value,
                                        ViReal64* result)
{
    ViStatus    error = VI_SUCCESS;
    ViInt32     int_result = 0;
    ViChar      model [RSSFU_IO_BUFFER_SIZE] = "";  
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckErr (rssfu_GetAttributeViString (instrSession, "",
                                               RS_ATTR_INSTRUMENT_MODEL,
                                               RSSFU_IO_BUFFER_SIZE, model));
    
    if ((value < RSSFU_VAL_PACKET_LENGTH) || (value > RSSFU_VAL_USEFUL_DRATE))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Value");
        
    switch (value){
        case RSSFU_VAL_PACKET_LENGTH:
            viCheckParm (rssfu_GetAttributeViInt32 (instrSession, "", RSSFU_ATTR_IQC_DVBS2_PACKET_LENGTH,
                                                    &int_result), 3, "Result"); 
            *result = (ViReal64)int_result;
        break;
    
        case RSSFU_VAL_MAX_USEFUL_DRATE:
            viCheckParm (rssfu_GetAttributeViReal64 (instrSession, "", RSSFU_ATTR_IQC_DVBS2_MAX_USEFUL_DATA_RATE,
                                                     result), 3, "Result");
        break;
        case RSSFU_VAL_MEAS_DRATE:
            viCheckParm (rssfu_GetAttributeViReal64 (instrSession, "", RSSFU_ATTR_IQC_DVBS2_MEASURED_DATA_RATE,
                                                     result), 3, "Result");
        break;
        case RSSFU_VAL_USEFUL_DRATE:
            viCheckParm (rssfu_GetAttributeViReal64 (instrSession, "", RSSFU_ATTR_IQC_DVBS2_USEFUL_DATA_RATE,
                                                     result), 3, "Result");
        break;
             
        case RSSFU_VAL_REQUIRED_SOURCE_DRATE:
            if (strstr (model, "SFU") != NULL) {  
                viCheckParm (rssfu_GetAttributeViReal64 (instrSession, "", RSSFU_ATTR_READ_DVBS2_CODING_REQUIRED_SOURCE_DATA_RATE ,
                                                     result), 3, "Result");
            }
        break;
     }           
     
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Configure DVBS2 Coding
 * Purpose:  This function configures DVBS2 coding parameters.
/============================================================================*/
ViStatus _VI_FUNC rssfu_confDVBS2Coding(ViSession instrSession,
                                        ViReal64 symbolRate,
                                        ViInt32 constellation,
                                        ViInt32 rollOff,
                                        ViInt32 codeRate,
                                        ViInt32 amc)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      model [RSSFU_IO_BUFFER_SIZE] = "";  

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckErr (rssfu_GetAttributeViString (instrSession, "",
                                               RS_ATTR_INSTRUMENT_MODEL,
                                               RSSFU_IO_BUFFER_SIZE, model));   
    
    if (strstr (model, "SFU") != NULL) {
       viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_IQC_DVBS2_AMC, amc),
                         6, "AMC"); 
    }

    viCheckParm (rssfu_SetAttributeViReal64 (instrSession, "", RSSFU_ATTR_IQC_DVBS2_SYMB, symbolRate),
                         2, "Symbol Rate"); 
    if (strstr (model, "BTC") == NULL) {
        viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_IQC_DVBS2_CONS, constellation),
                         3, "Constellation");
    
        viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_IQC_DVBS2_RATE, codeRate),
                         5, "Code Rate");
    }
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_IQC_DVBS2_ROLL, rollOff),
                         4, "Roll Off");
       
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Configure DVBS2 FEC Frame
 * Purpose:  This function configures the type of FECFRAME.
/============================================================================*/
ViStatus _VI_FUNC rssfu_confDVBS2FecFrame (ViSession instrSession,
                                           ViInt32 FECFrame)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_DVBS2_CODING_FECFRAME, FECFrame),
                         2, "FECFrame");                          
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}
  

  
/*============================================================================/
 * Function: Configure DVBS2 Pilots
 * Purpose:  This function configures DVBS2 pilots generation.
/============================================================================*/
ViStatus _VI_FUNC rssfu_confDVBS2Pilots(ViSession instrSession,
                                              ViBoolean pilots)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rssfu_SetAttributeViBoolean (instrSession, "", RSSFU_ATTR_IQC_DVBS2_PIL, pilots),
                         2, "Pilots");
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Enable DVBS2 Special Settings
 * Purpose:  This function enables use of special DVBS2 coding parameters.
/============================================================================*/
ViStatus _VI_FUNC rssfu_actEnableDVBS2Special(ViSession instrSession,
                                              ViBoolean state)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rssfu_SetAttributeViBoolean (instrSession, "", RSSFU_ATTR_IQC_DVBS2_SETT, state),
                         2, "State");
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Configure DVBS2 Special Settings
 * Purpose:  This function configures special DVBS coding parameters.
/============================================================================*/
ViStatus _VI_FUNC rssfu_confDVBS2Special(ViSession  instrSession,
                                         ViInt32    plScramblingSequenceID,
                                         ViBoolean  cwMode)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      model [RSSFU_IO_BUFFER_SIZE] = "";  

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckErr (rssfu_GetAttributeViString (instrSession, "",
                                               RS_ATTR_INSTRUMENT_MODEL,
                                               RSSFU_IO_BUFFER_SIZE, model));
    if (strstr (model, "SFC") == NULL) {
        viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_IQC_DVBS2_SCR_SEQ, plScramblingSequenceID),
                 2, "PL Scrambling Sequence ID");
    }
    if (strstr (model, "SFU") != NULL) {     
         viCheckParm (rssfu_SetAttributeViBoolean (instrSession, "", RSSFU_ATTR_IQC_DVBS2_CWM, cwMode),
                 3, "CW Mode");                 
    }
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Configure DVBS2 Error Insertion
 * Purpose:  This function configures error insertion in DVBS2 coding.
/============================================================================*/
ViStatus _VI_FUNC rssfu_confDVBS2ErrorInsertion(ViSession   instrSession,
                                                ViBoolean   crc8InHeader,
                                                ViBoolean   crc8InPayload,
                                                ViInt32 bch,
                                                ViInt32 ldpc_cc)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rssfu_SetAttributeViBoolean (instrSession, "", RSSFU_ATTR_IQC_DVBS2_ERR_HEAD, crc8InHeader),
                 2, "CRC-8 In Header"); 
    viCheckParm (rssfu_SetAttributeViBoolean (instrSession, "", RSSFU_ATTR_IQC_DVBS2_ERR_PAYL, crc8InPayload),
                 3, "CRC-8 In Payload");
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_IQC_DVBS2_ERR_BCH, bch),
                 4, "BCH");   
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_IQC_DVBS2_ERR_LDPC, ldpc_cc),
                 5, "LDPC/CC");
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Configure DVBS2 Phase Noise
 * Purpose:  This function configures phase noise parameters of DVBS2 
 *           coding.
 *           
 *           Note:
 *           Available only with option R&S SFU-K108 (AMC).
/============================================================================*/
ViStatus _VI_FUNC rssfu_confDVBS2PhaseNoise(ViSession instrSession,
                                            ViBoolean state,
                                            ViInt32 shape,
                                            ViInt32 magnitude)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rssfu_SetAttributeViBoolean (instrSession, "", RSSFU_ATTR_IQC_DVBS2_PHAS, state),
                 2, "State"); 
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_IQC_DVBS2_PHAS_SHAP, shape),
                 3, "Shape");
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_IQC_DVBS2_PHAS_MAGN, magnitude),
                 4, "Magnitude");   
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Configure DVBS2 Settings
 * Purpose:  This function configures DVBS2 coding settings.
/============================================================================*/
ViStatus _VI_FUNC rssfu_confDVBS2Settings(ViSession instrSession,
                                          ViInt32   testTSPacket,
                                          ViInt32   pidTestPacket,
                                          ViInt32   pid,
                                          ViInt32   payloadTestStuff,
                                          ViInt32   prbs)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      model [RSSFU_IO_BUFFER_SIZE] = "";  

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckErr (rssfu_GetAttributeViString (instrSession, "",
                                               RS_ATTR_INSTRUMENT_MODEL,
                                               RSSFU_IO_BUFFER_SIZE, model));   
    
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_IQC_DVBS2_TSP, testTSPacket),
                         2, "Test TS Packet");
    if (strstr (model, "SFC") == NULL) {
        viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_IQC_DVBS2_PIDT, pidTestPacket),
                             3, "PID Test Packet"); 
        viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_IQC_DVBS2_PID, pid),
                             4, "PID");
    }
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_IQC_DVBS2_PAYL, payloadTestStuff),
                         5, "Payload Test/Stuff");      
    switch (payloadTestStuff){
        case  RSSFU_IQC_PRBS:
            viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_IQC_DVBS2_PRBS, prbs),
                         6, "PRBS");                
    }
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}



/*****************************************************************************
 *--------------------------------- Directv ---------------------------------*
 *****************************************************************************/

/*============================================================================/
 * Function: Configure Directv Input Signal
 * Purpose:  This function configures Directv Input Signal parameters.
/============================================================================*/

ViStatus _VI_FUNC rssfu_confDirectvInputSignal (ViSession instrSession,
                                                ViInt32 source, ViInt32 input,
                                                ViInt32 stuffing,
                                                ViInt32 testSignal)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      model [RSSFU_IO_BUFFER_SIZE] = "";  

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckErr (rssfu_GetAttributeViString (instrSession, "",
                                               RS_ATTR_INSTRUMENT_MODEL,
                                               RSSFU_IO_BUFFER_SIZE, model));
                                                         
    if (strstr (model, "BTC") == NULL) {
        viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_IQC_DIR_SOUR, source),
                             2, "Source");
        viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_IQC_DIR_INP, input),
                             3, "Input"); 
    }
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_IQC_DIR_STUF, stuffing),
                         4, "Stuffing");
    if (source == RSSFU_IQC_SIG_SOUR_TEST)                     
        viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_IQC_DIR_TEST, testSignal),
                         5, "Test Signal");      
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Read Directv Packed Length
 * Purpose:  This function reads Directv packed length.
/============================================================================*/
ViStatus _VI_FUNC rssfu_readDirectvPackedLength (ViSession instrSession,
                                                 ViInt32* packedLength)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    if (!packedLength) viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Packed Length");
    viCheckParm (rssfu_GetAttributeViInt32 (instrSession, "", RSSFU_ATTR_IQC_DIR_PACKET_LENGTH,
                                            packedLength), 2, "Packed Length");
   Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Read Directv Values
 * Purpose:  This function reads selected coding value.
/============================================================================*/
ViStatus _VI_FUNC rssfu_readDirectvValues(ViSession instrSession,
                                        ViInt32 value,
                                        ViReal64* result)
{
    ViStatus    error = VI_SUCCESS;
    ViInt32     int_result = 0; 
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    
    if ((value < RSSFU_VAL_REQ_SOURCE_DRATE) || (value > RSSFU_VAL_USEFUL_DRATE))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Value");
        
    switch (value){
        case RSSFU_VAL_REQ_SOURCE_DRATE:
            viCheckParm (rssfu_GetAttributeViInt32 (instrSession, "", RSSFU_ATTR_IQC_DIR_REQ_SOURCE_DATA_RATE,
                                                    &int_result), 3, "Result"); 
            *result = (ViReal64)int_result;                                                    
        break;
        case RSSFU_VAL_MAX_USEFUL_DRATE:
            viCheckParm (rssfu_GetAttributeViReal64 (instrSession, "", RSSFU_ATTR_IQC_DIR_MAX_USEFUL_DATA_RATE,
                                                     result), 3, "Result");
        break;
        case RSSFU_VAL_MEAS_DRATE:
            viCheckParm (rssfu_GetAttributeViReal64 (instrSession, "", RSSFU_ATTR_IQC_DIR_MEASURED_DATA_RATE,
                                                     result), 3, "Result");
        break;
        case RSSFU_VAL_USEFUL_DRATE:
            viCheckParm (rssfu_GetAttributeViReal64 (instrSession, "", RSSFU_ATTR_IQC_DIR_USEFUL_DATA_RATE,
                                                     result), 3, "Result");
        break;
     }           
     
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Configure Directv Coding
 * Purpose:  This function configures Directv coding parameters.
/============================================================================*/
ViStatus _VI_FUNC rssfu_confDirectvCoding (ViSession instrSession,
                                           ViReal64 symbolRate,
                                           ViInt32 constellation, 
                                           ViInt32 rollOff,
                                           ViInt32 codeRate)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rssfu_SetAttributeViReal64 (instrSession, "", RSSFU_ATTR_IQC_DIR_SYMB, symbolRate),
                         2, "Symbol Rate"); 
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_IQC_DIR_CONS, constellation),
                         3, "Constellation");
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_IQC_DIR_ROLL, rollOff),
                         4, "Roll Off");
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_IQC_DIR_RATE, codeRate),
                         5, "Code Rate");                         
   
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Enable Directv Special Settings
 * Purpose:  This function enables use of special Directv coding parameters.
/============================================================================*/
ViStatus _VI_FUNC rssfu_actEnableDirectvSpecial(ViSession instrSession,
                                                ViBoolean state)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rssfu_SetAttributeViBoolean (instrSession, "", RSSFU_ATTR_IQC_DIR_SPEC_STAT, state),
                         2, "State");
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Configure Directv Special Settings
 * Purpose:  This function configures special Directv coding parameters.
/============================================================================*/
ViStatus _VI_FUNC rssfu_confDirectvSpecial (ViSession instrSession,
                                            ViInt32 insertErrorsAfterConvENC,
                                            ViBoolean CWMode)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      model [RSSFU_IO_BUFFER_SIZE] = ""; 
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckErr (rssfu_GetAttributeViString (instrSession, "",
                                               RS_ATTR_INSTRUMENT_MODEL,
                                               RSSFU_IO_BUFFER_SIZE, model));
    if (strstr (model, "SFC") == NULL) {
        viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_IQC_DIR_INSERT_ERR_CONV_ENC,
                            insertErrorsAfterConvENC), 2, "Insert Errors After Conv ENC");
    }
                         
    if (strstr (model, "SFU") != NULL) { 
        viCheckParm (rssfu_SetAttributeViBoolean(instrSession, "", RSSFU_ATTR_IQC_DIR_CW_MODE, CWMode), 3, "CW Mode");                         
    }
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Configure Directv Phase Noise
 * Purpose:  This function configures phase noise parameters of Directv
 *           coding.
/============================================================================*/
ViStatus _VI_FUNC rssfu_confDirectvPhaseNoise (ViSession instrSession,
                                               ViBoolean state, ViInt32 shape,
                                               ViInt32 magnitude)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      model [RSSFU_IO_BUFFER_SIZE] = "";  

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckErr (rssfu_GetAttributeViString (instrSession, "",
                                               RS_ATTR_INSTRUMENT_MODEL,
                                               RSSFU_IO_BUFFER_SIZE, model));
    
    viCheckParm (rssfu_SetAttributeViBoolean (instrSession, "", RSSFU_ATTR_DIRECTV_CODING_PHASE_NOISE, state),
                 2, "State"); 
    if (strstr (model, "SFU") != NULL) {
        viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_DIRECTV_CODING_PHASE_NOISE_SHAPE, shape),
                     3, "Shape");
    }
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_DIRECTV_CODING_PHASE_NOISE_MAGNITUDE, magnitude),
                 4, "Magnitude");   
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}   

/*============================================================================/
 * Function: Configure Directv Settings
 * Purpose:  This function configures Directv coding settings.
/============================================================================*/
ViStatus _VI_FUNC rssfu_confDirectvSettings (ViSession instrSession,
                                             ViInt32 testTSPacket,
                                             ViInt32 PRBS)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_IQC_DIR_TSP,
                            testTSPacket), 2, "Test TS Packet");
    viCheckParm (rssfu_SetAttributeViInt32(instrSession, "", RSSFU_ATTR_IQC_DIR_PRBS,
                            PRBS), 3, "PRBS");                         
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

 
/*****************************************************************************
 *--------------------------------- MediaFLO --------------------------------*
 *****************************************************************************/
 
/*============================================================================/
 * Function: Read MediaFLO Coder Status
 * Purpose:  This function reads the status of MediaFLO coder coder.
/============================================================================*/
ViStatus _VI_FUNC rssfu_readMfloCoderStatus (ViSession instrSession,
                                             ViChar  status[])
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSSFU_IO_BUFFER_SIZE]="\0";  
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    if (status == NULL) viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Status");
    viCheckErr (rssfu_GetAttributeViString (instrSession, "", RSSFU_ATTR_IQC_MFLO_LOCK,
                                            RSSFU_IO_BUFFER_SIZE, buffer));
    strncpy (status, buffer, strlen(buffer)); 
    if (status != NULL)
        status[strlen(buffer)] = '\0';
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}                                             

/*============================================================================/
 * Function: Configure MediaFLO Input Signal
 * Purpose:  This function configures MediaFLO Input Signal parameters.
/============================================================================*/
ViStatus _VI_FUNC rssfu_confMfloInputSignal (ViSession instrSession,
                                             ViInt32 source,
                                             ViInt32 input)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_IQC_MFLO_SOURCE,
                            source), 2, "Source");
    viCheckParm (rssfu_SetAttributeViInt32(instrSession, "", RSSFU_ATTR_IQC_MFLO_INPUT,
                            input), 3, "Input");                         
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Read Directv Values
 * Purpose:  This function reads selected coding value.
/============================================================================*/
ViStatus _VI_FUNC rssfu_readMfloMeasDataRate (ViSession instrSession,
                                              ViInt32 value,
                                              ViReal64* result)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    if ((value < RSSFU_VAL_MFLO_MEAS_DRATE) || (value > RSSFU_VAL_MFLO_MEAS_DRATE))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Value");
        
    switch (value){
        case RSSFU_VAL_MFLO_MEAS_DRATE:
            viCheckParm (rssfu_GetAttributeViReal64 (instrSession, "", RSSFU_ATTR_READ_MEDIAFLO_INPUT_SIGNAL_MEAS_DATA_RATE,
                                                    result), 3, "Result"); 
        break;
       /* case RSSFU_VAL_MFLO_WPID_DRATE:
            viCheckParm (rssfu_GetAttributeViReal64 (instrSession, "", RSSFU_ATTR_IQC_MFLO_WIDE_PID_DATA_RATE,
                                                     result), 3, "Result");
        break;
        case RSSFU_VAL_MFLO_LPID_DRATE:
            viCheckParm (rssfu_GetAttributeViReal64 (instrSession, "", RSSFU_ATTR_IQC_MFLO_LOCAL_PID_DATA_RATE,
                                                     result), 3, "Result");
        break;*/
     }           
     
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Configure MediaFLO Input Signal
 * Purpose:  This function configures MediaFLO delay parameters.
/============================================================================*/
ViStatus _VI_FUNC rssfu_confMfloDelaySettings (ViSession instrSession,
                                               ViReal64 networkDelay,
                                               ViReal64 transmitterDelay)

{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rssfu_SetAttributeViReal64 (instrSession, "", RSSFU_ATTR_IQC_MFLO_NETWORK_DELAY,
                            networkDelay), 2, "Network Delay");
    viCheckParm (rssfu_SetAttributeViReal64(instrSession, "", RSSFU_ATTR_IQC_MFLO_TRANSMITTER_DELAY,
                            transmitterDelay), 3, "Transmitter Delay");                         
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/*============================================================================/
 * Function: Read MediaFLO Delay SSF Arrival Time
 * Purpose:  This function queries the SSF arrival time related to the 1pps.
/============================================================================*/
/*ViStatus _VI_FUNC rssfu_readMfloDelaySsfArrivalTime (ViSession instrSession,
                                                     ViReal64* SSFArrivalTime)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rssfu_GetAttributeViReal64 (instrSession, "", RSSFU_ATTR_IQC_MFLO_SSF_ARRIVAL_TIME,
                                                     SSFArrivalTime), 2, "SSF Arrival Time");
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}
*/
/*============================================================================/
 * Function: Enable MediaFLO Special Settings
 * Purpose:  This function enables use of special MediaFLO coding parameters.
/============================================================================*/
ViStatus _VI_FUNC rssfu_actEnableMfloSpecial(ViSession instrSession,
                                             ViBoolean state)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rssfu_SetAttributeViBoolean (instrSession, "", RSSFU_ATTR_IQC_MFLO_SPECIAL_SETTING, state),
                         2, "State");
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Configure MediaFLO Special Settings
 * Purpose:  This function configures special MediaFLO coding parameters.
/============================================================================*/
ViStatus _VI_FUNC rssfu_confMfloSpecial (ViSession instrSession,
                                         ViBoolean specialTDM1Null,
                                         ViBoolean specialSinusoidTest,
                                         ViReal64 specialSinusoidTestFreq,
                                         ViReal64 gain)

{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rssfu_SetAttributeViBoolean (instrSession, "", RSSFU_ATTR_IQC_MFLO_TDM1_NULL,
                        specialTDM1Null), 2, "Special TDM1 Null");

   /* viCheckParm (rssfu_SetAttributeViBoolean (instrSession, "", RSSFU_ATTR_IQC_MFLO_SINUSOID_TEST,
                        specialSinusoidTest), 3, "Special Sinusoid Test");*/
    specialSinusoidTest;

   /* viCheckParm (rssfu_SetAttributeViReal64 (instrSession, "", RSSFU_ATTR_IQC_MFLO_SINUSOID_TEST_FREQ,
                        specialSinusoidTestFreq), 4, "Special Sinusoid Test Freq");*/
    specialSinusoidTestFreq;
    
   /* viCheckParm (rssfu_SetAttributeViReal64(instrSession, "", RSSFU_ATTR_IQC_MFLO_GAIN,
                        gain), 5, "Gain");   */                      
    gain;
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: MediaFLO Channel Bandwidth
 * Purpose:  This function sets the channel bandwidth of the output signal.
/============================================================================*/
ViStatus _VI_FUNC rssfu_setMfloChannelBandwidth (ViSession instrSession,
                                                 ViInt32 channelBandwidth)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_MEDIAFLO_CODING_CHANNEL_BANDWIDTH,
                            channelBandwidth), 2, "Channel Bandwidth");
                        
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/*============================================================================/
 * Function: Configure MediaFLO Area Settings
 * Purpose:  This function configures MediaFLO coding settings.
/============================================================================*/
ViStatus _VI_FUNC rssfu_confMfloAreaSettings (ViSession instrSession,
                                              ViInt32 wideAreaContentPID,
                                              ViInt32 localAreaContentPID,
                                              ViInt32 positioningPilotLength,
                                              ViInt32 wideAreaLength)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_IQC_MFLO_WIDE_AREA_CONTENT_PID,
                            wideAreaContentPID), 2, "Wide Area Content PID");
    viCheckParm (rssfu_SetAttributeViInt32(instrSession, "", RSSFU_ATTR_IQC_MFLO_LOCAL_AREA_CONTENT_PID,
                            localAreaContentPID), 3, "Local Area Content PID");
    /*viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_IQC_MFLO_POSITION_PILOT_LENGTH,
                            positioningPilotLength), 4, "Positioning Pilot Length");*/
    positioningPilotLength;
    /*viCheckParm (rssfu_SetAttributeViInt32(instrSession, "", RSSFU_ATTR_IQC_MFLO_WIDE_AREA_LENGTH,
                            wideAreaLength), 5, "Wide Area Length");*/                                                    
    wideAreaLength;
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/*============================================================================/
 * Function: Configure MediaFLO Length Settings
 * Purpose:  This function configures MediaFLO coding settings.
/============================================================================*/
ViStatus _VI_FUNC rssfu_confMfloIdSettings (ViSession instrSession,
                                                ViInt32 wideAreaIDValue,
                                                ViInt32 localAreaIDValue,
                                                ViInt32 infrastructureID,
                                                ViReal64 localTimeOffset,
                                                ViBoolean daylightSavTimeIndicator)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_IQC_MFLO_WIDE_AREA_ID_VALUE,
                            wideAreaIDValue), 2, "Wide Area ID Value");
    viCheckParm (rssfu_SetAttributeViInt32(instrSession, "", RSSFU_ATTR_IQC_MFLO_LOCAL_AREA_ID_VALUE,
                            localAreaIDValue), 3, "Local Area ID Value");
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_IQC_MFLO_INFRASTRUCTURE_ID,
                            infrastructureID), 4, "Infrastructure ID");
    viCheckParm (rssfu_SetAttributeViReal64 (instrSession, "", RSSFU_ATTR_IQC_MFLO_LOCAL_TIME_OFFSET,
                        localTimeOffset), 5, "Local Time Offset");
    viCheckParm (rssfu_SetAttributeViBoolean (instrSession, "", RSSFU_ATTR_IQC_MFLO_DAYLIGHT_SAV_TIME_IND,
                        daylightSavTimeIndicator), 6, "Daylight Saving Time Indicator");
                        
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Read MediaFLO Length Settings
 * Purpose:  This function reads selected coding value.
/============================================================================*/
ViStatus _VI_FUNC rssfu_readMfloLengthSettings (ViSession instrSession,
                                              ViInt32 value,
                                              ViInt32* result)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    if ((value < RSSFU_VAL_MFLO_MEAS_DRATE) || (value > RSSFU_VAL_MFLO_MEAS_DRATE))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Value");
        
    switch (value) {
       /* case RSSFU_VAL_MFLO_FRAME_LENGTH:
            viCheckParm (rssfu_GetAttributeViInt32 (instrSession, "", RSSFU_ATTR_IQC_MFLO_FRAME_LENGTH,
                                                    result), 3, "Result"); 
        break; */
        case RSSFU_VAL_MFLO_LOCAL_AREA_LENGTH:
            viCheckParm (rssfu_GetAttributeViInt32 (instrSession, "", RSSFU_ATTR_IQC_MFLO_LOCAL_AREA_LENGTH,
                                                     result), 3, "Result");
        break;
     }           
     
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}



/*============================================================================/
 * Function: Configure MediaFLO Input Signal
 * Purpose:  This function configures MediaFLO Input Signal parameters.
/============================================================================*/
ViStatus _VI_FUNC rssfu_ConfigureMediaFLOInputSignal (ViSession instrSession,
                                                      ViInt32 source,
                                                      ViInt32 input)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_IQC_MFLO_SOURCE,
                            source), 2, "Source");
    viCheckParm (rssfu_SetAttributeViInt32(instrSession, "", RSSFU_ATTR_IQC_MFLO_INPUT,
                            input), 3, "Input");                         
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Read MediaFLO Values
 * Purpose:  This function reads selected coding value.
/============================================================================*/
ViStatus _VI_FUNC rssfu_ReadMediaFLOValues (ViSession instrSession,
                                              ViInt32 value,
                                              ViReal64* result)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    if ((value < RSSFU_VAL_MFLO_MEAS_DRATE) || (value > RSSFU_VAL_MFLO_MEAS_LID_OIS))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Value");
        
    switch (value){
        case RSSFU_VAL_MFLO_MEAS_DRATE:
            viCheckParm (rssfu_GetAttributeViReal64 (instrSession, "", RSSFU_ATTR_READ_MEDIAFLO_INPUT_SIGNAL_MEAS_DATA_RATE,
                                                    result), 3, "Result"); 
        break;
        case RSSFU_VAL_MFLO_MEAS_WID_COND:
            viCheckParm (rssfu_GetAttributeViReal64 (instrSession, "", RSSFU_ATTR_READ_MEDIAFLO_INPUT_SIGNAL_WIDE_CONTENT_DATA_RATE,
                                                     result), 3, "Result");
        break;
        case RSSFU_VAL_MFLO_MEAS_WID_CTRD:
            viCheckParm (rssfu_GetAttributeViReal64 (instrSession, "", RSSFU_ATTR_READ_MEDIAFLO_INPUT_SIGNAL_WIDE_CONTROL_CH_DATA_RATE,
                                                     result), 3, "Result");
        break;
        case RSSFU_VAL_MFLO_MEAS_WID_OIS:
            viCheckParm (rssfu_GetAttributeViReal64 (instrSession, "", RSSFU_ATTR_READ_MEDIAFLO_INPUT_SIGNAL_WIDE_OIS_DATA_RATE,
                                                     result), 3, "Result");
        break;
        case RSSFU_VAL_MFLO_MEAS_LID_COND:
            viCheckParm (rssfu_GetAttributeViReal64 (instrSession, "", RSSFU_ATTR_READ_MEDIAFLO_INPUT_SIGNAL_LOCAL_CONTENT_DATA_RATE,
                                                     result), 3, "Result");
        break;
        case RSSFU_VAL_MFLO_MEAS_LID_CTRD:
            viCheckParm (rssfu_GetAttributeViReal64 (instrSession, "", RSSFU_ATTR_READ_MEDIAFLO_INPUT_SIGNAL_LOCAL_CONTROL_CH_DATA_RATE,
                                                     result), 3, "Result");
        break;
        case RSSFU_VAL_MFLO_MEAS_LID_OIS:
            viCheckParm (rssfu_GetAttributeViReal64 (instrSession, "", RSSFU_ATTR_READ_MEDIAFLO_INPUT_SIGNAL_LOCAL_OIS_DATA_RATE,
                                                     result), 3, "Result");
        break;
     }           
     
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Read Status MediaFLO Delay Parameters
 * Purpose:  This function reads selected delay parameters.
/============================================================================*/
ViStatus _VI_FUNC rssfu_ReadStatusMediaFLODelayParameters (ViSession instrSession,
                                                ViInt32 value, ViReal64* result)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if ((value < RSSFU_VAL_MFLO_STATUS_SSF_TIME) || (value > RSSFU_VAL_MFLO_STATUS_PPS_TDM))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Value");
        
    switch (value){
        case RSSFU_VAL_MFLO_STATUS_SSF_TIME:
            viCheckParm (rssfu_GetAttributeViReal64 (instrSession, "", RSSFU_ATTR_READ_MEDIAFLO_STATUS_SSF_ARRIVAL_TIME,
                                                    result), 3, "Result"); 
        break;
        case RSSFU_VAL_MFLO_STATUS_PPS_TDM:
            viCheckParm (rssfu_GetAttributeViReal64 (instrSession, "", RSSFU_ATTR_READ_MEDIAFLO_STATUS_1PPS_TDM1_ERROR,
                                                     result), 3, "Result");
        break;
     }           
     
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Read Status MediaFLO Count Parameters
 * Purpose:  This function reads selected count parameters.
/============================================================================*/
ViStatus _VI_FUNC rssfu_ReadStatusMediaFLOCountParameters (ViSession instrSession, 
                                            ViInt32 value, ViInt32* result)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if ((value < RSSFU_VAL_MFLO_STATUS_OUTF_COUNT) || (value > RSSFU_VAL_MFLO_STATUS_SCER_COUNT))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Value");
        
    switch (value){
        case RSSFU_VAL_MFLO_STATUS_OUTF_COUNT:
            viCheckParm (rssfu_GetAttributeViInt32 (instrSession, "", RSSFU_ATTR_READ_MEDIAFLO_STATUS_OUT_OF_FRAME_COUNT,
                                                    result), 3, "Result"); 
        break;
        case RSSFU_VAL_MFLO_STATUS_SEER_COUNT:
            viCheckParm (rssfu_GetAttributeViInt32 (instrSession, "", RSSFU_ATTR_READ_MEDIAFLO_STATUS_SEQUENCE_ERROR_COUNT,
                                                     result), 3, "Result");
        break;
        case RSSFU_VAL_MFLO_STATUS_SCER_COUNT:
            viCheckParm (rssfu_GetAttributeViInt32 (instrSession, "", RSSFU_ATTR_READ_MEDIAFLO_STATUS_SCHEDULING_ERROR_COUNT,
                                                     result), 3, "Result");
        break;
     }           
     
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/*============================================================================/
 * Function: Configure MediaFLO Coding
 * Purpose:  This function configures MediaFLO coding parameters.
/============================================================================*/
ViStatus _VI_FUNC rssfu_ConfigureMediaFLOCoding (ViSession instrSession,
                                                 ViInt32 channelBandwidth,
                                                 ViInt32 IFFTMode,
                                                 ViInt32 slotToInterlaceMapping,
                                                 ViInt32 cyclicPrefixLength)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_MEDIAFLO_CODING_CHANNEL_BANDWIDTH,
                            channelBandwidth), 2, "Channel Bandwidth");
    
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_MEDIAFLO_CODING_IFFT_MODE,
                            IFFTMode), 3, "IFFT Mode");
    
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_MEDIAFLO_CODING_SLOT_TO_INTERLACE_MAPPING,
                            slotToInterlaceMapping), 4, "Slot To Interlace Mapping");
    
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_MEDIAFLO_CODING_CYCLIC_PREFIX_LENGTH,
                            cyclicPrefixLength), 5, "Cyclic Prefix Length");
                        
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/*============================================================================/
 * Function: Configure MediaFLO Coding
 * Purpose:  This function configures MediaFLO coding parameters.
/============================================================================*/
ViStatus _VI_FUNC rssfu_ConfigureMediaFLOCodingParameters (ViSession instrSession,
              ViInt32 wideAreaIDValue,
              ViInt32 localAreaIDValue, ViInt32 wideMACSymbolsFrame,
              ViInt32 localMACSymbolsFrame, ViReal64 networkGeographyDelay)

{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_IQC_MFLO_WIDE_AREA_ID_VALUE,
                            wideAreaIDValue), 2, "Wide Area ID Value");
    
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_IQC_MFLO_LOCAL_AREA_ID_VALUE,
                            localAreaIDValue), 3, "Local Area ID Value");
    
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_MEDIAFLO_CODING_WIDE_MAC_SYMBOLS_FRAME,
                            wideMACSymbolsFrame), 4, "Wide MAC Symbols Frame");
    
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_MEDIAFLO_CODING_LOCAL_MAC_SYMBOLS_FRAME,
                            localMACSymbolsFrame), 5, "Local MAC Symbols Frame");

    viCheckParm (rssfu_SetAttributeViReal64 (instrSession, "", RSSFU_ATTR_MEDIAFLO_CODING_NETWORK_GEOGRAPHY_DELAY,
                            networkGeographyDelay), 6, "Network Geography Delay");
                        
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Enable MediaFLO Special Settings
 * Purpose:  This function enables use of special MediaFLO coding parameters.
/============================================================================*/
ViStatus _VI_FUNC rssfu_EnableMediaFLOSpecialSettings(ViSession instrSession,
                                             ViBoolean state)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rssfu_SetAttributeViBoolean (instrSession, "", RSSFU_ATTR_IQC_MFLO_SPECIAL_SETTING, state),
                         2, "State");
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Configure MediaFLO Special Settings
 * Purpose:  This function configures special MediaFLO coding parameters.
/============================================================================*/
ViStatus _VI_FUNC rssfu_ConfigureMediaFLOSpecialSettings (ViSession instrSession,
                                                          ViBoolean TDM1Null)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rssfu_SetAttributeViBoolean (instrSession, "", RSSFU_ATTR_IQC_MFLO_TDM1_NULL,
                        TDM1Null), 2, "TDM1 Null");
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/*============================================================================/
 * Function: Configure MediaFLO PID Settings
 * Purpose:  This function configures MediaFLO PID settings.
/============================================================================*/
ViStatus _VI_FUNC rssfu_ConfigureMediaFLOPIDSettings (ViSession instrSession,
                                                      ViInt32 PID, ViInt32 content,
                                                      ViInt32 controlChannel,
                                                      ViInt32 OIS)
{    
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
   if ((PID < RSSFU_VAL_IQC_MFLO_WIDE_PID) || (PID > RSSFU_VAL_IQC_MFLO_LOCAL_PID))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "PID");
        
    switch (PID){
        case RSSFU_VAL_IQC_MFLO_WIDE_PID:
            viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_MEDIAFLO_SETTINGS_WIDE_PID_CONTENT,
                            content), 3, "Content ");
            viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_MEDIAFLO_SETTINGS_WIDE_PID_CONTROL_CHANNEL,
                            controlChannel), 4, "Control Channel ");
            viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_MEDIAFLO_SETTINGS_WIDE_PID_OIS,
                            OIS), 5, "OIS ");
        break;
      
        case RSSFU_VAL_IQC_MFLO_LOCAL_PID:
            viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_MEDIAFLO_SETTINGS_LOCAL_PID_CONTENT,
                            content), 3, "Content");
            viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_MEDIAFLO_SETTINGS_LOCAL_PID_CONTROL_CHANNEL,
                            controlChannel), 4, "Control Channel");
            viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_MEDIAFLO_SETTINGS_LOCAL_PID_OIS,
                            OIS), 5, "OIS");
        break;
     }           
     
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Configure MediaFLO Local OIS System Settings  
 * Purpose:  This function configures MediaFLO local OIS system parameters.
/============================================================================*/
ViStatus _VI_FUNC rssfu_ConfigureMediaFLOLocalOISSystemSettings
             (ViSession instrSession, ViBoolean localOISInsertion,
              ViInt32 infrastructureID, ViReal64 localTimeOffset,
              ViBoolean daylightSavingTimeIndicator)

{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rssfu_SetAttributeViBoolean (instrSession, "", RSSFU_ATTR_MEDIAFLO_SETTINGS_LOCAL_OIS_INSERTION,
                            localOISInsertion), 2, "Local OIS Insertion");
        
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_MEDIAFLO_SETTINGS_INFRASTRUCTURE_ID,
                            infrastructureID), 3, "Infrastructure ID");
    
    viCheckParm (rssfu_SetAttributeViReal64 (instrSession, "", RSSFU_ATTR_MEDIAFLO_SETTINGS_LOCAL_TIME_OFFSET,
                            localTimeOffset), 4, "Local Time Offset");
        
    viCheckParm (rssfu_SetAttributeViBoolean (instrSession, "", RSSFU_ATTR_MEDIAFLO_SETTINGS_DAYLIGHT_SAVINGS_TIME_INDICATOR,
                            daylightSavingTimeIndicator), 5, "Daylight Saving Time Indicator");
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Configure MediaFLO Position Pilot Settings  
 * Purpose:  This function configures MediaFLO position pilot parameters.
/============================================================================*/
ViStatus _VI_FUNC rssfu_ConfigureMediaFLOPositionPilotSettings
             (ViSession instrSession, ViBoolean positionPilotSymbols,
              ViInt32 transmitterID, ViInt32 numberOfTransmitters)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rssfu_SetAttributeViBoolean (instrSession, "", RSSFU_ATTR_MEDIAFLO_SETTINGS_POSITION_PILOT_SYMBOLS,
                            positionPilotSymbols), 2, "Position Pilot Symbols");
        
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_MEDIAFLO_SETTINGS_TRANSMITTER_ID,
                            transmitterID), 3, "Transmitter ID");
    
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_MEDIAFLO_SETTINGS_NUMBER_OF_TRANSMITTERS,
                            numberOfTransmitters), 4, "Number Of Transmitters");
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/*============================================================================/
 * Function: Enable MediaFLO TX Position Settings
 * Purpose:  This function enables or disables the transmitter position field
 *           information on the PPC channel.
/============================================================================*/
ViStatus _VI_FUNC rssfu_EnableMediaFLOTXPositionSettings (ViSession instrSession,
                                                          ViBoolean TXPositionEnable)

{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rssfu_SetAttributeViBoolean (instrSession, "", RSSFU_ATTR_MEDIAFLO_SETTINGS_TX_POSITION_ENABLE, TXPositionEnable),
                         2, "TX Position Enable");
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Configure MediaFLO TX Settings
 * Purpose:  This function configures MediaFLO TX parameters.
/============================================================================*/
ViStatus _VI_FUNC rssfu_ConfigureMediaFLOTXSettings (ViSession instrSession,
                                                     ViInt32 TXPosition,
                                                     ViBoolean TXOffsetEnable,
                                                     ViReal64 TXOffset,
                                                     ViReal64 TXLatitude,
                                                     ViReal64 TXLongitude,
                                                     ViReal64 TXAltitude)

{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_MEDIAFLO_SETTINGS_TX_POSITION,
                            TXPosition), 2, "TX Position");
    
    viCheckParm (rssfu_SetAttributeViBoolean (instrSession, "", RSSFU_ATTR_MEDIAFLO_SETTINGS_TX_OFFSET_ENABLE,
                            TXOffsetEnable), 3, "TX Offset Enable");
    
    if (TXOffsetEnable == VI_TRUE) {
        viCheckParm (rssfu_SetAttributeViReal64 (instrSession, "", RSSFU_ATTR_MEDIAFLO_SETTINGS_TX_OFFSET,
                            TXOffset), 4, "TX Offset");
    }
    
    viCheckParm (rssfu_SetAttributeViReal64 (instrSession, "", RSSFU_ATTR_MEDIAFLO_SETTINGS_TX_LATITUDE,
                            TXLatitude), 5, "TX Latitude");
        
    viCheckParm (rssfu_SetAttributeViReal64 (instrSession, "", RSSFU_ATTR_MEDIAFLO_SETTINGS_TX_LONGITUDE,
                            TXLongitude), 6, "TX Longitude");
        
    viCheckParm (rssfu_SetAttributeViReal64 (instrSession, "", RSSFU_ATTR_MEDIAFLO_SETTINGS_TX_ALTITUDE,
                            TXAltitude), 7, "TX Altitude");
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

    
    
/*****************************************************************************
 *----------------------------------- T-DMB ---------------------------------*
 *****************************************************************************/
/*============================================================================/
 * Function: Read TDMB Coder Status
 * Purpose:  This function reads the status of T-DMB/DAB coder coder.
/============================================================================*/
ViStatus _VI_FUNC rssfu_readTDMBCoderStatus(ViSession instrSession,
                                            ViChar  status[])
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSSFU_IO_BUFFER_SIZE]="\0";  
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    if (status == NULL) viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Status");
    viCheckErr (rssfu_GetAttributeViString (instrSession, "", RSSFU_ATTR_IQC_TDMB_LOCK,
                                            RSSFU_IO_BUFFER_SIZE, buffer));
    strncpy (status, buffer, strlen(buffer)); 
    if (status != NULL)
        status[strlen(buffer)] = '\0';
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

 
/*============================================================================/
 * Function: Configure TDMB Input Signal
 * Purpose:  This function configures T-DMB Input Signal parameters.
/============================================================================*/
 
ViStatus _VI_FUNC rssfu_confTDMBInputSignal (ViSession instrSession,
                                             ViInt32 source)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_TDMB_SOURCE, source),
                         2, "Source");

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Read TDMB Values
 * Purpose:  This function reads selected coding value.
/============================================================================*/
ViStatus _VI_FUNC rssfu_readTDMBValues (ViSession instrSession, ViInt32 value,
                                        ViInt32* result)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    if ((value < RSSFU_VAL_TDMB_NUM_STREAMS) || (value > RSSFU_VAL_TDMB_MODE_IDENTITY))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Value");
        
    switch (value){
   
        case RSSFU_VAL_TDMB_NUM_STREAMS:
            viCheckParm (rssfu_GetAttributeViInt32 (instrSession, "", RSSFU_ATTR_TDMB_NUMBER_OF_STREAMS,
                                                     result), 3, "Result");
        break;
        case RSSFU_VAL_TDMB_MODE_IDENTITY:
            viCheckParm (rssfu_GetAttributeViInt32 (instrSession, "", RSSFU_ATTR_TDMB_MODE_IDENTITY,
                                                     result), 3, "Result");
        break;
     }           
     
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Read TDMB ETI Input
 * Purpose:  This function reads the ETI input signal.
/============================================================================*/
ViStatus _VI_FUNC rssfu_readTDMBEtiInput (ViSession instrSession,
                                          ViInt32* ETIInput)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rssfu_GetAttributeViInt32 (instrSession, "", RSSFU_ATTR_TDMB_ETI_INPUT,
                                            ETIInput), 2, "ETI Input");   
     
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/*============================================================================/
 * Function: Read Info STC SSTC
 * Purpose:  This function reads Info STC/SSTC.
/============================================================================*/
ViStatus _VI_FUNC rssfu_readTDMBInfo (ViSession instrSession, ViInt32 subchannel,
                                      ViInt32* SCID, ViInt32* protProfile,
                                      ViInt32* protLevel, ViInt32* dataRate)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buf1[RSSFU_TOKEN_BUFFER_SIZE] = "";
    ViInt32     l_subchannel = subchannel - 1;
    ViChar      buffer[RSSFU_IO_BUFFER_SIZE] = "\0";  
        
    if ((subchannel < 1) || (subchannel > 64))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Subchannel"); 

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    checkErr (rssfu_GetActualRepCaps (instrSession, RSSFU_ATTR_TDMB_SCID, "Subchannel", l_subchannel, buf1));

    viCheckParm (rssfu_GetAttributeViString (instrSession, buf1, RSSFU_ATTR_TDMB_SCID, RSSFU_IO_BUFFER_SIZE, buffer), 3, "SCID");
    sscanf (buffer, "0x%X", SCID);   
    
    
    viCheckParm (rssfu_GetAttributeViInt32 (instrSession, buf1, RSSFU_ATTR_TDMB_PROT_PROFILE, protProfile), 4, "Prot. Profile");  
        
        
    viCheckParm (rssfu_GetAttributeViInt32 (instrSession, buf1, RSSFU_ATTR_TDMB_PROT_LEVEL, protLevel), 5, "Prot. Level");  
        
        
    viCheckParm (rssfu_GetAttributeViInt32 (instrSession, buf1, RSSFU_ATTR_TDMB_DATA_RATE, dataRate), 6, "Data Rate");  
                
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}



/*============================================================================/
 * Function: Enable DMBT Special Settings
 * Purpose:  This function enables use of special T-DMB coding parameters.
/============================================================================*/
ViStatus _VI_FUNC rssfu_actEnableTDMBSpecial (ViSession instrSession,
                                              ViBoolean state)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rssfu_SetAttributeViBoolean (instrSession, "", RSSFU_ATTR_TDMB_SPECIAL_SETTINGS, state),
                         2, "State");
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/*============================================================================/
 * Function: Configure TDMB Special Settings
 * Purpose:  This function configures special T-DMB coding parameters.
/============================================================================*/
ViStatus _VI_FUNC rssfu_confTDMBSpecial (ViSession instrSession,
                                         ViInt32 transMode,
                                         ViInt32 selectTransMode,
                                         ViBoolean PRBSTestsignal,
                                         ViInt32 testsignalSCID)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_TDMB_TRANSMISSION_MODE, transMode),
                             2, "Transmission Mode");
    
    if (transMode == RSSFU_VAL_TDMB_TRAN_MODE_MANUAL)
            viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_TDMB_SELECT_TRANSMISSION_MODE, selectTransMode),
                             3, "Select Transmission Mode");
   
    viCheckParm (rssfu_SetAttributeViBoolean (instrSession, "", RSSFU_ATTR_TDMB_PRBS_TESTSIGNAL, PRBSTestsignal),
                         4, "PRBS Testsignal");                     
    if (PRBSTestsignal) 
        viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_TDMB_TESTSIGNAL_SCID, testsignalSCID),
                         5, "Testsignal SCID");
                         
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}
   
/*============================================================================/
 * Function: T-DMB Configuration
 * Purpose:  This function configures T-DMB coding settings.
/============================================================================*/
ViStatus _VI_FUNC rssfu_confTDMBConfiguration (ViSession instrSession,
                                          ViBoolean TDMBTTI,
                                          ViInt32 TIIMainID,ViInt32 TIISubID,
                                          ViInt32 TDMBNetwork,
                                          ViInt32 TDMBPRBS)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      model [RSSFU_IO_BUFFER_SIZE] = "";      

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckErr (rssfu_GetAttributeViString (instrSession, "",
                                               RS_ATTR_INSTRUMENT_MODEL,
                                               RSSFU_IO_BUFFER_SIZE, model));   
    
    viCheckParm (rssfu_SetAttributeViBoolean (instrSession, "", RSSFU_ATTR_TDMB_TII, TDMBTTI),
                         2, "TDMB TTI");
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_TDMB_TII_MAIN_ID, TIIMainID),
                         3, "TII Main ID");
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_TDMB_TII_SUB_ID, TIISubID),
                         4, "TII Sub ID");
    if (strstr (model, "SFC") == NULL) {
        viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_TDMB_NETWORK, TDMBNetwork),
                         5, "TDMB Network");
    }
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_TDMB_PRBS, TDMBPRBS),
                         6, "TDMB PRBS");
                         
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

   
/*============================================================================/
 * Function: T-DMB Settings - Obsolete
 * Purpose:  This function configures T-DMB coding settings.
/============================================================================*/
ViStatus _VI_FUNC rssfu_confTDMBSettings (ViSession instrSession,
                                          /*@unused@*/ViBoolean TDMBTTI,
                                          ViInt32 TIIMainSubID,
                                          /*@unused@*/ViInt32 TDMBNetwork,
                                          ViInt32 TDMBPRBS)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    /* Will be implemented in the future
    viCheckParm (rssfu_SetAttributeViBoolean (instrSession, "", RSSFU_ATTR_TDMB_TII, TDMBTTI),
                         2, "TDMB TTI");

    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_TDMB_NETWORK, TDMBNetwork),
                         4, "TDMB Network");
                         */
    TDMBTTI;
    TDMBNetwork;
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_TDMB_TII_MAIN_SUB_ID, TIIMainSubID),
                         3, "TII Main / Sub ID");
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_TDMB_PRBS, TDMBPRBS),
                         5, "TDMB PRBS");
                         
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


                                         
/*============================================================================/
 * Function: Configure Analog TV Audio Input Signal
 * Purpose:  This function selects the audio source of the R&S SFU.
/============================================================================*/
ViStatus _VI_FUNC rssfu_confAnalogTvAudioInputSignal (ViSession instrSession,
                                                      ViInt32 audioInputSignal)

{
    ViStatus    error = VI_SUCCESS;
    ViChar      model [RSSFU_IO_BUFFER_SIZE] = "";  
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
        
    viCheckErr (rssfu_GetAttributeViString (instrSession, "",
                                               RS_ATTR_INSTRUMENT_MODEL,
                                               RSSFU_IO_BUFFER_SIZE, model));
    if (strstr (model, "BTC") != NULL) {
      if ((audioInputSignal < RSSFU_VAL_IQC_ATV_AUDIO_SOURCE_AGEN) || (audioInputSignal > RSSFU_VAL_IQC_ATV_AUDIO_SOURCE_APLAY))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Audio Input Signal");
    }
    
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_ATV_AUDIO_INPUT_SIGNAL,
                            audioInputSignal), 2, "Audio Input Signal");
  
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Configure Analog TV Audio
 * Purpose:  This function configures analog TV audio parameters.
/============================================================================*/
ViStatus _VI_FUNC rssfu_configAnalogTvAudio (ViSession instrSession,
                                             ViInt32 frequencyParameter,
                                             ViReal64 frequencyValue,
                                             ViInt32 levelParameter,
                                             ViReal64 levelValue)
{                                             
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    if ((frequencyParameter < RSSFU_VAL_ATV_AUDIO_SOUND_FREQ_MONO) || (frequencyParameter > RSSFU_VAL_ATV_AUDIO_SOUND_FREQ_CH2))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Frequency Parameter");
        
    if ((levelParameter < RSSFU_VAL_ATV_AUDIO_SOUND_LEVEL_MONO) || (levelParameter > RSSFU_VAL_ATV_AUDIO_SOUND_LEVEL_CH2))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 4, "Level Parameter");        
  
    switch (frequencyParameter){
        case RSSFU_VAL_ATV_AUDIO_SOUND_FREQ_MONO:
            viCheckParm (rssfu_SetAttributeViReal64 (instrSession, "", RSSFU_ATTR_ATV_FREQ_SOUND_MONO,
                                                    frequencyValue), 3, "Frequency Value"); 
        break;
        case RSSFU_VAL_ATV_AUDIO_SOUND_FREQ_LEFT:
              viCheckParm (rssfu_SetAttributeViReal64 (instrSession, "", RSSFU_ATTR_ATV_FREQ_SOUND_LEFT,
                                                    frequencyValue), 3, "Frequency Value"); 
        break;
        case RSSFU_VAL_ATV_AUDIO_SOUND_FREQ_RIGHT:
               viCheckParm (rssfu_SetAttributeViReal64 (instrSession, "", RSSFU_ATTR_ATV_FREQ_SOUND_RIGHT,
                                                    frequencyValue), 3, "Frequency Value"); 
        break;
        case RSSFU_VAL_ATV_AUDIO_SOUND_FREQ_CH1:
              viCheckParm (rssfu_SetAttributeViReal64 (instrSession, "", RSSFU_ATTR_ATV_FREQ_SOUND_CH1,
                                                    frequencyValue), 3, "Frequency Value"); 
        break;
        case RSSFU_VAL_ATV_AUDIO_SOUND_FREQ_CH2:
               viCheckParm (rssfu_SetAttributeViReal64 (instrSession, "", RSSFU_ATTR_ATV_FREQ_SOUND_CH2,
                                                    frequencyValue), 3, "Frequency Value"); 
        break;
     }           

    switch (levelParameter){
        case RSSFU_VAL_ATV_AUDIO_SOUND_LEVEL_MONO:
            viCheckParm (rssfu_SetAttributeViReal64 (instrSession, "", RSSFU_ATTR_ATV_LEVEL_SOUND_MONO,
                                                    levelValue), 5, "Level Value"); 
        break;
        case RSSFU_VAL_ATV_AUDIO_SOUND_LEVEL_LEFT:
              viCheckParm (rssfu_SetAttributeViReal64 (instrSession, "", RSSFU_ATTR_ATV_LEVEL_SOUND_LEFT,
                                                    levelValue), 5, "Level Value"); 
        break;
        case RSSFU_VAL_ATV_AUDIO_SOUND_LEVEL_RIGHT:
               viCheckParm (rssfu_SetAttributeViReal64 (instrSession, "", RSSFU_ATTR_ATV_LEVEL_SOUND_RIGHT,
                                                    levelValue), 5, "Level Value"); 
        break;
        case RSSFU_VAL_ATV_AUDIO_SOUND_LEVEL_CH1:
              viCheckParm (rssfu_SetAttributeViReal64 (instrSession, "", RSSFU_ATTR_ATV_LEVEL_SOUND_CH1,
                                                    levelValue), 5, "Level Value"); 
        break;
        case RSSFU_VAL_ATV_AUDIO_SOUND_LEVEL_CH2:
               viCheckParm (rssfu_SetAttributeViReal64 (instrSession, "", RSSFU_ATTR_ATV_LEVEL_SOUND_CH2,
                                                    levelValue), 5, "Level Value"); 
        break;
     }           
  
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/*============================================================================/
 * Function: Configure Analog TV Audio AF
 * Purpose: This function switches audio channel 1 or 2 ON or OFF.
/============================================================================*/
ViStatus _VI_FUNC rssfu_configAnalogTvAudioAf (ViSession instrSession,
                                               ViInt32 audioChannel,
                                               ViBoolean state)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if ((audioChannel < RSSFU_VAL_ATV_AUDIO_GEN_AF1) || (audioChannel > RSSFU_VAL_ATV_AUDIO_GEN_AF2))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Audio Channel");
        
    switch (audioChannel){
        case RSSFU_VAL_ATV_AUDIO_GEN_AF1:
            viCheckParm (rssfu_SetAttributeViBoolean (instrSession, "", RSSFU_ATTR_ATV_AUDIO_AF_1, state),
                         3, "State"); 
        break;
        case RSSFU_VAL_ATV_AUDIO_GEN_AF2:
             viCheckParm (rssfu_SetAttributeViBoolean (instrSession, "", RSSFU_ATTR_ATV_AUDIO_AF_2, state),
                         3, "State"); 
        break;
    }
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}                                                  

/*============================================================================/
 * Function: Configure Analog TV Impedance
 * Purpose:  This function selects the input impedance of the external audio
 *           input.
/============================================================================*/
ViStatus _VI_FUNC rssfu_configAnalogTvAudioImpedance (ViSession instrSession,
                                                      ViInt32 inputImpedance)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_ATV_AUDIO_IMPEDANCE_SIGNAL_SOURCE,
                            inputImpedance), 2, "Input Impedance");
                        
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Read External Audio Input Level
 * Purpose:  This function reads the level at the external audio input. The
 *           level is referenced either to 50 ohms (in dBm) or 600 ohms (in
 *           dBu), depending on the set input impedance of the external audio
 *           input.
/============================================================================*/
ViStatus _VI_FUNC rssfu_readExternalAudioInputLevel (ViSession instrSession,
                                                     ViReal64* level)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (!level) viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Level");
    viCheckErr (rssfu_GetAttributeViReal64 (instrSession, "", RSSFU_ATTR_ATV_SOUND_MODUL_AUEX_LEVEL,
                                            level));
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/*============================================================================/
 * Function: Load Analog TV Audio Library
 * Purpose:  This function selects the audio player file.
/============================================================================*/
ViStatus _VI_FUNC rssfu_loadAnalogTvAudioLibrary(ViSession instrSession,
                                                    ViString loadLibrary)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rssfu_SetAttributeViString (instrSession, "",
        RSSFU_ATTR_ANALOG_TV_AUDIO_LOAD_AUDIO_PLAYER_FILE, 
                loadLibrary), 2, "Load Library");
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Analog TV Audio Library Catalog
 * Purpose:  This function reads out the files in the default directory.
/============================================================================*/
ViStatus _VI_FUNC rssfu_analogTvAudioLibraryCatalog (ViSession instrSession,
                                            ViString directory, ViInt32 bufferSize,
                                            ViChar output[],
                                            ViInt32 *returnedValues)
{
   
    ViChar*     pbuffer = NULL;
    ViUInt32    count = 0, len = 0;
    ViStatus    error   = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
  
    if (Rs_Simulating(instrSession) == 0){                /* call only when locked */

    if (bufferSize != 0 && output == VI_NULL)
        return VI_ERROR_PARAMETER5;

    viCheckErr( viPrintf (instrSession, ":MMEM:CDIR '%s'", directory));
    
    viCheckErr( viPrintf(instrSession, ":IQC:RATV:APL:LIBR:CAT?\n"));
    
    viCheckErr( rssfu_WaitForOPC (instrSession, MAX_OPC_TIME)); 
    viCheckErr( rssfu_ReadDataUnknownLength(instrSession, &pbuffer, &count));
    
    if (output != NULL) 
    {
        len = (ViUInt32) ((bufferSize > (ViInt32) count) ? (ViInt32) count : bufferSize);
        strncpy(output, pbuffer, len);
        output[len] = '\0';
    }
    
    free(pbuffer);
    if (returnedValues) *returnedValues = (ViInt32) count;
        
    viCheckErr(rssfu_CheckStatus (instrSession)); 
    }
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;   
}
                                            

/*============================================================================/
 * Function: Analog TV Audio Library Catalog Length
 * Purpose:  This function queries the number of files in the default
 *           directory for audio player files (D:\AUDIOPLAYER).
/============================================================================*/
ViStatus _VI_FUNC rssfu_analogTvAudioLibraryCatalogLength (ViSession instrSession,
                                                  ViString directory,
                                                  ViInt32* catalogLength)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSSFU_IO_BUFFER_SIZE] = ""; 

    checkErr( Rs_LockSession (instrSession, VI_NULL));
  
    if (Rs_Simulating(instrSession) == 0){                /* call only when locked */

    if (!catalogLength) viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Catalog Length");
    
    viCheckErr( viPrintf (instrSession, ":MMEM:CDIR '%s'", directory)); 
        
    sprintf (buffer,":RATV:APL:LIBR:CAT:LENG?");
    
    viCheckErr( viQueryf (instrSession, buffer, "%ld", catalogLength));
    }   
    
    viCheckErr(rssfu_CheckStatus (instrSession));
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}                                                    
    
/*============================================================================/
 * Function: Configure Analog TV Video Input Signal
 * Purpose:  This function selects the audio source of the R&S SFU.
/============================================================================*/
ViStatus _VI_FUNC rssfu_confAnalogTvVideoInputSignal (ViSession instrSession,
                                                      ViInt32 videoInputSignal)

{
    ViStatus    error = VI_SUCCESS;
    ViChar      model [RSSFU_IO_BUFFER_SIZE] = "";  
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckErr (rssfu_GetAttributeViString (instrSession, "",
                                               RS_ATTR_INSTRUMENT_MODEL,
                                               RSSFU_IO_BUFFER_SIZE, model));
    if (strstr (model, "BTC") != NULL) {
         if (videoInputSignal == RSSFU_VAL_IQC_ATV_VIDEO_SOURCE_EXT)
            viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Video Input Signal");
    }   
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_ATV_VIDEO_INPUT_SIGNAL,
                            videoInputSignal), 2, "Video Input Signal");
                        
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}
 
/*============================================================================/
 * Function: Configure Analog TV Video
 * Purpose:  This function configures the test pattern in the VIDEO GENERATOR
 *           menu.
/============================================================================*/
ViStatus _VI_FUNC rssfu_confAnalogTvVideo (ViSession instrSession,
                                           ViInt32 visionPicture)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_ATV_VISION_PICTURE, visionPicture),
                         2, "Vision Picture");

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}
 
/*============================================================================/
 * Function: Load Analog TV Video Library
 * Purpose:  This function selects video pattern from ATVVIDEOLIB.
/============================================================================*/
ViStatus _VI_FUNC rssfu_loadAnalogTvVideoLibrary(ViSession instrSession,
                                                    ViString loadLibrary)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rssfu_SetAttributeViString (instrSession, "", RSSFU_ATTR_ATV_LOAD_VIDEO_LIB, 
                loadLibrary), 2, "Load Library");
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Analog TV Video Library Catalog
 * Purpose:  This function reads out the files in the default directory.
/============================================================================*/
ViStatus _VI_FUNC rssfu_analogTvVideoLibraryCatalog (ViSession instrSession,
                                            ViString directory, ViInt32 bufferSize,
                                            ViChar output[],
                                            ViInt32 *returnedValues)
{
   
    ViChar*     pbuffer = NULL;
    ViUInt32    count = 0, len = 0;
    ViStatus    error   = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
  
    if (Rs_Simulating(instrSession) == 0){                /* call only when locked */

    if (bufferSize != 0 && output == VI_NULL)
        return VI_ERROR_PARAMETER5;

    viCheckErr( viPrintf (instrSession, ":MMEM:CDIR '%s'", directory));
    
    viCheckErr( viPrintf(instrSession, ":IQC:RATV:VIDG:LIBR:CAT?\n"));
    
    viCheckErr( rssfu_WaitForOPC (instrSession, MAX_OPC_TIME)); 
    viCheckErr( rssfu_ReadDataUnknownLength(instrSession, &pbuffer, &count));
    
    if (output != NULL) 
    {
        len = (ViUInt32) ((bufferSize > (ViInt32) count) ? (ViInt32) count : bufferSize);
        strncpy(output, pbuffer, len);
        output[len] = '\0';
    }
    
    free(pbuffer);
    if (returnedValues) *returnedValues = (ViInt32) count;
        
    viCheckErr(rssfu_CheckStatus (instrSession)); 
    }
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;   
}
                                            

/*============================================================================/
 * Function: Analog TV Video Library Catalog Length
 * Purpose:  This function reads out the number of files in the default
 *           directory (D:\ATVVIDEOLIB).
/============================================================================*/
ViStatus _VI_FUNC rssfu_analogTvVideoLibraryCatalogLength (ViSession instrSession,
                                                  ViString directory,
                                                  ViInt32* catalogLength)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSSFU_IO_BUFFER_SIZE] = ""; 

    checkErr( Rs_LockSession (instrSession, VI_NULL));
  
    if (Rs_Simulating(instrSession) == 0){                /* call only when locked */

    if (!catalogLength) viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Catalog Length");
    
    viCheckErr( viPrintf (instrSession, ":MMEM:CDIR '%s'", directory)); 
        
    sprintf (buffer,":IQC:RATV:VIDG:LIBR:CAT:LENG?");
    
    viCheckErr( viQueryf (instrSession, buffer, "%ld", catalogLength));
    }   
    
    viCheckErr(rssfu_CheckStatus (instrSession));
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}                                                    

/*============================================================================/
 * Function: Configure Analog TV Video Vision Picture
 * Purpose:  This function configures the sound modulation mode.
/============================================================================*/
ViStatus _VI_FUNC rssfu_confAnalogTvVideoVisionPicture (ViSession instrSession,
                                                        ViInt32 mode)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_ATV_SOUND_MODUL_MODE, mode),
                         2, "Mode");

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}                                                        
    
/*============================================================================/
 * Function: NICAM Modulation Signal
 * Purpose:  This function selects the NICAM modulation signal.
/============================================================================*/
ViStatus _VI_FUNC rssfu_confAnalogTvNICAMSignal (ViSession instrSession,
                                                   ViInt32 NICAMSignal)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_ATV_SOUND_MODUL_NICAM_SIGNAL,
                            NICAMSignal), 2, "NICAM Signal");

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}   

/*============================================================================/
 * Function: Read Analog TV Values
 * Purpose:  This function reads selected value.
/============================================================================*/
ViStatus _VI_FUNC rssfu_readAnalogTvValues (ViSession instrSession,
                                            ViInt32 value, ViReal64* result)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    if ((value < RSSFU_VAL_ATV_SOUND_FREQ_DEV_MONO) || (value > RSSFU_VAL_ATV_SOUND_FREQ_DEV_PILOT))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Value");
        
    switch (value){
        case RSSFU_VAL_ATV_SOUND_FREQ_DEV_MONO:
            viCheckParm (rssfu_GetAttributeViReal64 (instrSession, "", RSSFU_ATTR_ATV_SOUND_MODUL_FREQ_DEV_MONO,
                                                    result), 3, "Result"); 
        break;
        case RSSFU_VAL_ATV_SOUND_FREQ_DEV_SUBCAR1:
            viCheckParm (rssfu_GetAttributeViReal64 (instrSession, "", RSSFU_ATTR_ATV_SOUND_MODUL_FREQ_DEV_SUBCAR1,
                                                     result), 3, "Result");
        break;
        case RSSFU_VAL_ATV_SOUND_FREQ_DEV_SUBCAR2:
            viCheckParm (rssfu_GetAttributeViReal64 (instrSession, "", RSSFU_ATTR_ATV_SOUND_MODUL_FREQ_DEV_SUBCAR2,
                                                     result), 3, "Result");
        break;
        case RSSFU_VAL_ATV_SOUND_FREQ_DEV_PILOT:
            viCheckParm (rssfu_GetAttributeViReal64 (instrSession, "", RSSFU_ATTR_ATV_SOUND_MODUL_FREQ_DEV_PILOT,
                                                     result), 3, "Result");
        break;
     }           
     
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

 
/*============================================================================/
 * Function: Enable Analog TV Special Settings
 * Purpose:  This function enables use of special Analog TV coding
 *           parameters.
/============================================================================*/
ViStatus _VI_FUNC rssfu_actEnableAnalogTvSpecial (ViSession instrSession,
                                                  ViBoolean state)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rssfu_SetAttributeViBoolean (instrSession, "", RSSFU_ATTR_ATV_SPECIAL_SETTINGS, state),
                         2, "State");
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}                                                  

/*============================================================================/
 * Function: Configure Analog TV Preemphasis
 * Purpose:  This function configures the preemphasis for the AF sound
 *           channels.
/============================================================================*/
ViStatus _VI_FUNC rssfu_confAnalogTvPreemphasis (ViSession instrSession,
                                                 ViInt32 preemphasis)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_ATV_SPECIAL_PREEMPHASIS, preemphasis),
                         2, "Preemphasis");

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}
                                                  

/*============================================================================/
 * Function: Configure Analog TV Pilot
 * Purpose:  This function configures the pilot carrier on sound subcarrier 2.
/============================================================================*/
ViStatus _VI_FUNC rssfu_confAnalogTvPilot (ViSession instrSession,
                                           ViBoolean pilot,
                                           ViReal64 nomFreqDevPilot)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rssfu_SetAttributeViBoolean (instrSession, "", RSSFU_ATTR_ATV_SPECIAL_PILOT, pilot),
                         2, "Pilot");
    
    viCheckParm (rssfu_SetAttributeViReal64 (instrSession, "", RSSFU_ATTR_ATV_SPECIAL_NOM_FREQ_DEV_PILOT, nomFreqDevPilot),
                         3, "Nom. Freq. Dev. Pilot");

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}                                           
                                           
/*============================================================================/
 * Function: Configure Analog TV Subcarrier Settings
 * Purpose:  This function configures special Analog TV coding parameters.
/============================================================================*/
ViStatus _VI_FUNC rssfu_confAnalogTvSubcarrierSettings (ViSession instrSession,
                                                        ViInt32 subcarrierParameter,
                                                        ViReal64 nomFreqDev,
                                                        ViBoolean subcarrier,
                                                        ViReal64 frequencySubcarrier,
                                                        ViReal64 levelSubcarrier)
{                                             
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    if ((subcarrierParameter < RSSFU_VAL_ATV_SOUND_SPEC_SUBCAR_MONO) || (subcarrier > RSSFU_VAL_ATV_SOUND_SPEC_SUBCAR2))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Subcarrier");
 

    switch (subcarrierParameter){
        case RSSFU_VAL_ATV_SOUND_SPEC_SUBCAR_MONO:
            viCheckParm (rssfu_SetAttributeViReal64 (instrSession, "", RSSFU_ATTR_ATV_SPECIAL_NOM_FREQ_DEV_MONO,
                                                    nomFreqDev), 3, "Nom. Freq. Dev."); 
            
            viCheckParm (rssfu_SetAttributeViBoolean (instrSession, "", RSSFU_ATTR_ATV_SPECIAL_SUBCARRIER,
                                                    subcarrier), 4, "Subcarrier State");
                                                    
            viCheckParm (rssfu_SetAttributeViReal64 (instrSession, "", RSSFU_ATTR_ATV_SPECIAL_FREQ_SUBCARRIER,
                                                    frequencySubcarrier), 5, "Frequency Subcarrier");
        
            viCheckParm (rssfu_SetAttributeViReal64 (instrSession, "", RSSFU_ATTR_ATV_SPECIAL_LEVEL_SUBCARRIER,
                                                    levelSubcarrier), 6, "Level Subcarrier");                                                     
        break;
        
        
        case RSSFU_VAL_ATV_SOUND_SPEC_SUBCAR1:
            viCheckParm (rssfu_SetAttributeViReal64 (instrSession, "", RSSFU_ATTR_ATV_SPECIAL_NOM_FREQ_DEV_SUBCAR_1,
                                                    nomFreqDev), 3, "Nom. Freq. Dev."); 
            
            viCheckParm (rssfu_SetAttributeViBoolean (instrSession, "", RSSFU_ATTR_ATV_SPECIAL_SUBCARRIER_1,
                                                    subcarrier), 4, "Subcarrier State");
                                                    
            viCheckParm (rssfu_SetAttributeViReal64 (instrSession, "", RSSFU_ATTR_ATV_SPECIAL_FREQ_SUBCARRIER_1,
                                                    frequencySubcarrier), 5, "Frequency Subcarrier");
        
            viCheckParm (rssfu_SetAttributeViReal64 (instrSession, "", RSSFU_ATTR_ATV_SPECIAL_LEVEL_SUBCARRIER_1,
                                                    levelSubcarrier), 6, "Level Subcarrier");     
        break;
        
        
        case RSSFU_VAL_ATV_SOUND_SPEC_SUBCAR2:
            viCheckParm (rssfu_SetAttributeViReal64 (instrSession, "", RSSFU_ATTR_ATV_SPECIAL_NOM_FREQ_DEV_SUBCAR_2,
                                                    nomFreqDev), 3, "Nom. Freq. Dev."); 
            
            viCheckParm (rssfu_SetAttributeViBoolean (instrSession, "", RSSFU_ATTR_ATV_SPECIAL_SUBCARRIER_2,
                                                    subcarrier), 4, "Subcarrier State");
                                                    
            viCheckParm (rssfu_SetAttributeViReal64 (instrSession, "", RSSFU_ATTR_ATV_SPECIAL_FREQ_SUBCARRIER_2,
                                                    frequencySubcarrier), 5, "Frequency Subcarrier");
        
            viCheckParm (rssfu_SetAttributeViReal64 (instrSession, "", RSSFU_ATTR_ATV_SPECIAL_LEVEL_SUBCARRIER_2,
                                                    levelSubcarrier), 6, "Level Subcarrier");     
        break;
    }           
  
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

  
/*============================================================================/
 * Function: Configure Analog TV Special Settings
 * Purpose:  This function configures special Analog TV coding parameters.
/============================================================================*/
ViStatus _VI_FUNC rssfu_confAnalogTvSpecialSettings (ViSession instrSession,
                                                     ViBoolean VSBFilter,
                                                     ViBoolean GDPrecorrection,
                                                     ViReal64 residualCarrier)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rssfu_SetAttributeViBoolean (instrSession, "", RSSFU_ATTR_ATV_SPECIAL_VSB_FILTER, VSBFilter),
                         2, "VSB Filter");
    
    viCheckParm (rssfu_SetAttributeViBoolean (instrSession, "", RSSFU_ATTR_ATV_SPECIAL_GD_PRECORRECTION, GDPrecorrection),
                         3, "GD Precorrection");                         
    
    viCheckParm (rssfu_SetAttributeViReal64 (instrSession, "", RSSFU_ATTR_ATV_SPECIAL_RESIDUAL_CARRIER, residualCarrier),
                         4, "Residual Carrier");

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
} 
     
/*============================================================================/
 * Function: Configure Analog TV Special Video and Carrier
 * Purpose:  This function configures special Analog TV coding parameters.
/============================================================================*/
ViStatus _VI_FUNC rssfu_confAnalogTvSpecialVideoCarrier (ViSession instrSession,
                                                  ViBoolean video,
                                                  ViBoolean carrier)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rssfu_SetAttributeViBoolean (instrSession, "", RSSFU_ATTR_ATV_SPECIAL_VIDEO, video),
                         2, "Video");
    
    viCheckParm (rssfu_SetAttributeViBoolean (instrSession, "", RSSFU_ATTR_ATV_SPECIAL_CARRIER, carrier),
                         3, "Carrier");                         
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
} 
       
/*============================================================================/
 * Function: Configure Analog TV Special VSB Characteristic
 * Purpose:  This function configures the vestigial sideband characteristic
 *           of the respective standard.
/============================================================================*/
ViStatus _VI_FUNC rssfu_confAnalogTvSpecialVsbCharacteristic
             (ViSession instrSession, ViInt32 VSBCharacteristic)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_ATV_SPECIAL_VSB_CHARACTERISTIC,
                            VSBCharacteristic), 2, "VSB Characteristic");

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Set Analog TV Special AM Modulation Depth
 * Purpose:  This function sets the modulation depth of the L standard
 *           amplitude-modulated sound.
/============================================================================*/
ViStatus _VI_FUNC rssfu_setAnalogTvSpecialAmModulationDepth
             (ViSession instrSession, ViInt32 AMModulationDepth)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_ATV_SPECIAL_AM_MODULATION_DEPTH,
                            AMModulationDepth), 2, "AM Modulation Depth");

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/*****************************************************************************
 *----------------------------- CMMB ----------------------------------------*
 *****************************************************************************/

/*============================================================================/
 * Function: Configure CMMB Input Signal
 * Purpose:  This function configures CMMB Input Signal parameters.
/============================================================================*/
ViStatus _VI_FUNC rssfu_confCMMBInputSignal(ViSession   instrSession,
                                            ViInt32 source,
                                            ViInt32 input,
                                            ViInt32 stuffing)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      model [RSSFU_IO_BUFFER_SIZE] = ""; 

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    viCheckErr (rssfu_GetAttributeViString (instrSession, "",
                                               RS_ATTR_INSTRUMENT_MODEL,
                                               RSSFU_IO_BUFFER_SIZE, model));
    
    if (strstr (model, "BTC") == NULL) {  
        viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_CMMB_INPUT_SIGNAL_SOURCE, source),
                             2, "Source");
        if (source == RSSFU_VAL_IQC_CMMB_SOUR_EXT)
            viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_CMMB_INPUT_SIGNAL_INPUT, input),
                             3, "Input");
    }
    viCheckParm (rssfu_SetAttributeViBoolean (instrSession, "", RSSFU_ATTR_CMMB_INPUT_SIGNAL_STUFFING, (ViBoolean)stuffing),
                         4, "Stuffing");
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Configure CMMB Test Signal
 * Purpose:  This function sets the signal type for the test signal.
/============================================================================*/
ViStatus _VI_FUNC rssfu_configureCMMBTestSignal(ViSession instrSession,
                                           ViInt32 testSignal)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_CMMB_INPUT_SIGNAL_TEST_SIGNAL, testSignal),
                         2, "Test Signal"); 
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Read CMMB Values
 * Purpose:  This function reads selected coding value.
/============================================================================*/
ViStatus _VI_FUNC rssfu_readCMMBValues(ViSession instrSession,
                                        ViInt32 value,
                                        ViReal64* result)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      model [RSSFU_IO_BUFFER_SIZE] = "";  
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckErr (rssfu_GetAttributeViString (instrSession, "",
                                               RS_ATTR_INSTRUMENT_MODEL,
                                               RSSFU_IO_BUFFER_SIZE, model));
    
    if ((value < RSSFU_VAL_MAX_USEFUL_DRATE) || (value > RSSFU_VAL_MEAS_DRATE))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Value");
        
    switch (value){                                                       
        case RSSFU_VAL_MAX_USEFUL_DRATE:
            if (strstr (model, "BTC") == NULL) {  
                viCheckParm (rssfu_GetAttributeViReal64 (instrSession, "", RSSFU_ATTR_READ_CMMB_MAX_USEFUL_DATA_RATE,
                                                         result), 3, "Result");
            }
        break;
        case RSSFU_VAL_MEAS_DRATE:
            viCheckParm (rssfu_GetAttributeViReal64 (instrSession, "", RSSFU_ATTR_READ_CMMB_MEASURED_DATA_RATE,
                                                     result), 3, "Result");
        break;
     }           
     
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Set CMMB Number Of Services
 * Purpose:  This function sets the number of services.
/============================================================================*/
ViStatus _VI_FUNC rssfu_SetCMMBNumberOfServices(ViSession   instrSession,
                                            ViInt32 numberOfServices)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_CMMB_INPUT_SIGNAL_NUMBER_OF_SERVICES, numberOfServices),
                         2, "Number Of Services");
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
} 


/*============================================================================/
 * Function: Configure CMMB Coding
 * Purpose:  This function configures CMMB coding parameters.
/============================================================================*/
ViStatus _VI_FUNC rssfu_confCMMBCoding (ViSession instrSession,
                                        ViInt32 control, ViInt32 bandwidth) 

{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_CMMB_CODING_CONTROL, control),
                         2, "Control");
    
    if (control == RSSFU_VAL_IQC_CMMB_CONTROL_MANUAL)
        viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_CMMB_CODING_BANDWIDTH, bandwidth),
                         3, "Bandwidth");
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}                                         

/*============================================================================/
 * Function: Read CMMB OFDM Mode
 * Purpose:  This function returns number of carriers of the CMMB coder.
/============================================================================*/
ViStatus _VI_FUNC rssfu_ReadCMMBOFDMMode (ViSession instrSession, ViInt32* OFDMMode)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
   
    if (!OFDMMode) viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "OFDM Mode");
        
    viCheckParm (rssfu_GetAttributeViInt32 (instrSession, "", RSSFU_ATTR_CMMB_CODING_OFDM_MODE,
                                            OFDMMode), 2, "OFDM Mode");
   Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Configure CMMB PLCH Table
 * Purpose:  This function configures CMMB coding PLCH table parameters.
/============================================================================*/
ViStatus _VI_FUNC rssfu_confCMMBPLCHTable (ViSession instrSession, ViInt32 PLCH,
                                           ViInt32 RS, ViInt32 byteIL, ViInt32 LDPC,
                                           ViInt32 constel, ViInt32 SCRMode,
                                           ViInt32 startTS, ViInt32 stopTS)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSSFU_IO_BUFFER_SIZE] = ""; 

    checkErr( Rs_LockSession (instrSession, VI_NULL));
  
    if (Rs_Simulating(instrSession) == 0){                /* call only when locked */
   
    if ((PLCH < 0) || (PLCH > 39))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "PLCH");
    
    if ((RS < RSSFU_VAL_IQC_CMMB_RS_0) || (RS > RSSFU_VAL_IQC_CMMB_RS_3))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 3, "RS");
        
    if ((byteIL < 1) || (byteIL > 3))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 4, "Byte IL");     
    
    if ((LDPC < RSSFU_VAL_IQC_CMMB_LDPC_R1_2) || (LDPC > RSSFU_VAL_IQC_CMMB_LDPC_R3_4))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 5, "LDPC");     

    if ((constel < RSSFU_VAL_IQC_CMMB_CONST_C2) || (constel > RSSFU_VAL_IQC_CMMB_CONST_C16))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 6, "Constel");   
    
    if ((SCRMode < 0) || (SCRMode > 7))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 7, "SCR Mode");
    
    if ((startTS < 0) || (startTS > 39))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 8, "Start TS");  
        
    if ((stopTS < 0) || (stopTS > 39))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 9, "Stop TS");  
        
    sprintf (buffer,":CMMB:RS %ld,%s;BYIM %ld,%ld;LDPC %ld,%s;CONS %ld,%s;SCRM %ld,%ld;STAR %ld,%ld;STOP %ld,%ld",
        PLCH,cmmbPlchRsArr[RS],PLCH,byteIL,
        PLCH,cmmbPlchLdpdArr[LDPC],
        PLCH,cmmbPlchConstArr[constel],PLCH,SCRMode,
        PLCH,startTS,PLCH,stopTS);
  
    viCheckErr( viPrintf (instrSession, "%s", buffer));  
    }   
    
    viCheckErr(rssfu_CheckStatus (instrSession));    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
} 
  

/*============================================================================/
 * Function: Enable CMMB Special Settings
 * Purpose:  This function enables use of special AVSB coding parameters.
/============================================================================*/
ViStatus _VI_FUNC rssfu_actEnableCMMBSpecial(ViSession instrSession,
                                             ViBoolean state)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rssfu_SetAttributeViBoolean (instrSession, "", RSSFU_ATTR_CMMB_SPECIAL_SETTINGS_STATE, state),
                         2, "State");
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

 
/*============================================================================/
 * Function: Configure CMMB Special Settings
 * Purpose:  This function configures special CMMB coding parameters.
/============================================================================*/
ViStatus _VI_FUNC rssfu_confCMMBSpecial (ViSession instrSession,
                                         ViBoolean reedSolomon,
                                         ViBoolean byteInterleaver,
                                         ViBoolean bitInterleaver,
                                         ViBoolean scrambling)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rssfu_SetAttributeViBoolean (instrSession, "", RSSFU_ATTR_CMMB_SPECIAL_REED_SOLOMON, reedSolomon),
                 2, "Reed Solomon"); 
    viCheckParm (rssfu_SetAttributeViBoolean (instrSession, "", RSSFU_ATTR_CMMB_SPECIAL_BYTE_INTERLEAVER, byteInterleaver),
                 3, "Byte Interleaver");
    viCheckParm (rssfu_SetAttributeViBoolean (instrSession, "", RSSFU_ATTR_CMMB_SPECIAL_BIT_INTERLEAVER, bitInterleaver),
                 4, "Bit Interleaver");
    viCheckParm (rssfu_SetAttributeViBoolean (instrSession, "", RSSFU_ATTR_CMMB_SPECIAL_SCRAMBLING, scrambling),
                 5, "Scrambling");   
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Configure CMMB Settings
 * Purpose:  This function configures CMMB coding settings.
/============================================================================*/
ViStatus _VI_FUNC rssfu_confCMMBSettings (ViSession instrSession,
                                          ViInt32 transmitterID, ViInt32 areaID,
                                          ViInt32 payloadTestSignal, ViInt32 PRBS)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_CMMB_SETTINGS_TRANSMITTER_ID, transmitterID),
                         2, "Transmitter ID");
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_CMMB_SETTINGS_AREA_ID, areaID),
                         3, "Area ID"); 
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_CMMB_SETTINGS_PAYLOAD_TEST_SIGNAL, payloadTestSignal),
                         4, "Payload Test Signal"); 
    switch (payloadTestSignal){
        case  RSSFU_IQC_PRBS:
            viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_CMMB_SETTINGS_PRBS, PRBS),
                         5, "PRBS"); 
    }   
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}





/*****************************************************************************
 *--------------------------------- DVB-T2 ----------------------------------*
 *****************************************************************************/
/*===========================================================================*/
/* Function: Configure DVBT2 Input T2 MI Interface
/* Purpose:  This function activates or deactivates the T2 modulator
/*           interface.
/*===========================================================================*/
ViStatus _VI_FUNC rssfu_ConfigureDVBT2InputT2MIInterface (ViSession instrSession,
                                                          ViBoolean t2MIInterface, ViInt32 t2MIInput)
{
ViStatus    error = VI_SUCCESS;
ViChar      model [RSSFU_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
                    
    viCheckErr (rssfu_GetAttributeViString (instrSession, "",
                                               RS_ATTR_INSTRUMENT_MODEL,
                                               RSSFU_IO_BUFFER_SIZE, model));
    
    viCheckParm (rssfu_SetAttributeViBoolean (instrSession, "", RSSFU_ATTR_DVBT2_INPUT_SIGNAL_T2_MI_INTERFACE_STATE,
                        t2MIInterface), 2, "T2-MI Interface");
    if (strstr (model, "BTC") == NULL) { 
        if (t2MIInterface) 
            viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_DVBT2_INPUT_SIGNAL_T2_MI_INTERFACE_INPUT,
                        t2MIInput), 3, "T2-MI Input");  
    }
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/*===========================================================================*/
/* Function: Configure DVBT2 Input T2 MI PID
/* Purpose:  This function configures PID used for T2-MI decapsulation from
/*           the MPEG transport stream.
/*===========================================================================*/
ViStatus _VI_FUNC rssfu_ConfigureDVBT2InputT2MIPID(
    ViSession   instrSession, ViInt32 t2MIPID)
{
 ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_DVBT2_INPUT_SIGNAL_T2_MI_PID, t2MIPID),
                         2, "T2-MI PID");
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Configure DVBT2 Input T2 MI SID
/* Purpose:  This function configures T2-MI stream ID used when transmitting
/*           a composite signal.
/*===========================================================================*/
ViStatus _VI_FUNC rssfu_ConfigureDVBT2InputT2MISID(
    ViSession   instrSession, ViInt32 t2MISID)
{
 ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_DVBT2_INPUT_SIGNAL_T2_MI_SID, t2MISID),
                         2, "T2-MI SID");
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Configure DVBT2 Input Signal
/* Purpose:  This function configures DVB-T2 Input Signal parameters.
/*===========================================================================*/
ViStatus _VI_FUNC rssfu_confDVBT2InputSignal(
    ViSession   instrSession,
    ViInt32 t2MISource,
    ViInt32 numberOfPLP,
    ViInt32 cmType
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      model [RSSFU_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckErr (rssfu_GetAttributeViString (instrSession, "",
                                               RS_ATTR_INSTRUMENT_MODEL,
                                               RSSFU_IO_BUFFER_SIZE, model));
    if (strstr (model, "BTC") == NULL) { 
        viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_DVBT2_INPUT_SIGNAL_T2_MI_SOURCE, t2MISource),
                         2, "T2-MI Source");
    }
    
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_DVBT2_INPUT_SIGNAL_NUMBER_OF_PLP, numberOfPLP),
                         3, "Number Of PLP"); 
    /*viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_DVBT2_INPUT_SIGNAL_CM_TYPE, cmType),
                         4, "CM Type"); */
    cmType;
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Configure DVBT2 PLP
/* Purpose:  This function configures DVB-T2 PLP parameters.
/*===========================================================================*/
ViStatus _VI_FUNC rssfu_configureDVBT2Plp(
    ViSession   instrSession,
    ViInt32 plp,
    ViInt32 inputFormat,
    ViInt32 source, 
    ViInt32 input,
    ViBoolean   stuffing
)
{
 ViStatus    error = VI_SUCCESS;
 ViChar      buf1[RSSFU_TOKEN_BUFFER_SIZE] = "";
 ViChar      model [RSSFU_IO_BUFFER_SIZE] = ""; 

    if ((plp < 1) || (plp > 16))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "PLP"); 

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    checkErr (rssfu_GetActualRepCaps (instrSession, RSSFU_ATTR_DVBT2_INPUT_SIGNAL_INPUT_FORMAT, "Plp", plp-1, buf1));
    
    viCheckErr (rssfu_GetAttributeViString (instrSession, "",
                                               RS_ATTR_INSTRUMENT_MODEL,
                                               RSSFU_IO_BUFFER_SIZE, model));
    
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, buf1, RSSFU_ATTR_DVBT2_INPUT_SIGNAL_INPUT_FORMAT, inputFormat),
                         3, "Input Format");
    
    if (strstr (model, "BTC") == NULL) {
        viCheckParm (rssfu_SetAttributeViInt32 (instrSession, buf1, RSSFU_ATTR_DVBT2_INPUT_SIGNAL_SOURCE, source ),
                         4, "Source"); 
    }
    
    if (source == RSSFU_IQC_SIG_SOUR_EXT){
        if (strstr (model, "BTC") == NULL) { 
            viCheckParm (rssfu_SetAttributeViInt32 (instrSession, buf1, RSSFU_ATTR_DVBT2_INPUT_SIGNAL_INPUT, input),
                         5, "Input"); 
        }
    }   
    if (source != RSSFU_IQC_SIG_SOUR_TEST)   
        viCheckParm (rssfu_SetAttributeViBoolean (instrSession, buf1, RSSFU_ATTR_DVBT2_INPUT_SIGNAL_STUFFING, stuffing),
                         6, "Stuffing");                          
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;

}

/*===========================================================================*/
/* Function: Read DVBT2 Values
/* Purpose:  This function reads selected coding value.
/*===========================================================================*/
ViStatus _VI_FUNC rssfu_readDVBT2Values(
    ViSession   instrSession,
    ViInt32 plp,
    ViInt32 value,
    ViReal64* result
)
{
 ViStatus    error = VI_SUCCESS;
 ViChar      buf1[RSSFU_TOKEN_BUFFER_SIZE] = "";

    if ((plp < 1) || (plp > 16))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "PLP"); 

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    checkErr (rssfu_GetActualRepCaps (instrSession, RSSFU_ATTR_DVBT2_INPUT_SIGNAL_MAX_USEF_DR, "Plp", plp-1, buf1));

    
     switch (value){
        case RSSFU_VAL_MAX_USEFUL_DRATE:
            viCheckParm (rssfu_GetAttributeViReal64 (instrSession, buf1, RSSFU_ATTR_DVBT2_INPUT_SIGNAL_MAX_USEF_DR, 
                result), 4, "Result");
        break;
        case RSSFU_VAL_MEAS_DRATE:
            viCheckParm (rssfu_GetAttributeViReal64 (instrSession, buf1, RSSFU_ATTR_DVBT2_INPUT_SIGNAL_MEAS_DR, 
                result), 4, "Result");
        break;
        case RSSFU_VAL_USEFUL_DRATE:
            viCheckParm (rssfu_GetAttributeViReal64 (instrSession, buf1, RSSFU_ATTR_DVBT2_INPUT_SIGNAL_USEFUL_DR, 
                result), 4, "Result");
        break;
        
        
        default:
            viCheckParm( RS_ERROR_INVALID_PARAMETER, 3, "Value"); 
    }

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Configure DVBT2 Test Signal
/* Purpose:  This function configures DVB-T2 Test Signal.
/*===========================================================================*/
ViStatus _VI_FUNC rssfu_confDVBT2TestSignal(
    ViSession   instrSession,
    ViInt32 plp,
    ViInt32 testSignal
)
{
 ViStatus    error = VI_SUCCESS;
 ViChar      buf1[RSSFU_TOKEN_BUFFER_SIZE] = "";

    if ((plp < 1) || (plp > 16))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "PLP"); 

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    checkErr (rssfu_GetActualRepCaps (instrSession, RSSFU_ATTR_DVBT2_INPUT_SIGNAL_TEST_SIGNAL, "Plp", plp-1, buf1));

    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, buf1, RSSFU_ATTR_DVBT2_INPUT_SIGNAL_TEST_SIGNAL, testSignal),
                         3, "Test Signal");                        
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;

}


/*===========================================================================*/
/* Function: Reset DVBT2 T2 MI Analyzer Log File
/* Purpose:  This function resets the T2-MI analyzer log file.
/*===========================================================================*/
ViStatus _VI_FUNC rssfu_ResetDVBT2T2MIAnalyzerLogFile(ViSession instrSession)
{
 ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckErr (rssfu_SetAttributeViString (instrSession, "", RSSFU_ATTR_DVBT2_INPUT_SIGNAL_RESET_T2_MI_ANALYZER_LOG_FILE, ""));
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Read DVBT2 T2 MI Analyzer
 * Purpose:  This function reads the current error message from the T2-MI
/*           analyzer.

Attribute(s):

/============================================================================*/
ViStatus _VI_FUNC rssfu_ReadDVBT2T2MIAnalyzer (ViSession instrSession,
                                                       ViChar  t2MIAnalyzer[])
{
    ViStatus    error = VI_SUCCESS; 
    ViChar      buf2[RSSFU_IO_BUFFER_SIZE] = "\0";
   
                             
    checkErr( Rs_LockSession (instrSession, VI_NULL)); 
    

    if (t2MIAnalyzer == 0) viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "T2 MI Analyzer");
    viCheckErr (rssfu_GetAttributeViString (instrSession, "", RSSFU_ATTR_DVBT2_INPUT_SIGNAL_T2_MI_ANALYZER,
                                            RSSFU_IO_BUFFER_SIZE, buf2));
    strcpy (t2MIAnalyzer, buf2);                                           
        
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Configure DVBT2 Mode Stream Adaptation
/* Purpose:  This function configures DVB-T2 Mode Stream Adaptation 
/*           parameters.
/*===========================================================================*/
ViStatus _VI_FUNC rssfu_confDVBT2ModeStreamAdaptation(
    ViSession   instrSession,
    ViInt32 plp,
    ViInt32 plpID,
    ViInt32 plpType,
    ViInt32 bbMode,
    ViInt32 issy,
    ViBoolean nullPacketDel,
    ViBoolean inBandSignal
)
{
 ViStatus    error = VI_SUCCESS;
 ViChar      buf1[RSSFU_TOKEN_BUFFER_SIZE] = "";

    if ((plp < 1) || (plp > 16))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "PLP"); 

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    checkErr (rssfu_GetActualRepCaps (instrSession, RSSFU_ATTR_DVBT2_MODE_STREAM_ADAPT_PLP_ID, "Plp", plp-1, buf1));

    
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, buf1, RSSFU_ATTR_DVBT2_MODE_STREAM_ADAPT_PLP_ID, plpID),
                         3, "PLP ID");
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, buf1, RSSFU_ATTR_DVBT2_MODE_STREAM_ADAPT_PLP_TYPE, plpType),
                         4, "PLP Type");                          
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, buf1, RSSFU_ATTR_DVBT2_MODE_STREAM_ADAPT_BB_MODE, bbMode),
                         5, "BB Mode");
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, buf1, RSSFU_ATTR_DVBT2_MODE_STREAM_ADAPT_ISSY, issy),
                         6, "ISSY");
    viCheckParm (rssfu_SetAttributeViBoolean (instrSession, buf1, RSSFU_ATTR_DVBT2_MODE_STREAM_ADAPT_NULL_PACKET_DEL, nullPacketDel),
                         7, "Null Packet Del");
    viCheckParm (rssfu_SetAttributeViBoolean (instrSession, buf1, RSSFU_ATTR_DVBT2_MODE_STREAM_ADAPT_IN_BAND_SIGNAL, inBandSignal),
                         8, "In Band Signal");                         
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/*===========================================================================*/
/* Function: Configure DVBT2 Mode Stream Adaptation In Band Signal
/* Purpose: This function configures DVB-T2 mode stream In Band Signal.
/*===========================================================================*/
ViStatus _VI_FUNC rssfu_ConfigureDVBT2ModeStreamAdaptationInBandSignal
             (ViSession instrSession, ViInt32 plp, ViBoolean inBandSignalA,
              ViBoolean inBandSignalB)

{
 ViStatus    error = VI_SUCCESS;
 ViChar      buf1[RSSFU_TOKEN_BUFFER_SIZE] = "";

    if ((plp < 1) || (plp > 16))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "PLP"); 

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    checkErr (rssfu_GetActualRepCaps (instrSession, RSSFU_ATTR_DVBT2_MODE_STREAM_ADAPT_IN_BAND_SIGNAL_A, "Plp", plp-1, buf1));

    
    viCheckParm (rssfu_SetAttributeViBoolean (instrSession, buf1, RSSFU_ATTR_DVBT2_MODE_STREAM_ADAPT_IN_BAND_SIGNAL_A, inBandSignalA),
                         3, "In Band Signal A");
    viCheckParm (rssfu_SetAttributeViBoolean (instrSession, buf1, RSSFU_ATTR_DVBT2_MODE_STREAM_ADAPT_IN_BAND_SIGNAL_B, inBandSignalB),
                         4, "In Band Signal B");
                    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/*===========================================================================*/
/* Function: Configure DVBT2 Mode Stream Adaptation CM
/* Purpose:  This function sets the coding and modulation type.
/*===========================================================================*/
ViStatus _VI_FUNC rssfu_ConfigureDVBT2ModeStreamAdaptationCM(
    ViSession   instrSession,
    ViInt32 plp,  
    ViInt32 cmType
)
{
 ViStatus    error = VI_SUCCESS;
 ViChar      buf1[RSSFU_TOKEN_BUFFER_SIZE] = "";

    if ((plp < 1) || (plp > 16))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "PLP"); 

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    checkErr (rssfu_GetActualRepCaps (instrSession, RSSFU_ATTR_DVBT2_MODE_STREAM_ADAPT_CM_TYPE, "Plp", plp-1, buf1));

    
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, buf1, RSSFU_ATTR_DVBT2_MODE_STREAM_ADAPT_CM_TYPE, cmType),
                         3, "CM Type");
                    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Read DVBT2 Mode Stream Adaptation Values
/* Purpose:  This function reads selected coding value.
/*===========================================================================*/
ViStatus _VI_FUNC rssfu_ReadDVBT2ModeStreamAdaptationValues
             (ViSession instrSession, ViInt32 plp, ViInt32* PLPGroupID,
              ViInt32* otherInBandPLP)

{
 ViStatus    error = VI_SUCCESS;
 ViChar      buf1[RSSFU_TOKEN_BUFFER_SIZE] = "";

    if ((plp < 1) || (plp > 16))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "PLP"); 

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
     checkErr (rssfu_GetActualRepCaps (instrSession, RSSFU_ATTR_DVBT2_MODE_STREAM_ADAPT_PLP_GROUP_ID, "Plp", plp-1, buf1));    
    
    if (!PLPGroupID) viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "PLP Group ID");
    
    if (!otherInBandPLP) viCheckParm (RS_ERROR_INVALID_PARAMETER, 4, "Other In Band PLP");  
    
    viCheckParm (rssfu_GetAttributeViInt32 (instrSession, buf1, RSSFU_ATTR_DVBT2_MODE_STREAM_ADAPT_PLP_GROUP_ID, 
               PLPGroupID), 3, "PLP Group ID");
    viCheckParm (rssfu_GetAttributeViInt32 (instrSession,buf1, RSSFU_ATTR_DVBT2_MODE_STREAM_ADAPT_OTHER_IN_BAND_PLP, 
               otherInBandPLP), 4, "Other In Band PLP");
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Configure DVBT2 BICM Parameters
/* Purpose:  This function configures DVB-T2 Bit Interleaved Coding and 
/*           Modulation subitem parameters.
/*===========================================================================*/
ViStatus _VI_FUNC rssfu_confDVBT2BICMParameters(
    ViSession   instrSession,
    ViInt32 plp,
    ViInt32 fecFrame,
    ViInt32 codeRate,
    ViInt32 constellation,
    ViBoolean constellRotation
)
{
 ViStatus    error = VI_SUCCESS;
 ViChar      buf1[RSSFU_TOKEN_BUFFER_SIZE] = "";

    if ((plp < 1) || (plp > 16))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "PLP"); 

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    checkErr (rssfu_GetActualRepCaps (instrSession, RSSFU_ATTR_DVBT2_BICM_FEC_FRAME, "Plp", plp-1, buf1));

    
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, buf1, RSSFU_ATTR_DVBT2_BICM_FEC_FRAME, fecFrame),
                         3, "FEC Frame");
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, buf1, RSSFU_ATTR_DVBT2_BICM_CODE_RATE, codeRate),
                         4, "Code Rate");                          
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, buf1, RSSFU_ATTR_DVBT2_BICM_CONSTELL, constellation),
                         5, "Constellation");
    viCheckParm (rssfu_SetAttributeViBoolean (instrSession, buf1, RSSFU_ATTR_DVBT2_BICM_CONSTELL_ROTATION, constellRotation),
                         6, "Constell Rotation");
                           
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Configure DVBT2 BICM Time Interleaver Parameters
/* Purpose:  This function configures DVB-T2 Bit Interleaved Coding and 
/*           Modulation subitem parameters.
/*===========================================================================*/
ViStatus _VI_FUNC rssfu_confDVBT2BICMTimeInterleaverParameters(
    ViSession   instrSession,
    ViInt32 plp,
    ViInt32 timeInterleaverType,
    ViInt32 frameIntIJump,
    ViInt32 timeInterleaverLength
)
{
 ViStatus    error = VI_SUCCESS;
 ViChar      buf1[RSSFU_TOKEN_BUFFER_SIZE] = "";

    if ((plp < 1) || (plp > 16))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "PLP"); 

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    checkErr (rssfu_GetActualRepCaps (instrSession, RSSFU_ATTR_DVBT2_BICM_TIME_INTERL_TYPE, "Plp", plp-1, buf1));

    
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, buf1, RSSFU_ATTR_DVBT2_BICM_TIME_INTERL_TYPE, timeInterleaverType),
                         3, "Time Interleaver Type");
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, buf1, RSSFU_ATTR_DVBT2_BICM_FRAME_INT_I_JUMP, frameIntIJump),
                         4, "Frame Int I Jump");                          
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, buf1, RSSFU_ATTR_DVBT2_BICM_TIME_INTERL_LENGTH, timeInterleaverLength),
                         5, "Time Interleaver Length");  
                           
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/*===========================================================================*/
/* Function: Read DVBT2 BICM FEC Blocks Per IF
/* Purpose:  This function queries the FEC blocks per interleaving frame.
/*===========================================================================*/
ViStatus _VI_FUNC rssfu_readDVBT2BICMFECBlocksPerIF(
    ViSession   instrSession,
    ViInt32 plp,
    ViInt32*    fecBlocksPerIF
)
{
    ViStatus    error = VI_SUCCESS;   
    ViChar      buf1[RSSFU_TOKEN_BUFFER_SIZE] = "";

    if ((plp < 1) || (plp > 16))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "PLP"); 

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (!fecBlocksPerIF) viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "FEC Blocks Per IF");   
    
    checkErr (rssfu_GetActualRepCaps (instrSession, RSSFU_ATTR_DVBT2_BICM_FEC_BLOCKS_PER_IF, "Plp", plp-1, buf1));

    viCheckParm (rssfu_GetAttributeViInt32 (instrSession, buf1, RSSFU_ATTR_DVBT2_BICM_FEC_BLOCKS_PER_IF, 
               fecBlocksPerIF), 3, "FEC Blocks Per IF");        
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Read DVBT2 BICM Maximum FEC Blocks Per IF
/* Purpose:  This function queries the maximum FEC blocks per interleaving
/*           frame
/*===========================================================================*/
ViStatus _VI_FUNC rssfu_readDVBT2BICMMaxFECBlocksPerIF(
    ViSession   instrSession,
    ViInt32 plp,
    ViInt32*    MaxFecBlocksPerIF
)
{
    ViStatus    error = VI_SUCCESS;   
    ViChar      buf1[RSSFU_TOKEN_BUFFER_SIZE] = "";

    if ((plp < 1) || (plp > 16))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "PLP"); 

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (!MaxFecBlocksPerIF) viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Maximum FEC Blocks Per IF");   
    
    checkErr (rssfu_GetActualRepCaps (instrSession, RSSFU_ATTR_DVBT2_BICM_FEC_MAX_BLOCKS_PER_IF, "Plp", plp-1, buf1));

    viCheckParm (rssfu_GetAttributeViInt32 (instrSession, buf1, RSSFU_ATTR_DVBT2_BICM_FEC_MAX_BLOCKS_PER_IF, 
               MaxFecBlocksPerIF), 3, "Maximum FEC Blocks Per IF");        
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Configure DVBT2 Framing OFDM Parameters
/* Purpose:  This function configures FRAMING + OFDM parameters.
/*===========================================================================*/
ViStatus _VI_FUNC rssfu_confDVBT2BICMFramingOFDMParameters(
    ViSession   instrSession,
    ViInt32 channelBandwidth,
    ViInt32 fftSize,
    ViInt32 guardInterval,
    ViInt32 pilotPattern
)
{
 ViStatus    error = VI_SUCCESS;
 
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_DVBT2_FRAMING_CHANNEL_BANDWIDTH, channelBandwidth),
                         2, "Channel Bandwidth");
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_DVBT2_FRAMING_FFT_SIZE, fftSize),
                         3, "FFT Size");                          
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_DVBT2_FRAMING_GUARD_INTERVAL, guardInterval),
                         4, "Guard Interval");
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_DVBT2_FRAMING_PILOT_PATTERN, pilotPattern),
                         5, "Pilot Pattern");  
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/*===========================================================================*/
/* Function: Read DVBT2 Framing OFDM Used Bandwidth
/* Purpose:  This function queries the used channel bandwidth.
/*===========================================================================*/
ViStatus _VI_FUNC rssfu_readDVBT2FramingOFDMUsedBandwidth(
    ViSession   instrSession,
    ViReal64*   usedBandwidth
)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    if (!usedBandwidth) viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Used Bandwidth");
                                                               
    viCheckParm (rssfu_GetAttributeViReal64 (instrSession, "", RSSFU_ATTR_DVBT2_FRAMING_USED_BANDWIDTH, 
               usedBandwidth), 2, "Used Bandwidth");
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/*===========================================================================*/
/* Function: Configure DVBT2 Framing OFDM Bandwidth Variation
/* Purpose:  This function configures the bandwidth variation.
/*===========================================================================*/
ViStatus _VI_FUNC rssfu_ConfigureDVBT2FramingOFDMBandwidthVariation(
    ViSession   instrSession,ViInt32 bandwidthVariation)
{
    ViStatus    error = VI_SUCCESS;
 
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_DVBT2_FRAMING_BANDWIDTH_VARIATION, bandwidthVariation),
                         2, "Bandwidth Variation");
    Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Configure DVBT2 Framing OFDM
/* Purpose:  This function configures DVB-T2 Frames parameters.
/*===========================================================================*/
ViStatus _VI_FUNC rssfu_confDVBT2FramingOFDM(
    ViSession   instrSession,
    ViInt32 t2FramesPerSuperFrame,
    ViInt32 dataSymbolsPerT2Frame,
    ViInt32 subslicesPerT2Frame
)
{
 ViStatus    error = VI_SUCCESS;
 
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_DVBT2_FRAMING_T2_FRAMES_PER_SUPER_FRAME, t2FramesPerSuperFrame),
                         2, "T2 Frames Per Super Frame");
    
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_DVBT2_FRAMING_DATA_SYMBOLS_PER_T2_FRAME, dataSymbolsPerT2Frame),
                         3, "Data Symbols Per T2 Frame");
 
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_DVBT2_FRAMING_SUBSLICES_PER_T2_FRAME, subslicesPerT2Frame),
                         4, "Subslices Per T2 Frame");
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}
    
/*===========================================================================*/
/* Function: Read DVBT2 Framing OFDM Data Symbols
/* Purpose:  This function queries the data symbols per T2 frame (L_DATA).
/*===========================================================================*/
ViStatus _VI_FUNC rssfu_readDVBT2FramingOFDMDataSymbols(
    ViSession   instrSession,
    ViInt32*    dataSymbolsPerT2Frame
)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    if (!dataSymbolsPerT2Frame) viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Data Symbols Per T2 Frame");
    
    viCheckParm (rssfu_GetAttributeViInt32 (instrSession, "", RSSFU_ATTR_DVBT2_FRAMING_DATA_SYMBOLS_PER_T2_FRAME, 
               dataSymbolsPerT2Frame), 2, "Data Symbols Per T2 Frame");
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Read DVBT2 Framing OFDM Symbols Per T2 Frame
/* Purpose:  This function queries the OFDM symbols per T2 frame (L_F).
/*===========================================================================*/
ViStatus _VI_FUNC rssfu_readDVBT2FramingOFDMSymbolsPerT2Frame
             (ViSession instrSession, ViInt32* OFDMSymbolsPerT2Frame)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    if (!OFDMSymbolsPerT2Frame) viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "OFDM Symbols Per T2 Frame");
                                                               
    viCheckParm (rssfu_GetAttributeViInt32 (instrSession, "", RSSFU_ATTR_DVBT2_FRAMING_OFDM_SYMBOLS_PER_T2_FRAME, OFDMSymbolsPerT2Frame),
                         2, "OFDM Symbols Per T2 Frame"); 
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Configure DVBT2 Transmission System
/* Purpose:  This function configures transmission system parameters.
/*===========================================================================*/
ViStatus _VI_FUNC rssfu_confDVBT2TransmissionSystem(
    ViSession   instrSession,
    ViInt32 transmissionSystem,
    ViInt32 peakToAvragePowerRatio,
    ViBoolean   futureExtensionFrames,
    ViBoolean   timeFrequencySlicing
)
{
 ViStatus    error = VI_SUCCESS;
 
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_DVBT2_T2_SYSTEM_TRANSMISSION_SYSTEM, transmissionSystem),
                         2, "Transmission System");
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_DVBT2_T2_SYSTEM_PEAK_TO_AVRAGE_POWER_RATIO, peakToAvragePowerRatio),
                         3, "Peak To Avrage Power Ratio");                          
    viCheckParm (rssfu_SetAttributeViBoolean (instrSession, "", RSSFU_ATTR_DVBT2_T2_SYSTEM_FUTURE_EXTENSION_FRAMES, futureExtensionFrames),
                         4, "Future Extension Frames");
    viCheckParm (rssfu_SetAttributeViBoolean (instrSession, "", RSSFU_ATTR_DVBT2_T2_SYSTEM_TIME_FREQUENCY_SLICING, timeFrequencySlicing),
                         5, "Time Frequency Slicing");  
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Configure DVBT2 Transmission System Network
/* Purpose:  This function configures T2 system profile mode.
/*===========================================================================*/
ViStatus _VI_FUNC rssfu_ConfigureDVBT2TransmissionProfileMode
             (ViSession instrSession, ViInt32 profileMode)
{
 ViStatus    error = VI_SUCCESS;
 
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_DVBT2_T2_SYSTEM_PROFILE_MODE, profileMode),
                         2, "Profile Mode");                        
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Configure DVBT2 Transmission Profile Mode
/* Purpose:  This function configures T2 system network mode.
/*===========================================================================*/
ViStatus _VI_FUNC rssfu_ConfigureDVBT2TransmissionSystemNetwork
             (ViSession instrSession, ViInt32 networkMode)
{
 ViStatus    error = VI_SUCCESS;
 
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_DVBT2_T2_SYSTEM_NETWORK_MODE, networkMode),
                         2, "Network Mode");                        
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Configure DVBT2 Transmission System MISO
/* Purpose:  This function configures T2 system MISO group.
/*===========================================================================*/
ViStatus _VI_FUNC rssfu_ConfigureDVBT2TransmissionSystemMISO
             (ViSession instrSession, ViInt32 MISO)
{
 ViStatus    error = VI_SUCCESS;
 
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_DVBT2_T2_SYSTEM_MISO_GROUP, MISO),
                         2, "MISO");                        
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Configure DVBT2 Transmission MISO Group Mode
/* Purpose:  This function configures the MISO group operation mode.
/*===========================================================================*/
ViStatus _VI_FUNC rssfu_ConfigureDVBT2TransmissionMISOGroupMode (ViSession instrSession,
                                                     ViInt32 groupMode)
{
 ViStatus    error = VI_SUCCESS;
 
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_DVBT2_T2_SYSTEM_MISO_GROUP_MODE, groupMode),
                         2, "Group Mode");
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Configure DVBT2 Transmission Future Extension Frames
/* Purpose:  This function configures Future Extension Frames parameters.
/*===========================================================================*/
ViStatus _VI_FUNC rssfu_ConfigureDVBT2TransmissionFutureExtensionFrames
             (ViSession instrSession, ViInt32 FEFLength, ViInt32 FEFType,
              ViInt32 FEFInterval)
{
 ViStatus    error = VI_SUCCESS;
 
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_DVBT2_T2_SYSTEM_FUTURE_EXTENSION_FRAMES_LENGTH, FEFLength),
                         2, "FEF Length");
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_DVBT2_T2_SYSTEM_FUTURE_EXTENSION_FRAMES_TYPE, FEFType),
                         3, "FEF Type");                          
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_DVBT2_T2_SYSTEM_FUTURE_EXTENSION_FRAMES_INTERVAL, FEFInterval),
                         4, "FEF Interval");                          
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}
/*===========================================================================*/
/* Function: Configure DVBT2 Transmission Future Extension Payload
/* Purpose:  This function configures Future Extension Frames payload.
/*===========================================================================*/
ViStatus _VI_FUNC rssfu_ConfigureDVBT2TransmissionFutureExtensionPayload
             (ViSession instrSession, ViInt32 FEFPayload)
{
 ViStatus    error = VI_SUCCESS;
 
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_DVBT2_T2_SYSTEM_FUTURE_EXTENSION_FRAMES_PAYLOAD, FEFPayload),
                         2, "FEF Payload");
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/*===========================================================================*/
/* Function: Configure DVBT2 Transmission L1 Parameters
/* Purpose:  This function configures transmission system L1 parameters.
/*===========================================================================*/
ViStatus _VI_FUNC rssfu_confDVBT2TransmissionL1Param(
    ViSession   instrSession,
    ViInt32 l1PostModulation,
    ViBoolean   l1Repetition,
    ViBoolean   l1PostExtension,
    ViInt32 numAuxStreams,
    ViBoolean   l1RFSignalling
)
{
 ViStatus    error = VI_SUCCESS;
 
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_DVBT2_T2_SYSTEM_L1_POST_MODULATION, l1PostModulation),
                         2, "L1 Post Modulation");
    viCheckParm (rssfu_SetAttributeViBoolean (instrSession, "", RSSFU_ATTR_DVBT2_T2_SYSTEM_L1_REPETITION, l1Repetition),
                         3, "L1 Repetition");
    viCheckParm (rssfu_SetAttributeViBoolean (instrSession, "", RSSFU_ATTR_DVBT2_T2_SYSTEM_L1_POST_EXTENSION, l1PostExtension),
                         4, "L1 Post Extension");  
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_DVBT2_T2_SYSTEM_NUM_AUX_STREAMS, numAuxStreams),
                         5, "Num Aux Streams");                          
    viCheckParm (rssfu_SetAttributeViBoolean (instrSession, "", RSSFU_ATTR_DVBT2_T2_SYSTEM_L1_RF_SIGNALLING, l1RFSignalling),
                         6, "L1 RF Signalling");  
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Configure DVBT2 Transmission L Parameters L1T2
/* Purpose:  This function configures L1 post scrambling according to T2 and
/*           T2 base lite signaling version 1.3.1 ("L1 T2 VERSION").
/*===========================================================================*/
ViStatus _VI_FUNC rssfu_confDVBT2TransmissionLParametersL1T2(
    ViSession   instrSession, ViBoolean l1PostScrambled, ViBoolean t2BaseLite)

{
 ViStatus    error = VI_SUCCESS;
 
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rssfu_SetAttributeViBoolean (instrSession, "", RSSFU_ATTR_DVBT2_T2_SYSTEM_L1_POST_SCRAMBLED, l1PostScrambled),
                         2, "L1 Post Scrambled");
    viCheckParm (rssfu_SetAttributeViBoolean (instrSession, "", RSSFU_ATTR_DVBT2_T2_SYSTEM_T2_BASE_LITE, t2BaseLite),
                         3, "T2 Base Lite");  
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}
     
/*===========================================================================*/
/* Function: Configure DVBT2 Transmission L1 T2 Version
/* Purpose:  This function sets the T2 specification version used for
/*           transmission.
/*===========================================================================*/
ViStatus _VI_FUNC rssfu_ConfigureDVBT2TransmissionL1T2Version
             (ViSession instrSession, ViInt32 l1T2Version)

{
 ViStatus    error = VI_SUCCESS;
 
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_DVBT2_T2_SYSTEM_L1_T2_VERSION, l1T2Version),
                         2, "L1 T2 Version");

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Configure DVBT2 Transmission System Identification
/* Purpose:  This function configures transmission system identification 
/*           parameters.
/*===========================================================================*/
ViStatus _VI_FUNC rssfu_confDVBT2TransmissionSystemIdentification(
    ViSession   instrSession,
    ViInt32 cellID,
    ViInt32 networkID,
    ViInt32 systemID,
    ViBoolean   transmitterIDState
)
{
 ViStatus    error = VI_SUCCESS;
 
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_DVBT2_T2_SYSTEM_CELL_ID, cellID),
                         2, "Cell ID");
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_DVBT2_T2_SYSTEM_NETWORK_ID, networkID),
                         3, "Network ID");
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_DVBT2_T2_SYSTEM_ID, systemID),
                         4, "System ID");                          
  /*  viCheckParm (rssfu_SetAttributeViBoolean (instrSession, "", RSSFU_ATTR_DVBT2_T2_SYSTEM_TRANSMITTER_ID_STATE, transmitterIDState),
                         5, "Transmitter ID State");*/  
    transmitterIDState;
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Read DVBT2 T2 System Tx ID Availability
 * Purpose:  This function queries the availability of transmitter
 *           identification signals within the current geographic cell.
/============================================================================*/
ViStatus _VI_FUNC rssfu_ReadDVBT2T2SystemTxIDAvailability
             (ViSession instrSession, ViInt32* txIDAvailability)

{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    if (!txIDAvailability) viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Tx ID Availability");
    viCheckParm (rssfu_GetAttributeViInt32 (instrSession, "", RSSFU_ATTR_DVBT2_T2_SYSTEM_TX_ID_AVAILABILITY,
                                            txIDAvailability), 2, "Tx ID Availability");
   Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Read DVBT2 T2 System L1 Signalling Frequency
 * Purpose:  This function queries the frequency signaled via L1 signaling.
/============================================================================*/
ViStatus _VI_FUNC rssfu_ReadDVBT2T2SystemL1SignallingFrequency
             (ViSession instrSession, ViReal64* frequency)

{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    if (!frequency) viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Frequency");
    viCheckParm (rssfu_GetAttributeViReal64 (instrSession, "", RSSFU_ATTR_DVBT2_T2_SYSTEM_L1_SIGNALLING_FREQUENCY,
                                            frequency), 2, "Frequency");
   Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Configure DVBT2 SFN Delay Parametrs
/* Purpose:  This function configures SFN delay parameters.
/*===========================================================================*/
ViStatus _VI_FUNC rssfu_ConfigureDVBT2SFNDelayParameters (ViSession instrSession,
                                                          ViInt32 delayParameter,
                                                          ViReal64 timeValue)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    if ((delayParameter < RSSFU_VAL_DVBT2_SFN_MAX_DEVIATION_TIME) || (delayParameter > RSSFU_VAL_DVBT2_SFN_STATIC_DELAY))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Value");
        
     switch (delayParameter) {
         case RSSFU_VAL_DVBT2_SFN_MAX_DEVIATION_TIME:
            viCheckParm (rssfu_SetAttributeViReal64 (instrSession, "", RSSFU_ATTR_DVBT2_SFN_MAX_DEVIATION_TIME,
                                                     timeValue), 3, "Time Value");
        break;
         case RSSFU_VAL_DVBT2_SFN_STATIC_DELAY:
            viCheckParm (rssfu_SetAttributeViReal64 (instrSession, "", RSSFU_ATTR_DVBT2_SFN_STATIC_DELAY,
                                                     timeValue), 3, "Time Value");
        break;
  
     }           
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}
/*===========================================================================*/
/* Function: Configure DVBT2 SFN Mute P1 Symbol
/* Purpose:  This function mutes the P1 symbol of the first T2 frame in a
/*===========================================================================*/
ViStatus _VI_FUNC rssfu_ConfigureDVBT2SFNMuteP1Symbol (ViSession instrSession,
                                                       ViBoolean muteP1SymbolOfSuperFrame)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rssfu_SetAttributeViBoolean (instrSession, "", RSSFU_ATTR_DVBT2_SFN_MUTE_FIRST_P1_OF_SUPER_FRAME,
        muteP1SymbolOfSuperFrame), 2, "Mute P1 Symbol Of Super Frame");
                         
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Read DVBT2 SFN Delay Parameters
/* Purpose:  This function queries selected delay value.
/*===========================================================================*/
ViStatus _VI_FUNC rssfu_ReadDVBT2SFNDelayParameters (ViSession instrSession,
                                                     ViInt32 delayParameter,
                                                     ViReal64 *timeValue)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    if ((delayParameter < RSSFU_VAL_DVBT2_SFN_MAX_DEVIATION_TIME) || (delayParameter > RSSFU_VAL_DVBT2_SFN_PROCESS_DELAY))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Value");
        
    switch (delayParameter) {
         case RSSFU_VAL_DVBT2_SFN_MAX_DEVIATION_TIME:
            viCheckParm (rssfu_GetAttributeViReal64 (instrSession, "", RSSFU_ATTR_DVBT2_SFN_MAX_DEVIATION_TIME,
                                                     timeValue), 3, "Time Value");
        break;
         case RSSFU_VAL_DVBT2_SFN_STATIC_DELAY:
            viCheckParm (rssfu_GetAttributeViReal64 (instrSession, "", RSSFU_ATTR_DVBT2_SFN_STATIC_DELAY,
                                                     timeValue), 3, "Time Value");
        break;
        case RSSFU_VAL_DVBT2_SFN_PROCESS_DELAY:
            viCheckParm (rssfu_GetAttributeViReal64 (instrSession, "", RSSFU_ATTR_DVBT2_SFN_PROCESS_DELAY,
                                                     timeValue), 3, "Time Value");
        break;
        case RSSFU_VAL_DVBT2_SFN_DYNAMIC_DELAY:
            viCheckParm (rssfu_GetAttributeViReal64 (instrSession, "", RSSFU_ATTR_DVBT2_SFN_DYNAMIC_DELAY,
                                                     timeValue), 3, "Time Value");
        break;
        case RSSFU_VAL_DVBT2_SFN_TOTAL_DELAY:
            viCheckParm (rssfu_GetAttributeViReal64 (instrSession, "", RSSFU_ATTR_DVBT2_SFN_TOTAL_DELAY,
                                                     timeValue), 3, "Time Value");
        break;
     }           
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}
    
/*===========================================================================*/
/* Function: Configure DVBT2 SFN Timestamp Offset
/* Purpose:  This function sets the UTC/GPS leap seconds offset.
/*===========================================================================*/
ViStatus _VI_FUNC rssfu_ConfigureDVBT2SFNTimestampOffset (ViSession instrSession,
                                                       ViInt32 timestampOffset)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_DVBT2_SFN_TIMESTAMP_MODE_OFFSET,
        timestampOffset), 2, "Timestamp Offset");
                         
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Configure DVBT2 SFN Update Timestamp
 * Purpose:  This function updates the UTC time and date reference.
/============================================================================*/
ViStatus _VI_FUNC rssfu_ConfigureDVBT2SFNUpdateTimestamp (ViSession instrSession)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));    

    viCheckErr (rssfu_SetAttributeViString (instrSession, "", RSSFU_ATTR_DVBT2_SFN_UPDATE_TIMESTAMP, "")); 

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Read DVBT2 SFN Timestamp Mode
 * Purpose:  This function queries the type of the currently received T2-MI
 *           timestamps.
/============================================================================*/
ViStatus _VI_FUNC rssfu_ReadDVBT2SFNTimestampMode
             (ViSession instrSession, ViInt32* timestampMode)

{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    if (!timestampMode) viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Frequency");
    viCheckParm (rssfu_GetAttributeViInt32 (instrSession, "", RSSFU_ATTR_DVBT2_SFN_TIMESTAMP_MODE,
                                            timestampMode), 2, "Timestamp Mode");
   Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Read DVBT2 SFN Timestamp Parameters
 * Purpose:  This function queries the currently received T2-MI timestamps
 *           parameters.
/============================================================================*/
ViStatus _VI_FUNC rssfu_ReadDVBT2SFNTimestampParameters (ViSession instrSession,
                                                         ViInt32* UTCDate,
                                                         ViInt32* UTCTime,
                                                         ViInt32* secondsSince2000)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    if (!UTCDate) viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "UTC Date");
    if (!UTCTime) viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "UTC Time"); 
    if (!secondsSince2000) viCheckParm (RS_ERROR_INVALID_PARAMETER, 4, "Seconds Since 2000"); 
            
    viCheckParm (rssfu_GetAttributeViInt32 (instrSession, "", RSSFU_ATTR_DVBT2_SFN_TIMESTAMP_MODE_UTC_DATE,
                                            UTCDate), 2, "UTC Date");
    viCheckParm (rssfu_GetAttributeViInt32 (instrSession, "", RSSFU_ATTR_DVBT2_SFN_TIMESTAMP_MODE_UTC_TIME,
                                            UTCTime), 3, "UTC Time");
    viCheckParm (rssfu_GetAttributeViInt32 (instrSession, "", RSSFU_ATTR_DVBT2_SFN_TIMESTAMP_MODE_SECONDS_SINCE_2000,
                                            secondsSince2000), 4, "Seconds Since 2000");
  
   Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}
/*===========================================================================*/
/* Function: Configure DVBT2 Settings
/* Purpose:  This function configures DVB-T2 coding settings.
/*===========================================================================*/
ViStatus _VI_FUNC rssfu_confDVBT2Settings(
    ViSession   instrSession,
    ViInt32 testTSPacket,
    ViInt32 pidTestPacket,
    ViInt32 pid,
    ViInt32 payloadTestTSPacket,
    ViInt32 prbs
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      model [RSSFU_IO_BUFFER_SIZE] = "";  

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckErr (rssfu_GetAttributeViString (instrSession, "",
                                               RS_ATTR_INSTRUMENT_MODEL,
                                               RSSFU_IO_BUFFER_SIZE, model));
    
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_DVBT2_SETTINGS_TEST_TS_PACKET, testTSPacket),
                         2, "Test TS Packet");
    if (strstr (model, "SFC") == NULL) {
        viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_DVBT2_SETTINGS_PID_TEST_PACKET, pidTestPacket),
                             3, "PID Test Packet");
        if (pidTestPacket == RSSFU_IQC_PIDT_VAR)
        {
            viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_DVBT2_SETTINGS_PID, pid),
                             4, "PID");
        }
    }
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_DVBT2_SETTINGS_PAYLOAD_TEST_TS_PACKET, payloadTestTSPacket),
                         5, "Payload Test TS Packet"); 
  
    switch (payloadTestTSPacket){
        case  RSSFU_IQC_PRBS:
            viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_DVBT2_SETTINGS_PRBS, prbs),
                         6, "PRBS");    
    }   
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
} 


/*****************************************************************************
 *------------------------------ DVB-C2 -------------------------------------*
 *****************************************************************************/

/*===========================================================================*/
/* Function: Configure DVBC2 Input Signal
/* Purpose:  This function configures DVB-C2 input signal parameters.
/*===========================================================================*/
ViStatus _VI_FUNC rssfu_configureDVBC2InputSignal (ViSession instrSession, ViInt32 plp,
                                           ViInt32 inputFormat, ViInt32 source,
                                           ViInt32 input)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buf1[RSSFU_TOKEN_BUFFER_SIZE] = "";
    ViChar      model [RSSFU_IO_BUFFER_SIZE] = "";  
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));        

    viCheckErr (rssfu_GetAttributeViString (instrSession, "",
                                               RS_ATTR_INSTRUMENT_MODEL,
                                               RSSFU_IO_BUFFER_SIZE, model));
    if ((plp < 1) || (plp > 4))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "PLP"); 

    checkErr (rssfu_GetActualRepCaps (instrSession, RSSFU_ATTR_DVBC2_INPUT_SIGNAL_INPUT_FORMAT, "Plp", plp-1, buf1));  
    
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, buf1, RSSFU_ATTR_DVBC2_INPUT_SIGNAL_INPUT_FORMAT, inputFormat),
                         3, "Input Format");
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, buf1, RSSFU_ATTR_DVBC2_INPUT_SIGNAL_SOURCE, source),
                         4, "Source");
    if (strstr (model, "BTC") == NULL) {    
        viCheckParm (rssfu_SetAttributeViInt32 (instrSession, buf1, RSSFU_ATTR_DVBC2_INPUT_SIGNAL_INPUT, input),
                         5, "Input");
    }
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}   

/*===========================================================================*/
/* Function: Configure DVBC2 Number Of PLP
/* Purpose:  This function configures DVB-T2 physical layer pipe (PLP)
/*           number.
/*===========================================================================*/
ViStatus _VI_FUNC rssfu_ConfigureDVBC2NumberOfPLP (ViSession instrSession,
                                                   ViInt32 numberOfPLP)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));        

    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_DVBC2_INPUT_SIGNAL_NUMBER_OF_PLP, numberOfPLP),
                         2, "Number Of PLP");
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
} 

/*===========================================================================*/
/* Function: Read DVBT2 Values
/* Purpose:  This function reads selected coding value.
/*===========================================================================*/
ViStatus _VI_FUNC rssfu_ReadDVBC2Values(
    ViSession   instrSession,
    ViInt32 plp,
    ViInt32 value,
    ViReal64* result
)
{
 ViStatus    error = VI_SUCCESS;
 ViChar      buf1[RSSFU_TOKEN_BUFFER_SIZE] = "";

    if ((plp < 1) || (plp > 4))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "PLP"); 

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    checkErr (rssfu_GetActualRepCaps (instrSession, RSSFU_ATTR_READ_DVBC2_INPUT_SIGNAL_MAX_DR, "Plp", plp-1, buf1));

    switch (value){
        case RSSFU_VAL_MAX_USEFUL_DRATE:
            viCheckParm (rssfu_GetAttributeViReal64 (instrSession, buf1, RSSFU_ATTR_READ_DVBC2_INPUT_SIGNAL_MAX_DR, 
                result), 4, "Result");
        break;
        case RSSFU_VAL_MEAS_DRATE:
            viCheckParm (rssfu_GetAttributeViReal64 (instrSession, buf1, RSSFU_ATTR_READ_DVBC2_INPUT_SIGNAL_MEAS_DR, 
                result), 4, "Result");
        break;
        case RSSFU_VAL_USEFUL_DRATE:
            viCheckParm (rssfu_GetAttributeViReal64 (instrSession, buf1, RSSFU_ATTR_READ_DVBC2_INPUT_SIGNAL_USEFUL_DR, 
                result), 4, "Result");
        break;
        
        
        default:
            viCheckParm( RS_ERROR_INVALID_PARAMETER, 3, "Value"); 
    }

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Configure DVBC2 Test Signal
/* Purpose:  This function configures DVB-C2 Test Signal.
/*===========================================================================*/
ViStatus _VI_FUNC rssfu_ConfigureDVB2CTestSignal(
    ViSession   instrSession,
    ViInt32 plp,
    ViInt32 testSignal
)
{
 ViStatus    error = VI_SUCCESS;
 ViChar      buf1[RSSFU_TOKEN_BUFFER_SIZE] = "";

    if ((plp < 1) || (plp > 4))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "PLP"); 

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    checkErr (rssfu_GetActualRepCaps (instrSession, RSSFU_ATTR_DVBC2_INPUT_SIGNAL_TEST_SIGNAL, "Plp", plp-1, buf1));

    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, buf1, RSSFU_ATTR_DVBC2_INPUT_SIGNAL_TEST_SIGNAL, testSignal),
                         3, "Test Signal");                        
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;

}

/*===========================================================================*/
/* Function: Configure DVBC2 Mode Stream Adaptation
/* Purpose:  This function configures DVB-C2 Mode Stream Adaptation
/*           parameters.
/*===========================================================================*/
ViStatus _VI_FUNC rssfu_ConfigureDVBC2ModeStreamAdaptation
             (ViSession instrSession, ViInt32 plp, ViInt32 PLPID,
              ViInt32 BBMode, ViInt32 ISSY,
              ViBoolean nullPacketDel)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buf1[RSSFU_TOKEN_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));        

    if ((plp < 1) || (plp > 4))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "PLP"); 

    checkErr (rssfu_GetActualRepCaps (instrSession, RSSFU_ATTR_DVBC2_MODE_STREAM_ADAPT_PLP_ID, "Plp", plp-1, buf1));  
    
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, buf1, RSSFU_ATTR_DVBC2_MODE_STREAM_ADAPT_PLP_ID, PLPID),
                         3, "PLP ID");
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, buf1, RSSFU_ATTR_DVBC2_MODE_STREAM_ADAPT_BB_MODE, BBMode),
                         4, "BB Mode");
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, buf1, RSSFU_ATTR_DVBC2_MODE_STREAM_ADAPT_ISSY, ISSY),
                         5, "ISSY");
    viCheckParm (rssfu_SetAttributeViBoolean (instrSession, buf1, RSSFU_ATTR_DVBC2_MODE_STREAM_ADAPT_NULL_PACKET_DEL, nullPacketDel),
                         6, "Null Packet Del"); 
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
} 


/*===========================================================================*/
/* Function: Read DVBC2 Mode Stream Adaptation PLP Type
/* Purpose:  This function queries the PLP type.
/*===========================================================================*/
ViStatus _VI_FUNC rssfu_ReadDVBC2ModeStreamAdaptationPLPType(
    ViSession   instrSession,
    ViInt32 plp,
    ViInt32*    plpType
)
{
 ViStatus    error = VI_SUCCESS;
 ViChar      buf1[RSSFU_TOKEN_BUFFER_SIZE] = "";

    if ((plp < 1) || (plp > 4))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "PLP"); 

    checkErr( Rs_LockSession (instrSession, VI_NULL));
     if (!plpType) viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "PLP Type");   
     
    checkErr (rssfu_GetActualRepCaps (instrSession, RSSFU_ATTR_DVBC2_MODE_STREAM_ADAPT_PLP_TYPE, "Plp", plp-1, buf1));

    viCheckParm (rssfu_GetAttributeViInt32 (instrSession, buf1, RSSFU_ATTR_DVBC2_MODE_STREAM_ADAPT_PLP_TYPE, plpType),
                         3, "PLP Type");                        
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;

}

/*===========================================================================*/
/* Function: Configure DVBC2 BICM PLP Mapping Parameters
/* Purpose:  This function configures DVB-C2 Bit Interleaved Coding and PLP
/*           Mapping parameters.
/*===========================================================================*/
ViStatus _VI_FUNC rssfu_ConfigureDVBC2BICMPLPMappingParameters
             (ViSession instrSession, ViInt32 plp, ViInt32 FECFrame,
              ViInt32 codeRate, ViInt32 constellation, ViInt32 mapToDSlice)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buf1[RSSFU_TOKEN_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));        

    if ((plp < 1) || (plp > 4))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "PLP"); 

    checkErr (rssfu_GetActualRepCaps (instrSession, RSSFU_ATTR_DVBC2_BICM_FEC_FRAME, "Plp", plp-1, buf1));  
    
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, buf1, RSSFU_ATTR_DVBC2_BICM_FEC_FRAME, FECFrame),
                         3, "FEC Frame");
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, buf1, RSSFU_ATTR_DVBC2_BICM_CODE_RATE, codeRate),
                         4, "Code Rate");
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, buf1, RSSFU_ATTR_DVBC2_BICM_CONSTELL, constellation),
                         5, "Constellation");
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, buf1, RSSFU_ATTR_DVBC2_BICM_MAP_TO_DSLICE, mapToDSlice),
                         6, "Map To DSlice");
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/*===========================================================================*/
/* Function: Read DVBC2 BICM XFEC Frame Length
/* Purpose:  This function queries the length of an XFECFrame expressed as
/*           the number of OFDM cells.
/*===========================================================================*/
ViStatus _VI_FUNC rssfu_ReadDVBC2BICMXFECFrameLength (ViSession instrSession,
                                                      ViInt32 plp,
                                                      ViInt32* XFECFrameLength)

{
 ViStatus    error = VI_SUCCESS;
 ViChar      buf1[RSSFU_TOKEN_BUFFER_SIZE] = "";

    if ((plp < 1) || (plp > 4))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "PLP"); 

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    checkErr (rssfu_GetActualRepCaps (instrSession, RSSFU_ATTR_DVBC2_BICM_XFEC_FRAME_LENGTH, "Plp", plp-1, buf1));    
    
    if (!XFECFrameLength) viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "XFEC Frame Length");
    
    viCheckParm (rssfu_GetAttributeViInt32 (instrSession, buf1, RSSFU_ATTR_DVBC2_BICM_XFEC_FRAME_LENGTH, 
               XFECFrameLength), 3, "XFEC Frame Length");

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Read DVBC2 Data Slice Min Max
/* Purpose:  This function returns the minimum and maximum carrier index for
/*           the C2 signal.
/*===========================================================================*/
ViStatus _VI_FUNC rssfu_ReadDVBC2DataSliceMinMax (ViSession instrSession,
                                                  ViInt32* min_REL_DxTu,
                                                  ViInt32* max_REL_DxTu,
                                                  ViInt32* min_ABS_DxTu,
                                                  ViInt32* max_ABS_DxTu)
{
 ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (!min_REL_DxTu) viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Min REL Dx/Tu");
    if (!max_REL_DxTu) viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Max REL Dx/Tu"); 
    if (!min_ABS_DxTu) viCheckParm (RS_ERROR_INVALID_PARAMETER, 4, "Min ABS Dx/Tu"); 
    if (!max_ABS_DxTu) viCheckParm (RS_ERROR_INVALID_PARAMETER, 5, "Max ABS Dx/Tu"); 
    
    viCheckParm (rssfu_GetAttributeViInt32 (instrSession, "MIN", RSSFU_ATTR_DVBC2_DATA_SLICE_MIN_MAX_REL, 
               min_REL_DxTu), 2, "Min REL Dx/Tu");
    viCheckParm (rssfu_GetAttributeViInt32 (instrSession, "MAX", RSSFU_ATTR_DVBC2_DATA_SLICE_MIN_MAX_REL, 
               max_REL_DxTu), 3, "Max REL Dx/Tu");
    viCheckParm (rssfu_GetAttributeViInt32 (instrSession, "MIN", RSSFU_ATTR_DVBC2_DATA_SLICE_MIN_MAX_ABS, 
               min_ABS_DxTu), 4, "Min ABS Dx/Tu");
    viCheckParm (rssfu_GetAttributeViInt32 (instrSession, "MAX", RSSFU_ATTR_DVBC2_DATA_SLICE_MIN_MAX_ABS, 
               max_ABS_DxTu), 5, "Max ABS Dx/Tu");

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Configure DVBC2 Number Of Data Slices
/* Purpose:  This function configures number of data slices.
/*===========================================================================*/
ViStatus _VI_FUNC rssfu_ConfigureDVBC2NumberOfDataSlices (ViSession instrSession,
                                                          ViInt32 numberOfDataSlices)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));        

    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_DVBC2_DATA_SLICE_NUMBER_OF_DATA_SLICES,
        numberOfDataSlices), 2, "Number Of Data Slices");
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
} 

/*===========================================================================*/
/* Function: Configure DVBC2 Data Slice Parameters
/* Purpose:  This function configures DVB-C2 Data Slice parameters.
/*===========================================================================*/
ViStatus _VI_FUNC rssfu_ConfigureDVBC2DataSliceParameters
             (ViSession instrSession, ViInt32 DSlice, ViInt32 DSliceID_HEX,
              ViInt32 DSliceTIDepth, ViInt32 DSliceType, ViInt32 FECHeaderType,
              ViInt32 numXFECFrames)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buf1[RSSFU_TOKEN_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));        

    if ((DSlice < 0) || (DSlice > 3))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "DSlice"); 

    checkErr (rssfu_GetActualRepCaps (instrSession, RSSFU_ATTR_DVBC2_DATA_SLICE_DSLICE_ID, "DSlice", DSlice, buf1));  
    
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, buf1, RSSFU_ATTR_DVBC2_DATA_SLICE_DSLICE_ID, DSliceID_HEX),
                         3, "DS liceID HEX");
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, buf1, RSSFU_ATTR_DVBC2_DATA_SLICE_DSLICE_TI_DEPTH, DSliceTIDepth),
                         4, "DSlice TI Depth");
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, buf1, RSSFU_ATTR_DVBC2_DATA_SLICE_DSLICE_TYPE, DSliceType),
                         5, "DSlice Type");
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, buf1, RSSFU_ATTR_DVBC2_DATA_SLICE_FEC_HEADER_TYPE, FECHeaderType),
                         6, "FEC Header Type");
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, buf1, RSSFU_ATTR_DVBC2_DATA_SLICE_NUM_XFEC_FRAMES, numXFECFrames),
                         7, "Num XFEC Frames");
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Read DVBC2 Data Slice Num PLP
/* Purpose:  This function returns the number of PLPs assigned to this data
/*           slice. PLPs are assigned to data slices under MAP TO DSLICE.
/*===========================================================================*/
ViStatus _VI_FUNC rssfu_ReadDVBC2DataSliceNumPLP (ViSession instrSession,
                                                  ViInt32 DSlice,
                                                  ViInt32* DSliceNumPLP)
{
 ViStatus    error = VI_SUCCESS;
 ViChar      buf1[RSSFU_TOKEN_BUFFER_SIZE] = "";

    if ((DSlice < 0) || (DSlice > 3))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "DSlice"); 

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    checkErr (rssfu_GetActualRepCaps (instrSession, RSSFU_ATTR_DVBC2_DATA_SLICE_DSLICE_NUM_PLP, "DSlice", DSlice, buf1));    
    
    if (!DSliceNumPLP) viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "DSlice Num PLP");
    
    viCheckParm (rssfu_GetAttributeViInt32 (instrSession, buf1, RSSFU_ATTR_DVBC2_DATA_SLICE_DSLICE_NUM_PLP, 
               DSliceNumPLP), 3, "DSlice Num PLP");

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Configure DVBC2 Current Data Slice Parameters
/* Purpose:  This function configures DVB-C2 Current Data Slice parameters.
/*===========================================================================*/
ViStatus _VI_FUNC rssfu_ConfigureDVBC2CurrentDataSliceParameters
             (ViSession instrSession, ViInt32 DSlice, ViInt32 offsetLeftDxTu,
              ViInt32 tunePos_REL_DxTu, ViInt32 offsetRightDxTu)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buf1[RSSFU_TOKEN_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));        

    if ((DSlice < 0) || (DSlice > 3))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "DSlice"); 

    checkErr (rssfu_GetActualRepCaps (instrSession, RSSFU_ATTR_DVBC2_DATA_SLICE_OFFSET_LEFT, "DSlice", DSlice, buf1));  
    
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, buf1, RSSFU_ATTR_DVBC2_DATA_SLICE_OFFSET_LEFT, offsetLeftDxTu),
                         3, "Offset Left Dx/Tu");
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, buf1, RSSFU_ATTR_DVBC2_DATA_SLICE_TUNE_POS_REL, tunePos_REL_DxTu),
                         4, "Tune Pos REL Dx/Tu");
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, buf1, RSSFU_ATTR_DVBC2_DATA_SLICE_OFFSET_RIGHT, offsetRightDxTu),
                         5, "Offset Right Dx/Tu");

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Read DVBC2 Data Slice Values
/* Purpose:  This function reads selected data slice value.
/*===========================================================================*/
ViStatus _VI_FUNC rssfu_ReadDVBC2DataSliceValues(
    ViSession   instrSession,
    ViInt32 DSlice,
    ViInt32 DSliceValue,
    ViInt32* result
)
{
 ViStatus    error = VI_SUCCESS;
 ViChar      buf1[RSSFU_TOKEN_BUFFER_SIZE] = "";

    if ((DSlice < 0) || (DSlice > 3))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "DSlice");

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    checkErr (rssfu_GetActualRepCaps (instrSession, RSSFU_ATTR_DVBC2_DATA_SLICE_START_REL, "DSlice", DSlice, buf1));

    switch (DSliceValue){
        case RSSFU_VAL_DVBC2_DSLICE_START_REL:
            viCheckParm (rssfu_GetAttributeViInt32 (instrSession, buf1, RSSFU_ATTR_DVBC2_DATA_SLICE_START_REL, 
                result), 4, "Result");
        break;
        case RSSFU_VAL_DVBC2_DSLICE_STOP_REL:
            viCheckParm (rssfu_GetAttributeViInt32 (instrSession, buf1, RSSFU_ATTR_DVBC2_DATA_SLICE_STOP_REL, 
                result), 4, "Result");
        break;
        case RSSFU_VAL_DVBC2_DSLICE_WIDTH:
            viCheckParm (rssfu_GetAttributeViInt32 (instrSession, buf1, RSSFU_ATTR_DVBC2_DATA_SLICE_WIDTH, 
                result), 4, "Result");
        break;
        case RSSFU_VAL_DVBC2_DSLICE_START_ABS:
            viCheckParm (rssfu_GetAttributeViInt32 (instrSession, buf1, RSSFU_ATTR_DVBC2_DATA_SLICE_START_ABS, 
                result), 4, "Result");
        break;
        case RSSFU_VAL_DVBC2_DSLICE_TUNE_POS_ABS:
            viCheckParm (rssfu_GetAttributeViInt32 (instrSession, buf1, RSSFU_ATTR_DVBC2_DATA_SLICE_TUNE_POS_ABS, 
                result), 4, "Result");
        break;
        case RSSFU_VAL_DVBC2_DSLICE_TUNE_POS_REL:
            viCheckParm (rssfu_GetAttributeViInt32 (instrSession, buf1, RSSFU_ATTR_DVBC2_DATA_SLICE_TUNE_POS_REL, 
                result), 4, "Result");
        break;
        case RSSFU_VAL_DVBC2_DSLICE_STOP_ABS:
            viCheckParm (rssfu_GetAttributeViInt32 (instrSession, buf1, RSSFU_ATTR_DVBC2_DATA_SLICE_STOP_ABS, 
                result), 4, "Result");
        break;
        
        default:
            viCheckParm( RS_ERROR_INVALID_PARAMETER, 3, "DSlice Value"); 
    }

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Read DVBC2 Data Slice Frequency Values
/* Purpose:  This function reads selected data slice value.
/*===========================================================================*/
ViStatus _VI_FUNC rssfu_ReadDVBC2DataSliceFrequencyValues(
    ViSession   instrSession,
    ViInt32 DSlice,
    ViInt32 DSliceValue,
    ViReal64* result
)
{
 ViStatus    error = VI_SUCCESS;
 ViChar      buf1[RSSFU_TOKEN_BUFFER_SIZE] = "";

    if ((DSlice < 0) || (DSlice > 3))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "DSlice");

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    checkErr (rssfu_GetActualRepCaps (instrSession, RSSFU_ATTR_DVBC2_DATA_SLICE_START_REL, "DSlice", DSlice, buf1));

    switch (DSliceValue){
        case RSSFU_VAL_DVBC2_DSLICE_TUNE_FREQ:
            viCheckParm (rssfu_GetAttributeViReal64 (instrSession, buf1, RSSFU_ATTR_DVBC2_DATA_SLICE_TUNE_FREQ, 
                result), 4, "Result");
        break;
        case RSSFU_VAL_DVBC2_DSLICE_START_FREQ:
            viCheckParm (rssfu_GetAttributeViReal64 (instrSession, buf1, RSSFU_ATTR_DVBC2_DATA_SLICE_START_FREQ, 
                result), 4, "Result");
        break;
        case RSSFU_VAL_DVBC2_DSLICE_STOP_FREQ:
            viCheckParm (rssfu_GetAttributeViReal64 (instrSession, buf1, RSSFU_ATTR_DVBC2_DATA_SLICE_STOP_FREQ, 
                result), 4, "Result");
        break;
        
        default:
            viCheckParm( RS_ERROR_INVALID_PARAMETER, 3, "DSlice Value"); 
    }

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Configure DVBC2 Number Of Notches
/* Purpose:  This function configures Number of notches.
/*===========================================================================*/
ViStatus _VI_FUNC rssfu_ConfigureDVBC2NumberOfNotches (ViSession instrSession,
                                                  ViInt32 numberOfNotches)
{
 ViStatus    error = VI_SUCCESS;

   
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_DVBC2_NOTCH_NUMBER_OF_NOTCHES, 
               numberOfNotches), 3, "Number Of Notches");

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Read DVBC2 Number Of Notches
/* Purpose:  This function returns Number of notches.
/*===========================================================================*/
ViStatus _VI_FUNC rssfu_ReadDVBC2NumberOfNotches (ViSession instrSession,
                                                  ViInt32* numberOfNotches)
{
 ViStatus    error = VI_SUCCESS;

   
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    if (!numberOfNotches) viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Number Of Notches");
    
    viCheckParm (rssfu_GetAttributeViInt32 (instrSession, "", RSSFU_ATTR_DVBC2_NOTCH_NUMBER_OF_NOTCHES, 
               numberOfNotches), 3, "Number Of Notches");

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Read DVBC2 Notch Type
/* Purpose:  This function queries the Notch type.
/*===========================================================================*/
ViStatus _VI_FUNC rssfu_ReadDVBC2NotchType (ViSession instrSession,
                                            ViInt32 notch, ViInt32* notchType)

{
    ViStatus    error = VI_SUCCESS;
    ViChar      buf1[RSSFU_TOKEN_BUFFER_SIZE] = ""; 
     
    checkErr( Rs_LockSession (instrSession, VI_NULL));        
    
    if ((notch < 0) || (notch > 3))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Notch");
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    if (!notchType) viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Notch Type");
    
    checkErr (rssfu_GetActualRepCaps (instrSession, RSSFU_ATTR_DVBC2_NOTCH_TYPE, "Notch", notch, buf1));

     
    viCheckParm (rssfu_GetAttributeViInt32 (instrSession, buf1, RSSFU_ATTR_DVBC2_NOTCH_TYPE, notchType),
                         3, "Notch Type");
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Configure DVBC2 Notch Values
/* Purpose:  This function selects and sets the relevant notch parameters in
/*           the work view.
/*===========================================================================*/
ViStatus _VI_FUNC rssfu_ConfigureDVBC2NotchValues (ViSession instrSession,
                                                   ViInt32 notch,
                                                   ViInt32 start_REL_DxTu,
                                                   ViInt32 stop_REL_DxTu,
                                                   ViInt32 width_REL_DxTu)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buf1[RSSFU_TOKEN_BUFFER_SIZE] = ""; 
     
    checkErr( Rs_LockSession (instrSession, VI_NULL));        
    
    if ((notch < 0) || (notch > 3))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Notch");

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    checkErr (rssfu_GetActualRepCaps (instrSession, RSSFU_ATTR_DVBC2_NOTCH_START_REL, "Notch", notch, buf1));

     
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, buf1, RSSFU_ATTR_DVBC2_NOTCH_START_REL, start_REL_DxTu),
                         3, "Start REL [Dx/Tu]");
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, buf1, RSSFU_ATTR_DVBC2_NOTCH_STOP_REL, stop_REL_DxTu),
                         4, "Stop REL [Dx/Tu]");
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, buf1, RSSFU_ATTR_DVBC2_NOTCH_WIDTH, width_REL_DxTu),
                         5, "Width REL [Dx/Tu]");       
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}
/*===========================================================================*/
/* Function: Read DVBC2 Notch Values
/* Purpose:  This function reads selected data slice value.
/*===========================================================================*/
ViStatus _VI_FUNC rssfu_ReadDVBC2NotchValues(
    ViSession   instrSession,
    ViInt32 notch,
    ViInt32 notchParameter,
    ViInt32* result
)
{
 ViStatus    error = VI_SUCCESS;
 ViChar      buf1[RSSFU_TOKEN_BUFFER_SIZE] = "";

    if ((notch < 0) || (notch > 3))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Notch");

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    checkErr (rssfu_GetActualRepCaps (instrSession, RSSFU_ATTR_DVBC2_NOTCH_START_REL, "Notch", notch, buf1));

    switch (notchParameter){
        case RSSFU_VAL_DVBC2_NOTCH_START_REL:
            viCheckParm (rssfu_GetAttributeViInt32 (instrSession, buf1, RSSFU_ATTR_DVBC2_NOTCH_START_REL, 
                result), 4, "Result");
        break;
        case RSSFU_VAL_DVBC2_NOTCH_STOP_REL:
            viCheckParm (rssfu_GetAttributeViInt32 (instrSession, buf1, RSSFU_ATTR_DVBC2_NOTCH_STOP_REL, 
                result), 4, "Result");
        break;
        case RSSFU_VAL_DVBC2_NOTCH_WIDTH:
            viCheckParm (rssfu_GetAttributeViInt32 (instrSession, buf1, RSSFU_ATTR_DVBC2_NOTCH_WIDTH, 
                result), 4, "Result");
        break;
        case RSSFU_VAL_DVBC2_NOTCH_START_ABS:
            viCheckParm (rssfu_GetAttributeViInt32 (instrSession, buf1, RSSFU_ATTR_DVBC2_NOTCH_START_ABS, 
                result), 4, "Result");
        break;
        case RSSFU_VAL_DVBC2_NOTCH_STOP_ABS:
            viCheckParm (rssfu_GetAttributeViInt32 (instrSession, buf1, RSSFU_ATTR_DVBC2_NOTCH_STOP_ABS, 
                result), 4, "Result");
        break;
        
        default:
            viCheckParm( RS_ERROR_INVALID_PARAMETER, 3, "Notch Parameter"); 

    }

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}   

/*===========================================================================*/
/* Function: Read DVBC2 Notch Frequency Values
/* Purpose:  This function reads selected data slice value.
/*===========================================================================*/
ViStatus _VI_FUNC rssfu_ReadDVBC2NotchFrequencyValues(
    ViSession   instrSession,
    ViInt32 notch,
    ViInt32 notchParameter,
    ViReal64* result
)
{
 ViStatus    error = VI_SUCCESS;
 ViChar      buf1[RSSFU_TOKEN_BUFFER_SIZE] = "";

    if ((notch < 0) || (notch > 3))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Notch");

   checkErr( Rs_LockSession (instrSession, VI_NULL));
   
   checkErr (rssfu_GetActualRepCaps (instrSession, RSSFU_ATTR_DVBC2_NOTCH_START_FREQ, "Notch", notch, buf1));           

    switch (notchParameter){
        case RSSFU_VAL_DVBC2_NOTCH_START_FREQ:
            viCheckParm (rssfu_GetAttributeViReal64 (instrSession, buf1, RSSFU_ATTR_DVBC2_NOTCH_START_FREQ, 
                result), 4, "Result");
        break;
        case RSSFU_VAL_DVBC2_NOTCH_STOP_FREQ:
            viCheckParm (rssfu_GetAttributeViReal64 (instrSession, buf1, RSSFU_ATTR_DVBC2_NOTCH_STOP_FREQ, 
                result), 4, "Result");
        break;

        default:
            viCheckParm( RS_ERROR_INVALID_PARAMETER, 3, "Notch Parameter"); 

    }

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/*===========================================================================*/
/* Function: Configure DVBC2 Framing OFDM Parameters
/* Purpose:  This function configures FRAMING + OFDM parameters.
/*===========================================================================*/
ViStatus _VI_FUNC rssfu_ConfigureDVBC2FramingOFDMParameters
             (ViSession instrSession, ViInt32 channelBandwidth,
              ViInt32 numBundledCH, ViInt32 guardInterval)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));        

     
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_DVBC2_FRAMING_CHANNEL_BANDWIDTH, channelBandwidth),
                         2, "Channel Bandwidth");
   /* viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_DVBC2_FRAMING_NUM_BUNDLED_CH, numBundledCH),
                         3, "Num Bundled CH");
    */
    numBundledCH;
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_DVBC2_FRAMING_GUARD_INTERVAL, guardInterval),
                         4, "Guard Interval");
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Configure DVBC2 C2 Bandwidth
/* Purpose:  This function configures the total bandwidth of the C2 signal.
/*===========================================================================*/
ViStatus _VI_FUNC rssfu_ConfigureDVBC2FramingOFDMC2Bandwidth
             (ViSession instrSession, ViInt32 bandwidth)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));        

     
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_DVBC2_FRAMING_C2_BANDWIDTH, bandwidth),
                         2, "C2 Bandwidth");
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Read DVBC2 Framing OFDM Used Bandwidth
/* Purpose:  This function queries the used channel bandwidth.
/*===========================================================================*/
ViStatus _VI_FUNC rssfu_ReadDVBC2FramingOFDMUsedBandwidth
             (ViSession instrSession, ViReal64* usedBandwidth
)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    if (!usedBandwidth) viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Used Bandwidth");
                                                               
    viCheckParm (rssfu_GetAttributeViReal64 (instrSession, "", RSSFU_ATTR_DVBC2_FRAMING_USED_BANDWIDTH, 
               usedBandwidth), 2, "Used Bandwidth");
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Configure DVBC2 Framing OFDM Baseband Filter
/* Purpose:  This function configures the type of baseband filtering.
/*===========================================================================*/
ViStatus _VI_FUNC rssfu_ConfigureDVBC2FramingOFDMBasebandFilter (ViSession instrSession,
                                                  ViInt32 basebandFilter)
{
 ViStatus    error = VI_SUCCESS;

   
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_DVBC2_FRAMING_BASEBAND_FILTER,
                            basebandFilter), 2, "Baseband Filter");

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Configure DVBC2 Framing OFDM Adjust RF Frequency
/* Purpose:  This function sets the RF frequency to the calculated center
 *           frequency value.
/*===========================================================================*/
ViStatus _VI_FUNC rssfu_ConfigureDVBC2FramingOFDMAdjustRFFrequency (ViSession instrSession)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));    

    viCheckErr (rssfu_SetAttributeViString (instrSession, "", RSSFU_ATTR_DVBC2_FRAMING_ADJUST_RF_FREQUENCY, "")); 
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Read DVBC2 Framing OFDM Calculate Center Frequency
/* Purpose:  This function queries the calculated center frequency.
/*===========================================================================*/
ViStatus _VI_FUNC rssfu_ReadDVBC2FramingOFDMCalculateCenterFrequency
             (ViSession instrSession, ViReal64* calcCenterFrequency
)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    if (!calcCenterFrequency) viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Calculate Center Frequency");
                                                               
    viCheckParm (rssfu_GetAttributeViReal64 (instrSession, "", RSSFU_ATTR_DVBC2_FRAMING_CALC_CENTER_FREQUENCY, 
               calcCenterFrequency), 2, "Calculate Center Frequency");
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Read DVBC2 Framing OFDM Parameters
/* Purpose:  This function queries the OFDM symbols parameters.
/*===========================================================================*/
ViStatus _VI_FUNC rssfu_ReadDVBC2FramingOFDMParameters (ViSession instrSession,
                                                        ViInt32* DX,
                                                        ViReal64* startFrequency,
                                                        ViReal64* carrierSpacing_1Tu,
                                                        ViInt32* numberOfDataSymbolC2Frame,
                                                        ViInt32* numberOfPreambleSymbols)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      model [RSSFU_IO_BUFFER_SIZE] = "";  
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckErr (rssfu_GetAttributeViString (instrSession, "",
                                               RS_ATTR_INSTRUMENT_MODEL,
                                               RSSFU_IO_BUFFER_SIZE, model));

    viCheckParm (rssfu_GetAttributeViInt32 (instrSession, "", RSSFU_ATTR_DVBC2_FRAMING_DX, 
               DX), 2, "DX");
    viCheckParm (rssfu_GetAttributeViReal64 (instrSession, "", RSSFU_ATTR_DVBC2_FRAMING_START_FREQUENCY, 
               startFrequency), 3, "Start Frequency");
    
    viCheckParm (rssfu_GetAttributeViReal64 (instrSession, "", RSSFU_ATTR_DVBC2_FRAMING_CARRIER_SPACING, 
               carrierSpacing_1Tu), 4, "Carrier Spacing 1/Tu"); 
    
    if (strstr (model, "BTC") == NULL) {    
        viCheckParm (rssfu_GetAttributeViInt32 (instrSession, "", RSSFU_ATTR_DVBC2_FRAMING_NUMBER_OF_DATA_SYMBOLS_PER_C2_FRAME, 
                   numberOfDataSymbolC2Frame), 5, "Number Of Data Symbol C2 Frame");
    }   
    viCheckParm (rssfu_GetAttributeViInt32 (instrSession, "", RSSFU_ATTR_DVBC2_FRAMING_NUMBER_OF_PREAMBLE_SYMBOLS, 
               numberOfPreambleSymbols), 6, "Number Of Preamble Symbols");

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/*===========================================================================*/
/* Function: Configure DVBC2 C2 System Parameters
/* Purpose:  This function configures C2 system identification parameters.
/*===========================================================================*/
ViStatus _VI_FUNC rssfu_ConfigureDVBC2C2SystemParameters (ViSession instrSession,
                                                          ViInt32 networkID,
                                                          ViInt32 c2SystemID,
                                                          ViInt32 l1TimeInterleaver,
                                                          ViBoolean systemReservedTones)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));        

     
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_DVBC2_C2_SYSTEM_NETWORK_ID, networkID),
                         2, "Network ID");
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_DVBC2_C2_SYSTEM_ID, c2SystemID),
                         3, "C2 System ID");
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_DVBC2_C2_SYSTEM_L1_TIME_INTERLEAVER, l1TimeInterleaver),
                         4, "L1 Time Interleaver");
    viCheckParm (rssfu_SetAttributeViBoolean (instrSession, "", RSSFU_ATTR_DVBC2_C2_SYSTEM_RESERVED_TONES, systemReservedTones),
                         5, "System Reserved Tones");
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}   

/*============================================================================/
 * Function: Configure DVBC2 Settings
 * Purpose:  This function configures DVB-C2 coding settings.
/============================================================================*/
ViStatus _VI_FUNC rssfu_ConfigureDVBC2Settings (ViSession instrSession,
                                           ViInt32 testTSPacket,
                                           ViInt32 PIDTestPacket, ViInt32 pid,
                                           ViInt32 payloadTestTSPacket, ViInt32 prbs)

{
    ViStatus    error = VI_SUCCESS;
    ViChar      model [RSSFU_IO_BUFFER_SIZE] = "";  

    checkErr( Rs_LockSession (instrSession, VI_NULL));
        
    viCheckErr (rssfu_GetAttributeViString (instrSession, "",
                                               RS_ATTR_INSTRUMENT_MODEL,
                                               RSSFU_IO_BUFFER_SIZE, model));
              
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_DVBC2_SETTINGS_TEST_TS_PACKET, testTSPacket),
                         2, "Test TS Packet");
    
    if (strstr (model, "SFC") == NULL) {
        viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_DVBC2_SETTINGS_PID_TEST_PACKET, PIDTestPacket),
                             3, "PID Test Packet"); 
        viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_DVBC2_SETTINGS_PID, pid),
                             4, "PID");
    }
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_DVBC2_SETTINGS_PAYLOAD_TEST_TS_PACKET, payloadTestTSPacket),
                         5, "Payload Test TS Packet");      
    switch (payloadTestTSPacket){
        case  RSSFU_IQC_PRBS:
            viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_DVBC2_SETTINGS_PRBS, prbs),
                         6, "PRBS"); 
    }   
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
} 



/*****************************************************************************
 *----------------------------- ATSC-MH -------------------------------------*
 *****************************************************************************/
 
/*============================================================================/
 * Function: Configure ATSCMH Input Signal
 * Purpose:  This function configures ATSC-M/H Input Signal parameters.
/============================================================================*/
ViStatus _VI_FUNC rssfu_confATSCMHInputSignal (ViSession instrSession,
                                               ViInt32 source,
                                               ViInt32 input, ViInt32 stuffing)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      model [RSSFU_IO_BUFFER_SIZE] = "";

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckErr (rssfu_GetAttributeViString (instrSession, "",
                                               RS_ATTR_INSTRUMENT_MODEL,
                                               RSSFU_IO_BUFFER_SIZE, model));   
    if (strstr (model, "BTC") == NULL) { 
    
        viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_ATSCMH_INPUT_SIGNAL_SOURCE, source),
                             2, "Source"); 
        viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_ATSCMH_INPUT_SIGNAL_INPUT, input),
                             3, "Input");
    }
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_ATSCMH_INPUT_SIGNAL_STUFFING, stuffing),
                         4, "Stuffing");
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Configure ATSCMH Test Signal
 * Purpose:  This function configures ATSC-M/H Test Signal.
/============================================================================*/
ViStatus _VI_FUNC rssfu_confATSCMHTestSignal(ViSession   instrSession,
                                           ViInt32 testSignal)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_ATSCMH_INPUT_SIGNAL_TEST_SIGNAL, testSignal),
                         2, "Test Signal"); 
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Read ATSCMH Packed Length
 * Purpose:  This function reads ATSC-M/H packed length.
/============================================================================*/
ViStatus _VI_FUNC rssfu_readATSCMHPackedLength (ViSession instrSession,
                                                 ViInt32* packedLength)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    if (!packedLength) viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Packed Length");
    viCheckParm (rssfu_GetAttributeViInt32 (instrSession, "", RSSFU_ATTR_READ_ATSCMH_PACKET_LENGTH,
                                            packedLength), 2, "Packed Length");
   Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

 
/*============================================================================/
 * Function: Read ATSCMH Values
 * Purpose:  This function reads selected coding value.
/============================================================================*/
ViStatus _VI_FUNC rssfu_readATSCMHValues(ViSession instrSession,
                                        ViInt32 value,
                                        ViReal64* result)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    
    if ((value < RSSFU_VAL_MAX_USEFUL_DRATE) || (value > RSSFU_VAL_USEFUL_DRATE))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Value");
        
    switch (value){                                                       
        case RSSFU_VAL_MAX_USEFUL_DRATE:
            viCheckParm (rssfu_GetAttributeViReal64 (instrSession, "", RSSFU_ATTR_READ_ATSCMH_MAX_USEFUL_DATA_RATE,
                                                     result), 3, "Result");
        break;
        case RSSFU_VAL_MEAS_DRATE:
            viCheckParm (rssfu_GetAttributeViReal64 (instrSession, "", RSSFU_ATTR_READ_ATSCMH_MEASURED_DATA_RATE,
                                                     result), 3, "Result");
        break;
        case RSSFU_VAL_USEFUL_DRATE:
            viCheckParm (rssfu_GetAttributeViReal64 (instrSession, "", RSSFU_ATTR_READ_ATSCMH_USEFUL_DATA_RATE,
                                                     result), 3, "Result");
        break;
     }           
     
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Configure ATSCMH Coding Parameters
 * Purpose:  This function configures ATSC-M/H coding parameters.
/============================================================================*/
ViStatus _VI_FUNC rssfu_confATSCMHCodingParameters (ViSession instrSession,
                                                   ViReal64 symbolRate,
                                                   ViBoolean MHState)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
 
    viCheckParm (rssfu_SetAttributeViReal64 (instrSession, "", RSSFU_ATTR_ATSCMH_CODING_SYMBOL_RATE, symbolRate),
                         2, "Symbol Rate"); 

    viCheckParm (rssfu_SetAttributeViBoolean (instrSession, "", RSSFU_ATTR_ATSCMH_CODING_MH_STATE, MHState),
                         3, "MH State");                             
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}
 
/*============================================================================/
 * Function: Read ATSCMH Constellation
 * Purpose:  function queries the VSB constellation firmly set to 8VSB.
/============================================================================*/
ViStatus _VI_FUNC rssfu_readATSCMHConstellation (ViSession instrSession,
                                                 ViChar constellation[])
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSSFU_IO_BUFFER_SIZE]="\0";  
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    if (constellation == NULL) viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Constellation");
    viCheckErr (rssfu_GetAttributeViString (instrSession, "", RSSFU_ATTR_ATSCMH_CODING_CONSTELLATION,
                                            RSSFU_IO_BUFFER_SIZE, buffer));
    strncpy (constellation, buffer, strlen(buffer)); 
    if (constellation != NULL)
        constellation[strlen(buffer)] = '\0';
  
   Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}
 
/*============================================================================/
 * Function: Read ATSCMH Roll Off
 * Purpose:  This function queries the roll off.
/============================================================================*/
ViStatus _VI_FUNC rssfu_readATSCMHRollOff (ViSession instrSession,
                                         ViReal64* rollOff)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rssfu_GetAttributeViReal64 (instrSession, "", RSSFU_ATTR_ATSCMH_CODING_ROLL_OFF,
                                                     rollOff), 2, "Roll Off");
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Enable ATSCMH Special Settings
 * Purpose:  This function enables use of special ATSC-M/H coding parameters.
/============================================================================*/
ViStatus _VI_FUNC rssfu_enableATSCMHSpecial(ViSession instrSession,
                                             ViBoolean state)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rssfu_SetAttributeViBoolean (instrSession, "", RSSFU_ATTR_ATSCMH_SPECIAL_SETTINGS, state),
                         2, "State");
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

 
/*============================================================================/
 * Function: Configure ATSCMH Special Settings
 * Purpose:  This function configures special ATSC-M/H coding parameters.
/============================================================================*/
ViStatus _VI_FUNC rssfu_confATSCMHSpecial (ViSession instrSession,
                                           ViBoolean randomizer,
                                           ViBoolean reedSolomon,
                                           ViBoolean interleaver,
                                           ViBoolean SFNInitialization,
                                           ViBoolean trellisInitialization,
                                           ViBoolean pilot, ViReal64 pilotValue)

{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rssfu_SetAttributeViBoolean (instrSession, "", RSSFU_ATTR_ATSCMH_SPECIAL_RANDOMIZER, randomizer),
                 2, "Randomizer"); 
    viCheckParm (rssfu_SetAttributeViBoolean (instrSession, "", RSSFU_ATTR_ATSCMH_SPECIAL_REED_SOLOMON, reedSolomon),
                 3, "Reed Solomon");
    viCheckParm (rssfu_SetAttributeViBoolean (instrSession, "", RSSFU_ATTR_ATSCMH_SPECIAL_INTERLEAVER, interleaver),
                 4, "Interleaver");
    viCheckParm (rssfu_SetAttributeViBoolean (instrSession, "", RSSFU_ATTR_ATSCMH_SPECIAL_SFN_INITIALIZATION, SFNInitialization),
                 5, "SFN Initialization");   
    viCheckParm (rssfu_SetAttributeViBoolean (instrSession, "", RSSFU_ATTR_ATSCMH_SPECIAL_TRELLIS_INITIALIZATION, trellisInitialization),
                 6, "Trellis Initialization");   
    viCheckParm (rssfu_SetAttributeViBoolean (instrSession, "", RSSFU_ATTR_ATSCMH_SPECIAL_PILOT, pilot),
                 7, "Pilot");
    if (pilot)
        viCheckParm (rssfu_SetAttributeViReal64 (instrSession, "", RSSFU_ATTR_ATSCMH_SPECIAL_PILOT_VALUE, pilotValue),
                 8, "Pilot Value");                 
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Set ATSCMH MHE PID
 * Purpose:  This function determines the PID of the MPEG-2 packets that
 *           contain M/H data
/============================================================================*/
ViStatus _VI_FUNC rssfu_setAtscmhMhePid (ViSession instrSession, ViInt32 mhePID)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_ATSCMH_SETTINGS_MHE_PID, mhePID),
                 2, "MHE PID");
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Configure ATSCMH RF Watermark Settings
 * Purpose:  This function configures ATSC-M/H RF watermark settings.
/============================================================================*/
ViStatus _VI_FUNC rssfu_confATSCMHRfWatermarkSettings (ViSession instrSession,
                                                     ViBoolean RFWatermark,
                                                     ViInt32 txAddress,
                                                     ViInt32 networkID,
                                                     ViInt32 buryRatio)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rssfu_SetAttributeViBoolean (instrSession, "", RSSFU_ATTR_ATSCMH_SETTINGS_RF_WATERMARK, RFWatermark),
                 2, "RF Watermark");
    
    if (RFWatermark == VI_TRUE) {
        viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_ATSCMH_SETTINGS_TX_ADDRESS, txAddress),
                     3, "Tx Address");
        viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_ATSCMH_SETTINGS_NETWORK_ID, networkID),
                     4, "Network ID");
        viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_ATSCMH_SETTINGS_BURY_RATIO, buryRatio),
                     5, "Bury Ratio");
    }
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Configure ATSCMH MTXID Transmission
 * Purpose:  This function configures ATSC-M/H MTXID Transmission parameters.
/============================================================================*/
ViStatus _VI_FUNC rssfu_configATSCMHMTXIDTransmission (ViSession instrSession,
                                                       ViBoolean MTXIDTransmission,
                                                       ViInt32 transmitterID,
                                                       ViInt32 marketID)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rssfu_SetAttributeViBoolean (instrSession, "", RSSFU_ATTR_ATSCMH_SETTINGS_MTXID_TRANSMISSION, MTXIDTransmission),
                 2, "MTXID Transmission");
    
    if (MTXIDTransmission == VI_TRUE) {
        viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_ATSCMH_SETTINGS_TRANSMITTER_ID, transmitterID),
                     3, "Transmitter ID");
        viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_ATSCMH_SETTINGS_MARKET_ID, marketID),
                     4, "Market ID");
    }
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
} 

/*============================================================================/
 * Function: Configure ATSCMH Settings
 * Purpose:  This function configures ATSC-M/H coding settings.
/============================================================================*/
ViStatus _VI_FUNC rssfu_confATSCMHSettings (ViSession   instrSession,
                                         ViInt32 testTSPacket,
                                         ViInt32 pidTestPacket,
                                         ViInt32 pid,
                                         ViInt32 payloadTestStuff,
                                         ViInt32 prbs)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      model [RSSFU_IO_BUFFER_SIZE] = "";

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckErr (rssfu_GetAttributeViString (instrSession, "",
                                               RS_ATTR_INSTRUMENT_MODEL,
                                               RSSFU_IO_BUFFER_SIZE, model));   
    
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_ATSCMH_SETTINGS_TEST_TS_PACKET, testTSPacket),
                         2, "Test TS Packet");
    if (strstr (model, "SFC") == NULL) {
        viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_ATSCMH_SETTINGS_PID_TEST_PACKET, pidTestPacket),
                             3, "PID Test Packet"); 
    
        if (pidTestPacket == RSSFU_IQC_PIDT_VAR) {
            viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_ATSCMH_SETTINGS_PID, pid),
                                 4, "PID"); 
        }
    }
    
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_ATSCMH_SETTINGS_PAYLOAD_TEST_STUFF, payloadTestStuff),
                         5, "Payload Test/Stuff");      
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_ATSCMH_SETTINGS_PRBS, prbs),
                         6, "PRBS"); 
       
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 *----------------------- Audio Broadcasting --------------------------------*
 *****************************************************************************/

/*===========================================================================*/
/* Function: Configure Audio BC FM Input Signal
/* Purpose:  This function configures Audio Broadcasting FM Input Signal 
/*           parameters.
/*===========================================================================*/
ViStatus _VI_FUNC rssfu_configAudioBCFMInputSignal(
    ViSession   instrSession,
    ViInt32 sourceLeft,
    ViInt32 sourceRight,
    ViInt32 afMode,
    ViInt32 impedanceSignalSource
    
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      model [RSSFU_IO_BUFFER_SIZE] = "";  

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckErr (rssfu_GetAttributeViString (instrSession, "",
                                               RS_ATTR_INSTRUMENT_MODEL,
                                               RSSFU_IO_BUFFER_SIZE, model));   
    if (strstr (model, "BTC") != NULL) {
         
        if ((sourceLeft != RSSFU_VAL_AUDIO_BC_INPUT_AGEN) || (sourceLeft != RSSFU_VAL_AUDIO_BC_INPUT_APL))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Source Left");

    }
    
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_AUDIO_BC_SOURCE_LEFT, sourceLeft),
                         2, "Source Left"); 
    if (sourceLeft != RSSFU_VAL_AUDIO_BC_INPUT_MPX){
        if (strstr (model, "BTC") == NULL) { 
        viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_AUDIO_BC_SOURCE_RIGHT, sourceRight),
                         3, "Source Right");
        }
        viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_AUDIO_BC_AF_MODE, afMode),
                         4, "AF Mode");
    }
    if (strstr (model, "SFU") != NULL) {
        if((sourceLeft == RSSFU_VAL_AUDIO_BC_INPUT_EXT) || (sourceRight == RSSFU_VAL_AUDIO_BC_INPUT_EXT))
            viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_AUDIO_BC_IMPEDANCE_SIGNAL_SOURCE, impedanceSignalSource),
                         5, "Impedance Signal Source");   
    
    }
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;   
}

/*===========================================================================*/
/* Function: Configure Audio BC FM Source
/* Purpose:  This function configures Audio BC FM parameters.
/*===========================================================================*/
ViStatus _VI_FUNC rssfu_confAudioBCFMSource(
    ViSession   instrSession,
    ViInt32 source,
    ViReal64    frequency,
    ViReal64    level
)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    switch (source){
        case RSSFU_VAL_AUDIO_BC_SOUR_LEFT:
            viCheckParm (rssfu_SetAttributeViReal64 (instrSession, "", RSSFU_ATTR_AUDIO_BC_SOURCE_FREQUENCY_LEFT, 
                        frequency), 3, "Frequency");
            viCheckParm (rssfu_SetAttributeViReal64 (instrSession, "", RSSFU_ATTR_AUDIO_BC_SOURCE_LEVEL_LEFT, 
                        level), 4, "Level");
        break;
        case RSSFU_VAL_AUDIO_BC_SOUR_RIGHT:
            viCheckParm (rssfu_SetAttributeViReal64 (instrSession, "", RSSFU_ATTR_AUDIO_BC_SOURCE_FREQUENCY_RIGHT, 
                        frequency), 3, "Frequency");
            viCheckParm (rssfu_SetAttributeViReal64 (instrSession, "", RSSFU_ATTR_AUDIO_BC_SOURCE_LEVEL_RIGHT, 
                        level), 4, "Level");
        break;
        
        default:
            viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Source");
    }
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;   
}

/*===========================================================================*/
/* Function: Read Audio BC FM External Source
/* Purpose:  This function reads phase Audio BC FM parameters.
/*===========================================================================*/
ViStatus _VI_FUNC rssfu_readAudioBCFMExternalSource(
    ViSession   instrSession,
    ViInt32 source,
    ViReal64*   inputLevel
)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    switch (source){
        case RSSFU_VAL_AUDIO_BC_SOUR_LEFT:
            viCheckParm (rssfu_GetAttributeViReal64 (instrSession, "", RSSFU_ATTR_AUDIO_BC_SOURCE_INPUT_LEVEL_LEFT, 
                    inputLevel), 3, "Input Level");
        break;
        case RSSFU_VAL_AUDIO_BC_SOUR_RIGHT:
            viCheckParm (rssfu_GetAttributeViReal64 (instrSession, "", RSSFU_ATTR_AUDIO_BC_SOURCE_INPUT_LEVEL_RIGHT, 
                inputLevel), 3, "Input Level");
        break;
        case RSSFU_VAL_AUDIO_BC_SOUR_MPX:
            viCheckParm (rssfu_GetAttributeViReal64 (instrSession, "", RSSFU_ATTR_AUDIO_BC_SOURCE_INPUT_LEVEL_LEFT, 
                    inputLevel), 3, "Input Level");
        break;
        default:
            viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Source");
    }
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Configure Audio BC FM RDS RBDS
/* Purpose:  This function configures Audio Broadcasting FM RDS/RBDS
/*           parameters.
/*===========================================================================*/
ViStatus _VI_FUNC rssfu_configAudioBCFMRDSRBDS (ViSession instrSession,
                                                ViString groupSequence, ViInt32 RS_PI,
                                                ViString PS, ViBoolean TP,
                                                ViBoolean TA, ViInt32 PTY,
                                                ViString PTYN, ViString RT,
                                                ViInt32 MS)
{
 ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rssfu_SetAttributeViString (instrSession, "", RSSFU_ATTR_AUDIO_BC_RDS_GROUP_SEQUENCE, groupSequence),
                         2, "Group Sequence"); 
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_AUDIO_BC_RDS_PI, RS_PI),
                         3, "PI"); 
    viCheckParm (rssfu_SetAttributeViString (instrSession, "", RSSFU_ATTR_AUDIO_BC_RDS_PS, PS),
                         4, "PS"); 
    viCheckParm (rssfu_SetAttributeViBoolean (instrSession, "", RSSFU_ATTR_AUDIO_BC_RDS_TP, TP),
                         5, "TP");
    viCheckParm (rssfu_SetAttributeViBoolean (instrSession, "", RSSFU_ATTR_AUDIO_BC_RDS_TA, TA),
                         6, "TA");
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_AUDIO_BC_RDS_PTY, PTY),
                         7, "PTY");
    viCheckParm (rssfu_SetAttributeViString (instrSession, "", RSSFU_ATTR_AUDIO_BC_RDS_PTYN, PTYN),
                         8, "PTYN"); 
    viCheckParm (rssfu_SetAttributeViString (instrSession, "", RSSFU_ATTR_AUDIO_BC_RDS_RT, RT),
                         9, "RT"); 
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_AUDIO_BC_RDS_MS, MS),
                         10, "MS");

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;       
}

/*===========================================================================*/
/* Function: Read Audio BC FM RDS RBDS Radio Text Status
/* Purpose:  This function queries the data transfer status of the radio text RT.
/*===========================================================================*/
ViStatus _VI_FUNC rssfu_ReadAudioBCFMRDSRBDSRadioTextStatus(
    ViSession   instrSession,
    ViInt32*    rtReady
)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    if (!rtReady) viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "RT Ready");
    
    viCheckParm (rssfu_GetAttributeViInt32 (instrSession, "", RSSFU_ATTR_AUDIO_BC_RDS_RT_READY, 
               rtReady), 2, "RT Ready");

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Configure Audio BC FM RDS RBDS Decoder Identification
/* Purpose:  This function configures Audio Broadcasting FM RDS/RBDS Decoder
/*           Identification parameters.
/*===========================================================================*/
ViStatus _VI_FUNC rssfu_configAudioBCFMRDSRBDSDecoderIdn
             (ViSession instrSession, ViBoolean DIDynamicPTY,
              ViBoolean DICompressed, ViBoolean DIArtHead, ViBoolean DIStereo,
              ViBoolean CT, ViString CTOffset, ViInt32 AFMethod)
{
 ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rssfu_SetAttributeViBoolean (instrSession, "", RSSFU_ATTR_AUDIO_BC_RDS_DI_DYNAMIC_PTY, DIDynamicPTY),
                         2, "DI Dynamic PTY");
    viCheckParm (rssfu_SetAttributeViBoolean (instrSession, "", RSSFU_ATTR_AUDIO_BC_RDS_DI_COMPRESSED, DICompressed),
                         3, "DI Compressed");
    viCheckParm (rssfu_SetAttributeViBoolean (instrSession, "", RSSFU_ATTR_AUDIO_BC_RDS_DI_ART_HEAD, DIArtHead),
                         4, "DI Art Head");
    viCheckParm (rssfu_SetAttributeViBoolean (instrSession, "", RSSFU_ATTR_AUDIO_BC_RDS_DI_STEREO, DIStereo),
                         5, "DI Stereo");
    viCheckParm (rssfu_SetAttributeViBoolean (instrSession, "", RSSFU_ATTR_AUDIO_BC_RDS_DI_CT, CT),
                         6, "CT");
    if (CT)
        viCheckParm (rssfu_SetAttributeViString (instrSession, "", RSSFU_ATTR_AUDIO_BC_RDS_DI_CT_OFFSET, CTOffset),
                            7, "CT Offset"); 
    
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_AUDIO_BC_RDS_DI_AF_METHOD, AFMethod),
                         8, "AF Method");

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;       
}

/*===========================================================================*/
/* Function: Set Audio BC FM AF A Number Of Freq
/* Purpose:  This function sets the NUMBER OF FREQ. in AF method A (FM).
/*===========================================================================*/
ViStatus _VI_FUNC rssfu_SetAudioBCFMAFAFreq (ViSession instrSession, ViInt32 numberOfFreq)
{
 ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_AUDIO_BC_AF_A_NUMBER_OF_FREQ,
                    numberOfFreq), 2, "Number Of Freq");
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;       
}

 
/*===========================================================================*/
/* Function: Configure Audio BC FM AF A Frequency
/* Purpose:  This function sets the alternative frequencies 1 to 25 in AF
/*           method A (FM).
/*===========================================================================*/
ViStatus _VI_FUNC rssfu_ConfigureAudioBCFMAFAFrequency (ViSession instrSession,
                                                        ViInt32 af,
                                                        ViReal64 frequency)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buf1[RSSFU_TOKEN_BUFFER_SIZE] = "";

    if ((af < 1) || (af > 25))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "AF"); 

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    checkErr (rssfu_GetActualRepCaps (instrSession, RSSFU_ATTR_AUDIO_BC_AF_A_FREQUENCY, "AfFreq", af-1, buf1));

    viCheckParm (rssfu_SetAttributeViReal64 (instrSession, buf1, RSSFU_ATTR_AUDIO_BC_AF_A_FREQUENCY, frequency), 
                    3, "Frequency"); 
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
} 

/*===========================================================================*/
/* Function: Set Audio BC FM AF B Number Of Lists
/* Purpose:  This function sets the NUMBER OF LISTS in AF method B (FM).
/*===========================================================================*/
ViStatus _VI_FUNC rssfu_SetAudioBCFMAFBList (ViSession instrSession,
                                             ViInt32 numberOfLists)
{
 ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_AUDIO_BC_AF_B_NUMBER_OF_LISTS,
                    numberOfLists), 2, "Number Of Lists");
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;       
}

/*===========================================================================*/
/* Function: Set Audio BC FM AF B Number Of Freq
/* Purpose:  This function sets the NUMBER OF FREQ. and TUNING FREQ of a list
/*           in AF method B (FM).
/*===========================================================================*/
ViStatus _VI_FUNC rssfu_SetAudioBCFMAFBFreq (ViSession instrSession,
                                             ViInt32 list,
                                             ViInt32 numberOfFreq,
                                             ViReal64 tuningFreq)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buf1[RSSFU_TOKEN_BUFFER_SIZE] = "";

    if ((list < 1) || (list > 5))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "List"); 

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    checkErr (rssfu_GetActualRepCaps (instrSession, RSSFU_ATTR_AUDIO_BC_AF_B_NUMBER_OF_FREQ, "AfList", list-1, buf1));

    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, buf1, RSSFU_ATTR_AUDIO_BC_AF_B_NUMBER_OF_FREQ,
                    numberOfFreq), 3, "Number Of Freq");

    viCheckParm (rssfu_SetAttributeViReal64 (instrSession, buf1, RSSFU_ATTR_AUDIO_BC_AF_B_TUNING_FREQ,
                    tuningFreq),  4, "Tuning Freq"); 
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
} 


/*===========================================================================*/
/* Function: Configure Audio BC FM AF B Frequency
/* Purpose:  This function sets ascending or descendin alternative frequency
/*           of a list in AF method B (FM).
/*===========================================================================*/
ViStatus _VI_FUNC rssfu_ConfigureAudioBCFMAFBFrequency (ViSession instrSession,
                                                        ViInt32 list, ViInt32 af,
                                                        ViReal64 frequency,
                                                        ViInt32 order)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSSFU_IO_BUFFER_SIZE] = "";
    ViChar      buf1[RSSFU_TOKEN_BUFFER_SIZE] = "";
    ViChar      *p2buf; 

    if ((list < 1) || (list > 5))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "List"); 
    if ((af < 1) || (af > 12))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 3, "AF"); 
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    p2buf=buffer; 
    
    checkErr (rssfu_GetActualRepCaps (instrSession, RSSFU_ATTR_AUDIO_BC_AF_B_AF_FREQUENCY, "AfList", list-1, buf1));
    p2buf+=sprintf(p2buf,"%s,",buf1);   
    checkErr (rssfu_GetActualRepCaps (instrSession, RSSFU_ATTR_AUDIO_BC_AF_B_AF_FREQUENCY, "AfFreq", af-1, buf1));
    p2buf+=sprintf(p2buf,"%s",buf1); 

    viCheckParm (rssfu_SetAttributeViReal64 (instrSession, buffer, RSSFU_ATTR_AUDIO_BC_AF_B_AF_FREQUENCY,
                    frequency),  4, "Frequency"); 
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, buffer, RSSFU_ATTR_AUDIO_BC_AF_B_AF_ORDER,
                    order), 5, "order");
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
} 

/*===========================================================================*/
/* Function: Configure Audio BC FM Audio Modulation
/* Purpose:  This function configures Audio Broadcasting FM EON parameters.
/*===========================================================================*/
ViStatus _VI_FUNC rssfu_configAudioBCFMEONParameters (ViSession instrSession,
                                                      ViInt32   EONPI,
                                                      ViString  EONPS,
                                                      ViBoolean EONTP,
                                                      ViBoolean EONTA,
                                                      ViBoolean EONLA,
                                                      ViBoolean EONEG,
                                                      ViBoolean EONILS,
                                                      ViInt32 EONLSN,
                                                      ViInt32 EONPTY,
                                                      ViInt32 EONPIN,
                                                      ViInt32 EONAFMethod)
{
 ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_AUDIO_BC_EON_PI, EONPI),
                            2, "EON PI");
    viCheckParm (rssfu_SetAttributeViString (instrSession, "", RSSFU_ATTR_AUDIO_BC_EON_PS, EONPS),
                            3, "EON PS"); 
    viCheckParm (rssfu_SetAttributeViBoolean (instrSession, "", RSSFU_ATTR_AUDIO_BC_EON_TP, EONTP),
                            4, "EON TP");
    viCheckParm (rssfu_SetAttributeViBoolean (instrSession, "", RSSFU_ATTR_AUDIO_BC_EON_TA, EONTA),
                            5, "EON TA");
    viCheckParm (rssfu_SetAttributeViBoolean (instrSession, "", RSSFU_ATTR_AUDIO_BC_EON_LA, EONLA),
                            6, "EON LA");
    viCheckParm (rssfu_SetAttributeViBoolean (instrSession, "", RSSFU_ATTR_AUDIO_BC_EON_EG, EONEG),
                            7, "EON EG");
    viCheckParm (rssfu_SetAttributeViBoolean (instrSession, "", RSSFU_ATTR_AUDIO_BC_EON_ILS, EONILS),
                            8, "EON ILS");
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_AUDIO_BC_EON_LSN, EONLSN),
                            9, "EON LSN");
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_AUDIO_BC_EON_PTY, EONPTY),
                            10, "EON PTY");
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_AUDIO_BC_EON_PIN, EONPIN),
                            11, "EON PIN");
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_AUDIO_BC_EON_AF_METHOD, EONAFMethod),
                            12, "EON AF Method");
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;       
}


/*===========================================================================*/
/* Function: Set Audio BC FM EON AF A Number Of Freq
/* Purpose:  This function sets the NUMBER OF FREQ. in EON AF method A (FM).
/*===========================================================================*/
ViStatus _VI_FUNC rssfu_SetAudioBCFMEONAFAFreq (ViSession instrSession,
                                                ViInt32 numberOfFreq)
{
 ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_AUDIO_BC_EON_AF_A_NUMBER_OF_FREQ,
                    numberOfFreq), 2, "Number Of Freq");
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;       
}

 
/*===========================================================================*/
/* Function: Configure Audio BC FM EON AF A Frequency
/* Purpose:  This function sets one of 25 alternative frequencies in AF
/*           method A (FM).
/*===========================================================================*/
ViStatus _VI_FUNC rssfu_ConfigureAudioBCFMEONAFAFrequency
             (ViSession instrSession, ViInt32 af, ViReal64 frequency)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buf1[RSSFU_TOKEN_BUFFER_SIZE] = "";

    if ((af < 1) || (af > 25))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "AF"); 

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    checkErr (rssfu_GetActualRepCaps (instrSession, RSSFU_ATTR_AUDIO_BC_EON_AF_A_FREQUENCY, "AfFreq", af-1, buf1));

    viCheckParm (rssfu_SetAttributeViReal64 (instrSession, buf1, RSSFU_ATTR_AUDIO_BC_EON_AF_A_FREQUENCY, frequency), 
                    3, "Frequency"); 
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
} 

/*===========================================================================*/
/* Function: Set Audio BC FM AF Mapped Freq Number Of Freq
/* Purpose:  This function sets the NUMBER OF FREQ. and TUNING FREQ of EON,
/*           AF method MAPPED FREQ. (FM)
/*===========================================================================*/
ViStatus _VI_FUNC rssfu_SetAudioBCFMEONAFMappedFreqNumberOfFreq
             (ViSession instrSession, ViInt32 numberOfFreq,
              ViReal64 tuningFreq)
{
 ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_AUDIO_BC_EON_AF_MAPPED_FREQ_NUMBER_OF_FREQ,
                    numberOfFreq), 2, "Number Of Freq");
    viCheckParm (rssfu_SetAttributeViReal64 (instrSession, "", RSSFU_ATTR_AUDIO_BC_EON_AF_MAPPED_FREQ_TUNING_FREQ, 
                    tuningFreq), 3, "Tuning Freq");     

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;       
}

/*===========================================================================*/
/* Function: Configure Audio BC FM Audio Modulation
/* Purpose:  This function sets an alternative frequency in EON, AF method
/*           MAPPED FREQ. (FM).
/*===========================================================================*/
ViStatus _VI_FUNC rssfu_ConfigureAudioBCFMEONAFMappedFrequency
             (ViSession instrSession, ViInt32 af, ViReal64 frequency)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buf1[RSSFU_TOKEN_BUFFER_SIZE] = "";

    if ((af < 1) || (af > 4))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "AF"); 

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    checkErr (rssfu_GetActualRepCaps (instrSession, RSSFU_ATTR_AUDIO_BC_EON_AF_MAPPED_FREQ_AF_FREQUENCY, "AfFreq", af-1, buf1));

    viCheckParm (rssfu_SetAttributeViReal64 (instrSession, buf1, RSSFU_ATTR_AUDIO_BC_EON_AF_MAPPED_FREQ_AF_FREQUENCY, frequency), 
                    3, "Frequency"); 
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
} 

/*===========================================================================*/
/* Function: Enable Audio BC FM TMC
/* Purpose:  This function switches the TMC on or off (FM).
/*===========================================================================*/
ViStatus _VI_FUNC rssfu_EnableAudioBCFMTMC (ViSession instrSession,
                                            ViBoolean TMC)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rssfu_SetAttributeViBoolean (instrSession, "", RSSFU_ATTR_AUDIO_BC_TMC_TMC, TMC),
                         2, "TMC");
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;           
}

/*===========================================================================*/
/* Function: Audio BC FM TMC Send Data
/* Purpose:  This function sends the data of the traffic massege channel TMC (FM)
/*===========================================================================*/
ViStatus _VI_FUNC rssfu_AudioBCFMTMCSendData (ViSession instrSession)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));    

    viCheckErr (rssfu_SetAttributeViString (instrSession, "", RSSFU_ATTR_AUDIO_BC_TMC_APPLY, "")); 
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Set Audio BC TMC Group 3A VAR Parameters
/* Purpose:  This function sets the TMC group 3A VAR 1 or VAR 2 (FM).
/*===========================================================================*/
ViStatus _VI_FUNC rssfu_SetAudioBCTMCGroup3AVARParameters
             (ViSession instrSession, ViInt32 group3AVAR00,
              ViInt32 group3AVAR01)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_AUDIO_BC_TMC_GROUP_3A_VAR_00,
                            group3AVAR00), 2, "Group 3A VAR 00");
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_AUDIO_BC_TMC_GROUP_3A_VAR_01,
                            group3AVAR01), 3, "Group 3A VAR 01");
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Set Audio BC TMC Number Of 8A Groups
/* Purpose:  This function sets NUMBER OF 8A-GROUPS.
/*===========================================================================*/
ViStatus _VI_FUNC rssfu_SetAudioBCTMCNumberOf8AGroups (ViSession instrSession,
                                                       ViInt32 numberOf8AGroups)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_AUDIO_BC_TMC_NUMBER_OF_8A_GROUPS,
                            numberOf8AGroups), 2, "Number Of 8A Groups");
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/*===========================================================================*/
/* Function: Configure Audio BC FM TMC 8A Block
/* Purpose:  This function configures TMC group 8A.
/*===========================================================================*/
ViStatus _VI_FUNC rssfu_ConfigureAudioBCFMTMC8ABlock (ViSession instrSession,
                                                      ViInt32 group8A,
                                                      ViInt32 block,
                                                      ViInt32 value)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSSFU_IO_BUFFER_SIZE] = "";
    ViChar      buf1[RSSFU_TOKEN_BUFFER_SIZE] = "";
    ViChar      *p2buf; 

    if ((group8A < 1) || (group8A > 6))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Group 8A"); 
    if ((block < 2) || (block > 4))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 3, "Block"); 
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    p2buf=buffer; 
        
    checkErr (rssfu_GetActualRepCaps (instrSession, RSSFU_ATTR_AUDIO_BC_TMC_8A_BLOCK, "G8A", group8A-1, buf1));
    p2buf+=sprintf(p2buf,"%s,",buf1); 
    checkErr (rssfu_GetActualRepCaps (instrSession, RSSFU_ATTR_AUDIO_BC_TMC_8A_BLOCK, "Block", block-2, buf1));
    p2buf+=sprintf(p2buf,"%s",buf1); 
    
    if (block == 2) {
            if ((value < 0) || (value > 31))
            viCheckParm( RS_ERROR_INVALID_PARAMETER, 4, "Value"); 
    }
    
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, buffer, RSSFU_ATTR_AUDIO_BC_TMC_8A_BLOCK,
                    value),  4, "Value"); 
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
} 

/*===========================================================================*/
/* Function: Enable Audio BC FM Open Format
/* Purpose:  This function switches the open format on or off.
/*===========================================================================*/
ViStatus _VI_FUNC rssfu_EnableAudioBCFMOpenFormat (ViSession instrSession,
                                                   ViBoolean openFormat)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rssfu_SetAttributeViBoolean (instrSession, "", RSSFU_ATTR_AUDIO_BC_OPEN_FORMAT, openFormat),
                         2, "Open Format");
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;           
}

/*===========================================================================*/
/* Function: Audio BC FM Open Format Send Data
/* Purpose:  This function sends the open format data (FM).
/*===========================================================================*/
ViStatus _VI_FUNC rssfu_AudioBCFMOpenFormatSendData (ViSession instrSession)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));    

    viCheckErr (rssfu_SetAttributeViString (instrSession, "", RSSFU_ATTR_AUDIO_BC_OPEN_FORMAT_APPLY, "")); 
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}
 
/*===========================================================================*/
/* Function: Configure Audio BC Open Format Group A
/* Purpose:  This function configures open format Group A.
/*===========================================================================*/
ViStatus _VI_FUNC rssfu_ConfigureAudioBCOpenFormatGroupA (ViSession instrSession,
                                                          ViInt32 groupA,
                                                          ViInt32 block,
                                                          ViInt32 value)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSSFU_IO_BUFFER_SIZE] = "";
    ViChar      buf1[RSSFU_TOKEN_BUFFER_SIZE] = "";
    ViChar      *p2buf; 
              
    // only group: 1, 3, 5, 6, 7, 8, 9, 11, 12, 13, 15
    if ((groupA < 1) || (groupA > 15))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Group A"); 
    if ((block < 2) || (block > 4))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 3, "Block"); 
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    p2buf=buffer; 
        
    checkErr (rssfu_GetActualRepCaps (instrSession, RSSFU_ATTR_AUDIO_BC_OPEN_FORMAT_GROUP_A, "GroupA", groupA-1, buf1));
    p2buf+=sprintf(p2buf,"%s,",buf1); 
    checkErr (rssfu_GetActualRepCaps (instrSession, RSSFU_ATTR_AUDIO_BC_OPEN_FORMAT_GROUP_A, "Block", block-2, buf1));
    p2buf+=sprintf(p2buf,"%s",buf1); 
    
    if (block == 2) {
            if ((value < 0) || (value > 31))
            viCheckParm( RS_ERROR_INVALID_PARAMETER, 4, "Value"); 
    }
    
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, buffer, RSSFU_ATTR_AUDIO_BC_OPEN_FORMAT_GROUP_A,
                    value),  4, "Value"); 
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
} 

/*===========================================================================*/
/* Function: Configure Audio BC Open Format Group B
/* Purpose:  This function configures open format Group B.
/*===========================================================================*/
ViStatus _VI_FUNC rssfu_ConfigureAudioBCOpenFormatGroupB (ViSession instrSession,
                                                          ViInt32 groupB,
                                                          ViInt32 block,
                                                          ViInt32 value)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSSFU_IO_BUFFER_SIZE] = "";
    ViChar      buf1[RSSFU_TOKEN_BUFFER_SIZE] = "";
    ViChar      *p2buf; 
              
    // only group: 1, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13
    if ((groupB < 1) || (groupB > 13))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Group B"); 
    if ((block < 2) || (block == 3) || (block > 4))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 3, "Block"); 
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    p2buf=buffer; 
        
    checkErr (rssfu_GetActualRepCaps (instrSession, RSSFU_ATTR_AUDIO_BC_OPEN_FORMAT_GROUP_B, "GroupB", groupB-1, buf1));
    p2buf+=sprintf(p2buf,"%s,",buf1); 
    checkErr (rssfu_GetActualRepCaps (instrSession, RSSFU_ATTR_AUDIO_BC_OPEN_FORMAT_GROUP_B, "Block", block-2, buf1));
    p2buf+=sprintf(p2buf,"%s",buf1); 
    
    if (block == 2) {
            if ((value < 0) || (value > 31))
            viCheckParm( RS_ERROR_INVALID_PARAMETER, 4, "Value"); 
    }
    
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, buffer, RSSFU_ATTR_AUDIO_BC_OPEN_FORMAT_GROUP_B,
                    value),  4, "Value"); 
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
} 

/*===========================================================================*/
/* Function: Audio BC Open Format Group Data Transfer Status
/* Purpose:  This function reads the data transfer status of the indicated
/*           open format group.
/*===========================================================================*/
ViStatus _VI_FUNC rssfu_AudioBCOpenFormatGroupDataTransferStatus
             (ViSession instrSession, ViInt32 group, ViInt32 groupType,
              ViInt32* transferStatus)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buf1[RSSFU_TOKEN_BUFFER_SIZE] = "";


    checkErr( Rs_LockSession (instrSession, VI_NULL));
   
    switch (groupType){
        case RSSFU_VAL_AUDIO_BC_OF_GROUP_A: {
            if ((group < 1) || (group > 15))    // only group: 1, 3, 5, 6, 7, 8, 9, 11, 12, 13, 15 
                viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Group"); 
            checkErr (rssfu_GetActualRepCaps (instrSession, RSSFU_ATTR_AUDIO_BC_OPEN_FORMAT_GROUP_A_READY, "GroupA", group-1, buf1));   
            viCheckParm (rssfu_GetAttributeViInt32 (instrSession, buf1, RSSFU_ATTR_AUDIO_BC_OPEN_FORMAT_GROUP_A_READY, 
                        transferStatus), 4, "Transfer Status");
        }
        break;
        
        case RSSFU_VAL_AUDIO_BC_OF_GROUP_B: {
            if ((group < 1) || (group > 13))    // only group: 1, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13
                viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Group"); 
            checkErr (rssfu_GetActualRepCaps (instrSession, RSSFU_ATTR_AUDIO_BC_OPEN_FORMAT_GROUP_B_READY, "GroupB", group-1, buf1)); 
            viCheckParm (rssfu_GetAttributeViInt32 (instrSession, buf1, RSSFU_ATTR_AUDIO_BC_OPEN_FORMAT_GROUP_B_READY, 
                        transferStatus), 4, "Transfer Status");
        }
        break;

        default:
            viCheckParm( RS_ERROR_INVALID_PARAMETER, 3, "Group Type");
    }
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}





/*===========================================================================*/
/* Function: Configure Audio BC FM Audio Modulation
/* Purpose:  This function configures Audio Broadcasting modulation 
/*           parameters.
/*===========================================================================*/
ViStatus _VI_FUNC rssfu_configAudioBCFMAudioModulation(
    ViSession   instrSession,
    ViInt32 mode,
    ViInt32 preemphasis,
    ViReal64 nomFreqDevAudio
)
{
 ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_AUDIO_BC_MODULATION_MODE, mode),
                         2, "Mode"); 
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_AUDIO_BC_MODULATION_PREEMPHASIS, preemphasis),
                         3, "Preemphasis");
    viCheckParm (rssfu_SetAttributeViReal64 (instrSession, "", RSSFU_ATTR_AUDIO_BC_MODULATION_NOM_FREQ_DEV_AUDIO, nomFreqDevAudio),
                         4, "Nom Freq Dev Audio");

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;       
}


/*===========================================================================*/
/* Function: Configure Audio BC FM Modulation RDS
/* Purpose:  This function configures audio modulation coding parameters.
/*===========================================================================*/
ViStatus _VI_FUNC rssfu_configureAudioBCModulationRDS(
    ViSession   instrSession,
    ViBoolean   rds,
    ViReal64    freqDevRDS,
    ViReal64    freqDevPilot19kHz
)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rssfu_SetAttributeViBoolean (instrSession, "", RSSFU_ATTR_AUDIO_BC_MODULATION_RDS, rds),
                         2, "RDS");
    viCheckParm (rssfu_SetAttributeViReal64 (instrSession, "", RSSFU_ATTR_AUDIO_BC_MODULATION_FREQ_DEV_RDS, freqDevRDS),
                         3, "Freq Dev RDS");
    viCheckParm (rssfu_SetAttributeViReal64 (instrSession, "", RSSFU_ATTR_AUDIO_BC_MODULATION_FREQ_DEV_PILOT_19KHZ, freqDevPilot19kHz),
                         4, "Freq Dev Pilot 19kHz");

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;           
}

/*===========================================================================*/
/* Function: Configure Audio BC FM MPX Freq Deviation
/* Purpose:  This function sets the nominal deviation for MPX as source left 
/*           (FM).
/*===========================================================================*/
ViStatus _VI_FUNC rssfu_configureAudioMPXFreqDeviation(
    ViSession   instrSession,
    ViReal64    nomFreqDevMPX
)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rssfu_SetAttributeViReal64 (instrSession, "", RSSFU_ATTR_AUDIO_BC_MODULATION_NOM_FREQ_DEV_MPX, nomFreqDevMPX),
                         2, "Nom Freq Dev MPX");

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Read Audio BC FM Modulation Values
/* Purpose:  This function reads selected modulation value.
/*===========================================================================*/
ViStatus _VI_FUNC rssfu_readAudioBCFMModulationValues(
    ViSession   instrSession,
    ViInt32 value,
    ViReal64*   result
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      model [RSSFU_IO_BUFFER_SIZE] = ""; 

    checkErr( Rs_LockSession (instrSession, VI_NULL));
        
    viCheckErr (rssfu_GetAttributeViString (instrSession, "",
                                               RS_ATTR_INSTRUMENT_MODEL,
                                               RSSFU_IO_BUFFER_SIZE, model));
    switch (value){
        case RSSFU_VAL_AUDIO_FREQ_DEV_AUDIO:
            viCheckParm (rssfu_GetAttributeViReal64 (instrSession, "", RSSFU_ATTR_AUDIO_BC_MODULATION_FREQ_DEV_AUDIO, 
                result), 3, "Result");
        break;
        case RSSFU_VAL_AUDIO_FREQ_DEV_MPX:
            if (strstr (model, "SFU") != NULL) {            
                viCheckParm (rssfu_GetAttributeViReal64 (instrSession, "", RSSFU_ATTR_AUDIO_BC_MODULATION_FREQ_DEV_MPX, 
                result), 3, "Result");
            }               
        break;
        
        default:
            viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Value");
    }
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Enable Audio BC FM Special Settings
/* Purpose:  This function enables use of special Audio BC FM coding 
/*           parameters.
/*===========================================================================*/
ViStatus _VI_FUNC rssfu_enableAudioBCFMSpecialSettings(
    ViSession   instrSession,
    ViBoolean   state
)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    viCheckParm (rssfu_SetAttributeViBoolean (instrSession, "", RSSFU_ATTR_AUDIO_BC_FM_SPECIAL_SETTINGS, state),
                         2, "State");
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Configure  Audio BC FM Special Settings
/* Purpose:  This function configures special Audio BC coding parameters.
/*===========================================================================*/
ViStatus _VI_FUNC rssfu_confAudioBCFMSpecialSettings(
    ViSession   instrSession,
    ViBoolean   pilot,
    ViReal64    phaseOffsetPilot19kHz,
    ViReal64    phaseOffsetRDS
)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rssfu_SetAttributeViBoolean (instrSession, "", RSSFU_ATTR_AUDIO_BC_SPECIAL_PILOT, pilot),
                         2, "Pilot");
    viCheckParm (rssfu_SetAttributeViReal64 (instrSession, "", RSSFU_ATTR_AUDIO_BC_SPECIAL_PHASE_OFFSET_PILOT_19KHZ, phaseOffsetPilot19kHz),
                         3, "Phase Offset Pilot 19kHz");
    viCheckParm (rssfu_SetAttributeViReal64 (instrSession, "", RSSFU_ATTR_AUDIO_BC_SPECIAL_PHASE_OFFSET_RDS, phaseOffsetRDS),
                         4, "Phase Offset RDS");

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;   
}

/*===========================================================================*/
/* Function: Configure Audio BC AM Input Signal
/* Purpose:  This function configures Audio Broadcasting AM Input Signal 
/*           parameters.
/*===========================================================================*/
ViStatus _VI_FUNC rssfu_configAudioBCAMInputSignal(
    ViSession   instrSession,
    ViInt32 source,
    ViInt32 impedance
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      model [RSSFU_IO_BUFFER_SIZE] = ""; 
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckErr (rssfu_GetAttributeViString (instrSession, "",
                                               RS_ATTR_INSTRUMENT_MODEL,
                                               RSSFU_IO_BUFFER_SIZE, model));   

    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_AUDIO_BC_AM_SOURCE, source),
                         2, "Source"); 
            
    if (strstr (model, "SFU") != NULL) { 
        if (source == RSSFU_VAL_AUDIO_BC_INPUT_EXT)
            viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_AUDIO_BC_AM_IMPEDNACE_SIGNAL_SOURCE, impedance),
                         3, "Impedance Signal Source");
    }
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;   
}

/*===========================================================================*/
/* Function: Configure Audio BC AM Source
/* Purpose:  This function configures Audio BC AM parameters.
/*===========================================================================*/
ViStatus _VI_FUNC rssfu_confAudioBCAMSource(
    ViSession   instrSession,
    ViReal64    frequency,
    ViReal64    level
)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rssfu_SetAttributeViReal64 (instrSession, "", RSSFU_ATTR_AUDIO_BC_AM_INTERNAL_FREQUENCY, frequency),
                         2, "Frequency");
    viCheckParm (rssfu_SetAttributeViReal64 (instrSession, "", RSSFU_ATTR_AUDIO_BC_AM_INTERNAL_LEVEL, level),
                         3, "Level");
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;   
}

/*===========================================================================*/
/* Function: Read Audio BC AM External Source
/* Purpose:  This function returns the level of an external audio source (AM).
/*===========================================================================*/
ViStatus _VI_FUNC rssfu_readAudioBCAMExternalSource(
    ViSession   instrSession,
    ViReal64*   inputLevel
)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rssfu_GetAttributeViReal64 (instrSession, "", RSSFU_ATTR_AUDIO_BC_AM_INPUT_LEVEL, 
               inputLevel), 2, "Input Level");

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Configure Audio BC AM Modulation Depth
/* Purpose:  This function configures the nominal modulation depth (AM).
/*===========================================================================*/
ViStatus _VI_FUNC rssfu_confAudioBCAMModulationDepth(
    ViSession   instrSession,
    ViReal64    nomModDepth
)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rssfu_SetAttributeViReal64 (instrSession, "", RSSFU_ATTR_AUDIO_BC_AM_NOM_MOD_DEPTH, nomModDepth),
                         2, "Nom Mod Depth");
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Read Audio BC AM Mode Depth
/* Purpose:  This function queries the nominal modulation depth (AM).
/*===========================================================================*/
ViStatus _VI_FUNC rssfu_readAudioBCAMModeDepth(
    ViSession   instrSession,
    ViReal64*   modeDepth
)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rssfu_GetAttributeViReal64 (instrSession, "", RSSFU_ATTR_AUDIO_BC_AM_MOD_DEPTH, 
               modeDepth), 2, "Mode Depth");

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}
     
     
/*****************************************************************************
 *---------------------------- Impairments ----------------------------------*
 *****************************************************************************/
/*============================================================================/
 * Function: Configure Modulator Impairments
 * Purpose:  This function configures modulator impairments parameters.
/============================================================================*/
ViStatus _VI_FUNC rssfu_confImpModulator(ViSession  instrSession,
                                         ViBoolean  state,
                                         ViReal64   iOffset,
                                         ViReal64   qOffset,
                                         ViReal64   quadratureError,
                                         ViReal64   amplitudeImbalance
)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rssfu_SetAttributeViBoolean (instrSession, "", RSSFU_ATTR_MODULATOR_IMPAIRMENTS, state),
                         2, "State");
    viCheckParm (rssfu_SetAttributeViReal64 (instrSession, "", RSSFU_ATTR_MODULATOR_I_OFFSET, iOffset),
                         3, "I Offset"); 
    viCheckParm (rssfu_SetAttributeViReal64 (instrSession, "", RSSFU_ATTR_MODULATOR_Q_OFFSET, qOffset),
                         4, "Q Offset"); 
    viCheckParm (rssfu_SetAttributeViReal64 (instrSession, "", RSSFU_ATTR_MODULATOR_QUADRATURE_ERROR, quadratureError),
                         5, "Quadrature Error");    
    viCheckParm (rssfu_SetAttributeViReal64 (instrSession, "", RSSFU_ATTR_MODULATOR_AMPLITUDE_IMBALANCE, amplitudeImbalance),
                         6, "Amplitude Imbalance");                          
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Configure Baseband Impairments
 * Purpose:  This function configures baseband impairments parameters.
/============================================================================*/
ViStatus _VI_FUNC rssfu_confImpBaseband(ViSession   instrSession,
                                        ViBoolean   state,
                                        ViReal64    iOffset,
                                        ViReal64    qOffset,
                                        ViReal64    quadratureError,
                                        ViReal64    amplitudeImbalance,
                                        ViBoolean   optimizeBBImpforRFOutput)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      model [RSSFU_IO_BUFFER_SIZE] = "";  

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckErr (rssfu_GetAttributeViString (instrSession, "",
                                               RS_ATTR_INSTRUMENT_MODEL,
                                               RSSFU_IO_BUFFER_SIZE, model));
    if (strstr (model, "SFC") == NULL) {
        viCheckParm (rssfu_SetAttributeViBoolean (instrSession, "", RSSFU_ATTR_BB_IMPAIRMENTS, state),
                             2, "State");
        viCheckParm (rssfu_SetAttributeViReal64 (instrSession, "", RSSFU_ATTR_BB_I_OFFSET, iOffset),
                             3, "I Offset"); 
        viCheckParm (rssfu_SetAttributeViReal64 (instrSession, "", RSSFU_ATTR_BB_Q_OFFSET, qOffset),
                             4, "Q Offset"); 
        viCheckParm (rssfu_SetAttributeViReal64 (instrSession, "", RSSFU_ATTR_BB_QUADRATURE_ERROR, quadratureError),
                             5, "Quadrature Error");    
        viCheckParm (rssfu_SetAttributeViReal64 (instrSession, "", RSSFU_ATTR_BB_AMPLITUDE_IMBALANCE, amplitudeImbalance),
                             6, "Amplitude Imbalance"); 
    }
    if (strstr (model, "SFU") != NULL) {
        viCheckParm (rssfu_SetAttributeViBoolean (instrSession, "", RSSFU_ATTR_BB_OPTIMIZE, optimizeBBImpforRFOutput),
                         7, "Optimize BB Imp for RF Output");                          
    }
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Configure Impairments Optimization Mode
 * Purpose:  This function configures the optimization mode for the system
 *           error correction of the I/Q modulator for fast frequency setting
 *           FAST or high quality correction QHIGH.
/============================================================================*/
ViStatus _VI_FUNC rssfu_setImpOptimizationMode (ViSession instrSession,
                                                 ViInt32 impairmentsOptMode)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_BB_OPTIMIZATION_MODE,
                            impairmentsOptMode), 2, "Impairments Optimization Mode");

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/*****************************************************************************
 *---------------------------- Noise ----------------------------------------*
 *****************************************************************************/
/*============================================================================/
 * Function: Configure Noise
 * Purpose:  This function enables the noise generation and configures noise 
 *           mode.
/============================================================================*/
ViStatus _VI_FUNC rssfu_confNoise (ViSession instrSession, ViInt32 noise,
                                   ViBoolean AWGN, ViBoolean impulsiveNoise,
                                   ViBoolean phaseNoise)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      model [RSSFU_IO_BUFFER_SIZE] = "";

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckErr (rssfu_GetAttributeViString (instrSession, "",
                                               RS_ATTR_INSTRUMENT_MODEL,
                                               RSSFU_IO_BUFFER_SIZE, model));
    
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_NOISE, noise),
                         2, "Noise"); 
    
    if (strstr (model, "SFC") == NULL) {
        viCheckParm (rssfu_SetAttributeViBoolean (instrSession, "", RSSFU_ATTR_AWGN, AWGN),
                         3, "AWGN");
        viCheckParm (rssfu_SetAttributeViBoolean (instrSession, "", RSSFU_ATTR_NOISE_IMPULSIVE, impulsiveNoise),
                         4, "Impulsive Noise");
        viCheckParm (rssfu_SetAttributeViBoolean (instrSession, "", RSSFU_ATTR_PHASE_NOISE, phaseNoise),
                         5, "Phase Noise");
    }
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Read Gen Noise State
 * Purpose:  This function queries the noise generator state.
/============================================================================*/
ViStatus _VI_FUNC rssfu_readNoiseGenState (ViSession instrSession,
                                           ViInt32* noiseState)
{
    //ViChar noiseState[])  
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (!noiseState) viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Noise State");
    
    viCheckParm (rssfu_GetAttributeViInt32 (instrSession, "", RSSFU_ATTR_NOISE, 
                        noiseState), 2, "Noise State");
                        
   
    
    /*ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSSFU_IO_BUFFER_SIZE]="\0";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (!noiseState) viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Noise State");
    viCheckErr (rssfu_GetAttributeViString (instrSession, "", RSSFU_ATTR_NOISE,
                                            RSSFU_IO_BUFFER_SIZE, buffer));
    strncpy (noiseState, buffer, strlen(buffer));
    noiseState [strlen(buffer)] = '\0';
      */             
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Configure Noise CN
 * Purpose:  This function sets the carrier/noise ratio (C/N).
/============================================================================*/
ViStatus _VI_FUNC rssfu_confNoiseCn (ViSession instrSession, ViReal64 cN)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rssfu_SetAttributeViReal64 (instrSession, "", RSSFU_ATTR_NOISE_CN, cN), 4, "C/N");
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Read Noise CN
 * Purpose:  This function queries the carrier/noise ratio.
/============================================================================*/
ViStatus _VI_FUNC rssfu_readNoiseCN (ViSession instrSession, ViReal64 *cnr)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (!cnr) viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "CNR");
    viCheckErr (rssfu_GetAttributeViReal64 (instrSession, "", RSSFU_ATTR_NOISE_CN,
                                            cnr));
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Read Noise Eb/N0
 * Purpose:  This function queries the carrier/noise ratio.
/============================================================================*/
ViStatus _VI_FUNC rssfu_readNoiseEbN0 (ViSession instrSession, ViReal64 *ebN0)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (!ebN0) viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Eb/N0");
    viCheckErr (rssfu_GetAttributeViReal64 (instrSession, "", RSSFU_ATTR_NOISE_EB_N0_RATIO,
                                            ebN0));
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Configure Pulse Noise
 * Purpose:  This function configures pulse noise parameters.
/============================================================================*/
ViStatus _VI_FUNC rssfu_confNoisePulse (ViSession instrSession,
                                        ViReal64 cI,
                                        ViInt32 frameDuration,
                                        ViInt32 pulsesperBurst,
                                        ViReal64 pulseSpacingMin,
                                        ViReal64 pulseSpacingMax)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      model [RSSFU_IO_BUFFER_SIZE] = "";
    ViReal64    frameDurationArr[] = {0.01, 0.1, 1.0};
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckErr (rssfu_GetAttributeViString (instrSession, "",
                                               RS_ATTR_INSTRUMENT_MODEL,
                                               RSSFU_IO_BUFFER_SIZE, model));
    
    viCheckParm (rssfu_SetAttributeViReal64 (instrSession, "", RSSFU_ATTR_NOISE_CI, cI),
                         2, "C/I");
    if (strstr (model, "BTC") != NULL) {   
        viCheckParm (rssfu_SetAttributeViReal64 (instrSession, "", RSSFU_ATTR_NOISE_FRAME_DURATION_BTC, frameDurationArr[frameDuration]),
                             3, "Frame Duration");                      
    }
    else
        {  
        viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_NOISE_FRAME_DURATION, frameDuration),
                             3, "Frame Duration");                      
    }
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_NOISE_PULSES_PER_BURST, pulsesperBurst),
                         4, "Pulses per Burst"); 
    viCheckParm (rssfu_SetAttributeViReal64 (instrSession, "", RSSFU_ATTR_NOISE_PULSE_SPACING_MAX, pulseSpacingMax),
                         6, "Pulse Spacing Max");
    viCheckParm (rssfu_SetAttributeViReal64 (instrSession, "", RSSFU_ATTR_NOISE_PULSE_SPACING_MIN, pulseSpacingMin),
                         5, "Pulse Spacing Min"); 
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/*============================================================================/
 * Function: Configure Phase Noise
 * Purpose:  This function configures phase noise parameters.
/============================================================================*/
ViStatus _VI_FUNC rssfu_confNoisePhase(ViSession   instrSession,
                                       ViString    ssbShapeProfile,
                                       ViReal64    phaseNoiseat100Hz)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      model [RSSFU_IO_BUFFER_SIZE] = "";  

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckErr (rssfu_GetAttributeViString (instrSession, "",
                                               RS_ATTR_INSTRUMENT_MODEL,
                                               RSSFU_IO_BUFFER_SIZE, model));   
    
    if (strstr (model, "SFC") == NULL) { 
        viCheckParm (rssfu_SetAttributeViString (instrSession, "", RSSFU_ATTR_NOISE_SSB_SHAPE_PROFILE, ssbShapeProfile),
                             2, "SSB Shape Profile"); 
        viCheckParm (rssfu_SetAttributeViReal64 (instrSession, "", RSSFU_ATTR_PHASE_NOISE_LEVEL, phaseNoiseat100Hz),
                             3, "Phase Noise at 100 Hz"); 
    }
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Get Level Of Phase Noise
 * Purpose:  This function queries the level of phase noise at a specific
 *           frequency offset.
/============================================================================*/
ViStatus _VI_FUNC rssfu_GetLevelOfPhaseNoise (ViSession instrSession,
                                            ViInt32 offset, ViReal64* result)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSSFU_IO_BUFFER_SIZE] = ""; 
    ViChar      model [RSSFU_IO_BUFFER_SIZE] = ""; 
        
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckErr (rssfu_GetAttributeViString (instrSession, "",
                                               RS_ATTR_INSTRUMENT_MODEL,
                                               RSSFU_IO_BUFFER_SIZE, model));   

    if (strstr (model, "SFU") != NULL) { 
                
        if ((offset < RSSFU_VAL_PHASE_NOISE_1E2) || (offset > RSSFU_VAL_PHASE_NOISE_1E6))
            viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Offset");
        
        
        if (Rs_Simulating(instrSession) == 0) {                /* call only when locked */

        
        if (!result) viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Result");
    
        sprintf (buffer,":NOIS:PHAS:AT? %s", noiseOffsetArr[offset]);
    
        viCheckErr( viQueryf (instrSession, buffer, "%Le", result));
        }   
    }
    
    viCheckErr(rssfu_CheckStatus (instrSession));       
     
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Noise Bandwidth Settings
 * Purpose:  This function configures noise generation parameters.
/============================================================================*/
ViStatus _VI_FUNC rssfu_confNoiseBwSettings(ViSession   instrSession,
                                            ViBoolean   bandwidthCoupling,
                                            ViReal64    receiverBandwidth)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
   
    viCheckParm (rssfu_SetAttributeViBoolean (instrSession, "", RSSFU_ATTR_NOISE_BANDWIDTH_COUPLING, bandwidthCoupling),
                         2, "Bandwidth Coupling");
    if (!bandwidthCoupling)                         
        viCheckParm (rssfu_SetAttributeViReal64 (instrSession, "", RSSFU_ATTR_NOISE_RECEIVER_BANDWIDTH, receiverBandwidth),
                         3, "Receiver Bandwidth");                         
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Read Gen Noise Bandwidth
 * Purpose:  This function queries the generated noise bandwidth.
/============================================================================*/
ViStatus _VI_FUNC rssfu_readNoiseGenBand (ViSession instrSession,
                                          ViReal64 *generatedNoiseBandwidth)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (!generatedNoiseBandwidth) viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Generated Noise Bandwidth");
    viCheckErr (rssfu_GetAttributeViReal64 (instrSession, "", RSSFU_ATTR_GEN_NOISE_BANDWIDTH,
                                            generatedNoiseBandwidth));
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/*============================================================================/
 * Function: Noise File Operations
 * Purpose:  This function stores or recalls noise settings in the specified
 *           file.
/============================================================================*/
ViStatus _VI_FUNC rssfu_noiseFileOperations (ViSession instrSession,
                                                         ViInt32 operation,
                                                         ViString source)
{
    ViStatus    error   = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
  
    if (Rs_Simulating(instrSession) == 0){                /* call only when locked */
    switch (operation){
        case RSSFU_VAL_SAVE:
             viCheckParm (rssfu_SetAttributeViString (instrSession, "", RSSFU_ATTR_NOISE_STORE, source),
                         3, "Source"); 
        break;
        
        case RSSFU_VAL_RECALL:
             viCheckParm (rssfu_SetAttributeViString (instrSession, "", RSSFU_ATTR_NOISE_SELECT, source),
                         3, "Source"); 
        break;

        }
    }
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}                                                         
  

/*****************************************************************************
 *---------------------------- Obsolete Noise -------------------------------*
 *****************************************************************************/
/*===========================================================================*/
/* Function: Configure Noise
/* Purpose:  This function enables the noise generation and configures noise 
/*           mode.
/*===========================================================================*/
ViStatus _VI_FUNC rssfu_confNoise_obs(ViSession   instrSession,
                                  ViInt32 state,
                                  ViInt32 mode)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_NOISE, state),
                         2, "State"); 

    mode;
                          
Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Configure Noise Parameters
/* Purpose:  This function configures noise generation parameters.
/*===========================================================================*/
ViStatus _VI_FUNC rssfu_confNoiseParameters(ViSession   instrSession,
                                            ViReal64    cn,
                                            ViReal64    cnOffset,
                                            ViBoolean   bandwidthCoupling,
                                            ViReal64    receiverBandwidth)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rssfu_SetAttributeViReal64 (instrSession, "", RSSFU_ATTR_NOISE_CN, cn),
                         2, "C/N"); 
 
    viCheckParm (rssfu_SetAttributeViBoolean (instrSession, "", RSSFU_ATTR_NOISE_BANDWIDTH_COUPLING, bandwidthCoupling),
                         4, "Bandwidth Coupling");
    if (!bandwidthCoupling)
    viCheckParm (rssfu_SetAttributeViReal64 (instrSession, "", RSSFU_ATTR_NOISE_RECEIVER_BANDWIDTH, receiverBandwidth),
                         5, "Receiver Bandwidth");                         

    cnOffset;
Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Configure Pulse Noise
/* Purpose:  This function configures pulse noise parameters.
/*===========================================================================*/
ViStatus _VI_FUNC rssfu_confNoisePulse_obs(ViSession instrSession,
                                       ViInt32 pulsesperBurst,
                                       ViReal64 pulseSpacingMin,
                                       ViReal64 pulseSpacingMax)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_NOISE_PULSES_PER_BURST, pulsesperBurst),
                         2, "Pulses per Burst"); 
    viCheckParm (rssfu_SetAttributeViReal64 (instrSession, "", RSSFU_ATTR_NOISE_PULSE_SPACING_MAX, pulseSpacingMax),
                         4, "Pulse Spacing Max");
    viCheckParm (rssfu_SetAttributeViReal64 (instrSession, "", RSSFU_ATTR_NOISE_PULSE_SPACING_MIN, pulseSpacingMin),
                         3, "Pulse Spacing Min"); 
Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}



/*****************************************************************************
 *---------------------------- Fading ---------------------------------------*
 *****************************************************************************/
/*============================================================================/
 * Function: Enable Fading
 * Purpose:  This function enables/disables fading simulation.
/============================================================================*/
ViStatus _VI_FUNC rssfu_actEnableFading (ViSession instrSession,
                                         ViInt32 fader, ViBoolean state)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buf1[RSSFU_TOKEN_BUFFER_SIZE] = "";
    ViInt32     l_fader = fader - 1;

    if ((fader < 1) || (fader > 2))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Fader"); 

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    checkErr (rssfu_GetActualRepCaps (instrSession, RSSFU_ATTR_FADING_STATE, "Fader", l_fader, buf1));

    viCheckParm (rssfu_SetAttributeViBoolean (instrSession, buf1, RSSFU_ATTR_FADING_STATE, state),
                         3, "State");
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Restart Fading
 * Purpose:  This function restarts fading simulation.
/============================================================================*/
ViStatus _VI_FUNC rssfu_actRestartFading(ViSession instrSession, ViInt32 fader)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buf1[RSSFU_TOKEN_BUFFER_SIZE] = "";
    ViInt32     l_fader = fader - 1;

    if ((fader < 1) || (fader > 2))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Fader"); 

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    checkErr (rssfu_GetActualRepCaps (instrSession, RSSFU_ATTR_FADING_RESTART, "Fader", l_fader, buf1));

    viCheckErr (rssfu_SetAttributeViString (instrSession, buf1, RSSFU_ATTR_FADING_RESTART, "")); 
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Fading Configuration
 * Purpose:  This function enables/disables the selected path for the
 *           selected fading configuration.
/============================================================================*/
ViStatus _VI_FUNC rssfu_actFadingConfig(ViSession instrSession, ViInt32 fader,
                                              ViInt32 configuration,
                                              ViBoolean state)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buf1[RSSFU_TOKEN_BUFFER_SIZE] = "";
    ViInt32     l_fader = fader - 1;

    if ((fader < 1) || (fader > 2))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Fader"); 
    if ((configuration < RSSFU_VAL_FADING_STANDARD) || (configuration > RSSFU_VAL_FADING_2PATH_DYNAMIC))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 3, "Configuration"); 


    checkErr( Rs_LockSession (instrSession, VI_NULL));

 
    switch (configuration){
        case RSSFU_VAL_FADING_STANDARD:
            checkErr (rssfu_GetActualRepCaps (instrSession, RSSFU_ATTR_FADING_PAR_DEL_STATE_MODE, "Fader", l_fader, buf1));
            viCheckParm (rssfu_SetAttributeViBoolean (instrSession, buf1, RSSFU_ATTR_FADING_PAR_DEL_STATE_MODE, state), 
                    4, "State");     
        break;                    
        case RSSFU_VAL_FADING_DEL30:
            checkErr (rssfu_GetActualRepCaps (instrSession, RSSFU_ATTR_FADING_PAR_DEL30_STATE_MODE, "Fader", l_fader, buf1));
            viCheckParm (rssfu_SetAttributeViBoolean (instrSession, buf1, RSSFU_ATTR_FADING_PAR_DEL30_STATE_MODE, state), 
                    4, "State");
        break;                    
        case RSSFU_VAL_FADING_DEL50:
            checkErr (rssfu_GetActualRepCaps (instrSession, RSSFU_ATTR_FADING_PAR_DEL50_STATE_MODE, "Fader", l_fader, buf1));
            viCheckParm (rssfu_SetAttributeViBoolean (instrSession, buf1, RSSFU_ATTR_FADING_PAR_DEL50_STATE_MODE, state), 
                    4, "State");
        break;
        case RSSFU_VAL_FADING_BIRTH_DEATH:
            checkErr (rssfu_GetActualRepCaps (instrSession, RSSFU_ATTR_FADING_PAR_STATE_MODE_BIRTH_DEATH, "Fader", l_fader, buf1));
            viCheckParm (rssfu_SetAttributeViBoolean (instrSession, buf1, RSSFU_ATTR_FADING_PAR_STATE_MODE_BIRTH_DEATH, state), 
                    4, "State");
        break;
        case RSSFU_VAL_FADING_MOVING_PROP:
            checkErr (rssfu_GetActualRepCaps (instrSession, RSSFU_ATTR_FADING_PAR_MOVING_PATH_STATE_MODE, "Fader", l_fader, buf1));
            viCheckParm (rssfu_SetAttributeViBoolean (instrSession, buf1, RSSFU_ATTR_FADING_PAR_MOVING_PATH_STATE_MODE, state), 
                    4, "State");
        break;
        
        case RSSFU_VAL_FADING_2PATH_DYNAMIC:
            checkErr (rssfu_GetActualRepCaps (instrSession, RSSFU_ATTR_FADING_PAR_MOVING_PATH_STATE_MODE, "Fader", l_fader, buf1));
            viCheckParm (rssfu_SetAttributeViBoolean (instrSession, buf1, RSSFU_ATTR_FADING_2PATH_DYNAMIC_DELAY_STATE, state), 
                    4, "State");
        break;
    }   
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/*============================================================================/
 * Function: Configure Fading 
 * Purpose:  This function sets the fading configuration.
/============================================================================*/
ViStatus _VI_FUNC rssfu_configureFading (ViSession instrSession,
                                             ViInt32 fader, ViInt32 configuration)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buf1[RSSFU_TOKEN_BUFFER_SIZE] = "";
    ViInt32     l_fader = fader - 1;

    if ((fader < 1) || (fader > 2))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Fader"); 

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    checkErr (rssfu_GetActualRepCaps (instrSession, RSSFU_ATTR_FADING_CONFIGURATION, "Fader", l_fader, buf1));

    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, buf1, RSSFU_ATTR_FADING_CONFIGURATION, configuration), 
                    3, "Configuration"); 
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Store and Load Fading Settings
 * Purpose:  This function stores or recalls fading simulator settings in the
 *           specified file.
/============================================================================*/
ViStatus _VI_FUNC rssfu_storeLoadFadingSettings (ViSession instrSession,
                                                         ViInt32 fader,
                                                         ViInt32 operation,
                                                         ViString source)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buf1[RSSFU_TOKEN_BUFFER_SIZE] = "";
    ViInt32     l_fader = fader - 1;

    if ((fader < 1) || (fader > 2))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Fader"); 
    if ((operation < RSSFU_VAL_SAVE) || (operation > RSSFU_VAL_DELETE))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 3, "Operation"); 
        
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    checkErr (rssfu_GetActualRepCaps (instrSession, RSSFU_ATTR_FADING_STORE_FILE, "Fader", l_fader, buf1));

    if (Rs_Simulating(instrSession) == 0){                /* call only when locked */
    switch (operation){
        case RSSFU_VAL_SAVE:
             viCheckParm (rssfu_SetAttributeViString (instrSession, buf1, RSSFU_ATTR_FADING_STORE_FILE, source),
                         4, "Source"); 
        break;
        
        case RSSFU_VAL_RECALL:
             viCheckParm (rssfu_SetAttributeViString (instrSession, buf1, RSSFU_ATTR_FADING_LOAD_FILE, source),
                         4, "Source"); 
        break;
        
        case RSSFU_VAL_DELETE:
             viCheckParm (rssfu_SetAttributeViString (instrSession, buf1, RSSFU_ATTR_FADING_DELETE_FILE, source),
                         4, "Source"); 
        break;

        }
    }
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}                                                         
    

/*============================================================================/
 * Function: Fading Catalog
 * Purpose:  This function reads out the files in the default directory.
/============================================================================*/
ViStatus _VI_FUNC rssfu_fadingCatalog (ViSession instrSession, ViInt32 fader,
                                            ViString directory, ViInt32 bufferSize,
                                            ViChar output[],
                                            ViInt32 *returnedValues)
{
   
    ViChar*     pbuffer = NULL;
    ViUInt32    count = 0, len = 0;
    ViStatus    error   = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
  
    if ((fader < 1) || (fader > 2))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Fader"); 
    
    if (Rs_Simulating(instrSession) == 0){                /* call only when locked */

    if (bufferSize != 0 && output == VI_NULL)
        return VI_ERROR_PARAMETER5;

    viCheckErr( viPrintf (instrSession, ":MMEM:CDIR '%s'", directory)); 
    viCheckErr( viPrintf (instrSession, "SOUR%ld:FSIM:CAT?\n", fader));
   
    
    viCheckErr( rssfu_WaitForOPC (instrSession, MAX_OPC_TIME)); 
    viCheckErr( rssfu_ReadDataUnknownLength(instrSession, &pbuffer, &count));
    
    if (output != NULL) 
    {
        len = (ViUInt32) ((bufferSize > (ViInt32) count) ? (ViInt32) count : bufferSize);
        strncpy(output, pbuffer, len);
        output[len] = '\0';
    }
    
    free(pbuffer);
    if (returnedValues) *returnedValues = (ViInt32) count;
        
    viCheckErr(rssfu_CheckStatus (instrSession)); 
    }
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;   
}
 
/*===========================================================================*/
/* Function: Configure Fading Source
/* Purpose:  This function set the source for FSIMulator2 and as a query 
/*           show the input sources for FSIMulator1 and FSIMulator2.
/*===========================================================================*/
ViStatus _VI_FUNC rssfu_configureFadingSource(
    ViSession   instrSession,
    ViInt32 fader,
    ViInt32 source
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buf1[RSSFU_TOKEN_BUFFER_SIZE] = "";
    ViInt32     l_fader = fader - 1;

    if ((fader < 1) || (fader > 2))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Fader"); 

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    checkErr (rssfu_GetActualRepCaps (instrSession, RSSFU_ATTR_FADING_SOURCE, "Fader", l_fader, buf1));


    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, buf1, RSSFU_ATTR_FADING_SOURCE,
                                            source), 3, "Source"); 
                                           
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Configure Fading Standard Set
 * Purpose:  This function selects fading simulator standard set.
/============================================================================*/
ViStatus _VI_FUNC rssfu_confFadingStandardSet(ViSession instrSession, ViInt32 fader,
                                              ViInt32 standardSet)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buf1[RSSFU_TOKEN_BUFFER_SIZE] = "";
    ViInt32     l_fader = fader - 1;

    if ((fader < 1) || (fader > 2))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Fader"); 

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    checkErr (rssfu_GetActualRepCaps (instrSession, RSSFU_ATTR_FADING_PARAMETER_SET, "Fader", l_fader, buf1));

    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, buf1, RSSFU_ATTR_FADING_PARAMETER_SET, standardSet), 
                    3, "Standart Set"); 
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/*============================================================================/
 * Function: Enable Fading Path
 * Purpose:  This function enables/disables the selected path for the 
 *           selected fading configuration.
/============================================================================*/
ViStatus _VI_FUNC rssfu_actEnableFadingPath (ViSession instrSession,
                                             ViInt32 configuration,
                                             ViInt32 fader, ViInt32 group,
                                             ViInt32 path, ViBoolean state)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSSFU_IO_BUFFER_SIZE] = "";
    ViChar      buf1[RSSFU_TOKEN_BUFFER_SIZE] = "";
    ViChar      *p2buf;
    ViInt32     l_group, l_path, l_fader;
    l_group = group - 1;
    l_path  = path - 1;
    l_fader = fader - 1;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if ((configuration < RSSFU_VAL_FADING_STANDARD) || (configuration > RSSFU_VAL_FADING_DEL50))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Configuration");
    if ((fader < 1) || (fader > 2))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 3, "Fader");
    if ((group < 1) || (group > 8))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 4, "Group");
    if ((path < 1) || (path > 5))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 5, "Path"); 

    
    p2buf=buffer;
    switch (configuration){
        case RSSFU_VAL_FADING_STANDARD:
            checkErr (rssfu_GetActualRepCaps (instrSession, RSSFU_ATTR_FADING_PAR_DEL_STATE, "Fader", l_fader, buf1));
            p2buf+=sprintf(p2buf,"%s,",buf1);
            checkErr (rssfu_GetActualRepCaps (instrSession, RSSFU_ATTR_FADING_PAR_DEL_STATE, "Group", l_group, buf1));
            p2buf+=sprintf(p2buf,"%s,",buf1);
            checkErr (rssfu_GetActualRepCaps (instrSession, RSSFU_ATTR_FADING_PAR_DEL_STATE, "Path", l_path, buf1));
            p2buf+=sprintf(p2buf,"%s",buf1);
            viCheckParm (rssfu_SetAttributeViBoolean (instrSession, buffer, RSSFU_ATTR_FADING_PAR_DEL_STATE, state), 
                    6, "State");     
        break;                    
        case RSSFU_VAL_FADING_DEL30:
            checkErr (rssfu_GetActualRepCaps (instrSession, RSSFU_ATTR_FADING_PAR_DEL30_STATE, "Fader", l_fader, buf1));
            p2buf+=sprintf(p2buf,"%s,",buf1);
            checkErr (rssfu_GetActualRepCaps (instrSession, RSSFU_ATTR_FADING_PAR_DEL30_STATE, "Group", l_group, buf1));
            p2buf+=sprintf(p2buf,"%s,",buf1);
            checkErr (rssfu_GetActualRepCaps (instrSession, RSSFU_ATTR_FADING_PAR_DEL30_STATE, "Path", l_path, buf1));
            p2buf+=sprintf(p2buf,"%s",buf1);
            viCheckParm (rssfu_SetAttributeViBoolean (instrSession, buffer, RSSFU_ATTR_FADING_PAR_DEL30_STATE, state), 
                    6, "State");
        break;                    
        case RSSFU_VAL_FADING_DEL50:
            checkErr (rssfu_GetActualRepCaps (instrSession, RSSFU_ATTR_FADING_PAR_DEL50_STATE, "Fader", l_fader, buf1));
            p2buf+=sprintf(p2buf,"%s,",buf1);
            checkErr (rssfu_GetActualRepCaps (instrSession, RSSFU_ATTR_FADING_PAR_DEL50_STATE, "Group", l_group, buf1));
            p2buf+=sprintf(p2buf,"%s,",buf1);
            checkErr (rssfu_GetActualRepCaps (instrSession, RSSFU_ATTR_FADING_PAR_DEL50_STATE, "Path", l_path, buf1));
            p2buf+=sprintf(p2buf,"%s",buf1);
            viCheckParm (rssfu_SetAttributeViBoolean (instrSession, buffer, RSSFU_ATTR_FADING_PAR_DEL50_STATE, state), 
                    6, "State");
        break;                    
    }    
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Configure Fading Path Profile
 * Purpose:  This function select the fading profile for the paths.
/============================================================================*/
ViStatus _VI_FUNC rssfu_confFadingPathProfile(ViSession   instrSession,
                                              ViInt32 configuration,
                                              ViInt32 fader,
                                              ViInt32 group,
                                              ViInt32 path,
                                              ViInt32 profile)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSSFU_IO_BUFFER_SIZE] = "";
    ViChar      buf1[RSSFU_TOKEN_BUFFER_SIZE] = "";
    ViChar      *p2buf;
    ViInt32     l_group, l_path, l_fader;
    l_group = group - 1;
    l_path  = path - 1;
    l_fader = fader - 1; 
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    if ((configuration < RSSFU_VAL_FADING_STANDARD) || (configuration > RSSFU_VAL_FADING_DEL50))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Configuration");
    if ((fader < 1) || (fader > 2))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 3, "Fader"); 
    if ((group < 1) || (group > 8))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 4, "Group");
    if ((path < 1) || (path > 5))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 5, "Path"); 

    p2buf=buffer;
    
    switch (configuration){
        case RSSFU_VAL_FADING_STANDARD:
            checkErr (rssfu_GetActualRepCaps (instrSession, RSSFU_ATTR_FADING_PAR_DEL_PROFILE, "Fader", l_fader, buf1));
            p2buf+=sprintf(p2buf,"%s,",buf1);
            checkErr (rssfu_GetActualRepCaps (instrSession, RSSFU_ATTR_FADING_PAR_DEL_PROFILE, "Group", l_group, buf1));
            p2buf+=sprintf(p2buf,"%s,",buf1);
            checkErr (rssfu_GetActualRepCaps (instrSession, RSSFU_ATTR_FADING_PAR_DEL_PROFILE, "Path", l_path, buf1));
            p2buf+=sprintf(p2buf,"%s",buf1);
            viCheckParm (rssfu_SetAttributeViInt32 (instrSession, buffer, RSSFU_ATTR_FADING_PAR_DEL_PROFILE, profile), 
                    6, "Profile");     
        break;                    
        case RSSFU_VAL_FADING_DEL30:
            checkErr (rssfu_GetActualRepCaps (instrSession, RSSFU_ATTR_FADING_PAR_DEL30_PROFILE, "Fader", l_fader, buf1));
            p2buf+=sprintf(p2buf,"%s,",buf1);
            checkErr (rssfu_GetActualRepCaps (instrSession, RSSFU_ATTR_FADING_PAR_DEL30_PROFILE, "Group", l_group, buf1));
            p2buf+=sprintf(p2buf,"%s,",buf1);
            checkErr (rssfu_GetActualRepCaps (instrSession, RSSFU_ATTR_FADING_PAR_DEL30_PROFILE, "Path", l_path, buf1));
            p2buf+=sprintf(p2buf,"%s",buf1);
            viCheckParm (rssfu_SetAttributeViInt32 (instrSession, buffer, RSSFU_ATTR_FADING_PAR_DEL30_PROFILE, profile), 
                    6, "Profile");
        break;                    
        case RSSFU_VAL_FADING_DEL50:
            checkErr (rssfu_GetActualRepCaps (instrSession, RSSFU_ATTR_FADING_PAR_DEL50_PROFILE, "Fader", l_fader, buf1));
            p2buf+=sprintf(p2buf,"%s,",buf1);
            checkErr (rssfu_GetActualRepCaps (instrSession, RSSFU_ATTR_FADING_PAR_DEL50_PROFILE, "Group", l_group, buf1));
            p2buf+=sprintf(p2buf,"%s,",buf1);
            checkErr (rssfu_GetActualRepCaps (instrSession, RSSFU_ATTR_FADING_PAR_DEL50_PROFILE, "Path", l_path, buf1));
            p2buf+=sprintf(p2buf,"%s",buf1);
            viCheckParm (rssfu_SetAttributeViInt32 (instrSession, buffer, RSSFU_ATTR_FADING_PAR_DEL50_PROFILE, profile), 
                    6, "Profile");
        break;                    
    }    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Configure Fading Path Profile Table Numeric
 * Purpose:  This function sets the value of selected numeric parameter of 
 *           profile table.
/============================================================================*/
ViStatus _VI_FUNC rssfu_confFadingPathProfileTableNumeric(ViSession instrSession,
                                                          ViInt32 configuration,
                                                          ViInt32 fader,
                                                          ViInt32 group,
                                                          ViInt32 path,
                                                          ViInt32 parameterName,
                                                          ViReal64 value)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSSFU_IO_BUFFER_SIZE] = "";
    ViChar      buf1[RSSFU_TOKEN_BUFFER_SIZE] = "";
    ViChar      *p2buf;
    ViAttr     attrArr[] = {RSSFU_ATTR_FADING_PAR_DEL_PATH_LOSS,RSSFU_ATTR_FADING_PAR_DEL_BASIC_DELAY,
                             RSSFU_ATTR_FADING_PAR_DEL_ADDITIONAL_DELAY,RSSFU_ATTR_FADING_PAR_DEL_POWER_RATIO,
                             RSSFU_ATTR_FADING_PAR_DEL_CONST_PHASE,RSSFU_ATTR_FADING_PAR_DEL_SPEED,
                             RSSFU_ATTR_FADING_PAR_DEL_FREQ_RATIO,RSSFU_ATTR_FADING_PAR_DEL_COEFFICIENT,
                             RSSFU_ATTR_FADING_PAR_DEL_PHASE,RSSFU_ATTR_FADING_PAR_DEL_LOCAL_CONSTANT,
                             RSSFU_ATTR_FADING_PAR_DEL_STANDARD_DEVIATION};
    ViAttr     attr30Arr[] = {RSSFU_ATTR_FADING_PAR_DEL30_PATH_LOSS,RSSFU_ATTR_FADING_PAR_DEL30_BASIC_DELAY,
                             RSSFU_ATTR_FADING_PAR_DEL30_ADDITIONAL_DELAY,RSSFU_ATTR_FADING_PAR_DEL30_POWER_RATIO,
                             RSSFU_ATTR_FADING_PAR_DEL30_CONST_PHASE,RSSFU_ATTR_FADING_PAR_DEL30_SPEED,
                             RSSFU_ATTR_FADING_PAR_DEL30_FREQ_RATIO,RSSFU_ATTR_FADING_PAR_DEL30_COEFFICIENT,
                             RSSFU_ATTR_FADING_PAR_DEL30_PHASE,RSSFU_ATTR_FADING_PAR_DEL30_LOCAL_CONSTANT,
                             RSSFU_ATTR_FADING_PAR_DEL30_STANDARD_DEVIATION};   
    ViAttr     attr50Arr[] = {RSSFU_ATTR_FADING_PAR_DEL50_PATH_LOSS,RSSFU_ATTR_FADING_PAR_DEL50_BASIC_DELAY,
                             RSSFU_ATTR_FADING_PAR_DEL50_ADDITIONAL_DELAY,RSSFU_ATTR_FADING_PAR_DEL50_POWER_RATIO,
                             RSSFU_ATTR_FADING_PAR_DEL50_CONST_PHASE,RSSFU_ATTR_FADING_PAR_DEL50_SPEED,
                             RSSFU_ATTR_FADING_PAR_DEL50_FREQ_RATIO,RSSFU_ATTR_FADING_PAR_DEL50_COEFFICIENT,
                             RSSFU_ATTR_FADING_PAR_DEL50_PHASE,RSSFU_ATTR_FADING_PAR_DEL50_LOCAL_CONSTANT,
                             RSSFU_ATTR_FADING_PAR_DEL50_STANDARD_DEVIATION};   
                             
    ViInt32     l_group, l_path, l_fader;
    l_group = group - 1;
    l_path  = path - 1;
    l_fader = fader - 1;

    checkErr( Rs_LockSession (instrSession, VI_NULL)); 
    
    if ((configuration < RSSFU_VAL_FADING_STANDARD) || (configuration > RSSFU_VAL_FADING_DEL50))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Configuration");
    if ((fader < 1) || (fader > 2))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 3, "Fader"); 
    if ((group < 1) || (group > 8))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 4, "Group");
    if ((path < 1) || (path > 5))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 5, "Path"); 
    if ((parameterName < RSSFU_VAL_FADING_PATH_LOSS) || (parameterName > RSSFU_VAL_FADING_RES_STD_DEV))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 6, "Parameter Name");        
    
    p2buf=buffer;
    if (parameterName != RSSFU_VAL_FADING_BASIC_DELAY)
    {
        switch (configuration){
            case RSSFU_VAL_FADING_STANDARD:
                checkErr (rssfu_GetActualRepCaps (instrSession, attrArr[parameterName], "Fader", l_fader, buf1));
                p2buf+=sprintf(p2buf,"%s,",buf1);
                checkErr (rssfu_GetActualRepCaps (instrSession, attrArr[parameterName], "Group", l_group, buf1));
                p2buf+=sprintf(p2buf,"%s,",buf1);
                checkErr (rssfu_GetActualRepCaps (instrSession, attrArr[parameterName], "Path", l_path, buf1));
                p2buf+=sprintf(p2buf,"%s",buf1);
                viCheckParm (rssfu_SetAttributeViReal64 (instrSession, buffer, attrArr[parameterName], value), 
                            7, "Value");     
            break;                    
            case RSSFU_VAL_FADING_DEL30:
                checkErr (rssfu_GetActualRepCaps (instrSession, attr30Arr[parameterName], "Fader", l_fader, buf1));
                p2buf+=sprintf(p2buf,"%s,",buf1);
                checkErr (rssfu_GetActualRepCaps (instrSession, attr30Arr[parameterName], "Group", l_group, buf1));
                p2buf+=sprintf(p2buf,"%s,",buf1);
                checkErr (rssfu_GetActualRepCaps (instrSession, attr30Arr[parameterName], "Path", l_path, buf1));
                p2buf+=sprintf(p2buf,"%s",buf1);
                viCheckParm (rssfu_SetAttributeViReal64 (instrSession, buffer, attr30Arr[parameterName], value), 
                        7, "Value");
            break;                    
            case RSSFU_VAL_FADING_DEL50:
                checkErr (rssfu_GetActualRepCaps (instrSession, attr50Arr[parameterName], "Fader", l_fader, buf1));
                p2buf+=sprintf(p2buf,"%s,",buf1);
                checkErr (rssfu_GetActualRepCaps (instrSession, attr50Arr[parameterName], "Group", l_group, buf1));
                p2buf+=sprintf(p2buf,"%s,",buf1);
                checkErr (rssfu_GetActualRepCaps (instrSession, attr50Arr[parameterName], "Path", l_path, buf1));
                p2buf+=sprintf(p2buf,"%s",buf1);
                viCheckParm (rssfu_SetAttributeViReal64 (instrSession, buffer, attr50Arr[parameterName], value), 
                        7, "Value");
            break;                    
        }    
     }
     else
     {
        switch (configuration){
            case RSSFU_VAL_FADING_STANDARD:
                checkErr (rssfu_GetActualRepCaps (instrSession, attrArr[parameterName], "Fader", l_fader, buf1));
                p2buf+=sprintf(p2buf,"%s,",buf1);
                checkErr (rssfu_GetActualRepCaps (instrSession, attrArr[parameterName], "Group", l_group, buf1));
                p2buf+=sprintf(p2buf,"%s",buf1);
                viCheckParm (rssfu_SetAttributeViReal64 (instrSession, buffer, attrArr[parameterName], value), 
                        7, "Value");     
            break;                    
            case RSSFU_VAL_FADING_DEL30:
                checkErr (rssfu_GetActualRepCaps (instrSession, attr30Arr[parameterName], "Fader", l_fader, buf1));
                p2buf+=sprintf(p2buf,"%s,",buf1);
                checkErr (rssfu_GetActualRepCaps (instrSession, attr30Arr[parameterName], "Group", l_group, buf1));
                p2buf+=sprintf(p2buf,"%s",buf1);
                viCheckParm (rssfu_SetAttributeViReal64 (instrSession, buffer, attr30Arr[parameterName], value), 
                        7, "Value");
            break;                    
            case RSSFU_VAL_FADING_DEL50:
                checkErr (rssfu_GetActualRepCaps (instrSession, attr50Arr[parameterName], "Fader", l_fader, buf1));
                p2buf+=sprintf(p2buf,"%s,",buf1);
                checkErr (rssfu_GetActualRepCaps (instrSession, attr50Arr[parameterName], "Group", l_group, buf1));
                p2buf+=sprintf(p2buf,"%s",buf1);
                viCheckParm (rssfu_SetAttributeViReal64 (instrSession, buffer, attr50Arr[parameterName], value), 
                        7, "Value");
            break;                    
        }    
    }    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Configure Fading Path Profile Table Boolean
 * Purpose:  This function sets the value of selected boolean parameter of 
 *           profile table.
/============================================================================*/
ViStatus _VI_FUNC rssfu_confFadingPathProfileTableBoolean(ViSession instrSession,
                                                          ViInt32 configuration,
                                                          ViInt32 fader,
                                                          ViInt32 group,
                                                          ViInt32 path,
                                                          ViInt32 parameterName,
                                                          ViBoolean   value)
{
    ViStatus    error = VI_SUCCESS; 
    ViChar      buffer[RSSFU_IO_BUFFER_SIZE] = "";
    ViChar      buf1[RSSFU_TOKEN_BUFFER_SIZE] = "";
    ViChar      *p2buf;
    ViAttr     attrArr[] = {RSSFU_ATTR_FADING_PAR_DEL_CORRELATION_PATH,RSSFU_ATTR_FADING_PAR_DEL_LOGNORMAL_STATE};
    ViAttr     attr30Arr[] = {RSSFU_ATTR_FADING_PAR_DEL30_CORRELATION_PATH,RSSFU_ATTR_FADING_PAR_DEL30_LOGNORMAL_STATE}; 
    ViAttr     attr50Arr[] = {RSSFU_ATTR_FADING_PAR_DEL50_CORRELATION_PATH,RSSFU_ATTR_FADING_PAR_DEL50_LOGNORMAL_STATE}; 
    ViInt32     l_group, l_path, l_fader;
    l_group = group - 1;
    l_path  = path - 1;
    l_fader = fader - 1;  
                             
    checkErr( Rs_LockSession (instrSession, VI_NULL)); 
    
    if ((configuration < RSSFU_VAL_FADING_STANDARD) || (configuration > RSSFU_VAL_FADING_DEL50))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Configuration");
    if ((fader < 1) || (fader > 2))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 3, "Fader");
    if ((group < 1) || (group > 8))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 4, "Group");
    if ((path < 1) || (path > 5))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 5, "Path"); 
    if ((parameterName < RSSFU_VAL_FADING_CORR_PATH) || (parameterName > RSSFU_VAL_FADING_LOGNORMAL_STATE))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 6, "Parameter Name");        
    
    p2buf=buffer;
    switch (configuration){
        case RSSFU_VAL_FADING_STANDARD:
            checkErr (rssfu_GetActualRepCaps (instrSession, attrArr[parameterName], "Fader", l_fader, buf1));
            p2buf+=sprintf(p2buf,"%s,",buf1);
            checkErr (rssfu_GetActualRepCaps (instrSession, attrArr[parameterName], "Group", l_group, buf1));
            p2buf+=sprintf(p2buf,"%s,",buf1);
            checkErr (rssfu_GetActualRepCaps (instrSession, attrArr[parameterName], "Path", l_path, buf1));
            p2buf+=sprintf(p2buf,"%s",buf1);
            viCheckParm (rssfu_SetAttributeViBoolean (instrSession, buffer, attrArr[parameterName], value), 
                        7, "Value");     
        break;                    
        case RSSFU_VAL_FADING_DEL30:
            checkErr (rssfu_GetActualRepCaps (instrSession, attr30Arr[parameterName], "Fader", l_fader, buf1));
            p2buf+=sprintf(p2buf,"%s,",buf1);
            checkErr (rssfu_GetActualRepCaps (instrSession, attr30Arr[parameterName], "Group", l_group, buf1));
            p2buf+=sprintf(p2buf,"%s,",buf1);
            checkErr (rssfu_GetActualRepCaps (instrSession, attr30Arr[parameterName], "Path", l_path, buf1));
            p2buf+=sprintf(p2buf,"%s",buf1);
            viCheckParm (rssfu_SetAttributeViBoolean (instrSession, buffer, attr30Arr[parameterName], value), 
                        7, "Value");
        break;                    
        case RSSFU_VAL_FADING_DEL50:
            checkErr (rssfu_GetActualRepCaps (instrSession, attr50Arr[parameterName], "Fader", l_fader, buf1));
            p2buf+=sprintf(p2buf,"%s,",buf1);
            checkErr (rssfu_GetActualRepCaps (instrSession, attr50Arr[parameterName], "Group", l_group, buf1));
            p2buf+=sprintf(p2buf,"%s,",buf1);
            checkErr (rssfu_GetActualRepCaps (instrSession, attr50Arr[parameterName], "Path", l_path, buf1));
            p2buf+=sprintf(p2buf,"%s",buf1);
            viCheckParm (rssfu_SetAttributeViBoolean (instrSession, buffer, attr50Arr[parameterName], value), 
                        7, "Value");
        break;                    
    }    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Configure Fading Path Profile Resulting Doppler Frequency
 * Purpose:  This function set the Doppler frequency for the Standard Delay
 *           and Fine Delay 30 MHz / 50 MHz fading configurations.
/============================================================================*/
ViStatus _VI_FUNC rssfu_confFadingPathProfileResultDopplerFreq(ViSession instrSession,
                                                          ViInt32 configuration,
                                                          ViInt32 fader,
                                                          ViInt32 group,
                                                          ViInt32 path,
                                                          ViReal64 dopplerFreq)
{
    ViStatus    error = VI_SUCCESS; 
    ViChar      buffer[RSSFU_IO_BUFFER_SIZE] = "";
    ViChar      buf1[RSSFU_TOKEN_BUFFER_SIZE] = "";
    ViChar      *p2buf;
    ViInt32     l_group, l_path, l_fader;
    l_group = group - 1;
    l_path  = path - 1;
    l_fader = fader - 1;  
                             
    checkErr( Rs_LockSession (instrSession, VI_NULL)); 
    
    if ((configuration < RSSFU_VAL_FADING_STANDARD) || (configuration > RSSFU_VAL_FADING_DEL50))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Configuration");
    if ((fader < 1) || (fader > 2))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 3, "Fader");
    if ((group < 1) || (group > 8))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 4, "Group");
    if ((path < 1) || (path > 5))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 5, "Path"); 

    p2buf=buffer;
    switch (configuration){
        case RSSFU_VAL_FADING_STANDARD:
            checkErr (rssfu_GetActualRepCaps (instrSession,RSSFU_ATTR_FADING_PAR_DEL_RESULT_DOPPLER_SHIFT, "Fader", l_fader, buf1));
            p2buf+=sprintf(p2buf,"%s,",buf1);
            checkErr (rssfu_GetActualRepCaps (instrSession, RSSFU_ATTR_FADING_PAR_DEL_RESULT_DOPPLER_SHIFT, "Group", l_group, buf1));
            p2buf+=sprintf(p2buf,"%s,",buf1);
            checkErr (rssfu_GetActualRepCaps (instrSession, RSSFU_ATTR_FADING_PAR_DEL_RESULT_DOPPLER_SHIFT, "Path", l_path, buf1));
            p2buf+=sprintf(p2buf,"%s",buf1);
            viCheckParm (rssfu_SetAttributeViReal64 (instrSession, buffer, RSSFU_ATTR_FADING_PAR_DEL_RESULT_DOPPLER_SHIFT, 
                        dopplerFreq), 6, "Doppler Frequency");        
        break;                    
        case RSSFU_VAL_FADING_DEL30:
            checkErr (rssfu_GetActualRepCaps (instrSession, RSSFU_ATTR_FADING_PAR_DEL30_RESULT_DOPPLER_SHIFT, "Fader", l_fader, buf1));
            p2buf+=sprintf(p2buf,"%s,",buf1);
            checkErr (rssfu_GetActualRepCaps (instrSession, RSSFU_ATTR_FADING_PAR_DEL30_RESULT_DOPPLER_SHIFT, "Group", l_group, buf1));
            p2buf+=sprintf(p2buf,"%s,",buf1);
            checkErr (rssfu_GetActualRepCaps (instrSession, RSSFU_ATTR_FADING_PAR_DEL30_RESULT_DOPPLER_SHIFT, "Path", l_path, buf1));
            p2buf+=sprintf(p2buf,"%s",buf1);
            viCheckParm (rssfu_SetAttributeViReal64 (instrSession, buffer, RSSFU_ATTR_FADING_PAR_DEL30_RESULT_DOPPLER_SHIFT, 
                        dopplerFreq), 6, "Doppler Frequency");   
        break;                    
        case RSSFU_VAL_FADING_DEL50:
            checkErr (rssfu_GetActualRepCaps (instrSession, RSSFU_ATTR_FADING_PAR_DEL50_RESULT_DOPPLER_SHIFT, "Fader", l_fader, buf1));
            p2buf+=sprintf(p2buf,"%s,",buf1);
            checkErr (rssfu_GetActualRepCaps (instrSession, RSSFU_ATTR_FADING_PAR_DEL50_RESULT_DOPPLER_SHIFT, "Group", l_group, buf1));
            p2buf+=sprintf(p2buf,"%s,",buf1);
            checkErr (rssfu_GetActualRepCaps (instrSession, RSSFU_ATTR_FADING_PAR_DEL50_RESULT_DOPPLER_SHIFT, "Path", l_path, buf1));
            p2buf+=sprintf(p2buf,"%s",buf1);
            viCheckParm (rssfu_SetAttributeViReal64 (instrSession, buffer, RSSFU_ATTR_FADING_PAR_DEL50_RESULT_DOPPLER_SHIFT, 
                        dopplerFreq), 6, "Doppler Frequency");   
        break;                    
    }    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}   
    
/*============================================================================/
 * Function: Read Fading Path Profile Table Results
 * Purpose:  This function queries the value of selected resulting parameter 
 *           of profile table.
/============================================================================*/
ViStatus _VI_FUNC rssfu_readFadingPathProfileTableResults(ViSession   instrSession,
                                                          ViInt32 configuration,
                                                          ViInt32 fader,
                                                          ViInt32 group,
                                                          ViInt32 path,
                                                          ViInt32 parameterName,
                                                          ViReal64*   value)
{
    ViStatus    error = VI_SUCCESS; 
    ViChar      buffer[RSSFU_IO_BUFFER_SIZE] = "";
    ViChar      buf1[RSSFU_TOKEN_BUFFER_SIZE] = "";
    ViChar      *p2buf;
    ViAttr     attrArr[] = {RSSFU_ATTR_FADING_PAR_DEL_RESULTING_DELAY,RSSFU_ATTR_FADING_PAR_DEL_RESULT_DOPPLER_SHIFT};
    ViAttr     attr30Arr[] = {RSSFU_ATTR_FADING_PAR_DEL30_RESULTING_DELAY,RSSFU_ATTR_FADING_PAR_DEL30_RESULT_DOPPLER_SHIFT};
    ViAttr     attr50Arr[] = {RSSFU_ATTR_FADING_PAR_DEL50_RESULTING_DELAY,RSSFU_ATTR_FADING_PAR_DEL50_RESULT_DOPPLER_SHIFT};
    ViInt32     l_group, l_path, l_fader;
    l_group = group - 1;
    l_path  = path - 1;
    l_fader = fader - 1;    
                             
    checkErr( Rs_LockSession (instrSession, VI_NULL)); 
    
    if ((configuration < RSSFU_VAL_FADING_STANDARD) || (configuration > RSSFU_VAL_FADING_DEL50))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Configuration");
    if ((fader < 1) || (fader > 2))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 3, "Fader");    
    if ((group < 1) || (group > 8))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 4, "Group");
    if ((path < 1) || (path > 5))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 5, "Path"); 
    if ((parameterName < RSSFU_VAL_FADING_RES_DELAY) || (parameterName > RSSFU_VAL_FADING_RES_DOPP_SHIFT))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 6, "Parameter Name");        
    
    p2buf=buffer;
    switch (configuration){
        case RSSFU_VAL_FADING_STANDARD:
            checkErr (rssfu_GetActualRepCaps (instrSession, attrArr[parameterName], "Fader", l_fader, buf1));
            p2buf+=sprintf(p2buf,"%s,",buf1);
            checkErr (rssfu_GetActualRepCaps (instrSession, attrArr[parameterName], "Group", l_group, buf1));
            p2buf+=sprintf(p2buf,"%s,",buf1);
            checkErr (rssfu_GetActualRepCaps (instrSession, attrArr[parameterName], "Path", l_path, buf1));
            p2buf+=sprintf(p2buf,"%s",buf1);
            viCheckParm (rssfu_GetAttributeViReal64 (instrSession, buffer, attrArr[parameterName], value), 
                        7, "Value");     
        break;                    
        case RSSFU_VAL_FADING_DEL30:
            checkErr (rssfu_GetActualRepCaps (instrSession, attr30Arr[parameterName], "Fader", l_fader, buf1));
            p2buf+=sprintf(p2buf,"%s,",buf1);
            checkErr (rssfu_GetActualRepCaps (instrSession, attr30Arr[parameterName], "Group", l_group, buf1));
            p2buf+=sprintf(p2buf,"%s,",buf1);
            checkErr (rssfu_GetActualRepCaps (instrSession, attr30Arr[parameterName], "Path", l_path, buf1));
            p2buf+=sprintf(p2buf,"%s",buf1);
            viCheckParm (rssfu_GetAttributeViReal64 (instrSession, buffer, attr30Arr[parameterName], value), 
                        7, "Value");
        break;                    
        case RSSFU_VAL_FADING_DEL50:
            checkErr (rssfu_GetActualRepCaps (instrSession, attr50Arr[parameterName], "Fader", l_fader, buf1));
            p2buf+=sprintf(p2buf,"%s,",buf1);
            checkErr (rssfu_GetActualRepCaps (instrSession, attr50Arr[parameterName], "Group", l_group, buf1));
            p2buf+=sprintf(p2buf,"%s,",buf1);
            checkErr (rssfu_GetActualRepCaps (instrSession, attr50Arr[parameterName], "Path", l_path, buf1));
            p2buf+=sprintf(p2buf,"%s",buf1);
            viCheckParm (rssfu_GetAttributeViReal64 (instrSession, buffer, attr50Arr[parameterName], value), 
                        7, "Value");
        break;                    
    }    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Configure Fading Birth Death Path Loss
 * Purpose:  This function sets the loss of the paths with birth death
 *           propagation.
/============================================================================*/
ViStatus _VI_FUNC rssfu_confFadingBirthDeathPathLoss (ViSession instrSession,
                                                       ViInt32 fader, ViInt32 path,
                                                       ViReal64 pathLoss)
{
    ViStatus    error = VI_SUCCESS; 
    ViChar      buffer[RSSFU_IO_BUFFER_SIZE] = "";
    ViChar      buf1[RSSFU_TOKEN_BUFFER_SIZE] = "";
    ViChar      *p2buf;
    ViInt32     l_path, l_fader;    
    l_path  = path - 1;
    l_fader = fader - 1;    
                             
    checkErr( Rs_LockSession (instrSession, VI_NULL)); 
    
    if ((fader < 1) || (fader > 2))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Fader");    
    if ((path < 1) || (path > 5))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 3, "Path"); 
    
    p2buf=buffer;

    checkErr (rssfu_GetActualRepCaps (instrSession, RSSFU_ATTR_FADING_PAR_PATH_LOSS_BIRTH_DEATH, "Fader", l_fader, buf1));
    p2buf+=sprintf(p2buf,"%s,",buf1);
    checkErr (rssfu_GetActualRepCaps (instrSession, RSSFU_ATTR_FADING_PAR_PATH_LOSS_BIRTH_DEATH, "Path", l_path, buf1));
    p2buf+=sprintf(p2buf,"%s",buf1);
    viCheckParm (rssfu_SetAttributeViReal64 (instrSession, buffer, RSSFU_ATTR_FADING_PAR_PATH_LOSS_BIRTH_DEATH, 
                        pathLoss), 4, "Path Loss");                                           
        
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Read Fading Birth Death Path Profile
 * Purpose:  This function displays the fading profile for birth death
 *           propagation. The fading profile has a fixed setting to Pure
 *           Doppler.
/============================================================================*/
ViStatus _VI_FUNC rssfu_readFadingBirthDeathPathProfile (ViSession instrSession,
                                                       ViInt32 fader, ViInt32 path,
                                                       ViChar  profile[])
{
    ViStatus    error = VI_SUCCESS; 
    ViChar      buffer[RSSFU_IO_BUFFER_SIZE] = "";
    ViChar      buf1[RSSFU_TOKEN_BUFFER_SIZE] = "";
    ViChar      buf2[RSSFU_IO_BUFFER_SIZE] = "\0";
    ViChar      *p2buf;
    ViInt32     l_path, l_fader;    
    l_path  = path - 1;
    l_fader = fader - 1;    
                             
    checkErr( Rs_LockSession (instrSession, VI_NULL)); 
    
    if ((fader < 1) || (fader > 2))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Fader");    
    if ((path < 1) || (path > 5))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 3, "Path"); 
    
    p2buf=buffer;

    checkErr (rssfu_GetActualRepCaps (instrSession, RSSFU_ATTR_FADING_PAR_PROFILE_BIRTH_DEATH, "Fader", l_fader, buf1));
    p2buf+=sprintf(p2buf,"%s,",buf1);
    checkErr (rssfu_GetActualRepCaps (instrSession, RSSFU_ATTR_FADING_PAR_PROFILE_BIRTH_DEATH, "Path", l_path, buf1));
    p2buf+=sprintf(p2buf,"%s",buf1);
    if (profile == 0) viCheckParm (RS_ERROR_INVALID_PARAMETER, 4, "Profile");
    viCheckErr (rssfu_GetAttributeViString (instrSession, buffer, RSSFU_ATTR_FADING_PAR_PROFILE_BIRTH_DEATH,
                                            RSSFU_IO_BUFFER_SIZE, buf2));
    strcpy (profile, buf2);                                           
        
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/*============================================================================/
 * Function: Configure Fading Birth Death Path Profile Numeric
 * Purpose:  This function sets the value of selected numeric parameter of
 *           Birth Death Propagation configuration.
/============================================================================*/
ViStatus _VI_FUNC rssfu_confFadingBirthDeathPathProfile (ViSession instrSession,
                                                        ViInt32 fader,
                                                        ViReal64 minDelay,
                                                        ViReal64 delayGrid,
                                                        ViInt32  delayPositions,
                                                        ViReal64 startOffset,
                                                        ViReal64 hoppingDwell,
                                                        ViReal64 speed,
                                                        ViReal64 freqRatio)
{
    ViStatus    error = VI_SUCCESS; 
    ViChar      buf1[RSSFU_TOKEN_BUFFER_SIZE] = "";
    ViInt32     l_fader;    
    l_fader = fader - 1;    
                             
    checkErr( Rs_LockSession (instrSession, VI_NULL)); 
    
    if ((fader < 1) || (fader > 2))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Fader");    
   
    checkErr (rssfu_GetActualRepCaps (instrSession, RSSFU_ATTR_FADING_PAR_MIN_DELAY_BIRTH_DEATH, "Fader", l_fader, buf1));

    viCheckParm (rssfu_SetAttributeViReal64 (instrSession, buf1, RSSFU_ATTR_FADING_PAR_MIN_DELAY_BIRTH_DEATH, 
                        minDelay), 3, "Min. Delay");
    viCheckParm (rssfu_SetAttributeViReal64 (instrSession, buf1, RSSFU_ATTR_FADING_PAR_DELAY_GRID_BIRTH_DEATH, 
                        delayGrid), 4, "Delay Grid");                        
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, buf1, RSSFU_ATTR_FADING_PAR_POSITIONS_BIRTH_DEATH, 
                        delayPositions), 5, "Delay Positions");
    viCheckParm (rssfu_SetAttributeViReal64 (instrSession, buf1, RSSFU_ATTR_FADING_PAR_START_OFFSET_BIRTH_DEATH, 
                        startOffset), 6, "Start Offset");
    viCheckParm (rssfu_SetAttributeViReal64 (instrSession, buf1, RSSFU_ATTR_FADING_PAR_HOPPING_DWELL_BIRTH_DEATH, 
                        hoppingDwell), 7, "Hopping Dwell");
    viCheckParm (rssfu_SetAttributeViReal64 (instrSession, buf1, RSSFU_ATTR_FADING_PAR_SPEED_BIRTH_DEATH, 
                        speed), 8, "Speed");
    viCheckParm (rssfu_SetAttributeViReal64 (instrSession, buf1, RSSFU_ATTR_FADING_PAR_FREQ_RATIO_BIRTH_DEATH, 
                        freqRatio), 9, "Freq. Ratio");                        
                                                
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Configure Fading Birth Death Doppler Frequency
 * Purpose:  This function sets the Doppler frequency with birth death
 *           propagation.
/============================================================================*/
ViStatus _VI_FUNC rssfu_confFadingBirthDeathDopplerFrequency (ViSession instrSession,
                                                 ViInt32 fader, ViReal64 dopplerFreq)
{
  ViStatus    error = VI_SUCCESS; 
    ViChar      buf1[RSSFU_TOKEN_BUFFER_SIZE] = "";
    ViInt32     l_fader;    
    l_fader = fader - 1;    
                             
    checkErr( Rs_LockSession (instrSession, VI_NULL)); 
    
    if ((fader < 1) || (fader > 2))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Fader");    
   
    checkErr (rssfu_GetActualRepCaps (instrSession, RSSFU_ATTR_FADING_PAR_RES_DOPPLER_SHIFT_BIRTH_DEATH, "Fader", l_fader, buf1));

    viCheckParm (rssfu_SetAttributeViReal64 (instrSession, buf1, RSSFU_ATTR_FADING_PAR_RES_DOPPLER_SHIFT_BIRTH_DEATH, 
                        dopplerFreq), 3, "Doppler Frequency");
        
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}    
     
     
/*============================================================================/
 * Function: Read Fading Birth Death Path Profile Results
 * Purpose:  This function queries the value of selected resulting parameter
 *           of Birth Death Propagation configuration.
/============================================================================*/
ViStatus _VI_FUNC rssfu_readFadingBirthDeathPathProfileResults (ViSession instrSession,
                                                        ViInt32 fader,
                                                        ViInt32 parameterName,
                                                        ViReal64* value)
{
    ViStatus    error = VI_SUCCESS; 
    ViChar      buf1[RSSFU_TOKEN_BUFFER_SIZE] = "";
    ViInt32     l_fader;    
    l_fader = fader - 1;    
                             
    checkErr( Rs_LockSession (instrSession, VI_NULL)); 
    
    if ((fader < 1) || (fader > 2))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Fader");
    if ((parameterName < RSSFU_VAL_FADING_RES_MAX_DELAY) || (parameterName > RSSFU_VAL_FADING_RES_DOPP_SHIFT))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 3, "Parameter Name"); 
   
    if (parameterName == RSSFU_VAL_FADING_RES_MAX_DELAY)
    {
        checkErr (rssfu_GetActualRepCaps (instrSession, RSSFU_ATTR_FADING_PAR_MAX_DELAY_BIRTH_DEATH, "Fader", l_fader, buf1));
        viCheckParm (rssfu_GetAttributeViReal64 (instrSession, buf1, RSSFU_ATTR_FADING_PAR_MAX_DELAY_BIRTH_DEATH, value), 
                        4, "Value");
    }
    
    if (parameterName == RSSFU_VAL_FADING_RES_DOPP_SHIFT)
    {
        checkErr (rssfu_GetActualRepCaps (instrSession, RSSFU_ATTR_FADING_PAR_RES_DOPPLER_SHIFT_BIRTH_DEATH, "Fader", l_fader, buf1));
        viCheckParm (rssfu_GetAttributeViReal64 (instrSession, buf1, RSSFU_ATTR_FADING_PAR_RES_DOPPLER_SHIFT_BIRTH_DEATH, value), 
                        4, "Value");
    }
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/*============================================================================/
 * Function: Configure Fading Moving Propagation Path Reference
 * Purpose:  This function sets the value of selected numeric parameter of
 *           the Moving Propagation configuration.
/============================================================================*/
ViStatus _VI_FUNC rssfu_confFadingMovingPropagPathReference (ViSession instrSession,
                                                 ViInt32 fader,
                                                 ViBoolean referencePathState,
                                                 ViReal64 referencePathPathLoss,
                                                 ViReal64 referencePathDelay)
{
    ViStatus    error = VI_SUCCESS; 
    ViChar      buf1[RSSFU_TOKEN_BUFFER_SIZE] = "";
    ViInt32     l_fader;    
    l_fader = fader - 1;    
                             
    checkErr( Rs_LockSession (instrSession, VI_NULL)); 
    
    if ((fader < 1) || (fader > 2))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Fader");    
   
    checkErr (rssfu_GetActualRepCaps (instrSession, RSSFU_ATTR_FADING_PAR_REF_PATH_STATE, "Fader", l_fader, buf1));
    
    viCheckParm (rssfu_SetAttributeViBoolean (instrSession, buf1, RSSFU_ATTR_FADING_PAR_REF_PATH_STATE, 
                        referencePathState), 3, "Reference Path State");
    viCheckParm (rssfu_SetAttributeViReal64 (instrSession, buf1, RSSFU_ATTR_FADING_PAR_REF_PATH_LOSS, 
                        referencePathPathLoss), 4, "Reference Path Path Loss");                        
    viCheckParm (rssfu_SetAttributeViReal64 (instrSession, buf1, RSSFU_ATTR_FADING_PAR_REF_PATH_DELAY, 
                        referencePathDelay), 5, "Reference Path Delay");
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/*============================================================================/
 * Function: Configure Fading Moving Propagation Path
 * Purpose:  This function sets the value of selected numeric parameter of
 *           the Moving Propagation configuration.
/============================================================================*/
ViStatus _VI_FUNC rssfu_confFadingMovingPropagPath (ViSession instrSession,
                                              ViInt32 fader,
                                              ViBoolean movingPathState,
                                              ViReal64 movingPathPathLoss,
                                              ViReal64 movingPathDelay,
                                              ViReal64 movingPathVariationPK,
                                              ViReal64 movingPathVariationPeriod)
{
    ViStatus    error = VI_SUCCESS; 
    ViChar      buf1[RSSFU_TOKEN_BUFFER_SIZE] = "";
    ViInt32     l_fader;    
    l_fader = fader - 1;    
                             
    checkErr( Rs_LockSession (instrSession, VI_NULL)); 
    
    if ((fader < 1) || (fader > 2))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Fader");    

    checkErr (rssfu_GetActualRepCaps (instrSession, RSSFU_ATTR_FADING_PAR_MOVING_PATH_STATE, "Fader", l_fader, buf1));
    
    viCheckParm (rssfu_SetAttributeViBoolean (instrSession, buf1, RSSFU_ATTR_FADING_PAR_MOVING_PATH_STATE, 
                        movingPathState), 3, "Moving Path State");
    viCheckParm (rssfu_SetAttributeViReal64 (instrSession, buf1, RSSFU_ATTR_FADING_PAR_MOVING_PATH_LOSS, 
                        movingPathPathLoss), 4, "Moving Path Path Loss");                        
    viCheckParm (rssfu_SetAttributeViReal64 (instrSession, buf1, RSSFU_ATTR_FADING_PAR_MOVING_PATH_DELAY, 
                        movingPathDelay), 5, "Moving Path Delay");
    viCheckParm (rssfu_SetAttributeViReal64 (instrSession, buf1, RSSFU_ATTR_FADING_PAR_MOVING_PATH_VARIATION_PK, 
                        movingPathVariationPK), 6, "Moving Path Variation PK");
    viCheckParm (rssfu_SetAttributeViReal64 (instrSession, buf1, RSSFU_ATTR_FADING_PAR_MOVING_PATH_VARIATION_PERIOD, 
                        movingPathVariationPeriod), 7, "Moving Path Variation Period");
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/*============================================================================/
 * Function: Configure Fading 2 Path Dynamic Delay Path
 * Purpose:  This function sets the value of the 2 Path Dynamic Delay
 *           configuration.
/============================================================================*/
ViStatus _VI_FUNC rssfu_confFading2pathDynamicDelayPath (ViSession instrSession,
                                                         ViInt32 fader,
                                                         ViInt32 configurePath,
                                                         ViBoolean state,
                                                         ViInt32 profile,
                                                         ViReal64 pathLoss,
                                                         ViReal64 speed,
                                                         ViReal64 frequencyRatio,
                                                         ViReal64 minDelay)
{                                             
    ViStatus    error = VI_SUCCESS; 
    ViChar      buf1[RSSFU_TOKEN_BUFFER_SIZE] = "";
    ViInt32     l_fader;    
    l_fader = fader - 1;    
                             
    checkErr( Rs_LockSession (instrSession, VI_NULL)); 
    
    if ((fader < 1) || (fader > 2))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Fader");    

    
    if ((configurePath < RSSFU_VAL_FADING_P2DYN_REF_PATH) || (configurePath > RSSFU_VAL_FADING_P2DYN_MOVING_PATH))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 3, "Configure Path");
 
    checkErr (rssfu_GetActualRepCaps (instrSession, RSSFU_ATTR_FADING_2PATH_DYN_DEL_REF_PATH_STATE, "Fader", l_fader, buf1));
   
    switch (configurePath){
        case RSSFU_VAL_FADING_P2DYN_REF_PATH:
            
            viCheckParm (rssfu_SetAttributeViBoolean (instrSession, buf1, RSSFU_ATTR_FADING_2PATH_DYN_DEL_REF_PATH_STATE,
                                                    state), 4, "State");
                                                    
            viCheckParm (rssfu_SetAttributeViInt32 (instrSession, buf1, RSSFU_ATTR_FADING_2PATH_DYN_DEL_REF_PATH_PROFILE, 
                                                    profile), 5, "Profile");;
        
            viCheckParm (rssfu_SetAttributeViReal64 (instrSession, buf1, RSSFU_ATTR_FADING_2PATH_DYN_DEL_REF_PATH_LOSS,
                                                    pathLoss), 6, "Path Loss");
        
            viCheckParm (rssfu_SetAttributeViReal64 (instrSession, buf1, RSSFU_ATTR_FADING_2PATH_DYN_DEL_REF_PATH_FREQ_RATIO,
                                                    frequencyRatio), 8, "Frequency Ratio");
            
            viCheckParm (rssfu_SetAttributeViReal64 (instrSession, buf1, RSSFU_ATTR_FADING_2PATH_DYN_DEL_REF_PATH_MIN_DELAY,
                                                    minDelay), 9, "Min Delay");                                                    
        break;
        
        case RSSFU_VAL_FADING_P2DYN_MOVING_PATH:
            viCheckParm (rssfu_SetAttributeViBoolean (instrSession, buf1, RSSFU_ATTR_FADING_2PATH_DYN_DEL_MOVING_PATH_STATE,
                                                    state), 4, "State");
                                                    
            viCheckParm (rssfu_SetAttributeViInt32 (instrSession, buf1, RSSFU_ATTR_FADING_2PATH_DYN_DEL_MOVING_PATH_PROFILE, 
                                                    profile), 5, "Profile");;
        
            viCheckParm (rssfu_SetAttributeViReal64 (instrSession, buf1, RSSFU_ATTR_FADING_2PATH_DYN_DEL_MOVING_PATH_LOSS,
                                                    pathLoss), 6, "Path Loss");
        
            viCheckParm (rssfu_SetAttributeViReal64 (instrSession, buf1, RSSFU_ATTR_FADING_2PATH_DYN_DEL_MOVING_PATH_FREQ_RATIO,
                                                    frequencyRatio), 8, "Frequency Ratio");
            
            viCheckParm (rssfu_SetAttributeViReal64 (instrSession, buf1, RSSFU_ATTR_FADING_2PATH_DYN_DEL_MOVING_PATH_MIN_DELAY,
                                                    minDelay), 9, "Min Delay");    
        break;
    }
    
    viCheckParm (rssfu_SetAttributeViReal64 (instrSession, buf1, RSSFU_ATTR_FADING_2PATH_DYN_DEL_SPEED,
                                                    speed), 7, "Speed");   
  
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Configure Fading 2 Path Dynamic Delay
 * Purpose:  This function configures the 2 Path Dynamic Delay parametrs.
/============================================================================*/
ViStatus _VI_FUNC rssfu_confFading2pathDynamicDelay (ViSession instrSession,
                                                     ViInt32 fader,
                                                     ViInt32 movingMode,
                                                     ViReal64 movingPathMaxDelay,
                                                     ViReal64 period,
                                                     ViReal64 dwell)
{                                             
    ViStatus    error = VI_SUCCESS; 
    ViChar      buf1[RSSFU_TOKEN_BUFFER_SIZE] = "";
    ViInt32     l_fader;    
    l_fader = fader - 1;    
                             
    checkErr( Rs_LockSession (instrSession, VI_NULL)); 
    
    if ((fader < 1) || (fader > 2))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Fader"); 
       
    if ((movingMode < RSSFU_VAL_FADING_P2DYN_MOV_MODE_HOPP) || (movingMode > RSSFU_VAL_FADING_P2DYN_MOV_MODE_SLID))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 3, "Moving Mode");    


    checkErr (rssfu_GetActualRepCaps (instrSession, RSSFU_ATTR_FADING_2PATH_DYN_DEL_MOVING_PATH_MAX_DELAY, "Fader", l_fader, buf1));
    
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, buf1, RSSFU_ATTR_FADING_2PATH_DYN_DEL_MOVING_MODE, 
                                                    movingMode), 3, "Moving Mode");;
        
    viCheckParm (rssfu_SetAttributeViReal64 (instrSession, buf1, RSSFU_ATTR_FADING_2PATH_DYN_DEL_MOVING_PATH_MAX_DELAY,
                                                    movingPathMaxDelay), 4, "Moving Path Max Delay");
    
    switch (movingMode){
        case RSSFU_VAL_FADING_P2DYN_MOV_MODE_SLID:
            viCheckParm (rssfu_SetAttributeViReal64 (instrSession, buf1, RSSFU_ATTR_FADING_2PATH_DYN_DEL_MOVING_PERIOD,
                                                    period), 5, "Period");
        break;                                                  
                                                    
        case RSSFU_VAL_FADING_P2DYN_MOV_MODE_HOPP:
            viCheckParm (rssfu_SetAttributeViReal64 (instrSession, buf1, RSSFU_ATTR_FADING_2PATH_DYN_DEL_MOVING_DWELL,
                                                    dwell), 6, "Dwell");
        break;
    }
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}
      

/*============================================================================/
 * Function: Configure Fading 2 Path Dynamic Delay Doppler Shift
 * Purpose:  This function sets the actual Doppler shift of the Moving path.
/============================================================================*/
ViStatus _VI_FUNC rssfu_confFading2pathDynamicDelayDopplerShift (ViSession instrSession,
                                                 ViInt32 fader, ViInt32 configurePath,
                                                 ViReal64 dopplerShift)
{
  ViStatus    error = VI_SUCCESS; 
    ViChar      buf1[RSSFU_TOKEN_BUFFER_SIZE] = "";
    ViInt32     l_fader;    
    l_fader = fader - 1;    
                             
    checkErr( Rs_LockSession (instrSession, VI_NULL)); 
    
    if ((fader < 1) || (fader > 2))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Fader");    

    if ((configurePath < RSSFU_VAL_FADING_P2DYN_REF_PATH) || (configurePath > RSSFU_VAL_FADING_P2DYN_MOVING_PATH))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 3, "Configure Path");

    checkErr (rssfu_GetActualRepCaps (instrSession, RSSFU_ATTR_FADING_2PATH_DYN_DEL_REF_PATH_DOPLER_SHIFT, "Fader", l_fader, buf1));
    
    switch (configurePath){
        case RSSFU_VAL_FADING_P2DYN_REF_PATH:
    
            viCheckParm (rssfu_SetAttributeViReal64 (instrSession, buf1, RSSFU_ATTR_FADING_2PATH_DYN_DEL_REF_PATH_DOPLER_SHIFT, 
                        dopplerShift), 4, "Doppler Shift");
        break;
        
        case RSSFU_VAL_FADING_P2DYN_MOVING_PATH:
            viCheckParm (rssfu_SetAttributeViReal64 (instrSession, buf1, RSSFU_ATTR_FADING_2PATH_DYN_DEL_MOVING_PATH_DOPLER_SHIFT, 
                        dopplerShift), 4, "Doppler Shift");
        break;
    }
                
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}    

/*============================================================================/
 * Function: Read Fading 2 Path Dynamic Delay Doppler Shift
 * Purpose:  Queries the actual Doppler shift of the Moving path.
/============================================================================*/
ViStatus _VI_FUNC rssfu_readFading2pathDynamicDelayDopplerShift
             (ViSession instrSession, ViInt32 fader, ViReal64* resDopplerShift)
{
    ViStatus    error = VI_SUCCESS; 
    ViChar      buf1[RSSFU_TOKEN_BUFFER_SIZE] = "";
    ViInt32     l_fader;    
    l_fader = fader - 1;    
                             
    checkErr( Rs_LockSession (instrSession, VI_NULL)); 
    
    if ((fader < 1) || (fader > 2))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Fader");    

    checkErr (rssfu_GetActualRepCaps (instrSession, RSSFU_ATTR_FADING_2PATH_DYN_DEL_MOVING_PATH_DOPLER_SHIFT, "Fader", l_fader, buf1));
    

    if (!resDopplerShift) viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Res Doppler Shift");
    viCheckErr (rssfu_GetAttributeViReal64 (instrSession, buf1, RSSFU_ATTR_FADING_2PATH_DYN_DEL_MOVING_PATH_DOPLER_SHIFT,
                                            resDopplerShift));
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}             






/*============================================================================/
 * Function: Configure Fading Common Settings
 * Purpose:  This function selects fading simulator common settings.
/============================================================================*/
ViStatus _VI_FUNC rssfu_confFadingCommonSettings(ViSession instrSession,
                                                 ViInt32 fader,
                                                 ViInt32 speedUnit,
                                                 ViBoolean localConstantCoupled,
                                                 ViBoolean standardDeviationCoupled,
                                                 ViBoolean ignoreRFFrequency,
                                                 ViInt32 signalDedicatedTo,
                                                 ViReal64 virtualRF,
                                                 ViInt32 restartMode)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buf1[RSSFU_TOKEN_BUFFER_SIZE] = "";
    ViInt32     l_fader = fader - 1;
    
    if ((fader < 1) || (fader > 2))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Fader");


    checkErr( Rs_LockSession (instrSession, VI_NULL));

    checkErr (rssfu_GetActualRepCaps (instrSession, RSSFU_ATTR_FADING_SPEED_UNIT, "Fader", l_fader, buf1));
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, buf1, RSSFU_ATTR_FADING_SPEED_UNIT, speedUnit), 
                    3, "Speed Unit");
    /*   Fw 2.0 
    viCheckParm (rssfu_SetAttributeViBoolean (instrSession, buf1, RSSFU_ATTR_FADING_LOCAL_CONSTANT_COUPLED, 
                        localConstantCoupled), 4, "Local Constant Coupled");                    
    
    viCheckParm (rssfu_SetAttributeViBoolean (instrSession, buf1, RSSFU_ATTR_FADING_STANDARD_DEVIATION_COUPLED, 
                        standardDeviationCoupled), 5, "Standard Deviation Coupled");                        
    */
    localConstantCoupled;
    standardDeviationCoupled;
    viCheckParm (rssfu_SetAttributeViBoolean (instrSession, buf1, RSSFU_ATTR_FADING_IGNORE_RF_CHANGES, 
                        ignoreRFFrequency), 6, "Ignore RF Frequency < 5%"); 
    
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, buf1, RSSFU_ATTR_FADING_SIGNAL_DEDICATED_TO, 
                        signalDedicatedTo), 7, "Signal Dedicated To");  
    
    viCheckParm (rssfu_SetAttributeViReal64 (instrSession, buf1, RSSFU_ATTR_FADING_VIRTUAL_RF, 
                        virtualRF), 8, "Virtual RF");
    
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, buf1, RSSFU_ATTR_FADING_RESTART_MODE, 
                        restartMode), 9, "Restart Mode");                        
                        
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/*============================================================================/
 * Function: Configure Fading Common Coupled Settings
 * Purpose:  This function selects fading simulator common coupled settings.
/============================================================================*/
ViStatus _VI_FUNC rssfu_confFadingCommonCoupledSettings(ViSession instrSession,
                                                 ViInt32 fader,
                                                 ViBoolean localConstantCoupled,
                                                 ViBoolean standardDeviationCoupled)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buf1[RSSFU_TOKEN_BUFFER_SIZE] = "";
    ViInt32     l_fader = fader - 1;
    
    if ((fader < 1) || (fader > 2))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Fader");


    checkErr( Rs_LockSession (instrSession, VI_NULL));

    checkErr (rssfu_GetActualRepCaps (instrSession, RSSFU_ATTR_FADING_STANDARD_DEVIATION_COUPLED, "Fader", l_fader, buf1));
 
    /*
    viCheckParm (rssfu_SetAttributeViBoolean (instrSession, buf1, RSSFU_ATTR_FADING_LOCAL_CONSTANT_COUPLED, 
                        localConstantCoupled), 3, "Local Constant Coupled");                    
    */
    localConstantCoupled;
    viCheckParm (rssfu_SetAttributeViBoolean (instrSession, buf1, RSSFU_ATTR_FADING_STANDARD_DEVIATION_COUPLED, 
                        standardDeviationCoupled), 4, "Standard Deviation Coupled");                        
                        
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}



/*============================================================================/
 * Function: Configure Fading Signal Delay Shift
 * Purpose:  This function sets the Signal(B) Delay Shift.
/============================================================================*/
ViStatus _VI_FUNC rssfu_confFadingSignalDelayShift (ViSession instrSession,
                                                    ViInt32 fader,
                                                    ViReal64 signal_B_DelayShift)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buf1[RSSFU_TOKEN_BUFFER_SIZE] = "";
    ViInt32     l_fader = fader - 1;

   if ((fader < 1) || (fader > 2))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Fader"); 
 
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    
    checkErr (rssfu_GetActualRepCaps (instrSession, RSSFU_ATTR_FADING_SIGNAL_DELAY_SHIFT, "Fader", l_fader, buf1));

    viCheckParm (rssfu_SetAttributeViReal64 (instrSession, buf1, RSSFU_ATTR_FADING_SIGNAL_DELAY_SHIFT, 
                        signal_B_DelayShift), 3, "Signal(B) Delay Shift");

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Configure Fading Insertion Loss Settings
 * Purpose:  This function selects fading simulator insertion loss settings.
/============================================================================*/
ViStatus _VI_FUNC rssfu_confFadingInsertionLossSettings(ViSession instrSession,
                                                        ViInt32 fader,
                                                        ViInt32 mode,
                                                        ViReal64 insertionLoss)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buf1[RSSFU_TOKEN_BUFFER_SIZE] = "";
    ViInt32     l_fader = fader - 1;

   if ((fader < 1) || (fader > 2))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Fader"); 
 
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    
    checkErr (rssfu_GetActualRepCaps (instrSession, RSSFU_ATTR_FADING_INSERT_LOSS_MODE, "Fader", l_fader, buf1));

    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, buf1, RSSFU_ATTR_FADING_INSERT_LOSS_MODE, mode), 
                    3, "Mode");
                    
    if (mode == RSSFU_VAL_FADING_LOSS_MODE_USER) {
    viCheckParm (rssfu_SetAttributeViReal64 (instrSession, buf1, RSSFU_ATTR_FADING_INSERT_LOSS, 
                        insertionLoss), 4, "Insertion Loss");
    }
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Configure Fading Delay Settings
 * Purpose:  This function selects fading simulator delayed path settings.
/============================================================================*/
ViStatus _VI_FUNC rssfu_confFadingDelaySettings(ViSession   instrSession,
                                                ViInt32 fader,
                                                ViBoolean commonSpeedAllPath)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buf1[RSSFU_TOKEN_BUFFER_SIZE] = "";
    ViInt32     l_fader = fader - 1;
    ViChar      model [RSSFU_IO_BUFFER_SIZE] = "";      


        
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    viCheckErr (rssfu_GetAttributeViString (instrSession, "",
                                               RS_ATTR_INSTRUMENT_MODEL,
                                               RSSFU_IO_BUFFER_SIZE, model));
    if ((fader < 1) || (fader > 2))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Fader");   
      
    if (strstr (model, "SFC") == NULL) { 
        checkErr (rssfu_GetActualRepCaps (instrSession, RSSFU_ATTR_FADING_COMMON_SPEED_ALL_PATH, "Fader", l_fader, buf1));
  
        viCheckParm (rssfu_SetAttributeViBoolean (instrSession, buf1, RSSFU_ATTR_FADING_COMMON_SPEED_ALL_PATH, 
                    commonSpeedAllPath), 3, "Common Speed All Path");
    }
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Set Fading Settings to Default
 * Purpose:  This function sets the default settings (preset) for fading 
 *           simulation.
/============================================================================*/
ViStatus _VI_FUNC rssfu_actFadingSettingsDefault(ViSession instrSession, ViInt32 fader)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buf1[RSSFU_TOKEN_BUFFER_SIZE] = "";
    ViInt32     l_fader = fader - 1;

    if ((fader < 1) || (fader > 2))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Fader"); 
        
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    checkErr (rssfu_GetActualRepCaps (instrSession, RSSFU_ATTR_FADING_DEFAULT, "Fader", l_fader, buf1));

    viCheckErr (rssfu_SetAttributeViString (instrSession, buf1, RSSFU_ATTR_FADING_DEFAULT,""));
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Configure Fading Reference Settings
 * Purpose:  This function configures fading simulator reference path
 *           settings.
/============================================================================*/
ViStatus _VI_FUNC rssfu_confFadingReferenceSettings (ViSession instrSession,
                                                     ViInt32 fader,
                                                     ViInt32 reference,
                                                     ViBoolean commonSpeedAllPath,
                                                     ViBoolean commonDopplerShiftAllPath)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buf1[RSSFU_TOKEN_BUFFER_SIZE] = "";
    ViInt32     l_fader = fader - 1;
    ViChar      model [RSSFU_IO_BUFFER_SIZE] = ""; 
    ViUInt32    attr = RSSFU_ATTR_FADING_REFERENCE;  

  
        
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    viCheckErr (rssfu_GetAttributeViString (instrSession, "",
                                               RS_ATTR_INSTRUMENT_MODEL,
                                               RSSFU_IO_BUFFER_SIZE, model));
    if ((fader < 1) || (fader > 2))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Fader");   
    
    if (strstr (model, "BTC") != NULL) { 
        attr = RSSFU_ATTR_FADING_REFERENCE_BTC;
    }
        
    if (strstr (model, "SFC") == NULL) {
        checkErr (rssfu_GetActualRepCaps (instrSession, RSSFU_ATTR_FADING_REFERENCE, "Fader", l_fader, buf1));
    
        viCheckParm (rssfu_SetAttributeViInt32 (instrSession, buf1, RSSFU_ATTR_FADING_REFERENCE, 
                        reference), 3, "Reference");
    }
    switch (reference){
        case RSSFU_VAL_FADING_REFERENCE_SPE:
            if (strstr (model, "SFC") == NULL) { 
                viCheckParm (rssfu_SetAttributeViBoolean (instrSession, buf1, RSSFU_ATTR_FADING_COMMON_SPEED_ALL_PATH, 
                commonSpeedAllPath), 4, "Common Speed All Path");
            }
        break;
        case RSSFU_VAL_FADING_REFERENCE_FDOP:
            if (strstr (model, "SFU") != NULL) {
                viCheckParm (rssfu_SetAttributeViBoolean (instrSession, buf1, RSSFU_ATTR_FADING_COMMON_DOPPLER_SHIFT_ALL_PATH, 
                commonDopplerShiftAllPath), 5, "Common Doppler Shift All Path");
            }
        break;
        
        default:
            viCheckParm( RS_ERROR_INVALID_PARAMETER, 3, "Reference");
    }
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}
    
/*============================================================================/
 * Function: Configure Fading Max Paths
 * Purpose:  This function configures fading simulator Max Path.
/============================================================================*/
ViStatus _VI_FUNC rssfu_confFadingMaxPaths (ViSession instrSession,
                                            ViInt32 fader, ViBoolean maxPaths)

{
    ViStatus    error = VI_SUCCESS;
    ViChar      buf1[RSSFU_TOKEN_BUFFER_SIZE] = "";
    ViInt32     l_fader = fader - 1;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));   
    if ((fader < 1) || (fader > 2))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Fader"); 
        
    checkErr (rssfu_GetActualRepCaps (instrSession, RSSFU_ATTR_FADING_MAX_PATHS, "Fader", l_fader, buf1));
    
    viCheckParm (rssfu_SetAttributeViBoolean (instrSession, buf1, RSSFU_ATTR_FADING_MAX_PATHS,
        maxPaths), 2, "Max Paths");  
 
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/*============================================================================/
 * Function: Fading Coupling Settings
 * Purpose:  This function couples the setting for the Speed or Dopplershift
 *           for the paths of both faders.
/============================================================================*/
ViStatus _VI_FUNC rssfu_confFadingCouplingSettings (ViSession instrSession,
                                                    ViInt32 fader, ViInt32 couplingSel,
                                                    ViBoolean speedCoupled_AB,
                                                    ViBoolean dopplerShiftCoupled_AB)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buf1[RSSFU_TOKEN_BUFFER_SIZE] = "";
    ViInt32     l_fader = fader - 1;
    ViChar      model [RSSFU_IO_BUFFER_SIZE] = "";      

    checkErr( Rs_LockSession (instrSession, VI_NULL));   
    
    if ((fader < 1) || (fader > 2))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Fader");
    
    viCheckErr (rssfu_GetAttributeViString (instrSession, "",
                                               RS_ATTR_INSTRUMENT_MODEL,
                                               RSSFU_IO_BUFFER_SIZE, model));
    checkErr (rssfu_GetActualRepCaps (instrSession, RSSFU_ATTR_FADING_SPEED_COUPLED, "Fader", l_fader, buf1));
   
    switch (couplingSel){
        case RSSFU_VAL_FADING_REFERENCE_SPE:
            if (strstr (model, "SFC") == NULL) {            
                viCheckErr (rssfu_SetAttributeViBoolean (instrSession, buf1, RSSFU_ATTR_FADING_SPEED_COUPLED, speedCoupled_AB));
            }
        break;
        case RSSFU_VAL_FADING_REFERENCE_FDOP:
            if (strstr (model, "SFU") != NULL) {
             viCheckErr (rssfu_SetAttributeViBoolean (instrSession, buf1, RSSFU_ATTR_FADING_DOPPLER_SHIFT_COUPLED, dopplerShiftCoupled_AB));
            }
        break;
        
      
        default:
            viCheckParm( RS_ERROR_INVALID_PARAMETER, 3, "Coupling Selection");
    }   
        
    checkErr( Rs_LockSession (instrSession, VI_NULL));
 
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Read Fading Clipped Sample
 * Purpose:  This function queries the share of samples which were clipped 
 *           due to the insertion loss setting.
/============================================================================*/
ViStatus _VI_FUNC rssfu_readFadingClippedSample(ViSession   instrSession,
                                                ViInt32 fader,
                                                ViReal64*   clippedSample)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buf1[RSSFU_TOKEN_BUFFER_SIZE] = "";  
    ViInt32     l_fader = fader - 1;

    checkErr( Rs_LockSession (instrSession, VI_NULL));      
    if ((fader < 1) || (fader > 2))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Fader"); 

    checkErr (rssfu_GetActualRepCaps (instrSession, RSSFU_ATTR_FADING_CLIPPED_SAMPLE, "Fader", l_fader, buf1));

    viCheckParm (rssfu_GetAttributeViReal64 (instrSession, buf1, RSSFU_ATTR_FADING_CLIPPED_SAMPLE, 
                clippedSample), 3, "Clipped Sample");
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 *---------------------------- Power Meter ----------------------------------*
 *****************************************************************************/
/*============================================================================/
 * Function: Enable Power Meter
 * Purpose:  This function enables/disables the level measurement.
/============================================================================*/
ViStatus _VI_FUNC rssfu_actEnablePowerMeter (ViSession instrSession,
                                             ViBoolean state)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rssfu_SetAttributeViBoolean (instrSession, "", RSSFU_ATTR_POWER_METER_STATE, state),
                         2, "State");
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Power Meter Configuration
 * Purpose:  This function selects the unit used for measured value and
 *           sets the reference value for use in computation of relative
 *           power value.
/============================================================================*/
ViStatus _VI_FUNC rssfu_powerMeterConfiguration (ViSession instrSession,
                                                 ViInt32 unit,
                                                 ViReal64 referencePower)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_POWER_METER_UNIT, unit),
                         2, "Unit");
    viCheckParm (rssfu_SetAttributeViReal64 (instrSession, "", RSSFU_ATTR_POWER_METER_REF_POWER, 
                        referencePower), 3, "Reference Power");

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}
    
/*============================================================================/
 * Function: Configure Power Meter Frequency
 * Purpose:  This function configures the power meter frequency.
/============================================================================*/
ViStatus _VI_FUNC rssfu_confPowerMeterFrequency (ViSession instrSession,
                                                 ViInt32 source,
                                                 ViReal64 frequency)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_POWER_METER_SOURCE, source),
                         2, "Source");
    viCheckParm (rssfu_SetAttributeViReal64 (instrSession, "", RSSFU_ATTR_POWER_METER_FREQUENCY, 
                        frequency), 3, "Frequency");

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Relative Power State
 * Purpose:  This function selects the state of the relative power
 *           measurement and a returns the relative output power measured
 *           with power sensor.
/============================================================================*/
ViStatus _VI_FUNC rssfu_relativePowerState (ViSession instrSession,
                                            ViInt32 relativePowerState,
                                            ViReal64* relativePower)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_POWER_METER_REL_POWER_STATE,
                        relativePowerState), 2, "Relative Power State");
    
    viCheckParm (rssfu_GetAttributeViReal64 (instrSession, "", RSSFU_ATTR_POWER_METER_REL_POWER, 
                relativePower), 3, "Relative Power");

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Read Absolute Output Power
 * Purpose:  This function returns the absolute output power measured with
 *           power sensor.
/============================================================================*/
ViStatus _VI_FUNC rssfu_readAbsoluteOutputPower (ViSession instrSession,
                                                 ViReal64 *power)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rssfu_GetAttributeViReal64 (instrSession, "", RSSFU_ATTR_POWER_METER_POWER, 
                power), 2, "Power");

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Power Meter M2Ref
 * Purpose:  This function stores the current power value for use in
 *           computation of relative power value.
/============================================================================*/
ViStatus _VI_FUNC rssfu_actPowerMeterM2Ref (ViSession instrSession)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));    

    viCheckErr (rssfu_SetAttributeViString (instrSession, "", RSSFU_ATTR_POWER_METER_M2REF, "")); 

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/*============================================================================/
 * Function: Power Meter Zero
 * Purpose:  This function activates the auto zero function.
/============================================================================*/
ViStatus _VI_FUNC rssfu_actPowerMeterZero (ViSession instrSession)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));    

    viCheckErr (rssfu_SetAttributeViString (instrSession, "", RSSFU_ATTR_POWER_METER_ZERO, "")); 

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/*============================================================================/
 * Function: Read Power Sensor Type
 * Purpose:  This function queries the type of the power sensor in use.
/============================================================================*/
ViStatus _VI_FUNC rssfu_readPowerSensorType (ViSession instrSession,
                                             ViInt32 bufferSize,
                                             ViChar sensorType[])

{
    ViStatus    error = VI_SUCCESS;
       
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rssfu_GetAttributeViString (instrSession, "", RSSFU_ATTR_POWER_METER_SENSOR_TYPE, 
                        bufferSize, sensorType), 2, "Sensor Type");  
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Configure Average Filter
 * Purpose:  This function configures the filter length used for measurement.
/============================================================================*/
ViStatus _VI_FUNC rssfu_confAverageFilter (ViSession instrSession,
                                           ViInt32 filterType,
                                           ViInt32 filterLength)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_POWER_METER_FILTER_TYPE,
                        filterType), 2, "Filter Type");
    if(filterType == RSSFU_VAL_PWR_METER_FILTER_USER)
    {
        viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_POWER_METER_FILTER_LENGTH,
                        filterLength), 3, "Filter Length");
    }   
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Average Auto Once
 * Purpose:  This function activates a single search for the optimum filter
 *           length for the current measurement conditions.
/============================================================================*/
ViStatus _VI_FUNC rssfu_actAverageAutoOnce (ViSession instrSession)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));    

    viCheckErr (rssfu_SetAttributeViString (instrSession, "", RSSFU_ATTR_POWER_METER_AUTO_ONCE, "")); 

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 *------------------------- BER Application ---------------------------------*
 *****************************************************************************/
/*============================================================================/
 * Function: Select BER Input
 * Purpose:  This function select the input for BER measurement.
/============================================================================*/
ViStatus _VI_FUNC rssfu_actBERSelect(ViSession instrSession,
                                     ViInt32 input)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_BER_SEL_MEAS, 
                input), 2, "Input");
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Run BER Measurement
 * Purpose:  This function configures common parameters of BER measurement.
/============================================================================*/
ViStatus _VI_FUNC rssfu_actBER(ViSession   instrSession,
                               ViInt32 action)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    switch (action){
        case RSSFU_VAL_BER_STOP:
            viCheckErr (rssfu_SetAttributeViBoolean (instrSession, "", RSSFU_ATTR_BER_MEAS_STAT, VI_FALSE));
        break;
        case RSSFU_VAL_BER_START:
            viCheckErr (rssfu_SetAttributeViBoolean (instrSession, "", RSSFU_ATTR_BER_MEAS_STAT, VI_TRUE));
        break;
        case RSSFU_VAL_BER_RESTART:
            viCheckErr (rssfu_SetAttributeViString (instrSession, "", RSSFU_ATTR_BER_RESTART, ""));
        break;
        default:
            viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Action");
    }   
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Configure BER Serial Clock Data Input
 * Purpose:  This function configures parameters of BER measurement for 
 *           Serial Clock Data Input.
/============================================================================*/
ViStatus _VI_FUNC rssfu_confBERSerial(ViSession   instrSession,
                                      ViInt32 clock,
                                      ViInt32 data,
                                      ViInt32 enable)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_BER_INP_CLK, 
                clock), 2, "Clock");
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_BER_DATA, 
                data), 3, "Data");
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_BER_FUNC_INP, 
                enable), 4, "Enable");
                
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Configure BER Inputs
 * Purpose:  This function configures parameters of BER measurement for 
 *           other inputs than Serial Clock Data Input.
/============================================================================*/
ViStatus _VI_FUNC rssfu_confBERInputs(ViSession   instrSession,
                                      ViInt32 signal,
                                      ViInt32 spiDVALID,
                                      ViInt32 gatingMode)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      model [RSSFU_IO_BUFFER_SIZE] = "";  

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckErr (rssfu_GetAttributeViString (instrSession, "",
                                               RS_ATTR_INSTRUMENT_MODEL,
                                               RSSFU_IO_BUFFER_SIZE, model));;
   
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_BER_SIGNAL , 
                signal), 2, "Signal");
    if (strstr (model, "SFU") != NULL) { 
        viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_BER_PARALLEL, 
                    spiDVALID), 3, "SPI DVALID");
    }
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_BER_GATING_MODE, 
                gatingMode), 4, "Gating Mode");
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Configure BER Gating Time
/* Purpose:  This function defines the gating time in hh:mm:ss format. Only 
/*           effective if gating mode with user defined time is selected.
/*===========================================================================*/
ViStatus _VI_FUNC rssfu_confBERGatingTime(
    ViSession   instrSession,
    ViString    gatingTime
)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));    

    viCheckErr (rssfu_SetAttributeViString (instrSession, "", RSSFU_ATTR_BER_USER_GATING_TIME, gatingTime)); 

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;   
}

/*============================================================================/
 * Function: Configure BER Settings
 * Purpose:  This function configures settings of BER measurement.
/============================================================================*/
ViStatus _VI_FUNC rssfu_confBERSettings(ViSession   instrSession,
                                        ViInt32 payload,
                                        ViInt32 prbs)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_BER_BIT_PATT , 
                payload), 2, "Payload");
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_BER_PRBS_SEQ, 
                prbs), 3, "PRBS");

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Get BER Measurement
 * Purpose:  This function queries selected BER measurement.
/============================================================================*/
ViStatus _VI_FUNC rssfu_readBER(ViSession   instrSession,
                                ViInt32 measurement,
                                ViInt32 bufferSize,
                                ViChar  result[])
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if ((measurement < RSSFU_VAL_BER_BER) || (measurement > RSSFU_VAL_BER_ALL))  
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Measurement");
    if (bufferSize<=0)
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 3, "Buffer Size");                                 
    
    switch (measurement){
        case RSSFU_VAL_BER_BER:
            viCheckParm (rssfu_GetAttributeViString (instrSession, "", RSSFU_ATTR_BER_READ_RESULT, 
                        bufferSize, result), 4, "Result");  
        break;
        case RSSFU_VAL_BER_EVAL:
            viCheckParm (rssfu_GetAttributeViString (instrSession, "", RSSFU_ATTR_BER_READ_EVAL, 
                        bufferSize, result), 4, "Result");  
        break;
        case RSSFU_VAL_BER_STATE:
            viCheckParm (rssfu_GetAttributeViString (instrSession, "", RSSFU_ATTR_BER_STATE, 
                        bufferSize, result), 4, "Result");  
        break;
        case RSSFU_VAL_BER_ERROR_COUNT:
            viCheckParm (rssfu_GetAttributeViString (instrSession, "", RSSFU_ATTR_BER_ERROR_COUNT, 
                        bufferSize, result), 4, "Result");  
        break;
        case RSSFU_VAL_BER_GATING_TIME:
            viCheckParm (rssfu_GetAttributeViString (instrSession, "", RSSFU_ATTR_BER_GATING_TIME, 
                        bufferSize, result), 4, "Result");  
        break;
        case RSSFU_VAL_BER_ALL:         
            viCheckParm (rssfu_GetAttributeViString (instrSession, "", RSSFU_ATTR_BER_READ_ALL_RESULTS, 
                        bufferSize, result), 4, "Result");  
        break;      
    }   
        
Error:      
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/*****************************************************************************
 *------------------------- ARB Generator -----------------------------------*
 *****************************************************************************/
/*============================================================================/
 * Function: Enable ARB Modulation
 * Purpose:  This function enables/disables ARB modulation.
/============================================================================*/
ViStatus _VI_FUNC rssfu_actEnableARB(ViSession   instrSession,
                                     ViBoolean   state)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rssfu_SetAttributeViBoolean (instrSession, "", RSSFU_ATTR_ARB , 
                state), 2, "State");
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Load ARB Waveform
 * Purpose:  This function selects the waveform to be loaded.
/============================================================================*/
ViStatus _VI_FUNC rssfu_actLoadARBWaveform(ViSession   instrSession,
                                           ViString    loadWaveform)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rssfu_SetAttributeViString (instrSession, "", RSSFU_ATTR_ARB_LOAD_WAVEFORM, 
                loadWaveform), 2, "Load Waveform");
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Delete ARB Waveform
 * Purpose:  This function deletes the specified waveform file.
/============================================================================*/
ViStatus _VI_FUNC rssfu_DeleteARBWaveform (ViSession instrSession,
                                           ViString deleteWaveform)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rssfu_SetAttributeViString (instrSession, "", RSSFU_ATTR_ARB_WAVEFORM_DELETE, 
                deleteWaveform), 2, "Delete Waveform");
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}                                           
                                           
/*============================================================================/
 * Function: ARB Waveform Samples
 * Purpose:  This function queries the number of samples in the waveform file
 *           selected using command :ARB:WAV:SEL.
/============================================================================*/
ViStatus _VI_FUNC rssfu_ARBWaveformSamples (ViSession instrSession,
                                            ViString waveformFilename,
                                            ViInt32* numberOfSamples)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSSFU_IO_BUFFER_SIZE] = ""; 
    ViChar      model [RSSFU_IO_BUFFER_SIZE] = "";

    checkErr( Rs_LockSession (instrSession, VI_NULL));
   
    viCheckErr (rssfu_GetAttributeViString (instrSession, "",
                                               RS_ATTR_INSTRUMENT_MODEL,
                                               RSSFU_IO_BUFFER_SIZE, model));
    if (strstr (model, "SFU") != NULL) { 
        if (Rs_Simulating(instrSession) == 0){                /* call only when locked */

        if (!numberOfSamples) viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Number Of Samples");
    
        viCheckErr( viPrintf (instrSession, ":MMEM:CDIR '%s'", waveformFilename)); 
        //sprintf (buffer,":BB:ARB:WAV:POIN? '%s'", waveformFilename);
        sprintf (buffer,":BB:ARB:WAV:POIN?");   
    
        viCheckErr( viQueryf (instrSession, buffer, "%ld", numberOfSamples));
        }   
    }    
    viCheckErr(rssfu_CheckStatus (instrSession));    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}                                            

/*============================================================================/
 * Function: ARB Waveform File Tag
 * Purpose:  This function selects the waveform to be loaded.
/============================================================================*/
ViStatus _VI_FUNC rssfu_actARBWaveformFileTag (ViSession instrSession,
                                               ViString waveformFileName,
                                               ViInt32 tag, ViInt32 bufferSize,
                                               ViChar output[],
                                               ViInt32 *returnedValues)
{
   
    ViChar*     pbuffer = NULL;
    ViUInt32    count = 0, len = 0;
    ViStatus    error   = VI_SUCCESS;
    ViChar      model [RSSFU_IO_BUFFER_SIZE] = "";

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    viCheckErr (rssfu_GetAttributeViString (instrSession, "",
                                               RS_ATTR_INSTRUMENT_MODEL,
                                               RSSFU_IO_BUFFER_SIZE, model));   
  
    if (strstr (model, "SFC") == NULL) { 
        if (Rs_Simulating(instrSession) == 0){                /* call only when locked */

        if (bufferSize != 0 && output == VI_NULL)
            return VI_ERROR_PARAMETER4;

        viCheckErr( viPrintf(instrSession, ":BB:ARB:WAV:DATA? '%s','%s'\n",
            waveformFileName, waveformFileTagArr[tag]));
   
    
        viCheckErr( rssfu_WaitForOPC (instrSession, MAX_OPC_TIME)); 
        viCheckErr( rssfu_ReadDataUnknownLength(instrSession, &pbuffer, &count));
    
        if (output != NULL) 
        {
            len = (ViUInt32) ((bufferSize > (ViInt32) count) ? (ViInt32) count : bufferSize);
            strncpy(output, pbuffer, len);
            output[len] = '\0';
        }
    
        free(pbuffer);
        if (returnedValues) *returnedValues = (ViInt32) count;
    }    
    viCheckErr(rssfu_CheckStatus (instrSession)); 
    }
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;   
}

/*============================================================================/
 * Function: ARB Waveform Catalog
 * Purpose:  This function reads out the files in the default directory.
/============================================================================*/
ViStatus _VI_FUNC rssfu_ARBWaveformCatalog (ViSession instrSession,
                                            ViString directory, ViInt32 bufferSize,
                                            ViChar output[],
                                            ViInt32 *returnedValues)
{
   
    ViChar*     pbuffer = NULL;
    ViUInt32    count = 0, len = 0;
    ViStatus    error   = VI_SUCCESS;
    ViChar      model [RSSFU_IO_BUFFER_SIZE] = "";  

    checkErr( Rs_LockSession (instrSession, VI_NULL));
        
    viCheckErr (rssfu_GetAttributeViString (instrSession, "",
                                               RS_ATTR_INSTRUMENT_MODEL,
                                               RSSFU_IO_BUFFER_SIZE, model));
    if (strstr (model, "SFC") == NULL) { 
        if (Rs_Simulating(instrSession) == 0){                /* call only when locked */

        if (bufferSize != 0 && output == VI_NULL)
            return VI_ERROR_PARAMETER5;

        viCheckErr( viPrintf (instrSession, ":MMEM:CDIR '%s'", directory)); 
        viCheckErr( viPrintf (instrSession, ":BB:ARB:WAV:CAT?\n"));
       // viCheckErr( viPrintf(instrSession, ":BB:ARB:WAV:CAT? '%s'\n", directory));
   
    
        viCheckErr( rssfu_WaitForOPC (instrSession, MAX_OPC_TIME)); 
        viCheckErr( rssfu_ReadDataUnknownLength(instrSession, &pbuffer, &count));
    
        if (output != NULL) 
        {
            len = (ViUInt32) ((bufferSize > (ViInt32) count) ? (ViInt32) count : bufferSize);
            strncpy(output, pbuffer, len);
            output[len] = '\0';
        }
    
        free(pbuffer);
        if (returnedValues) *returnedValues = (ViInt32) count;
        }
    viCheckErr(rssfu_CheckStatus (instrSession)); 
    }
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;   
}
                                            

/*============================================================================/
 * Function: ARB Waveform Catalog Length
 * Purpose:  This function reads out the number of files in the default
 *           directory for waveform files (D: ARBWaveforms).
/============================================================================*/
ViStatus _VI_FUNC rssfu_ARBWaveformCatalogLength (ViSession instrSession,
                                                  ViString directory,
                                                  ViInt32* catalogLength)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSSFU_IO_BUFFER_SIZE] = ""; 
    ViChar      model [RSSFU_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckErr (rssfu_GetAttributeViString (instrSession, "",
                                               RS_ATTR_INSTRUMENT_MODEL,
                                               RSSFU_IO_BUFFER_SIZE, model));   
    if (strstr (model, "SFC") == NULL) {
        if (Rs_Simulating(instrSession) == 0){                /* call only when locked */

        if (!catalogLength) viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Catalog Length");
    
        viCheckErr( viPrintf (instrSession, ":MMEM:CDIR '%s'", directory)); 
        //sprintf (buffer,":BB:ARB:WAV:CAT:LENG? '%s'", directory);
        sprintf (buffer,":BB:ARB:WAV:CAT:LENG?");    
    
        viCheckErr( viQueryf (instrSession, buffer, "%ld", catalogLength));
        }   
    }
    viCheckErr(rssfu_CheckStatus (instrSession));
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}                                                    

/*============================================================================/
 * Function: ARB Memory Size
 * Purpose:  This function returns the size of the internal ARB memory in MByte.
/============================================================================*/
ViStatus _VI_FUNC rssfu_ARBMemorySize (ViSession instrSession, 
                                       ViInt32 *size)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckErr (rssfu_GetAttributeViInt32 (instrSession, "", RSSFU_ATTR_ARB_MEMORY_SIZE,
                                            size));
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: ARB Samples Maximum
 * Purpose:  This function returns the maximum number of samples that can be 
 *           loaded into the internal ARB memory.
/============================================================================*/
ViStatus _VI_FUNC rssfu_ARBSamplesMaximum (ViSession instrSession,
                                           ViInt32 *samples)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckErr (rssfu_GetAttributeViInt32 (instrSession, "", RSSFU_ATTR_ARB_MAXIMUM_SAMPLES,
                                            samples));
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Configure ARB Clock
 * Purpose:  This function configures settings of ARB clock.
/============================================================================*/
ViStatus _VI_FUNC rssfu_confARBClock(ViSession instrSession,
                                     ViReal64 frequency,
                                     ViInt32 source)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      model [RSSFU_IO_BUFFER_SIZE] = "";  

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    viCheckErr (rssfu_GetAttributeViString (instrSession, "",
                                               RS_ATTR_INSTRUMENT_MODEL,
                                               RSSFU_IO_BUFFER_SIZE, model));   
    
    if (strstr (model, "SFC") == NULL) { 
        viCheckParm (rssfu_SetAttributeViReal64 (instrSession, "", RSSFU_ATTR_ARB_CLOCK_FREQUENCY, 
                frequency), 2, "Frequency");
    }

    if (strstr (model, "SFU") != NULL) { 
        viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_ARB_CLOCK_SOURCE, 
                source), 3, "Source");  
    }
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Configure ARB External Clock
 * Purpose:  This function configures settings of ARB external clock.
/============================================================================*/
ViStatus _VI_FUNC rssfu_confARBExternalClock (ViSession instrSession,
                                              ViInt32 clockMode,
                                              ViInt32 sampleClockMultiplier)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_ARB_EXT_CLOCK_MODE, 
                clockMode), 2, "Clock Mode");
    
    if (clockMode == RSSFU_VAL_ARB_EXT_MODE_MSAM)
        viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_ARB_EXT_SAMPLE_CLOCK_MULTIPLIER, 
                        sampleClockMultiplier), 3, "Sample Clock Multiplier");              
    Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}                                              

/*============================================================================/
 * Function: Read ARB External Clock Frequency
 * Purpose:  This function indicates the measured frequency of the external
 *           clock signal.
/============================================================================*/
ViStatus _VI_FUNC rssfu_readARBExternalClockFrequency (ViSession instrSession,
                                                       ViReal64* measuredExtClock)
{
    ViStatus    error = VI_SUCCESS;

                                                         
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rssfu_GetAttributeViReal64 (instrSession, "", RSSFU_ATTR_ARB_EXT_MEASURED_EXTERNAL_CLOCK, 
                measuredExtClock), 2, "Measured Ext Clock");
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}                                                       


/*============================================================================/
 * Function: Configure ARB Trigger
 * Purpose:  This function configures settings of ARB Trigger.
/============================================================================*/
ViStatus _VI_FUNC rssfu_confARBTrigger (ViSession instrSession, ViInt32 mode,
                                        ViInt32 trigegrSource, ViReal64 triggerDelay,
                                        ViReal64 triggerInhibit)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_ARB_TRIGGER_MODE, 
                        mode), 2, "Mode");
    
    if (mode != RSSFU_ARB_VAL_TRIG_AUTO) {
        viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_ARB_TRIGGER_SOURCE, 
                        trigegrSource), 3, "Trigegr Source");              
        
        viCheckParm (rssfu_SetAttributeViReal64 (instrSession, "", RSSFU_ATTR_ARB_TRIGGER_DELAY, 
                        triggerDelay), 4, "Trigger Delay");                     
        
        viCheckParm (rssfu_SetAttributeViReal64 (instrSession, "", RSSFU_ATTR_ARB_TRIGGER_INHIBIT, 
                        triggerInhibit), 5, "Trigger Inhibit");                     
    }                       
                        
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}                                           

/*============================================================================/
 * Function: ARB Trigger Execute
 * Purpose:  This function executes selected ARB Trigger.
/============================================================================*/
ViStatus _VI_FUNC rssfu_actARBTriggerExecute (ViSession instrSession,
                                              ViInt32 trigger)
{
    ViStatus    error = VI_SUCCESS;
     
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    if ((trigger < RSSFU_ARB_VAL_TRIGGER_ARM) || (trigger > RSSFU_ARB_VAL_TRIGGGER_EXECUTE))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Trigger");
    
    switch (trigger){
        case RSSFU_ARB_VAL_TRIGGER_ARM:
            viCheckErr (rssfu_SetAttributeViString (instrSession, "", RSSFU_ATTR_ARB_TRIGGER_ARM, "")); 

        break;
        
        case RSSFU_ARB_VAL_TRIGGGER_EXECUTE:
            viCheckErr (rssfu_SetAttributeViString (instrSession, "", RSSFU_ATTR_ARB_TRIGGER_EXECUTE, "")); 

        break;
    }    
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}                       
                                              
/*============================================================================/
 * Function: ARB Trigger Status 
 * Purpose:  This function displays the status of waveform output for all
 *           trigger modes.
/============================================================================*/
ViStatus _VI_FUNC rssfu_readARBTriggerStatus (ViSession instrSession,
                                              ViChar triggerStatus[])
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSSFU_IO_BUFFER_SIZE]="\0";

                                                         
    checkErr( Rs_LockSession (instrSession, VI_NULL));
          

    if (triggerStatus == NULL) viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Trigger Status");
        viCheckErr (rssfu_GetAttributeViString (instrSession, "", RSSFU_ATTR_ARB_TRIGGER_STATUS,
                                            RSSFU_IO_BUFFER_SIZE, buffer));
    if (triggerStatus != NULL)
        strncpy (triggerStatus, buffer, strlen(buffer)); 
       
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}                                              


/*============================================================================/
 * Function: Configure ARB Marker
 * Purpose:  This function configures settings of ARB marker.
/============================================================================*/
ViStatus _VI_FUNC rssfu_confARBMarker (ViSession instrSession, ViInt32 marker1,
                                       ViInt32 delay)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_ARB_MARKER1, 
                marker1), 2, "Marker 1");
                
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_ARB_MARKER_DELAY, 
                delay), 3, "Delay");
              
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}                                       
      

/*============================================================================/
 * Function: Configure ARB Test Signal
 * Purpose:  This function configures settings of ARB test signal.
/============================================================================*/
ViStatus _VI_FUNC rssfu_confARBTestSignal(ViSession instrSession,
                                          ViReal64 frequency,
                                          ViInt32 samplesperPeriod,
                                          ViReal64 phaseOffset)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rssfu_SetAttributeViReal64 (instrSession, "", RSSFU_ATTR_ARB_TST_SIG_FREQUENCY, 
                frequency), 2, "Frequency");
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_ARB_TST_SIG_SAMPLES_PER_PERIOD, 
                samplesperPeriod), 3, "Samples Per Period");
    viCheckParm (rssfu_SetAttributeViReal64 (instrSession, "", RSSFU_ATTR_ARB_TST_SIG_PHASE_OFFSET_Q, 
                phaseOffset), 4, "Phase Offset");
                
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Generate ARB Test Signal
 * Purpose:  This function generates a signal and uses it as output straight 
 *           away.
/============================================================================*/
ViStatus _VI_FUNC rssfu_actARBGenerateTest(ViSession instrSession)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckErr (rssfu_SetAttributeViString (instrSession, "", RSSFU_ATTR_ARB_TST_SIG_GENERATE, ""));
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/*****************************************************************************
 *------------------------- TSGEN ---------------------------------*
 *****************************************************************************/
 
/*============================================================================/
 * Function: Select Application
 * Purpose:  TS Player and Recorder are sharing the same resources and
 *           therefore only one of these applications are available at the
 *           same time. The TS-Player is selectable only if the TS-Recorder
 *           is stopped. The same applies to the TS-Player. The TS-Recorder
 *           is only available when the TS-Player is stopped.This command
 *           make the preselection on one of the applications. Possible
 *           selections are TS Player (PLAY) or TS Record (RECord).
/============================================================================*/
ViStatus _VI_FUNC rssfu_confTsgenApplication (ViSession instrSession,
                                              ViInt32 application)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    

    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_TSGEN_PREPARE, 
                application), 2, "Application");
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: TSPlayer and Recorder Status
 * Purpose:  Select the status of the TSPlayer/Recorder.
/============================================================================*/
ViStatus _VI_FUNC rssfu_actTsgenStatus (ViSession instrSession,
                                        ViInt32 command)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      model [RSSFU_IO_BUFFER_SIZE] = "";
    ViUInt32    attr = RSSFU_ATTR_TSGEN_COMMAND;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    if (strstr (model, "BTC") != NULL) {
        if ((command < RSSFU_VAL_TSGEN_STOP) || (command > RSSFU_VAL_TSGEN_PLAY))
            viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Command");
        
        attr = RSSFU_ATTR_TSGEN_COMMAND_BTC;
    }
    
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", attr, command), 2, "Command");
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Read TSPlayer and Recorder Status
 * Purpose:  Queries the actual state of the TSPlayer or Recorder.
/============================================================================*/
ViStatus _VI_FUNC rssfu_readTsgenStatus (ViSession instrSession,
                                         ViInt32* command)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rssfu_GetAttributeViInt32 (instrSession, "", RSSFU_ATTR_TSGEN_COMMAND_STATE, 
                command), 2, "Command");
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Read Player Parameters
 * Purpose:  This function reads player parameters.
 *****************************************************************************/
ViStatus _VI_FUNC rssfu_readTsgenPlayerParam (ViSession instrSession,
                                         ViChar  fileDateSize[],
                                         ViChar  originalLoopTime[],
                                         ViReal64*  originalTSDataRate)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSSFU_IO_BUFFER_SIZE]="\0";
 
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (fileDateSize == VI_NULL)
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Null address for File Date / Size");
    if (originalLoopTime == VI_NULL)
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 3, "Null address for Original Loop Time");
    if (originalTSDataRate == VI_NULL)
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 4, "Null address for Original TS Data Rate");
        

    viCheckParm (rssfu_GetAttributeViString (instrSession, "", RSSFU_ATTR_TSGEN_FILE_DATE, 
                        RSSFU_IO_BUFFER_SIZE, buffer), 2, "File Date / Size");
    strncpy (fileDateSize, buffer, strlen(buffer));
    
    
    viCheckParm (rssfu_GetAttributeViString (instrSession, "", RSSFU_ATTR_TSGEN_ORIGINAL_LOOP_TIME, 
                        RSSFU_IO_BUFFER_SIZE, buffer), 3, "Original Loop Time");
    strncpy (originalLoopTime, buffer, strlen(buffer));  
    
    
    viCheckParm (rssfu_GetAttributeViReal64 (instrSession, "", RSSFU_ATTR_TSGEN_ORIGINAL_TS_DATA_RATE, 
                        originalTSDataRate), 4, "Original TS Data Rate");
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: File Operations
 * Purpose:  The MMEMory subsystem (Mass Memory) contains the commands for
 *           managing files and directories as well as for loading and
 *           storing complete instrument settings in files.
 *****************************************************************************/
ViStatus _VI_FUNC rssfu_actFileOperations (ViSession instrSession,
                                           ViInt32 operation,
                                           ViString source,
                                           ViString destination)
{
    ViStatus    error   = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
  
    if (Rs_Simulating(instrSession) == 0){                /* call only when locked */
    switch (operation){
        case RSSFU_VAL_MMEM_COPY:
            viCheckErr( viPrintf (instrSession, "MMEM:COPY '%s','%s'", source, destination)); 
        break;
        
        case RSSFU_VAL_MMEM_DELETE:
            viCheckErr( viPrintf (instrSession, "MMEM:DEL '%s'", source)); 
        break;
        
        case RSSFU_VAL_MMEM_RENAME:
            viCheckErr( viPrintf (instrSession, "MMEM:MOVE '%s','%s'", source, destination)); 
        break;
        
        case RSSFU_VAL_MMEM_MDIR:
            viCheckErr( viPrintf (instrSession, "MMEM:MDIR '%s'", source)); 
        break;
        
        case RSSFU_VAL_MMEM_CDIR:
            viCheckErr( viPrintf (instrSession, "MMEM:CDIR '%s'", source)); 
        break;        
        
        case RSSFU_VAL_MMEM_RDIR:
            viCheckErr( viPrintf (instrSession, "MMEM:RDIR '%s'", source)); 
        break;
        
        case RSSFU_VAL_MMEM_MSIS:
            viCheckErr( viPrintf (instrSession, "MMEM:MSIS '%s'", source)); 
        break;        
        }
    viCheckErr(rssfu_CheckStatus (instrSession));
    }
    

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================
 * Function: File Directory Path                                             
 * Purpose:  This fuction reads out the subdirectories or subdirectories and
 *           files in the specified directory.                        
/============================================================================*/
ViStatus _VI_FUNC rssfu_actFileCatPath (ViSession instrSession, ViInt32 read,
                                        ViString directory, ViInt32 bufferSize,
                                        ViChar output[], ViInt32 *returnedValues)
{
   
    ViChar*     pbuffer = NULL;
    ViUInt32    count = 0;
    ViStatus    error   = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
  
    if (Rs_Simulating(instrSession) == 0){                /* call only when locked */

    if (bufferSize != 0 && output == VI_NULL)
        return VI_ERROR_PARAMETER5;

    if (read == RSSFU_VAL_MMEM_CATALOG_FILES)
    {
        viCheckErr( viPrintf(instrSession, "*CLS;:MMEM:CAT? '%s'\n", directory));
    }
    else
        viCheckErr( viPrintf(instrSession, "*CLS;:MMEM:DCAT? '%s'\n", directory)); 
    
    
    viCheckErr( rssfu_WaitForOPC (instrSession, MAX_OPC_TIME)); 
    viCheckErr( rssfu_ReadDataUnknownLength(instrSession, &pbuffer, &count));
    
    if (output != NULL) strncpy(output, pbuffer, (size_t) ((bufferSize > (ViInt32) count) ? (ViInt32) count : bufferSize));
    free(pbuffer);
    if (returnedValues) *returnedValues = (ViInt32) count;
        
    viCheckErr(rssfu_CheckStatus (instrSession)); 
    }
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;   
}

/*============================================================================/
 * Function: Catalog Length
 * Purpose:  This function reads out the number of files or subdirectories in
 *           the specified directory.
/============================================================================*/
ViStatus _VI_FUNC rssfu_readCatalogLength (ViSession instrSession, ViInt32 read,
                                         ViInt32 *catalogLength)
{
   ViStatus    error   = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
  
    if (Rs_Simulating(instrSession) == 0){                /* call only when locked */

    if (!catalogLength) viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Catalog Length");
    
    if (read == RSSFU_VAL_MMEM_CATALOG_FILES)
    {
        viCheckErr( viQueryf (instrSession, "MMEM:CAT:LENG?", "%ld", catalogLength));
    }
    else
        viCheckErr( viQueryf (instrSession, "MMEM:DCAT:LENG?", "%ld", catalogLength));
 }   
    viCheckErr(rssfu_CheckStatus (instrSession));
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


///////////////////////////////////////////////////////////////////////////////////

/*============================================================================/
 * Function: Read To File From Instrument
 * Purpose:  This function is used to read data from the instrument and
 *           write it to a user specified file.
/============================================================================*/
ViStatus _VI_FUNC rssfu_readToFile (ViSession instrSession, ViString source,
                ViString destination)
{
    ViUInt32    retCnt = 0;
    ViChar      buffer[RSSFU_IO_BUFFER_SIZE] = "";
    ViChar      *exBuf;
    ViInt32     lenLength = 0;
    size_t      count = 0;
    ViInt32     cnt = 0;
    FILE        *targetFile;
    //exBuf = 0;

    ViStatus error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
  
    if (Rs_Simulating(instrSession) == 0){                /* call only when locked */
    //
    sprintf (buffer, "*CLS;:MMEM:DATA? '%s'\n", source);


    viCheckErr( viPrintf (instrSession, "%s", buffer));    
    viCheckErr( rssfu_WaitForOPC (instrSession, MAX_OPC_TIME)); 

    /* GPIB Interface specific operations */
   

    viCheckErr( viRead (instrSession, (ViBuf) buffer, 2, &retCnt));
    
    if (sscanf (buffer, "#%1ld", &lenLength) != 1) {
        error = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        return error;
        }

    viCheckErr( viRead (instrSession, (ViBuf) buffer, (ViUInt32) lenLength, &retCnt));

        buffer [lenLength]='\0';

    if (sscanf (buffer, "%ld", &cnt) != 1) {
        error = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        return error;
        }

    count = (size_t) cnt;
    /* Allocate memory space */
    exBuf = malloc (count + 1);
    if (exBuf == NULL)
       return VI_ERROR_ALLOC;
    else
        exBuf [0] = '\0';
    
    if ((error = viRead (instrSession, (ViBuf) exBuf, (ViUInt32) (count + 1), &retCnt)) < 0)
    {
    /* If not NULL pointer, then deallocate memory. */
        if (exBuf != NULL) free (exBuf);
           return error;
        }

    /* Store data from instrument to file on host computer */
    targetFile = fopen (destination, "wb");
    if (targetFile == NULL)
        {
        if (exBuf != NULL)
            free (exBuf);
        return VI_ERROR_INSTR_FILE_OPEN;
        }

    if (count != fwrite (exBuf, sizeof (ViByte), count, targetFile))
        {
        (void) fclose (targetFile);
        if (exBuf != NULL)
            free (exBuf);
        return VI_ERROR_INSTR_FILE_WRITE;
        }

    (void) fclose (targetFile);

    if (exBuf != NULL) /* If not NULL pointer, then deallocate memory. */
        free (exBuf);

    viCheckErr(rssfu_CheckStatus (instrSession)); 
    }
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;  
}

/*============================================================================/
 * Function: Write From File To Instrument
 * Purpose:  This function is used to read data from a user specified file
 *           and write it to the instrument.
/============================================================================*/
ViStatus _VI_FUNC rssfu_writeFromFile (ViSession instrSession, ViString source,
                ViString destination)
{
    ViInt32     digits;

    ViUInt32    fileSize            = 0;
    ViUInt32    bytesWrite          = 0;
    ViChar      *exBuf;
    ViChar      *p2buf;
    FILE        *sourceFile;

//

   ViStatus    error   = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
  
    if (Rs_Simulating(instrSession) == 0){                /* call only when locked */

    /* Open source file from host controller */
    sourceFile = fopen (source, "rb");
    if (sourceFile == NULL)
        return VI_ERROR_INSTR_FILE_OPEN;

    /* Get size of file */
    (void)fseek (sourceFile, 0, SEEK_END);
    fileSize = (ViUInt32) ftell (sourceFile);
    (void)fseek (sourceFile, 0, SEEK_SET);

    /* Allocate memory space (add 64 bytes for command header) */
    exBuf = malloc ((size_t)(fileSize + strlen (destination) + 64));
    if (exBuf == NULL)
        return VI_ERROR_ALLOC;
    else
        exBuf[0] = '\0';

    /* Create command header */
    digits = ((int)floor(log10((double)fileSize))) + 1;
    p2buf = exBuf;
    bytesWrite = (ViUInt32) sprintf (p2buf, ":MMEM:DATA '%s',#%ld%lu", destination, digits, fileSize);
    p2buf += bytesWrite;

    if (fileSize != (ViUInt32) fread (p2buf, sizeof (ViByte), (size_t)fileSize, sourceFile))
        {
        free (exBuf);
        return VI_ERROR_INSTR_FILE_OPEN;
        }

    bytesWrite += fileSize;
    (void)fclose (sourceFile);

  
             if ((error = viWrite (instrSession,
                (ViBuf) ":SYST:COMM:GPIB:LTER EOI\n", 25, VI_NULL)) < 0)
                 
                {
                /* If not NULL pointer, then deallocate memory. */
                free (exBuf);
                return error;
                }
            if ((error = viWrite (instrSession, (ViBuf) exBuf, bytesWrite, VI_NULL)) < 0)
                {
                /* If not NULL pointer, then deallocate memory. */
                free (exBuf);
                return error;
                }
            if ((error = viWrite (instrSession,
                (ViBuf) ":SYST:COMM:GPIB:LTER LSTAN\n", 27, VI_NULL)) < 0)
                {
                /* If not NULL pointer, then deallocate memory. */
                free (exBuf);
                return error;
                }



    if (exBuf) /* If not NULL pointer, then deallocate memory. */
        free (exBuf);

    viCheckErr(rssfu_CheckStatus (instrSession));
    }
    

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Select Play File
 * Purpose:  This function selects the file to be loaded into the TS Player
/============================================================================*/
ViStatus _VI_FUNC rssfu_confTsgenPlayfile (ViSession instrSession,
                                           ViString fileName)
{ 
    ViStatus    error = VI_SUCCESS; 
    ViChar      model [RSSFU_IO_BUFFER_SIZE] = "";
    ViUInt32    attr = RSSFU_ATTR_TSGEN_PLAYER_PLAYFILE;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    if (strstr (model, "BTC") != NULL) {
    
        attr = RSSFU_ATTR_MM_GENERATOR_PLAYER_FILE;
    }
        
    viCheckParm (rssfu_SetAttributeViString (instrSession, "", attr, fileName),
                         2, "File name");                           
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Set TS Byte Rate
 * Purpose:  This function sets the TS byte rate.
/============================================================================*/
ViStatus _VI_FUNC rssfu_setTsByteRate (ViSession instrSession,
                                          ViReal64 byteRate)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rssfu_SetAttributeViReal64 (instrSession, "", RSSFU_ATTR_TSGEN_PLAYER_BYTERATE, byteRate),
                         2, "Byte Rate"); 
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/*============================================================================/
 * Function: Timing Settings
 * Purpose:  This function configures timing settings.
/============================================================================*/
ViStatus _VI_FUNC rssfu_confTsgenTimingSettings (ViSession instrSession,
                                            ViReal64 TSDataRate,
                                            ViReal64 playWindowStart,
                                            ViReal64 playWindowStop,
                                            ViBoolean PCRJitter)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      model [RSSFU_IO_BUFFER_SIZE] = "";  

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckErr (rssfu_GetAttributeViString (instrSession, "",
                                              RS_ATTR_INSTRUMENT_MODEL,
                                               RSSFU_IO_BUFFER_SIZE, model));
    if (strstr (model, "BTC") != NULL) {

        viCheckParm (rssfu_SetAttributeViReal64 (instrSession, "", RSSFU_ATTR_MM_GENERATOR_PLAYER_TS_DATA_RATE, TSDataRate),
                             2, "TS Data Rate"); 
        viCheckParm (rssfu_SetAttributeViReal64 (instrSession, "", RSSFU_ATTR_MM_GENERATOR_PLAY_WINDOW_START, playWindowStart),
                             3, "Play Window Start"); 
        viCheckParm (rssfu_SetAttributeViReal64 (instrSession, "", RSSFU_ATTR_MM_GENERATOR_PLAY_WINDOW_STOP, playWindowStop),
                             4, "Play Window Stop"); 
    }   
    else
    {
        viCheckParm (rssfu_SetAttributeViReal64 (instrSession, "", RSSFU_ATTR_TSGEN_TS_DATA_RATE, TSDataRate),
                             2, "TS Data Rate"); 
        viCheckParm (rssfu_SetAttributeViReal64 (instrSession, "", RSSFU_ATTR_TSGEN_PLAY_WINDOW_START, playWindowStart),
                             3, "Play Window Start"); 
        viCheckParm (rssfu_SetAttributeViReal64 (instrSession, "", RSSFU_ATTR_TSGEN_PLAY_WINDOW_STOP, playWindowStop),
                             4, "Play Window Stop"); 
    }
    
    if (strstr (model, "SFU") != NULL) {   
        viCheckParm (rssfu_SetAttributeViBoolean (instrSession, "", RSSFU_ATTR_TSGEN_PCR_JITTER, PCRJitter),
                5, "PCR Jitter");
        }
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Configure PCR Jitter
 * Purpose:  This function configures PCR Jitter settings.
/============================================================================*/
ViStatus _VI_FUNC rssfu_confTsgenPcrJitter (ViSession instrSession,
                                       ViInt32 waveform,
                                       ViReal64 amplitude,
                                       ViReal64 frequency)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_TSGEN_WAVEFORM, waveform),
                         2, "Waveform"); 
    viCheckParm (rssfu_SetAttributeViReal64 (instrSession, "", RSSFU_ATTR_TSGEN_AMPLITUDE, amplitude),
                         3, "Amplitude"); 
    viCheckParm (rssfu_SetAttributeViReal64 (instrSession, "", RSSFU_ATTR_TSGEN_FREQUENCY, frequency),
                         4, "Frequency");    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: TSGen Nullpacket Stuffing
 * Purpose:  This function sets the output data rate of the TS Player.
/============================================================================*/
ViStatus _VI_FUNC rssfu_TSGenNullpacketStuffing (ViSession instrSession,
                                         ViBoolean NullpacketStuffing)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
                                                                  
    viCheckParm (rssfu_SetAttributeViBoolean (instrSession, "", RSSFU_ATTR_TSGEN_PLAYER_NULLPACKET_STUFFING , 
                NullpacketStuffing), 2, "Nullpacket Stuffing");
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Interface Settings (GEN)
 * Purpose:  This function configures generator interface.
/============================================================================*/
ViStatus _VI_FUNC rssfu_confTsgenInterfaceSetGen (ViSession instrSession,
                                             ViInt32 packetLength,
                                             ViInt32 TSSerialOutput)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      model [RSSFU_IO_BUFFER_SIZE] = "";  
    ViUInt32    attr = RSSFU_ATTR_TSGEN_TS_SERIAL_OUTPUT;

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckErr (rssfu_GetAttributeViString (instrSession, "",
                                               RS_ATTR_INSTRUMENT_MODEL,
                                               RSSFU_IO_BUFFER_SIZE, model));
   
    if (strstr (model, "SFU") != NULL) { /* read only for BTC, SFC */ 
        viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_TSGEN_PACKET_LENGTH, packetLength),
                             2, "Packet Length");
    
    }
    if (strstr (model, "BTC") != NULL) {
            attr = RSSFU_ATTR_MM_GENERATOR_PLAYER_OUTPUT_MODE;   
        
    }
        viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", attr, TSSerialOutput),
                             3, "TS Serial Output");
    
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}
 
/*============================================================================/
 * Function: Interface Settings Stop Data
 * Purpose:  This function determines whether the TS player generates data
 *           when it is in Pause or Stop mode.
/============================================================================*/
ViStatus _VI_FUNC rssfu_confTsgenInterfaceStopData (ViSession instrSession,
                                                    ViInt32 stopData)

{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_TSGEN_STOP_DATA, stopData),
                         2, "Stop Data"); 
   Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Configure TSGEN Packet Settings
 * Purpose:  This function configures TSGEN packet settings.
/============================================================================*/
ViStatus _VI_FUNC rssfu_confTSGENPacketSettings (ViSession instrSession,
                                                 ViInt32 testTSPacket,
                                                 ViInt32 payloadTestStuff,
                                                 ViInt32 PRBS)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    if ((payloadTestStuff < RSSFU_IQC_PRBS) || (payloadTestStuff > RSSFU_IQC_HFF))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 3, "Payload Test Stuff"); 
    
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_TSGEN_PLAYER_TEST_TS_PACKET, testTSPacket),
                         2, "Test TS Packet"); 
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_TSGEN_PAYLOAD_TEST_STUFF, payloadTestStuff),
                         3, "Payload Test Stuff");
                        
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_TSGEN_PLAYER_PRBS, PRBS),
                         4, "PRBS");    
    
   
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Configure TSGen Seamless Loop
 * Purpose:  This function configures seamless loop parameters.
/============================================================================*/
ViStatus _VI_FUNC rssfu_configureTSGenSeamlessLoop (ViSession instrSession,
                                                    ViBoolean continuityCounter,
                                                    ViBoolean PCRDTSPTS,
                                                    ViBoolean TDTTOT)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rssfu_SetAttributeViBoolean (instrSession, "", RSSFU_ATTR_TSGEN_SEAMLESS_LOOP_CONT_COUNT, 
                continuityCounter), 2, "continuityCounter");
    viCheckParm (rssfu_SetAttributeViBoolean (instrSession, "", RSSFU_ATTR_TSGEN_SEAMLESS_LOOP_PCR_DTS_PTS, 
                PCRDTSPTS), 3, "PCR,DTS/PTS");
    viCheckParm (rssfu_SetAttributeViBoolean (instrSession, "", RSSFU_ATTR_TSGEN_SEAMLESS_LOOP_TDT_TOT, 
                TDTTOT), 4, "TDT/TOT");
                    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: New Record Stream
 * Purpose:  This function selects the file to be recorded.
/============================================================================*/
ViStatus _VI_FUNC rssfu_confTsgenNewRecordStream (ViSession instrSession,
                                                  ViString fileName)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    viCheckParm (rssfu_SetAttributeViString (instrSession, "", RSSFU_ATTR_TSGEN_RECORD_FILENAME, fileName),
                         2, "File name");                           
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Read Recorder Parameters
 * Purpose:  This function reads recorder parameters.
/============================================================================*/
ViStatus _VI_FUNC rssfu_readTsgenRecParam (ViSession instrSession,
                                      ViReal64* freeMemory,
                                      ViReal64* inputTSDataRate,
                                      ViReal64* recordLength)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rssfu_GetAttributeViReal64 (instrSession, "", RSSFU_ATTR_TSGEN_FREE_MEMORY, freeMemory),
                                2, "Free Memory");    
    viCheckParm (rssfu_GetAttributeViReal64 (instrSession, "", RSSFU_ATTR_TSGEN_INPUT_TS_DATA_RATE, inputTSDataRate),
                                3, "Input TS Data Rate");
    viCheckParm (rssfu_GetAttributeViReal64 (instrSession, "", RSSFU_ATTR_TSGEN_RECORD_LENGTH, recordLength),
                                4, "Record Length");                                    
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Interface Settings (GEN)
 * Purpose:  This function configures recorder interface.
/============================================================================*/
ViStatus _VI_FUNC rssfu_confTsgenInterfaceSetRec (ViSession instrSession,
                                             ViInt32 input,
                                             ViReal64 recordSize,
                                             ViInt32 recordMode)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_TSGEN_INPUT, input),
                         2, "Input"); 
    viCheckParm (rssfu_SetAttributeViReal64 (instrSession, "", RSSFU_ATTR_TSGEN_RECORD_SIZE, recordSize),
                         3, "Record Size");
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_TSGEN_RECORD_MODE, recordMode),
                         4, "Record Mode");                          
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Record Progress
 * Purpose:  This function queries the current record progress in %.
/============================================================================*/
ViStatus _VI_FUNC rssfu_readTsgenRecordProgress (ViSession instrSession,
                                                 ViReal64* recordProgress)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rssfu_GetAttributeViReal64 (instrSession, "", RSSFU_ATTR_TSGEN_RECORD_PROGRESS,
                                            recordProgress), 2, "Record Progress");    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}                                                 

/*===========================================================================*/
/* Function: TSGen Seek Position
/* Purpose:  Queries the measured data rate of the transport stream at the 
/*           selected recorder input.
/*===========================================================================*/
ViStatus _VI_FUNC rssfu_readTSGenSeekPosition(
    ViSession   instrSession,
    ViReal64*   position
)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rssfu_GetAttributeViReal64 (instrSession, "", RSSFU_ATTR_TSGEN_SEEK_POSITION,
                                            position), 2, "Position");    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;   
}

/*****************************************************************************
 *------------------------- Adjustment --------------------------------------*
 *****************************************************************************/
/*============================================================================/
 * Function: Configure Internal Level Correction
 * Purpose:  This function switches internal level correction on or off.
/============================================================================*/
ViStatus _VI_FUNC rssfu_confCalInternal(ViSession instrSession,
                                        ViBoolean internalLevelCorrection)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rssfu_SetAttributeViBoolean (instrSession, "", RSSFU_ATTR_CAL_LEV_STATE, 
                internalLevelCorrection), 2, "Internal Level Correction");
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Run Calibration
 * Purpose:  This function runs selected calibration.
/============================================================================*/
ViStatus _VI_FUNC rssfu_actCal(ViSession   instrSession,
                               ViInt32 calibrationType,
                               ViInt32*    result)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      model [RSSFU_IO_BUFFER_SIZE] = "";  
    
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckErr (rssfu_GetAttributeViString (instrSession, "",
                                               RS_ATTR_INSTRUMENT_MODEL,
                                               RSSFU_IO_BUFFER_SIZE, model));
    
    if ((calibrationType < RSSFU_VAL_CAL_ALL) || (calibrationType > RSSFU_VAL_CAL_VMF))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Calibration Type");

    switch (calibrationType){
        case RSSFU_VAL_CAL_ALL:
                viCheckErr( rssfu_SetAttributeViString (instrSession, "", RSSFU_ATTR_CAL_ALL, ""));
                viCheckParm (rssfu_GetAttributeViInt32 (instrSession, "", RSSFU_ATTR_CAL_ALL_RESULT, 
                            result), 3, "Result");
    break;
        case RSSFU_VAL_CAL_SYNTHESIS:
            if (strstr (model, "SFU") != NULL) {  
                viCheckErr( rssfu_SetAttributeViString (instrSession, "", RSSFU_ATTR_CAL_FREQ, ""));
                viCheckParm (rssfu_GetAttributeViInt32 (instrSession, "", RSSFU_ATTR_CAL_FREQ_RESULT, 
                            result), 3, "Result");
            }
        break;
        case RSSFU_VAL_CAL_LEVEL:
            if (strstr (model, "SFU") != NULL) { 
                viCheckErr( rssfu_SetAttributeViString (instrSession, "", RSSFU_ATTR_CAL_LEV, ""));
                viCheckParm (rssfu_GetAttributeViInt32 (instrSession, "", RSSFU_ATTR_CAL_LEV_RESULT, 
                            result), 3, "Result");
            }
        break;
        case RSSFU_VAL_CAL_VML:
            viCheckErr( rssfu_SetAttributeViString (instrSession, "", RSSFU_ATTR_CAL_LOCAL, ""));
            viCheckParm (rssfu_GetAttributeViInt32 (instrSession, "", RSSFU_ATTR_CAL_LOCAL_RESULT, 
                        result), 3, "Result");
        break;
        case RSSFU_VAL_CAL_VMF:
            if (strstr (model, "BTC") == NULL) { 
                viCheckErr( rssfu_SetAttributeViString (instrSession, "", RSSFU_ATTR_CAL_IQM_FULL, ""));
                viCheckParm (rssfu_GetAttributeViInt32 (instrSession, "", RSSFU_ATTR_CAL_IQM_FULL_RESULT, 
                            result), 3, "Result");
            }
        break;
    }

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Calibration Date
 * Purpose:  This function queries the date of the last system error 
 *           correction.
/============================================================================*/
ViStatus _VI_FUNC rssfu_getCalDate(ViSession   instrSession,
                                   ViInt32 calibrationType,
                                   ViChar  date[])
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSSFU_IO_BUFFER_SIZE] = "";
    ViChar      model [RSSFU_IO_BUFFER_SIZE] = ""; 
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckErr (rssfu_GetAttributeViString (instrSession, "",
                                               RS_ATTR_INSTRUMENT_MODEL,
                                               RSSFU_IO_BUFFER_SIZE, model));   
    
    if ((calibrationType < RSSFU_VAL_CAL_SYNTHESIS) || (calibrationType > RSSFU_VAL_CAL_VMF))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Calibration Type");
        
    switch (calibrationType){
        case RSSFU_VAL_CAL_SYNTHESIS:
            if (strstr (model, "SFU") != NULL) {
            viCheckErr (rssfu_GetAttributeViString (instrSession, "", RSSFU_ATTR_CAL_FREQ_DATE, RSSFU_IO_BUFFER_SIZE,
                        buffer));
            }
        break;
        case RSSFU_VAL_CAL_LEVEL:
            if (strstr (model, "SFC") == NULL) {  
                viCheckErr (rssfu_GetAttributeViString (instrSession, "", RSSFU_ATTR_CAL_LEV_DATE, RSSFU_IO_BUFFER_SIZE,
                        buffer));
            }
        break;
        case RSSFU_VAL_CAL_VML:
            if (strstr (model, "BTC") == NULL) { 
                viCheckErr (rssfu_GetAttributeViString (instrSession, "", RSSFU_ATTR_CAL_LOCAL_DATE, RSSFU_IO_BUFFER_SIZE,
                        buffer));
            }
        break;
        case RSSFU_VAL_CAL_VMF:
            if (strstr (model, "BTC") == NULL) {  
                viCheckErr (rssfu_GetAttributeViString (instrSession, "", RSSFU_ATTR_CAL_IQM_FULL_DATE, RSSFU_IO_BUFFER_SIZE,
                        buffer));
            }   
        break;
        default: buffer[0] = '\0';
    }   
    strcpy (date, buffer);
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/*****************************************************************************
 *------------------------- System  -----------------------------------------*
 *****************************************************************************/
/*============================================================================/
 * Function: System Display Update
 * Purpose:  This function activates/deactivates updating of the display.
 *           Upon changing to manual control, the status is automatically set
 *           to ON.
/============================================================================*/
ViStatus _VI_FUNC rssfu_systemDisplayUpdate (ViSession instrSession,
                                             ViBoolean displayUpdate)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rssfu_SetAttributeViBoolean (instrSession, "", RSSFU_ATTR_SYSTEM_DISPLAY_UPDATE, 
                displayUpdate), 2, "Display Update");
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: System Keypad Lock
 * Purpose:  This function locks or unlocks the front panel keypad of the
 *           R&S SFU including the LOCAL key.
/============================================================================*/
ViStatus _VI_FUNC rssfu_systemKeypadLock (ViSession instrSession,
                                          ViInt32 keypadLock)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_SYSTEM_KLOCK, 
                keypadLock), 2, "Keypad Lock");
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/*============================================================================/
 * Function: Set Status Register
 * Purpose:  This function sets the Enable, NTransition, and PTransition bits
 *           of status questionable and operating registers.
/============================================================================*/
ViStatus _VI_FUNC rssfu_setStatusRegister (ViSession instrSession,
                                           ViInt32 registerOperation,
                                           ViInt32 enable, ViInt32 PTransition,
                                           ViInt32 NTransition)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if ((registerOperation < RSSFU_VAL_PRESET_STAT_REG) || (registerOperation > RSSFU_VAL_OPER_STAT_REG))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Register Operation");
        
    if ((enable < 0) || (enable > 32767))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 3, "Enable");
    
    if ((PTransition < 0) || (PTransition > 32767))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 4, "PTransition");     
    
    if ((NTransition < 0) || (NTransition > 32767))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 5, "NTransition");     
            
        
    if (Rs_Simulating(instrSession) == 0) {                /* call only when locked */
    
        switch (registerOperation) {
            case RSSFU_VAL_PRESET_STAT_REG:
                viCheckErr(viPrintf (instrSession, ":STAT:PRES")); 
            break;
            
            case RSSFU_VAL_QUEST_STAT_REG:
                viCheckErr(viPrintf (instrSession, ":STAT:QUES:ENAB %ld;PTR %ld;NTR %ld",
                    enable, PTransition, NTransition)); 
            break;
            
            case RSSFU_VAL_OPER_STAT_REG:
                viCheckErr(viPrintf (instrSession, ":STAT:OPER:ENAB %ld;PTR %ld;NTR %ld",
                    enable, PTransition, NTransition)); 
            break;
        }
    }   
    
    viCheckErr(rssfu_CheckStatus (instrSession));       
     
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}
 

/*============================================================================/
 * Function: Get Status Register
 * Purpose:  This function contains the commands for the querying status
 *           reporting system.
/============================================================================*/
ViStatus _VI_FUNC rssfu_getStatusRegister (ViSession instrSession,
                                           ViInt32 registerOperation,
                                           ViInt32 statusRegistersQuery,
                                           ViInt32* registerValue)

{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if ((registerOperation < RSSFU_VAL_QUEST_STAT_REG   ) || (registerOperation > RSSFU_VAL_OPER_STAT_REG))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Register Operation");
    
    if ((statusRegistersQuery < RSSFU_VAL_REG_CONDITION   ) || (statusRegistersQuery > RSSFU_VAL_REG_NTRANSITION))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 3, "Register Operation");  
            
        
    if (Rs_Simulating(instrSession) == 0) {                /* call only when locked */
    
        switch (registerOperation) {
            case RSSFU_VAL_QUEST_STAT_REG:
                viCheckErr (viQueryf (instrSession, ":STAT:QUES:%s?", "%ld",
                    statusRegisterArr[statusRegistersQuery], registerValue)); 
            break;
            
            case RSSFU_VAL_OPER_STAT_REG:
                viCheckErr (viQueryf (instrSession, ":STAT:OPER:%s?", "%ld",
                    statusRegisterArr[statusRegistersQuery], registerValue)); 
            break;
        }
    }   

    viCheckErr(rssfu_CheckStatus (instrSession));       
     
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}                                          
                                           

/*============================================================================/
 * Function: Set User-definable Status Register
 * Purpose:  This function sets the Enable, NTransition and PTransition
 *           user-definable bits of status questionable and operating registers.
/============================================================================*/
ViStatus _VI_FUNC rssfu_setUserDefStatusRegister (ViSession instrSession,
                                                  ViInt32 registerOperation,
                                                  ViInt32 userdefinableBit,
                                                  ViInt32 enable,
                                                  ViInt32 PTransition,
                                                  ViInt32 NTransition)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      model [RSSFU_IO_BUFFER_SIZE] = "";  
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckErr (rssfu_GetAttributeViString (instrSession, "",
                                               RS_ATTR_INSTRUMENT_MODEL,
                                               RSSFU_IO_BUFFER_SIZE, model));

    if ((registerOperation < RSSFU_VAL_QUEST_STAT_REG) || (registerOperation > RSSFU_VAL_OPER_STAT_REG))
            viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Register Operation");
    
    if (strstr (model, "BTC") == NULL) {  
        if (registerOperation == RSSFU_VAL_QUEST_STAT_REG) {
            if ((userdefinableBit < 9) || (userdefinableBit > 12))
                viCheckParm( RS_ERROR_INVALID_PARAMETER, 3, "User-definable Bit");
        }
        else {
            if ((userdefinableBit < 8) || (userdefinableBit > 12))
                viCheckParm( RS_ERROR_INVALID_PARAMETER, 3, "User-definable Bit");
        }
    }
    if ((enable < 0) || (enable > 32767))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 4, "Enable");
    
    if ((PTransition < 0) || (PTransition > 32767))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 5, "PTransition");     
    
    if ((NTransition < 0) || (NTransition > 32767))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 6, "NTransition");     
            
        
    if (Rs_Simulating(instrSession) == 0) {                /* call only when locked */
    
        switch (registerOperation) {

            case RSSFU_VAL_QUEST_STAT_REG:
                viCheckErr(viPrintf (instrSession, ":STAT:QUES:BIT%ld:ENAB %ld;PTR %ld;NTR %ld",
                    userdefinableBit, enable, PTransition, NTransition)); 
            break;
            
            case RSSFU_VAL_OPER_STAT_REG:
                viCheckErr(viPrintf (instrSession, ":STAT:OPER:BIT%ld:ENAB %ld;PTR %ld;NTR %ld",
                    userdefinableBit, enable, PTransition, NTransition)); 
            break;
        }
    }   
    
    viCheckErr(rssfu_CheckStatus (instrSession));       
     
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}
 

/*============================================================================/
 * Function: Get User-definable Status Register
 * Purpose:  This function contains the commands for the querying status
 *           reporting system.
/============================================================================*/
ViStatus _VI_FUNC rssfu_getUserDefStatusRegister (ViSession instrSession,
                                                  ViInt32 registerOperation,
                                                  ViInt32 userdefinableBit,
                                                  ViInt32 statusRegistersQuery,
                                                  ViInt32* registerValue)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      model [RSSFU_IO_BUFFER_SIZE] = "";  
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckErr (rssfu_GetAttributeViString (instrSession, "",
                                               RS_ATTR_INSTRUMENT_MODEL,
                                               RSSFU_IO_BUFFER_SIZE, model));



    if ((registerOperation < RSSFU_VAL_QUEST_STAT_REG   ) || (registerOperation > RSSFU_VAL_OPER_STAT_REG))
            viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Register Operation");
    
    if (strstr (model, "BTC") == NULL) {      
        if (registerOperation == RSSFU_VAL_QUEST_STAT_REG) {
            if ((userdefinableBit < 9) || (userdefinableBit > 12))
                viCheckParm( RS_ERROR_INVALID_PARAMETER, 3, "User-definable Bit");
        }
        else {
            if ((userdefinableBit < 8) || (userdefinableBit > 12))
                viCheckParm( RS_ERROR_INVALID_PARAMETER, 3, "User-definable Bit");
        }
    }
    if ((statusRegistersQuery < RSSFU_VAL_REG_CONDITION   ) || (statusRegistersQuery > RSSFU_VAL_REG_NTRANSITION))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 4, "Register Operation");  
            
        
    if (Rs_Simulating(instrSession) == 0) {                /* call only when locked */
    
        switch (registerOperation) {
            case RSSFU_VAL_QUEST_STAT_REG:
                viCheckErr (viQueryf (instrSession, ":STAT:QUES:BIT%ld:%s?", "%ld",
                    userdefinableBit, statusRegisterArr[statusRegistersQuery], registerValue)); 
            break;
            
            case RSSFU_VAL_OPER_STAT_REG:
                viCheckErr (viQueryf (instrSession, ":STAT:OPER:BIT%ld:%s?", "%ld",
                    userdefinableBit, statusRegisterArr[statusRegistersQuery], registerValue)); 
            break;
        }
    }   

    viCheckErr(rssfu_CheckStatus (instrSession));       
     
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}                                          
                                           
    
                                           
/*****************************************************************************
 *--------------------- Hardware Settings -----------------------------------*
 *****************************************************************************/

/*****************************************************************************
 * Function: Configure Reference Oscillator
 * Purpose:  This function configures reference oscillator.
 *****************************************************************************/
ViStatus _VI_FUNC rssfu_confReferenceOscillator (ViSession instrSession,
                                                 ViInt32 source,
                                                 ViInt32 externalRefFrequency,
                                                 ViInt32 synchronizationBandwidth,
                                                 ViBoolean adjustmentState,
                                                 ViInt32 adjustmentFrequency)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      model [RSSFU_IO_BUFFER_SIZE] = "";
    ViUInt32    attr = RSSFU_ATTR_ROSC_SOURCE;
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckErr (rssfu_GetAttributeViString (instrSession, "",
                                               RS_ATTR_INSTRUMENT_MODEL,
                                               RSSFU_IO_BUFFER_SIZE, model));
    
    if (strstr (model, "BTC") != NULL)
        attr = RSSFU_ATTR_ROSC_SOURCE_BTC;
    
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", attr, source),
                         2, "Source"); 
    
    if (source == RSSFU_VAL_ROSC_SOURCE_EXT) {
        if (strstr (model, "SFC") == NULL) {
            viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_ROSC_FREQ, externalRefFrequency),
                                 3, "External Ref Frequency"); 
            viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_ROSC_SYNC_BANDWIDTH, synchronizationBandwidth),
                                 4, "Synchronization Bandwidth");
        }
    }
    else { 
        viCheckParm (rssfu_SetAttributeViBoolean (instrSession, "", RSSFU_ATTR_ROSC_ADJ_STATE, adjustmentState),
                             5, "Adjustment State");                          
        viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_ROSC_ADJ_FREQUENCY, adjustmentFrequency),
                             6, "Adjustment Frequency"); 
    }                     
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/*****************************************************************************
 * Function: Configure Trigger IO
 * Purpose:  This function configures Trigger I/O ports.
 *****************************************************************************/        
ViStatus _VI_FUNC rssfu_confTriggerIO (ViSession instrSession,
                                       ViInt32 tio, ViInt32 triggerIOState,
                                       ViInt32 triggerIOSource,
                                       ViInt32 destination)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buf1[RSSFU_TOKEN_BUFFER_SIZE] = "";
    ViInt32     l_tio = tio - 1;

    if ((tio < 1) || (tio > 15))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Trigger IO Pin"); 

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    checkErr (rssfu_GetActualRepCaps (instrSession, RSSFU_ATTR_TRIGGER_IO_STATE, "TriggerIO", l_tio, buf1));

    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, buf1, RSSFU_ATTR_TRIGGER_IO_STATE, triggerIOState), 
                    3, "State");
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, buf1, RSSFU_ATTR_TRIGGER_IO_SOURCE, triggerIOSource), 
                        4, "Source"); 
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, buf1, RSSFU_ATTR_TRIGGER_IO_DESTINATION, destination), 
                        5, "Destination");                     
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}
                                                 

/*****************************************************************************
 *---------------------------------------------------------------------------*
 *****************************************************************************/

/*============================================================================/
 * Function: Store and Load Application
 * Purpose:  This function stores or recalls application settings in the
 *           specified file.
/============================================================================*/
ViStatus _VI_FUNC rssfu_storeLoadApplication (ViSession instrSession,
                                                         ViInt32 operation,
                                                         ViString source)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSSFU_IO_BUFFER_SIZE] = ""; 

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if ((operation < RSSFU_VAL_SAVE) || (operation > RSSFU_VAL_RECALL))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Operation");
        
    if (Rs_Simulating(instrSession) == 0) {                /* call only when locked */

        switch (operation){
            case RSSFU_VAL_SAVE:
                 sprintf (buffer,"*SAV 1;:MMEMory:STORe:STATe 1,\"%s\";*OPC", source);
            break;
        
            case RSSFU_VAL_RECALL:
                sprintf (buffer,":MMEMory:LOAD:STATe 1,\"%s\";*RCL 1;*OPC", source);
            break;
        }   
    }
    viCheckErr( viPrintf (instrSession, "%s", buffer));
    viCheckErr (rssfu_WaitForOPC (instrSession, 60000));
    viCheckErr(rssfu_CheckStatus (instrSession));       
     
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}
     

/*============================================================================/
 * Function: Store and Load Instrument Data
 * Purpose:  This function stores or recalls instrument data.
/============================================================================*/
ViStatus _VI_FUNC rssfu_storeLoadInstrumentData (ViSession instrSession,
                                                 ViInt32 operation,
                                                 ViInt32 stateNumber)
{
    ViStatus    error   = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
  
    if (Rs_Simulating(instrSession) == 0) {                /* call only when locked */
    switch (operation){
        case RSSFU_VAL_SAVE:
             viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_SAV, stateNumber),
                         3, "State Number"); 
        break;
        
        case RSSFU_VAL_RECALL:
             viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_RCL, stateNumber),
                         3, "State Number"); 
        break;
        }
    }
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/* --- Obsolete functions --- */ 
/*****************************************************************************
 *----------------------------------- DMB-TH --------------------------------*
 *****************************************************************************/
/*============================================================================/
 * Function: Read ISDBT Coder Status
 * Purpose:  This function reads the status of DMB-TH coder.
/============================================================================*/
ViStatus _VI_FUNC rssfu_readDMBTHCoderStatus (ViSession instrSession,
                                              ViChar status[])

{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSSFU_IO_BUFFER_SIZE]="\0";  
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    if (status == NULL) viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Status");
    viCheckErr (rssfu_GetAttributeViString (instrSession, "", RSSFU_ATTR_IQC_DMBTH_LOCK,
                                            RSSFU_IO_BUFFER_SIZE, buffer));
    if (status != NULL)
        strncpy (status, buffer, strlen(buffer)); 
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
} 


/*============================================================================/
 * Function: Configure DMBTH Input Signal
 * Purpose:  This function configures DMB-TH Input Signal parameters.
/============================================================================*/
ViStatus _VI_FUNC rssfu_confDMBTHInputSignal (ViSession instrSession,
                                              ViInt32 source, ViInt32 input,
                                              ViInt32 stuffing)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_IQC_DMBTH_SOURCE, source),
                         2, "Source"); 
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_IQC_DMBTH_INPUT_SIGNAL, input),
                         3, "Input");
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_IQC_DMBTH_STUFFING, stuffing),
                         4, "Stuffing");
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}
 
/*============================================================================/
 * Function: Configure DMBTH Test Signal
 * Purpose:  This function configures DMB-TH Test Signal.
/============================================================================*/
ViStatus _VI_FUNC rssfu_confDMBTHTestSignal (ViSession instrSession,
                                             ViInt32 testSignal)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_IQC_DMBTH_TEST, testSignal),
                         2, "Test Signal"); 
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}
 

/*============================================================================/
 * Function: Read DMBTH Packed Length
 * Purpose:  This function reads DMB-TH packed length.
/============================================================================*/
ViStatus _VI_FUNC rssfu_readDMBTHPackedLength (ViSession instrSession,
                                               ViInt32* packedLength)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    if (!packedLength) viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Packed Length");
    viCheckParm (rssfu_GetAttributeViInt32 (instrSession, "", RSSFU_ATTR_IQC_DMBTH_PACKET_LENGTH,
                                            packedLength), 2, "Packed Length");
   Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/*============================================================================/
 * Function: Read DMBTH Values
 * Purpose:  This function reads selected coding value.
/============================================================================*/
ViStatus _VI_FUNC rssfu_readDMBTHValues (ViSession instrSession, ViInt32 value,
                                         ViReal64* result)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    
    if ((value < RSSFU_VAL_MAX_USEFUL_DRATE) || (value > RSSFU_VAL_USEFUL_DRATE))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Value");
        
    switch (value){                                                       
        case RSSFU_VAL_MAX_USEFUL_DRATE:
            viCheckParm (rssfu_GetAttributeViReal64 (instrSession, "", RSSFU_ATTR_IQC_DMBTH_MAX_USEFUL_DATA_RATE,
                                                     result), 3, "Result");
        break;
        case RSSFU_VAL_MEAS_DRATE:
            viCheckParm (rssfu_GetAttributeViReal64 (instrSession, "", RSSFU_ATTR_IQC_DMBTH_MEASURED_DATA_RATE,
                                                     result), 3, "Result");
        break;
        case RSSFU_VAL_USEFUL_DRATE:
            viCheckParm (rssfu_GetAttributeViReal64 (instrSession, "", RSSFU_ATTR_IQC_DMBTH_USEFUL_DATA_RATE,
                                                     result), 3, "Result");
        break;
       
     }           
     
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Configure DMBTH Network
 * Purpose:  This function configures DMB-TH Network parameters.
/============================================================================*/
ViStatus _VI_FUNC rssfu_confDMBTHNetwork (ViSession instrSession,
                                          ViInt32 networkMode)

{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_IQC_DMBTH_NETWORK_MODE, networkMode),
                         2, "Network Mode"); 
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Configure DMBTH Control
 * Purpose:  This function configures DMB-TH coding control.
/============================================================================*/
ViStatus _VI_FUNC rssfu_confDMBTHControl (ViSession instrSession,
                                          ViInt32 channelBandwidth)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_IQC_DMBTH_CHANNEL_BANDWIDTH, channelBandwidth),
                         2, "Channel Bandwidth");                          
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Configure DMBTH Coding
 * Purpose:  This function configures DMB-TH coding parameters.
/============================================================================*/
ViStatus _VI_FUNC rssfu_confDMBTHCoding (ViSession instrSession,
                                         ViInt32 constellation, ViInt32 codeRate,
                                         ViInt32 guardInterval,
                                         ViInt32 timeInterleaver)

{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_IQC_DMBTH_CONSTELLATION, constellation),
                         2, "Constellation"); 
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_IQC_DMBTH_CODE_RATE, codeRate),
                         3, "Code Rate");
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_IQC_DMBTH_GUARD_INTERVAL, guardInterval),
                         4, "Guard Interval");
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_IQC_DMBTH_TIME_INTERLEAVER, timeInterleaver),
                         5, "Time Interleaver");                         
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/*============================================================================/
 * Function: Read DMBTH SFN Values
 * Purpose:  This function reads selected SFN value.
/============================================================================*/
ViStatus _VI_FUNC rssfu_readDMBTHSFNValues (ViSession instrSession,
                                            ViInt32 value, ViReal64* result)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    if ((value < RSSFU_VAL_SIGN_TIMESTAMP) || (value > RSSFU_VAL_ADD_DELAY))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Value");
    
    if (!result) viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Result");
    
    switch (value){ 
       /* case RSSFU_VAL_SIGN_TIMESTAMP:
            viCheckParm (rssfu_GetAttributeViReal64 (instrSession, "", RSSFU_ATTR_IQC_DMBTH_SIG_TIME_STAMP,
            result), 4, "Result");                 
        break;
       */
        case RSSFU_VAL_MAX_DELAY:
            viCheckParm (rssfu_GetAttributeViReal64 (instrSession, "", RSSFU_ATTR_IQC_DMBTH_MAXIMUM_DELAY,
            result), 4, "Result");                 
        break;
    
        case RSSFU_VAL_LOC_DELAY:
            viCheckParm (rssfu_GetAttributeViReal64 (instrSession, "", RSSFU_ATTR_IQC_DMBTH_LOCAL_DELAY,
            result), 4, "Result");                 
        break;

       /* case RSSFU_VAL_ADD_DELAY:
            viCheckParm (rssfu_GetAttributeViReal64 (instrSession, "", RSSFU_ATTR_IQC_DMBTH_ADDITIONAL_DELAY,
            result), 4, "Result");                 
        break; 
        */
    }
        
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
} 


/*============================================================================/
 * Function: Enable DMBTH Special Settings
 * Purpose:  This function enables use of special DMB-TH coding parameters.
/============================================================================*/
ViStatus _VI_FUNC rssfu_actEnableDMBTHSpecial (ViSession instrSession,
                                               ViBoolean state)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rssfu_SetAttributeViBoolean (instrSession, "", RSSFU_ATTR_IQC_DMBTH_SPECIAL_SETTINGS, state),
                         2, "State");
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Configure DMBTH Special Settings
 * Purpose:  This function configures special DMB-TH coding parameters.
/============================================================================*/
ViStatus _VI_FUNC rssfu_confDMBTHSpecial (ViSession instrSession,
                                          ViBoolean GIPowerBoost,
                                          ViInt32 LDPCOutput, ViInt32 PNPolarity,
                                          ViInt32 QAMConstellation,
                                          ViInt32 QPSKConstellation)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rssfu_SetAttributeViBoolean (instrSession, "", RSSFU_ATTR_IQC_DMBTH_GI_POWER_BOOST, GIPowerBoost),
                 2, "GI Power Boost"); 
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_IQC_DMBTH_LDPC_OUTPUT, LDPCOutput),
                 3, "LDPC Output");
    viCheckParm (rssfu_SetAttributeViInt32(instrSession, "", RSSFU_ATTR_IQC_DMBTH_PN_POLARITY, PNPolarity),
                 4, "PN Polarity");   
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_IQC_DMBTH_QAM_CONSTELLATION, QAMConstellation),
                 5, "QAM Constellation");
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_IQC_DMBTH_QPSK_CONSTELLATION, QPSKConstellation),
                 6, "QPSK Constellation");
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}
 
/*============================================================================/
 * Function: Set DMBTH Special Single Carrier
 * Purpose:  This function sets the special single carrier.
/============================================================================*/
ViStatus _VI_FUNC rssfu_setDMBTHSpecialSingleCarrier (ViSession instrSession,
                                                      ViBoolean singleCarrier)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rssfu_SetAttributeViBoolean (instrSession, "", RSSFU_ATTR_IQC_DMBTH_SPECIAL_SINGLE_CARRIER,
                        singleCarrier), 2, "Single Carrier"); 
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}
 
/*============================================================================/
 * Function: Configure DMBTH Settings
 * Purpose:  This function configures DMB-TH coding settings.
/============================================================================*/
ViStatus _VI_FUNC rssfu_confDMBTHSettings (ViSession instrSession,
                                           ViBoolean controlFrames,
                                           ViInt32 testTSPacket,
                                           ViInt32 PIDTestPacket, ViInt32 pid,
                                           ViInt32 payloadTestStuff, ViInt32 prbs)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_IQC_DMBTH_CONTROL_FRAMES, controlFrames),
                         2, "Control Frames");
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_IQC_DMBTH_TEST_TS_PACKET, testTSPacket),
                         3, "Test TS Packet");
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_IQC_DMBTH_PID_TEST_PACKED, PIDTestPacket),
                         4, "PID Test Packet"); 
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_IQC_DMBTH_PID, pid),
                         5, "PID Test Packet"); 
    viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_IQC_DMBTH_PAYLOAD_TEST, payloadTestStuff),
                         6, "Payload Test/Stuff");      
    switch (payloadTestStuff){
        case  RSSFU_IQC_PRBS:
            viCheckParm (rssfu_SetAttributeViInt32 (instrSession, "", RSSFU_ATTR_IQC_DMBTH_PRBS, prbs),
                         7, "PRBS"); 
    }   
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
} 




/*****************************************************************************
 * Function: rssfu_GetAttribute<type> Functions
 * Purpose:  These functions enable the instrument driver user to get
 *           attribute values directly.  There are typesafe versions for
 *           ViInt32, ViReal64, ViString, ViBoolean, and ViSession attributes.
 *****************************************************************************/

/* ViInt32 */
ViStatus _VI_FUNC rssfu_GetAttributeViInt32 (ViSession instrSession, ViString channelName,
                                                ViUInt32 attributeId, ViInt32 *value)
{
    return Rs_GetAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL,
                            (ViInt32) sizeof (ViInt32), (ViInt32 *)value);
}

/* ViReal64 */
ViStatus _VI_FUNC rssfu_GetAttributeViReal64 (ViSession instrSession, ViString channelName,
                                                 ViUInt32 attributeId, ViReal64 *value)
{
    return Rs_GetAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL,
                            (ViInt32) sizeof (ViReal64), (ViReal64 *)value);
}

/* ViString */
ViStatus _VI_FUNC rssfu_GetAttributeViString (ViSession instrSession, ViString channelName,
                                                 ViUInt32 attributeId, ViInt32 bufSize,
                                                 ViChar value[])
{
    return Rs_GetAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL,
                            bufSize, value);
}

/* ViBoolean */
ViStatus _VI_FUNC rssfu_GetAttributeViBoolean (ViSession instrSession, ViString channelName,
                                                  ViUInt32 attributeId, ViBoolean *value)
{
    return Rs_GetAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL,
                            (ViInt32) sizeof (ViBoolean), (ViBoolean *)value);
}

/* ViSession */
ViStatus _VI_FUNC rssfu_GetAttributeViSession (ViSession instrSession, ViString channelName,
                                                  ViUInt32 attributeId, ViSession *value)
{
    return Rs_GetAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL,
                            (ViInt32) sizeof (ViSession), (ViSession *)value);
}

/*****************************************************************************
 * Function: rssfu_SetAttribute<type> Functions
 * Purpose:  These functions enable the instrument driver user to set
 *           attribute values directly.  There are typesafe versions for
 *           ViInt32, ViReal64, ViString, ViBoolean, and ViSession datatypes.
 *****************************************************************************/

/* ViInt32 */
ViStatus _VI_FUNC rssfu_SetAttributeViInt32 (ViSession instrSession, ViString channelName,
                                                ViUInt32 attributeId, ViInt32 value)
{
    return Rs_SetAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, &value);
}

/* ViReal64 */
ViStatus _VI_FUNC rssfu_SetAttributeViReal64 (ViSession instrSession, ViString channelName,
                                                 ViUInt32 attributeId, ViReal64 value)
{
    return Rs_SetAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, &value);
}

/* ViString */
ViStatus _VI_FUNC rssfu_SetAttributeViString (ViSession instrSession, ViString channelName,
                                                 ViUInt32 attributeId, ViString value)
{
    return Rs_SetAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, value);
}

/* ViBoolean */
ViStatus _VI_FUNC rssfu_SetAttributeViBoolean (ViSession instrSession, ViString channelName,
                                                  ViUInt32 attributeId, ViBoolean value)
{
    return Rs_SetAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, &value);
}

/* ViSession */
ViStatus _VI_FUNC rssfu_SetAttributeViSession (ViSession instrSession, ViString channelName,
                                                  ViUInt32 attributeId, ViSession value)
{
    return Rs_SetAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, &value);
}

/*****************************************************************************
 * Function: rssfu_CheckAttribute<type> Functions
 * Purpose:  These functions enable the instrument driver user to check
 *           attribute values directly.  These functions check the value you
 *           specify even if you set the RSSFU_ATTR_RANGE_CHECK
 *           attribute to VI_FALSE.  There are typesafe versions for ViInt32,
 *           ViReal64, ViString, ViBoolean, and ViSession datatypes.
 *****************************************************************************/

/* ViInt32 */
ViStatus _VI_FUNC rssfu_CheckAttributeViInt32 (ViSession instrSession, ViString channelName,
                                                  ViUInt32 attributeId, ViInt32 value)
{
    return Rs_CheckAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, &value);
}

/* ViReal64 */
ViStatus _VI_FUNC rssfu_CheckAttributeViReal64 (ViSession instrSession, ViString channelName,
                                                   ViUInt32 attributeId, ViReal64 value)
{
    return Rs_CheckAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, &value);
}

/* ViString */
ViStatus _VI_FUNC rssfu_CheckAttributeViString (ViSession instrSession, ViString channelName,
                                                   ViUInt32 attributeId, ViString value)
{
    return Rs_CheckAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, value);
}

/* ViBoolean */
ViStatus _VI_FUNC rssfu_CheckAttributeViBoolean (ViSession instrSession, ViString channelName,
                                                    ViUInt32 attributeId, ViBoolean value)
{
    return Rs_CheckAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, &value);
}

/* ViSession */
ViStatus _VI_FUNC rssfu_CheckAttributeViSession (ViSession instrSession, ViString channelName,
                                                    ViUInt32 attributeId, ViSession value)
{
    return Rs_CheckAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, &value);
}

/*****************************************************************************
 * Function: Rs_GetAttributeRepCapNameId
 * Purpose:  This function returns the repeated capability id(s) that
 *           belongs to the attribute you specify. More then one id is listed
 *           in comma separated string.
 *****************************************************************************/
ViStatus _VI_FUNC rssfu_GetAttributeRepeatedCapabilityIds (ViSession instrSession,
                                                            ViUInt32 attributeID,
                                                            ViInt32 bufferSize,
                                                            ViChar _VI_FAR repeatedCapabilityIds[])
{
    ViStatus    error = VI_SUCCESS;
    ViChar      repCapNameId[RS_MAX_MESSAGE_BUF_SIZE] = "";

    if (bufferSize <= 0)
        viCheckParm(RS_ERROR_INVALID_PARAMETER, 3, "Buffer Size");

    viCheckErr (Rs_GetAttributeRepCapNameId (instrSession, attributeID, repCapNameId));

    strncpy (repeatedCapabilityIds, repCapNameId, RS_MAX_MESSAGE_BUF_SIZE);
    repeatedCapabilityIds[(bufferSize > RS_MAX_MESSAGE_BUF_SIZE) ?
        RS_MAX_MESSAGE_BUF_SIZE : bufferSize] = '\0';

Error:
    return error;
}

/*****************************************************************************
 * Function: Get Attribute Repeated Capability Id Name(s)
 * Purpose:  This function returns the repeated capability name(s) that
 *           belongs to the attribute and single repeated capability id you
 *           may specify. More then one name is listed in comma separated
 *           string.
 *****************************************************************************/
ViStatus _VI_FUNC rssfu_GetAttributeRepeatedCapabilityIdNames (ViSession instrSession,
                                                                /*@unused@*/ViUInt32 attributeID,
                                                                ViString repeatedCapabilityId,
                                                                ViInt32 bufferSize,
                                                                ViChar _VI_FAR repeatedCapabilityIdNames[])
{
    ViStatus    error = VI_SUCCESS;
    RsSessionPropertiesPtr  sessionProperties = Rs_ViSession (instrSession);
    ViUInt32     rcap;
    attributeID;

    if (bufferSize <= 0)
        viCheckParm(RS_ERROR_INVALID_PARAMETER, 4, "Buffer Size");

    for (rcap = 0; rcap < sessionProperties -> numberOfRepCaps; rcap++)
        if (strncmp (sessionProperties -> repCapTable[rcap].repCapNameId, repeatedCapabilityId, strlen (repeatedCapabilityId)) == 0)
            break;

    strncpy (repeatedCapabilityIdNames, sessionProperties -> repCapTable[rcap].repCapName, RS_MAX_MESSAGE_BUF_SIZE);
    repeatedCapabilityIdNames[(bufferSize > RS_MAX_MESSAGE_BUF_SIZE) ?
        RS_MAX_MESSAGE_BUF_SIZE - 1 : bufferSize - 1] = '\0';

Error:
    return error;
}

/*****************************************************************************
 * Function: WriteInstrData and ReadInstrData Functions
 * Purpose:  These functions enable the instrument driver user to
 *           write and read commands directly to and from the instrument.
 *****************************************************************************/
ViStatus _VI_FUNC rssfu_WriteInstrData (ViSession instrSession, ViString writeBuffer)
{
    return Rs_WriteInstrData (instrSession, writeBuffer);
}

ViStatus _VI_FUNC rssfu_ReadInstrData (ViSession instrSession, ViInt32 numBytes,
                                          ViChar rdBuf[], ViInt32 *bytesRead)
{
    return Rs_ReadInstrData (instrSession, numBytes, rdBuf, (ViUInt32 *) bytesRead);
}

/*****************************************************************************
 *-------------------- Utility Functions (Not Exported) ---------------------*
 *****************************************************************************/

/*****************************************************************************
 * Function: rssfu_CheckStatus
 * Purpose:  This function checks the status of the instrument to detect
 *           whether the instrument has encountered an error. This function
 *           is called at the end of most exported driver functions. If the
 *           instrument reports an error, this function returns
 *           RS_ERROR_INSTRUMENT_SPECIFIC. The user can set the
 *           RSSFU_ATTR_QUERY_INSTRUMENT_STATUS attribute to VI_FALSE to disable this
 *           check and increase execution speed.
 *
 *           Note:  Call this function only when the session is locked.
 *****************************************************************************/
static ViStatus rssfu_CheckStatus (ViSession instrSession)
{
    ViStatus    error = VI_SUCCESS;

    if ((Rs_QueryInstrStatus (instrSession) != 0) && (Rs_Simulating (instrSession) == 0))
        {
        checkErr( rssfu_CheckStatusCallback (instrSession));
        }

Error:
    return error;
}

/*****************************************************************************
 * Function: rssfu_WaitForOPC
 * Purpose:  This function waits for the instrument to complete the
 *           execution of all pending operations. This function is a
 *           wrapper for the WaitForOPCCallback. It can be called from
 *           other instrument driver functions.
 *
 *           The maxTime parameter specifies the maximum time to wait for
 *           operation complete in milliseconds.
 *
 *           Note:  Call this function only when the session is locked.
 *****************************************************************************/
static ViStatus rssfu_WaitForOPC (ViSession instrSession, ViInt32 maxTime)
{
    ViStatus    error = VI_SUCCESS;

    if (Rs_Simulating(instrSession) == 0)
        {
        ViInt32  oldOPCTimeout = 0;

        checkErr( Rs_GetAttribute (instrSession, VI_NULL, RSSFU_ATTR_OPC_TIMEOUT, 0,
                                   (ViInt32) sizeof (oldOPCTimeout), &oldOPCTimeout));

        (void)Rs_SetAttribute (instrSession, VI_NULL, RSSFU_ATTR_OPC_TIMEOUT, 0, &maxTime);

        error = rssfu_WaitForOPCCallback (instrSession);

        (void)Rs_SetAttribute (instrSession, VI_NULL, RSSFU_ATTR_OPC_TIMEOUT, 0, &oldOPCTimeout);

        viCheckErr( error);
        }
Error:
    return error;
}

/*****************************************************************************
 * Function: rssfu_DefaultInstrSetup
 * Purpose:  This function sends a default setup to the instrument. The
 *           rssfu_reset function calls this function. The
 *           rssfu_RsInit function calls this function when the
 *           user passes VI_FALSE for the reset parameter. This function is
 *           useful for configuring settings that other instrument driver
 *           functions require.
 *
 *           Note:  Call this function only when the session is locked.
 *****************************************************************************/
static ViStatus rssfu_DefaultInstrSetup (ViSession instrSession)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      msg[RS_MAX_MESSAGE_BUF_SIZE] = "";
                
    /* Set all the attributes to the default state. Do not update inherent attributes! */
    checkErr (Rs_ApplyAttributeDefaults (instrSession, VI_FALSE));

    /* Init attributes */
    checkErr( rssfu_InitAttributes (instrSession));

    if (Rs_Simulating(instrSession) == 0)
        {   
        viCheckErr( viPrintf (instrSession, "*CLS;*ESE 1;*SRE 32;*WAI"));
        viCheckErr (viQueryf (instrSession, ":SYST:ERR:ALL?\n", "%[^\r\n]", msg));

        }

Error:
    return error;
}

/******************************************************************************
 * Function: Read Data With Unknown length
 * Description: This function allocate buffer and read data from the instrument
 * WARNING:  You must deallocate buffer, if any pointer has been returned.
 *           This function does not work correctly, when you read bin data via
 *           serial interface
 *****************************************************************************/
static ViStatus rssfu_ReadDataUnknownLength(ViSession instrSession,
                ViChar **outBuffer, ViUInt32 *count)
{
    ViStatus error = VI_SUCCESS;
    ViUInt32 actSize, retCnt = 0;
    ViChar   *buffer = VI_NULL, *p2buffer;

    // *outBuffer = VI_NULL;
    if (count) *count = 0;

    /* Allocate Buffer */
    actSize = (ViUInt32) RSSFU_IO_BUFFER_SIZE;
    buffer = (ViChar *)calloc((size_t)actSize, sizeof(ViChar));
    if (buffer == VI_NULL) return VI_ERROR_ALLOC;

    p2buffer = buffer;
    do
        {
        if ((error = viRead (instrSession, (ViBuf) p2buffer, (ViUInt32) RSSFU_IO_BUFFER_SIZE, &retCnt)) < 0)
            {
            free(buffer);
            return error;
            }
        if (retCnt == (ViUInt32) RSSFU_IO_BUFFER_SIZE) /* Buffer was small, reallocate it */
        {
            actSize += RSSFU_IO_BUFFER_SIZE;
            buffer = (ViChar *)realloc(buffer, (size_t) actSize);
            if (buffer == VI_NULL) return VI_ERROR_ALLOC;
            p2buffer = buffer + (actSize - RSSFU_IO_BUFFER_SIZE); /* Set pointer to end of data in rallocated buffer */
        }
        else
        {
            *(p2buffer+retCnt) = '\0'; /* For sure set end of string on the end of data */
        }
       } while (error == 0x3FFF0006); /* if buffer was small, continue with next read */

    *outBuffer = buffer;
    if (count) *count = (actSize - RSSFU_IO_BUFFER_SIZE) + retCnt;

    return error;
}

/*===========================================================================*/
/* Function: Set OPC Timeout                                                 */
/* Purpose:  Sets new OPC timeout                                            */
/*===========================================================================*/
ViStatus _VI_FUNC rssfu_SetOPCTimeout(ViSession instrSession, ViInt32 timeout)
{
    ViStatus    error = VI_SUCCESS; 
    
    checkErr( Rs_SetAttribute (instrSession, VI_NULL, RSSFU_ATTR_OPC_TIMEOUT, 0, &timeout));  
    
Error:    
    return error;
}

/*===========================================================================*/
/* Function: Get OPC Timeout                                                 */
/* Purpose:  Gets new OPC timeout                                            */
/*===========================================================================*/
ViStatus _VI_FUNC rssfu_GetOPCTimeout(ViSession instrSession, ViInt32 *timeout)
{
    ViStatus    error = VI_SUCCESS; 
    
    checkErr( Rs_GetAttribute (instrSession, VI_NULL, RSSFU_ATTR_OPC_TIMEOUT, 0, sizeof(timeout), timeout));  
    
Error:    
    return error;
}

/*****************************************************************************
 *------------------------ Global Session Callbacks -------------------------*
 *****************************************************************************/

/*****************************************************************************
 * Function: Rs_CheckStatusCallback
 * Purpose:  This function queries the instrument to determine if it has
 *           encountered an error. If the instrument has encountered an
 *           error, this function returns the RS_ERROR_INSTRUMENT_SPECIFIC
 *           error code. This function is called by the
 *           Rs_CheckStatus utility function. The
 *           Rs_SetAttribute and Rs_GetAttribute functions invoke this
 *           function when the optionFlags parameter includes the
 *           RS_VAL_DIRECT_USER_CALL flag.
 *
 *           The user can disable calls to this function by setting the
 *           rssfu_QUERY_INSTRUMENT_STATUS attribute to VI_FALSE.  The driver can
 *           disable the check status callback for a particular attribute by
 *           setting the RS_VAL_DONT_CHECK_STATUS flag.
 *
 *****************************************************************************/
ViStatus rssfu_CheckStatusCallback (ViSession instrSession)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer [10] = "*ESR?\n";
    ViInt16     esr     = 0;

    viCheckErr (viWrite (instrSession, (ViBuf) buffer, (ViUInt32) strlen (buffer), NULL));
    viCheckErr (viRead (instrSession, (ViPBuf) buffer, 10, NULL));
    
    sscanf (buffer, "%hd", &esr);

    if ((esr & IEEE_488_2_ERROR_MASK) != 0)
        {
        viCheckErr (RS_ERROR_INSTR_SPECIFIC);
        }

Error:

    return error;
}

/*****************************************************************************
 * Function: Rs_WaitForOPCCallback
 * Purpose:  This function waits until the instrument has finished processing
 *           all pending operations. This function is called by the
 *           Rs_WaitForOPC utility function. The RS engine invokes
 *           this function in the following two cases:
 *           - Before invoking the read callback for attributes for which the
 *             RS_VAL_WAIT_FOR_OPC_BEFORE_READS flag is set.
 *           - After invoking the write callback for attributes for which the
 *             RS_VAL_WAIT_FOR_OPC_AFTER_WRITES flag is set.
 *
 *****************************************************************************/
ViStatus rssfu_WaitForOPCCallback (ViSession instrSession)
{
    ViStatus    error = VI_SUCCESS;
    ViUInt16    response    = 0;
    ViInt32     counter     = 0;
    ViUInt16    intf        = VI_INTF_GPIB;
    ViInt32     opcTimeout = 0;

    checkErr( Rs_GetAttribute (instrSession, VI_NULL, RS_ATTR_OPC_TIMEOUT, 0,
        (ViInt32) sizeof (opcTimeout), &opcTimeout));

    viCheckErr( viGetAttribute (instrSession, VI_ATTR_INTF_TYPE, &intf));

    /* Interface specific operations */
    switch (intf)
        {
        case VI_INTF_ASRL: /* Serial interface */

            /* Not supported */

        break;
        default: /* Default case */

            /* Check STB for errors. */
            do
                {
                error = viReadSTB (instrSession, &response);

                if (counter < 1000)
                {
                    Rs_Delay (0.005);
                    counter += 5;
                }
                else if (counter < 5000)
                {
                    Rs_Delay (0.01);
                    counter += 10;
                }
                else if (counter < 20000)
                {
                    Rs_Delay (0.05);
                    counter += 50;
                }
                else
                {
                    Rs_Delay (1.0);
                    counter += 1000;
                }

                if (counter >= opcTimeout)
                    return VI_ERROR_TMO;

                /* Wait while STB is non zero value. Only Error/Event Queue
                   and MAV bits are taken into consideration. */
            
                } while ((response & 0x34) == 0);

            /* (check Error(Event) Queue bit in STB) */
            return ((response & 0x4) != 0)? RS_ERROR_INSTRUMENT_STATUS : error;
        }

Error:
    return error;
}

/*****************************************************************************
 *- User Defined Attribute Callbacks ----------------------------------------*
 *****************************************************************************/
/*****************************************************************************
 * Function: rssfu_IqcPacketLength_readCallback
 * Purpose:  This function returns the number of packet length.
 *****************************************************************************/
ViStatus rssfu_IqcPacketLength_ReadCallback(ViSession instrSession, /*@unused@*/ViConstString repCapName, ViAttr attributeId, /*@unused@*/ViInt32 bufSize, void *value)
{
    ViInt32                 length;
    RsSessionPropertiesPtr  sessionProperties = Rs_ViSession (instrSession); 
    ViInt32                 idx             = 0;   
    ViStatus                error           = VI_SUCCESS;    
    ViChar                  buffer[20]      = "";
    repCapName;
    bufSize;
    
    viCheckErr (Rs_GetAttributeIndex (instrSession, attributeId, &idx));

    /* If no command available, exit the function */
    if (sessionProperties -> attr[idx].command == VI_NULL)
        return VI_SUCCESS;
    
    viCheckErr (viQueryf (instrSession, "%s?", "%[^\n\r]", sessionProperties -> attr[idx].command, buffer));
    
    if (strncmp (buffer,"INV",3) != 0)
    {
        (void) sscanf (buffer,"P%ld", &length);
        *(ViInt32 *)value = length;
    }
    else
    {
        *(ViInt32 *)value = 0;
    }

Error:   
    return error;
}




/*****************************************************************************
 *- Coerce Callbacks --------------------------------------------------------*
 *****************************************************************************/


/*****************************************************************************
 *- Range Table Callbacks ---------------------------------------------------*
 *****************************************************************************/
         
             
/*****************************************************************************
 * Function: rssfu_inputStuffing_RangeTableCallback
 * Purpose:  This function overrides standard calback funtion.
 *****************************************************************************/
ViStatus rssfu_inputStuffing_RangeTableCallback (ViSession instrSession, ViConstString repCapName, ViAttr attributeId, RsRangeTablePtr *rangeTablePtr)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      model [RSSFU_IO_BUFFER_SIZE] = ""; 
    
    checkErr( Rs_LockSession (instrSession, VI_NULL)); 
    
    viCheckErr (rssfu_GetAttributeViString (instrSession, "",
                                               RS_ATTR_INSTRUMENT_MODEL,
                                               RSSFU_IO_BUFFER_SIZE, model));
    if (strstr (model, "SFU"))
    {
        *rangeTablePtr=&rssfu_rngIqcStuf;
    }
    else
    {
        *rangeTablePtr=&rssfu_rngIqcStuffingSFC;   
    }
   
Error:
    Rs_UnlockSession(instrSession, VI_NULL); 
    return error;
}
         
 
/*****************************************************************************
 * Function: rssfu_InputSignalSource_RangeTableCallback
 * Purpose:  This function overrides standard calback funtion.
 *****************************************************************************/
ViStatus rssfu_InputSignalSource_RangeTableCallback (ViSession instrSession, ViConstString repCapName, ViAttr attributeId, RsRangeTablePtr *rangeTablePtr)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      model [RSSFU_IO_BUFFER_SIZE] = ""; 
    
    checkErr( Rs_LockSession (instrSession, VI_NULL)); 
    
    viCheckErr (rssfu_GetAttributeViString (instrSession, "",
                                               RS_ATTR_INSTRUMENT_MODEL,
                                               RSSFU_IO_BUFFER_SIZE, model));
    if (strstr (model, "BTC"))
    {
        *rangeTablePtr=&rssfu_rngBTCInputSignalSource;
    }
    else
    {
        *rangeTablePtr=&rssfu_rngIqcSigSour;   
    }
   
Error:
    Rs_UnlockSession(instrSession, VI_NULL); 
    return error;

}    
/*****************************************************************************
 * Function: rssfu_Rosc_Freq_RangeTableCallback
 * Purpose:  This function overrides standard calback funtion.
 *****************************************************************************/
ViStatus rssfu_Rosc_Freq_RangeTableCallback (ViSession instrSession, ViConstString repCapName, ViAttr attributeId, RsRangeTablePtr *rangeTablePtr)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      model [RSSFU_IO_BUFFER_SIZE] = ""; 
    
    checkErr( Rs_LockSession (instrSession, VI_NULL)); 
    
    viCheckErr (rssfu_GetAttributeViString (instrSession, "",
                                               RS_ATTR_INSTRUMENT_MODEL,
                                               RSSFU_IO_BUFFER_SIZE, model));
    if (strstr (model, "BTC"))
    {
        *rangeTablePtr=&rssfu_rngBTCRoscFreq;
    }
    else
    {
        *rangeTablePtr=&rssfu_rngRoscFreq;   
    }
   
Error:
    Rs_UnlockSession(instrSession, VI_NULL); 
    return error;
}

/*****************************************************************************
 * Function: rssfu_InitAttributes
 * Purpose:  This function inits attributes to the desired values if needed.
 *****************************************************************************/
static ViStatus rssfu_InitAttributes (ViSession instrSession)
{
    ViStatus    error           = VI_SUCCESS;
    ViInt32     p2value         = 0;
    ViChar      buffer[RS_MAX_MESSAGE_BUF_SIZE] = "",
                revision[RS_MAX_MESSAGE_BUF_SIZE] = "",
                manufacturer[RS_MAX_MESSAGE_BUF_SIZE] = "",
                model[RS_MAX_MESSAGE_BUF_SIZE] = "",
                idQ[RS_MAX_MESSAGE_BUF_SIZE] = "",
                options_list[2*RS_MAX_MESSAGE_BUF_SIZE] = "";
    void*       p2void = NULL;

    buffer[RS_MAX_MESSAGE_BUF_SIZE -1] = '\0';

    /*- Misc -*/

    if (Rs_Simulating(instrSession) == 0)
        {
        /* Instrument identification */
        viCheckErr (viQueryf (instrSession, "*IDN?\n", "%[^\r\n]", idQ));
        idQ[RS_MAX_MESSAGE_BUF_SIZE - 1] = '\0';
        checkErr (Rs_SetAttribute (instrSession, "", (ViAttr) RSSFU_ATTR_ID_QUERY_RESPONSE, 0, idQ));
        
        /* Option(s) string */
        viCheckErr (viQueryf (instrSession, "*OPT?\n", "%[^\r\n]", options_list));
        //options_list[2*RS_MAX_MESSAGE_BUF_SIZE - 1] = '\0';
       
        }
    else /* Simulating */
        {
        /* Instrument identification */
        strncpy (idQ, RSSFU_SIMULATION_ID_QUERY, RS_MAX_MESSAGE_BUF_SIZE - 1);
        checkErr (Rs_SetAttribute (instrSession, "", (ViAttr) RSSFU_ATTR_ID_QUERY_RESPONSE, 0, idQ));
        
        /* Option(s) string */
        strncpy (options_list, RSSFU_SIMULATION_OPT_QUERY, 2*RS_MAX_MESSAGE_BUF_SIZE - 1);
        }

    /*- Inherent Instrument Specific Attributes ----------------------------*/

    /*- User Options -*/

    /*
    RS_ATTR_RANGE_CHECK             ... Rs_SpecificDriverNew
    RS_ATTR_QUERY_INSTRUMENT_STATUS ... Rs_SpecificDriverNew
    RS_ATTR_CACHE                   ... Rs_SpecificDriverNew
    RS_ATTR_SIMULATE                ... Rs_SpecificDriverNew
    RS_ATTR_RECORD_COERCIONS        ... not supported
    RS_ATTR_INTERCHANGE_CHECK       ... not supported
    RS_ATTR_SPY                     ... not supported
    RS_ATTR_USE_SPECIFIC_SIMULATION ... not supported
    */

    /*- Class Driver Identification -*/

    checkErr (Rs_SetAttribute (instrSession, "", RS_ATTR_CLASS_DRIVER_DESCRIPTION, 0, RSSFU_CLASS_DRIVER_DESCRIPTION));
    checkErr (Rs_SetAttribute (instrSession, "", RS_ATTR_CLASS_DRIVER_PREFIX, 0, RSSFU_CLASS_DRIVER_PREFIX));
    checkErr (Rs_SetAttribute (instrSession, "", RS_ATTR_CLASS_DRIVER_VENDOR, 0, RSSFU_CLASS_DRIVER_VENDOR));
    checkErr (Rs_SetAttribute (instrSession, "", RS_ATTR_CLASS_DRIVER_REVISION, 0, RSSFU_CLASS_DRIVER_REVISION));
    p2value = RSSFU_CLASS_SPEC_MAJOR_VERSION;
    checkErr (Rs_SetAttribute (instrSession, "", RS_ATTR_CLASS_DRIVER_CLASS_SPEC_MAJOR_VERSION, 0, &p2value));
    p2value = RSSFU_CLASS_SPEC_MINOR_VERSION;
    checkErr (Rs_SetAttribute (instrSession, "", RS_ATTR_CLASS_DRIVER_CLASS_SPEC_MINOR_VERSION, 0, &p2value));

    /*- Driver Identification -*/
                                                        
    checkErr (Rs_SetAttribute (instrSession, "", RS_ATTR_SPECIFIC_DRIVER_DESCRIPTION, 0, RSSFU_SPECIFIC_DRIVER_DESCRIPTION));
    checkErr (Rs_SetAttribute (instrSession, "", RS_ATTR_SPECIFIC_DRIVER_PREFIX, 0, RSSFU_SPECIFIC_DRIVER_PREFIX));
    checkErr (Rs_SetAttribute (instrSession, "", RS_ATTR_SPECIFIC_DRIVER_LOCATOR, 0, RSSFU_SPECIFIC_DRIVER_LOCATOR));
    checkErr (Rs_SetAttribute (instrSession, "", RS_ATTR_SPECIFIC_DRIVER_VENDOR, 0, RSSFU_SPECIFIC_DRIVER_VENDOR));

    sprintf (buffer, 
             "Driver: %s %ld.%ld.%ld, Compiler: %s %3.2f, "
             "Components: RSEngine %.2f, VISA-Spec %.2f",
             RSSFU_SPECIFIC_DRIVER_PREFIX,
             RSSFU_MAJOR_VERSION,RSSFU_MINOR_VERSION,RSSFU_MINOR_MINOR_VERSION, 
             RS_COMPILER_NAME, RS_COMPILER_VER_NUM, 
             (double)(RS_ENGINE_MAJOR_VERSION + RS_ENGINE_MINOR_VERSION)/10.0, 
             (double)Rs_ConvertVISAVer(VI_SPEC_VERSION));

    checkErr (Rs_SetAttribute (instrSession, "", RS_ATTR_SPECIFIC_DRIVER_REVISION, 0, buffer));
    p2value = RSSFU_CLASS_SPEC_MAJOR_VERSION;
    checkErr (Rs_SetAttribute (instrSession, "", RS_ATTR_SPECIFIC_DRIVER_CLASS_SPEC_MAJOR_VERSION, 0, &p2value));
    p2value = RSSFU_CLASS_SPEC_MINOR_VERSION;
    checkErr (Rs_SetAttribute (instrSession, "", RS_ATTR_SPECIFIC_DRIVER_CLASS_SPEC_MINOR_VERSION, 0, &p2value));

    /*- Driver Capabilities -*/
                                   
    checkErr (Rs_SetAttribute (instrSession, "", RS_ATTR_SUPPORTED_INSTRUMENT_MODELS, 0, RSSFU_SUPPORTED_INSTRUMENT_MODELS));
    checkErr (Rs_SetAttribute (instrSession, "", RS_ATTR_GROUP_CAPABILITIES, 0, RSSFU_GROUP_CAPABILITIES));
    checkErr (Rs_SetAttribute (instrSession, "", RS_ATTR_FUNCTION_CAPABILITIES, 0, RSSFU_FUNCTION_CAPABILITIES));
    /* RS_ATTR_CHANNEL_COUNT        ... Rs_BuildRepCapTable */

    /*- Driver Setup -*/

    /* RS_ATTR_DRIVER_SETUP         ... Rs_SpecificDriverNew */

    /*- Instrument Identification -*/

    checkErr (Rs_GetAttribute (instrSession, "", (ViAttr) RSSFU_ATTR_ID_QUERY_RESPONSE, 0, RS_MAX_MESSAGE_BUF_SIZE, idQ));
    (void) sscanf (idQ, "%[^,],%[^,],%*[^,],%256[^\n]", manufacturer, model, revision);
     
    if (strstr (model, "R&S BTC")) {
        strcpy (model, "BTC");
    }
    
    checkErr (Rs_SetAttribute (instrSession, "", (ViAttr) RS_ATTR_INSTRUMENT_MANUFACTURER, 0, manufacturer));
    checkErr (Rs_SetAttribute (instrSession, "", (ViAttr) RS_ATTR_INSTRUMENT_MODEL, 0, model));
    checkErr (Rs_SetAttribute (instrSession, "", (ViAttr) RS_ATTR_INSTRUMENT_FIRMWARE_REVISION, 0, revision));
    
        
    checkErr (rssfu_ReplaceString (options_list, model, "SFU"));
        
    checkErr (Rs_SetAttribute (instrSession, "", (ViAttr) RS_ATTR_OPTIONS_LIST, 0, options_list));

    /*- Advanced Session Information -*/
                                  
    /*  
    RS_ATTR_IO_RESOURCE_DESCRIPTOR  ... rssfu_InitWithOptions
    RS_ATTR_LOGICAL_NAME            ... not supported
    */

    /*- Inherent Vendor Specific Attributes --------------------------------*/

    /*- Error Info -*/

    /*
    RS_ATTR_PRIMARY_ERROR           ... Rs_SetErrorInfo
    RS_ATTR_SECONDARY_ERROR         ... Rs_SetErrorInfo
    RS_ATTR_ERROR_ELABORATION       ... Rs_SetErrorInfo
    */

    /*- Session I/O -*/

    /*
    RS_ATTR_VISA_RM_SESSION         ... rssfu_RsInit
    RS_ATTR_IO_SESSION              ... Rs_UpdateIOSession
    RS_ATTR_IO_SESSION_TYPE         ... not supported
    */

    /*- Version Info -*/                     

    p2value = RSSFU_MAJOR_VERSION;
    checkErr (Rs_SetAttribute (instrSession, "", RS_ATTR_SPECIFIC_DRIVER_MAJOR_VERSION, 0, &p2value));
    p2value = RSSFU_MINOR_VERSION;
    checkErr (Rs_SetAttribute (instrSession, "", RS_ATTR_SPECIFIC_DRIVER_MINOR_VERSION, 0, &p2value));
    p2value = RSSFU_MINOR_MINOR_VERSION;
    checkErr (Rs_SetAttribute (instrSession, "", RS_ATTR_SPECIFIC_DRIVER_MINOR_MINOR_VERSION, 0, &p2value));
    p2value = RSSFU_CLASS_SPEC_MAJOR_VERSION;
    checkErr (Rs_SetAttribute (instrSession, "", RS_ATTR_CLASS_DRIVER_MAJOR_VERSION, 0, &p2value));
    p2value = RSSFU_CLASS_SPEC_MINOR_VERSION;
    checkErr (Rs_SetAttribute (instrSession, "", RS_ATTR_CLASS_DRIVER_MINOR_VERSION, 0, &p2value));
    p2value = RS_ENGINE_MAJOR_VERSION;
    checkErr (Rs_SetAttribute (instrSession, "", RS_ATTR_ENGINE_MAJOR_VERSION, 0, &p2value));
    p2value = RS_ENGINE_MINOR_VERSION;
    checkErr (Rs_SetAttribute (instrSession, "", RS_ATTR_ENGINE_MINOR_VERSION, 0, &p2value));
    checkErr (Rs_SetAttribute (instrSession, "", RS_ATTR_ENGINE_REVISION, 0, RS_ENGINE_REVISION));

    /*- Session Callbacks -*/                

    p2void = rssfu_WaitForOPCCallback;
    checkErr (Rs_SetAttribute (instrSession, "", RS_ATTR_OPC_CALLBACK, 0, &p2void));
    p2void = rssfu_CheckStatusCallback;
    checkErr (Rs_SetAttribute (instrSession, "", RS_ATTR_CHECK_STATUS_CALLBACK, 0, &p2void));
    p2value = RSSFU_OPC_TIMEOUT;
    checkErr (Rs_SetAttribute (instrSession, "", RS_ATTR_OPC_TIMEOUT, 0, &p2value));


    /*----------------------------------------------------------------------*/

Error:
    return error;
}

#if 1
/*****************************************************************************
 * Function: rssfu_GetActualRepCaps
 * Purpose:  This function returns the actual repeated capability string
 *****************************************************************************/
static ViStatus rssfu_GetActualRepCaps (ViSession instrSession, 
                                        ViAttr attribute, 
                                        ViString repCap,
                                        ViInt32 index,
                                        ViChar retVal[])
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr (rssfu_GetAttributeRepeatedCapabilityIdNames (instrSession, attribute, 
                                                repCap, RS_MAX_MESSAGE_BUF_SIZE, buffer));
    checkErr (Rs_GetTokenAtIndex(buffer,",",(ViUInt32) index, retVal, RSSFU_TOKEN_BUFFER_SIZE));
  
Error:
    return error;  
}
#else

ViStatus rssfu_CreateLayerRepCap (ViInt32 layer, ViChar buf[], ViInt32 param)
{
    if ((layer < RSSFU_VAL_IQC_ISDBT_LAYER_A) || (layer > RSSFU_VAL_IQC_ISDBT_LAYER_C))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, param, "Layer"); 
    
    buf[0] = 'A' + layer;
    
    return VI_SUCCESS;
}
#endif

static ViStatus rssfu_ReplaceString (char *str, char *orig, char *rep)
{

    int error = VI_ERROR_INV_PARAMETER;
    static char buffer[4096];
    char *b = buffer;
    char *p;
    char *r = str;
    
    if(strlen(str)>4000)
     return error;

    while((p = strstr(r, orig))){
        strncpy(b, r, p-r);
        b=b+(p-r);
        r = p+strlen(orig);
        sprintf(b, "%s", rep);
        b=b+strlen(rep);
    }
    sprintf(b, "%s", r);
    sprintf(str, "%s", buffer);

    return VI_SUCCESS; 
}
 
 

/*****************************************************************************
 *------------------- End Instrument Driver Source Code ---------------------*
 *****************************************************************************/
